<?

 /*****************************************************************************
 * Filename: include/api/api.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class API
 {
  function __construct($mode=false, $config=false)
  {
   global $_path, $_config;

   $this->e=new Errors();

   $this->clear();

   $this->mode=false;
   if(!isset($this->modes))
    $this->modes=Array("server", "client");

   if(!$mode)
    $mode=$this->modes[0];

   $this->version="1.0";
   $this->encoding="ISO-8859-1";
   $this->contentType="text/plain";

   if(get_class($this)=="API")
    $this->type=false;
   else if(preg_match("/^API_([A-Za-z]+)$/", get_class($this), $m))
   {
    $this->type=$m[1];
    unset($m);
   }
   else
   {
    $this->e->add("Unable to determine API type from: \"".get_class($this)."\"", "system");
    return;
   }

   //store default request headers for API version detection
   $this->setRequestResponseHeaders(Headers::requestS());

   //get API config
   $this->config=$_config->merge("API", $config);

   if(!in_array($mode, $this->modes))
   {
    $this->e->add($this->type." API unknown mode: \"".$mode."\"", "system");
    return;
   }

   $this->mode=$mode;
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }


  function clear()
  {
   $this->output="";
   $this->filename=false;

   $this->commands=Array(
   /*
     Array("type" => "data", "data" => Data::cache("module", "action", "extra"), "count" => true/false),
     Array("type" => "language", "language" => Language(), "lang" => "en", "set" => "set"),
     Array("type" => "modules", "modules" => JAS::modules()),
     Array("type" => "projects", "projects" => JAS::projects(), "activate" => "project" / false),
   */
   );
  }

  function error()
  {
   $this->output="";
   for($c=0;$c<$this->e->getErrors();$c++)
   {
    $this->output.=$this->e->getLevel($c).": ";
    $this->output.=$this->e->getError($c)."\n";
   }

   return true;
  }

  function filename($filename)
  {
   $this->filename=$filename;
   return true;
  }

  function setRequestResponseHeaders($headers=false)
  {
   if($headers)
   {
    if(is_string($headers)) //headers string
     $headers=explode("\n", $headers); //make lines

    if(array_type($headers)=="indexed") //lines
    {
     $tmp=Array(); //get key/value pairs
     foreach($headers as $header)
      if(preg_match("/^([^:]+)\s*:\s(.*)$/", $header, $m))
       $tmp[$m[1]]=$m[2];

     $headers=$tmp;
    }

    if(array_type($headers)!="associative")
    {
     $this->e->add($this->type." API headers array isn't an associative array", "system");
     return false;
    }
   }
   else //empty headers clear requestHeaders
    $headers=Array();

   $this->requestResponseHeaders=$headers;
   return true;
  }

  function getHeaderVersion()
  {
   foreach($this->requestResponseHeaders as $header => $value)
    if($header=="X-JAS-API") //API header found
     return ($this->type?str_replace($this->type."-", "", $value):$value);

   return false;
  }

  function output($headers=true)
  {
   if($headers&&$this->mode=="server")
   {
    $h=new Headers();
    $h->set("X-JAS-API", ($this->type?$this->type."-":"").$this->version);
    if($this->filename)
     $h->file($this->filename, $this->contentType, strlen($this->output));
    else
    {
     $h->type($this->contentType);
     $h->size(strlen($this->output));
    }
   }

   return $this->output;
  }

  function parse($request)
  {
   //convert request commands list

   $this->e->add($this->type." API function: \"parse\" not defined", "system");
   return false;
  }

  function format()
  {
   //convert commands to output format

   $this->e->add($this->type." API function: \"format\" not defined", "system");
   return false; //should return format
  }


  function enabled($cmd=false, $module=false, $action=false, $extra=false)
  {
   switch($cmd)
   {
    case false:
    {
     if(!$this->config[strtolower($this->type)])
     {
      $this->e->add($this->type." API is disabled", "error");
      return false;
     }

     break;
    }
    case "data":
    {
     if($action=="view"||$action=="tree"||$action=="database") //extra relevant
     {
      if(!$module||!$action||!$extra)
      {
       $this->e->add($this->type." API data action requires module, action and extra to be set", "error");
       return false;
      }

      $type=$action.ucfirst($extra);
     }
     else if($action=="search") //configAPI search field name hack
      $type="searching";
     else if($action=="user")
     {
      if($extra=="password")
       $type=$action."Password";
      else //login/logout
       $type=$action."LoginLogout";
     }
     else //extra not relevant
     {
      if(!$module||!$action)
      {
       $this->e->add($this->type." API data action requires module and action to be set", "error");
       return false;
      }

      $type=$action;
     }

     if(!isset($this->config[$type]))
     {
      $this->e->add($this->type." API unknown action: ".$action." / ".$extra, "error");
      return false;
     }

     if(!$this->config[$type])
     {
      $this->e->add($this->type." API action: ".$action." / ".$extra." is disabled", "error");
      return false;
     }

     if(isset($this->config[$type."AllModules"])&&!$this->config[$type."AllModules"]) //module whitelist
     {
      if(!is_array($this->config[$type."Modules"])||!in_array($module, $this->config[$type."Modules"]))
      {
       $this->e->add($this->type." API action: ".$action." / ".$extra." is disabled for module: ".$module, "error");
       return false;
      }
     }
     //else all modules allowed

     break;
    }
    default: //language, modules, projects, etc
    {
     if(!isset($this->config[$cmd]))
     {
      $this->e->add($this->type." API unknown command: ".$cmd." ", "error");
      return false;
     }

     if(!$this->config[$cmd])
     {
      $this->e->add($this->type." API ".$cmd." command is disabled", "error");
      return false;
     }

     break;
    }
   }

   return true;
  }


  function addData(&$data, $count=false)
  {
   $nr=sizeof($this->commands);
   return $this->setData($nr, $data, $count);
  }
  function setData($nr, &$data, $count=false)
  {
   $type="data";

   if(!is_object($data)||!get_class($data)=="Data")
   {
    $this->e->add($this->type." API ".$type." command has no data object", "system");
    return false;
   }

   $this->commands[$nr]["type"]=$type;
   $this->commands[$nr]["data"]=&$data;
   $this->commands[$nr]["count"]=$count;
   return true;
  }

  function addLanguage($set, $lang, $language=false)
  {
   $nr=sizeof($this->commands);
   return $this->setLanguage($nr, $set, $lang, $language);
  }
  function setLanguage($nr, $set, $lang, $language=false)
  {
   $type="language";

   if($language===false)
    $language=new Language();

   if(!is_object($language)||!get_class($language)=="Language")
   {
    $this->e->add($this->type." API ".$type." command has no language object", "system");
    return false;
   }

   $this->commands[$nr]["type"]=$type;
   $this->commands[$nr]["set"]=$set;
   $this->commands[$nr]["lang"]=$lang;
   $this->commands[$nr]["language"]=$language;
   return true;
  }

  function addModules($modules=false, $all=false)
  {
   $nr=sizeof($this->commands);
   return $this->setModules($nr, $modules, $all);
  }
  function setModules($nr, $modules=false, $all=false)
  {
   $type="modules";

   if($modules===false)
    $modules=Array();

   if(!is_array($modules))
   {
    $this->e->add($this->type." API ".$type." command has no modules list", "system");
    return false;
   }

   //validate array
   $l=sizeof($modules);
   for($c=0;$c<$l;$c++)
   {
    if(is_object($modules[$c]))
    {
     if(!get_class($modules[$c])=="Data")
     {
      $this->e->add($this->type." API ".$type." command modules list contains an incorrect module", "system");
      return false;
     }
    }
    else if(!is_string($modules[$c]))
    {
     $this->e->add($this->type." API ".$type." command modules list contains an incorrect module", "system");
     return false;
    }
   }

   $this->commands[$nr]["all"]=$all;
   $this->commands[$nr]["type"]=$type;
   $this->commands[$nr]["modules"]=$modules;
   return true;
  }
  function mergeModules(&$d)
  {
   //look up module in commands
   $found=false;
   $l=sizeof($this->commands);
   for($c=0;$c<$l;$c++)
   {
    //validate module
    if($this->commands[$c]["type"]=="modules") //modules list
    {
     $l2=sizeof($this->commands[$c]["modules"]);
     for($x=0;$x<$l2;$x++)
     {
      if(is_object($this->commands[$c]["modules"][$x])) //module object
      {
       if($this->commands[$c]["modules"][$x]->module==$d->module) //name match
       {
        if(!$this->compareModules($this->commands[$c]["modules"][$x], $d)) //validate module against local definition
         return false;

        $found=true;
        break;
       }
      }
      else //module name
      {
       if($this->commands[$c]["modules"][$x]==$d->module) //name match
       {
        $this->commands[$c]["modules"][$x]=&$d; //set module definition
        $found=true;
        break;
       }
      }
     }
    }

    if($found)
     break;
   }

   if(!$found) //module not found in command list
   {
    //find modules command
    for($c=0;$c<$l;$c++)
    {
     //validate module
     if($this->commands[$c]["type"]=="modules") //modules list
     {
      //add module definition to modules command
      $this->commands[$c]["modules"][]=&$d;
      $found=true;
     }
    }

    if(!$found) //modules command found
    {
     if(!$this->addModules(Array(&$d)))
      return false;
    }
   }

   return true;
  }
  function compareModules(&$expected, &$received)
  {
   $success=true;

   if($expected->module!=$received->module)
   {
    $this->e->add($this->type." API compare module expected module: \"".$expected->module."\" doesn't match \"".$received->module."\"", "error");
    $success=false;
   }

   if($expected->title!=$received->title)
   {
    $this->e->add($this->type." API compare module expected title: \"".$expected->title."\" doesn't match \"".$received->title."\"", "error");
    $success=false;
   }

   //check fields
   foreach(array_keys($expected->field) as $field)
   {
    if($expected->check("properties", "control", $field))
     continue;

    if(!isset($received->field[$field]))
    {
     $this->e->add($this->type." API compare module expected field: \"".$field."\" not found", "error");
     $success=false;
     continue;
    }

    foreach(Array("type", "use", "properties", "min", "max", "title", "text") as $type)
    {
     if($expected->field[$field][$type]!=$received->field[$field][$type])
     {
      $this->e->add($this->type." API compare module expected ".$type.": \"".$expected->field[$field][$type]."\" doesn't match \"".$received->field[$field][$type]."\" for field: \"".$field."\"", "error");
      $success=false;
     }
    }

    if($expected->field[$field]["original"]["defaultvalue"]!=$received->field[$field]["original"]["defaultvalue"])
    {
     $this->e->add($this->type." API compare module expected default: \"".$expected->field[$field]["original"]["defaultvalue"]."\" doesn't match \"".$received->field[$field]["original"]["defaultvalue"]."\" for field: \"".$field."\"", "error");
     $success=false;
    }
   }

   //check reference
   foreach(array_keys($expected->reference) as $reference)
   {
    if(!isset($received->reference[$reference]))
    {
     $this->e->add($this->type." API compare module expected reference: \"".$reference."\" not found", "error");
     $success=false;
     continue;
    }

    foreach(Array("module", "to", "type", "from") as $type)
    {
     if($expected->reference[$reference][$type]!=$received->reference[$reference][$type])
     {
      $this->e->add($this->type." API compare module expected ".$type.": \"".$expected->reference[$reference][$type]."\" doesn't match \"".$received->reference[$reference][$type]."\" for reference: \"".$reference."\"", "error");
      $success=false;
     }
    }
   }

   return $success;
  }


  function addProjects($activate=false, $projects=false)
  {
   $nr=sizeof($this->commands);
   return $this->setProjects($nr, $activate, $projects);
  }
  function setProjects($nr, $activate=false, $projects=false)
  {
   $type="projects";

   if($projects===false)
    $projects=Array();

   if(!is_array($projects))
   {
    $this->e->add($this->type." API ".$type." command has no projects list", "system");
    return false;
   }

   //validate array
   $l=sizeof($projects);
   for($c=0;$c<$l;$c++)
   {
    if(!is_string($projects[$c]))
    {
     $this->e->add($this->type." API ".$type." command projects list contains an incorrect project", "system");
     return false;
    }
   }

   $this->commands[$nr]["type"]=$type;
   $this->commands[$nr]["activate"]=$activate;
   $this->commands[$nr]["projects"]=$projects;
   return true;
  }


  function deleteCommand($nr)
  {
   if(!isset($this->commands[$nr]))
   {
    $this->e->add($this->type." API can't delete command #".$nr.", out of range", "system");
    return false;
   }

   $this->commands=array_merge(array_slice($this->commands, 0, $nr), array_slice($this->commands, $nr+1));
   return true;
  }


  function commands($nr=false)
  {
   if($this->mode!="server")
   {
    $this->e->add($this->type." API won't execute commands in \"".$this->mode."\" mode", "system");
    return false;
   }

   if(!$this->enabled())
    return false;

   if($nr===false) //all commands
   {
    $success=true;
    $l=sizeof($this->commands);

    for($c=0;$c<$l;$c++)
     if(!$this->commands($c))
      $success=false;

    return $success;
   }

   if(!isset($this->commands[$nr]))
   {
    $this->e->add($this->type." API command #".$nr." doesn't exist", "system");
    return false;
   }

   //check if command is allowed
   if($this->commands[$nr]["type"]=="data")
   {
    $d=&$this->commands[$nr]["data"];

    if(!$this->enabled($this->commands[$nr]["type"], $d->module, $d->action, $d->extra))
     return false;
   }
   else //language, modules, projects, etc
   {
    if(!$this->enabled($this->commands[$nr]["type"]))
     return false;
   }

   //execute commands
   $function="command_".$this->commands[$nr]["type"];
   if(!in_array($function, get_class_methods($this)))
   {
    $this->e->add($this->type." API unsupported command \"".$this->commands[$nr]["type"]."\"", "system");
    return false;
   }

   if(!$this->$function($nr))
    return false;

   return true;
  }


  /*
   * Data
   */

  function command_data($nr=0)
  {
   if($this->commands[$nr]["type"]!="data")
   {
    $this->e->add($this->type." command #".$nr." isn't a data command", "error");
    return false;
   }

   $d=&$this->commands[$nr]["data"];

   if(($d->action=="view"||$d->action=="tree"||$d->action=="search"))
    $submit=false;
   else if(($d->action=="add"||$d->action=="edit"||$d->action=="move"||$d->action=="delete"||$d->action=="undelete"))
    $submit=true;
   else if(($d->action=="user"&&($d->extra=="login"||$d->extra=="logout"||$d->extra=="password")))
    $submit=true;
   else if(($d->action=="database"&&($d->extra=="info")))
    $submit=false;
   else
   {
    $this->e->add($this->type." API unsupported action: \"".$d->action." / ".$d->extra."\"", "error");
    return false;
   }

   if($submit)
   {
    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }
    else
    {
     $this->e->add($d->e);
     return true;
    }
   }

   if($d->action=="database"&&$d->extra=="info"&&$d->check("valuetype", "set", "info_updated")&&$d->getValue("info_updated")) //poll table for updates
   {
    session_write_close(); //don't write session data after this, this will open up the server for other requests instead of queuing them after this potentially long request

    $updates=false;
    $previousUpdate=DT::dateConvertS($d->getValue("info_updated"), $d->field["info_updated"]["format"][$d->dataformat]["datetimeformat"]); //get previous update time from XML

    $timeout=ini_get("max_execution_time"); //polling timeout after max_execution_time
    $stoptime=$timeout+time();

    //poll till timeout occurs
    while(true)
    {
     if(!$d->get()) //get info
     {
      $this->e->add($d->e);
      return false;
     }

     $updated=DT::dateConvertS($d->getValue("info_updated"), $d->field["info_updated"]["format"][$d->dataformat]["datetimeformat"]); //get current update time

     if($updated>$previousUpdate) //updated after previous update time
     {
      $updates=true; //set updated, returns new update time
      break;
     }

     if($stoptime<=time()) //timeout happend
      break; //stop polling

     usleep(0.1*1000000); //sleep .1 second before next poll
    }

    if(!$updates)
    {
     $this->e->add($this->type." API no updates for: \"".$d->module."\"", "note", "Timeout: ".$timeout." second(s)", "note");
     return false;
    }
   }
   else
   {
    if(!$d->get(0, $this->commands[$nr]["count"]))
    {
     $this->e->add($d->e);
     return false;
    }
   }

   if($d->action=="user") //login / logout
   {
    $this->e->add($d->e);
    return true;
   }

   return true;
  }


  /*
   * Language
   */

  function command_language($nr=0)
  {
   if($this->commands[$nr]["type"]!="language")
   {
    $this->e->add($this->type." command #".$nr." isn't a language command", "error");
    return false;
   }

   $l=&$this->commands[$nr]["language"];
   $l->clear();

   if($this->commands[$nr]["lang"]==$l->original)
   {
    $this->e->add($this->type." API not getting set \"".$this->commands[$nr]["set"]."\" for original language \"".$l->original."\"", "warning");
    return false;
   }

   if(!$l->read($this->commands[$nr]["set"], $this->commands[$nr]["lang"]))
   {
    $this->e->add($this->type." API unable to read language set: \"".$this->commands[$nr]["set"]."\" for language: \"".$this->commands[$nr]["lang"]."\"", "error");
    return false;
   }

   if(!isset($l->translations["raw"][$l->original][$this->commands[$nr]["lang"]][$this->commands[$nr]["set"]]))
   {
    $this->e->add($this->type." API no translation found for set \"".$this->commands[$nr]["set"]."\" for language \"".$this->commands[$nr]["lang"]."\" ", "error");
    return false;
   }

   return true;
  }


  /*
   * Modules
   */

  function command_modules($nr=0)
  {
   if($this->commands[$nr]["type"]!="modules")
   {
    $this->e->add($this->type." command #".$nr." isn't a modules command", "error");
    return false;
   }

   if(!$this->commands[$nr]["modules"]) //module list
   {
    foreach(JAS::modules() as $module)
    {
     $this->commands[$nr]["modules"][]=$module;

     if($this->commands[$nr]["all"]) //all modules
     {
      //get *:* sub modules
      $d=Data::cache($module, "view", "item", Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true));
      foreach(array_keys($d->reference) as $ref)
       if($d->reference[$ref]["type"]=="*:*")
        $this->commands[$nr]["modules"][]=$d->module.":".$d->reference[$ref]["data"]->module;
     }
    }
   }
   else //module information
   {
    $l=sizeof($this->commands[$nr]["modules"]);
    $allModules=JAS::modules();

    for($c=0;$c<$l;$c++)
    {
     $module=explode(":", $this->commands[$nr]["modules"][$c]);
     if(!in_array($module[0], $allModules))
     {
      $this->e->add($this->type." API unknown module: \"".$module[0]."\"", "error");
      return false;
     }

     $d=Data::cache($module[0], "view", "item", Array("skipCheckTables" => true));

     if(isset($module[1])) //*:* referenced module
     {
      if(!isset($d->reference[$module[1]]))
      {
       $this->error("Unknown referenced module: \"".$module[1]."\"");
       return false;
      }

      if($d->reference[$module[1]]["type"]!="*:*")
      {
       $this->error("Referenced module: \"".$module[1]."\" isn't \"*:*\" type");
       return false;
      }

      $d=&$d->reference[$module[1]]["data"];
     }

     $this->commands[$nr]["modules"][$c]=&$d;;
     unset($d);
    }
   }

   return true;
  }


  /*
   * Projects
   */

  function command_projects($nr=0)
  {
   global $_session, $_config, $_path;

   if($this->commands[$nr]["type"]!="projects")
   {
    $this->e->add($this->type." command #".$nr." isn't a projects command", "error");
    return false;
   }

   $this->commands[$nr]["projects"]=JAS::projects();

   if($this->commands[$nr]["activate"])
   {
    if(!in_array($this->commands[$nr]["activate"], $this->commands[$nr]["projects"]))
    {
     $this->e->add($this->type." API can't activate unknown project: \"".$this->commands[$nr]["activate"]."\"", "error");
     return false;
    }

    $_session->set("project", $this->commands[$nr]["activate"]); //ovewrite project variable

    //reload path
    unset($_path["project"]);
    require($_path["include"]."path.php");

    $_config->set(false); //clear config (including user/groups)
    $_config->load();
   }

   $this->commands[$nr]["activate"]=$_session->get("project");
   return true;
  }
 }

?>
