<?

 /*****************************************************************************
 * Filename: include/api/csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");
 require_once($_path["format"]."csv.php");
 require_once($_path["api"]."api.php");

 class API_CSV extends API
 {
  function __construct($mode=false, $config=false)
  {
   parent::__construct($mode, $config);

   $this->contentType="text/csv";
   $this->csvSeparators=Array(false, false, false); //use CSV defaults
  }


  function error()
  {
   switch($this->version)
   {
    case "1.0":
    {
     if($this->output=$this->format_errors_1_0())
      return true;

     break;
    }
    default: break;
   }

   return parent::error();
  }


  function parse($csv)
  {
   if(!$csv)
   {
    $this->e->add($this->type." API no parse ".($this->mode=="client"?"response":"request")." provided", "error");
    return false;
   }

   if($this->mode=="server")
   {
    if($version=$this->getHeaderVersion()) //header version
     $this->version=$version; //use header version
   }
   else //client
   {
    if(!($version=$this->getHeaderVersion())) //no header version
     $this->e->add($this->type." API header version not set, using version: \"".$this->version."\"", "warning");
    else if($this->version!=$version) //version mismatch
    {
     $this->e->add($this->type." API header version: \"".$version."\" doesn't match version: \"".$this->version."\"", "error");
     return false;
    }
   }

   //detect separators
   if(in_array(ltrim($csv)[0], Array("\"", "'", "`"))) //use quote
    $this->csvSeparators[1]=ltrim($csv)[0];
   else //no quotes
    $this->csvSeparators[1]="";

   if(preg_match("/^".$this->csvSeparators[1]."[a-z]+".$this->csvSeparators[1]."([^\n\r])?/", $csv, $m)&&isset($m[1])) //find separator
    $this->csvSeparators[0]=$m[1]; //use custom separator

   if($linebreak=Mutilate::detectLineBreaksS($csv)) //get linebreask
    $this->csvSeparators[2]=$linebreak; //use custom linebreak

   $tmp=new Format_CSV();
   if(!$tmp->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   switch($this->version)
   {
    case "1.0":
    {
     $cmds=Array();
     if($this->csvSeparators[2]) //multiline
      $cmds=explode($this->csvSeparators[2].$this->csvSeparators[2], $csv); //get command sections (2 linebreaks)
     else //single line
      $cmds=Array($csv);

     foreach($cmds as $cmd)
     {
      if(!$cmd) //excess linebreaks
       continue;

      $tmp->clear();
      if(!$tmp->parse($cmd, false, (preg_match("/[\r\n]/", $cmd)?true:false)))
      {
       $this->e->add($tmp->e);
       return false;
      }

      $function="parse_".strtolower($tmp->data[0][0])."_".strtr($this->version, ".", "_");
      if(!in_array($function, get_class_methods($this)))
      {
       $this->e->add($this->type." API unsupported parse ".strtolower($tmp->data[0][0])." ".($this->mode=="client"?"response":"request")." for version: \"".$this->version."\"", "error");
       return false;
      }

      $tmp->reset();
      if(!$this->$function($tmp))
       return false;
     }

     break;
    }
    default:
    {
     $this->e->add($this->type." API unsupported version: \"".$this->version."\"", "error");
     return false;
    }
   }


   return true;
  }


  function format($nr=false)
  {
   if($this->csvSeparators[1]==="") //no quotes
    $this->csvSeparators[1]=false; //use default

   if($nr===false) //all commands
   {
    $this->output="";

    $success=true;
    $l=sizeof($this->commands);

    for($c=0;$c<$l;$c++)
     if(!$this->format($c))
      $success=false;

    return $success;
   }

   //format command
   switch($this->version)
   {
    case "1.0":
    {
     $function="format_".$this->commands[$nr]["type"]."_".strtr($this->version, ".", "_");
     if(!in_array($function, get_class_methods($this)))
     {
      $this->e->add($this->type." API unsupported format ".$this->commands[$nr]["type"]." ".($this->mode=="client"?"request":"response")." for version: \"".$this->version."\"", "error");
      return false;
     }

     if(!($output=$this->$function($nr)))
      return false;

     $this->output.=$output;
     break;
    }
    default:
    {
     $this->e->add($this->type." API unsupported version: \"".$this->version."\"", "error");
     return false;
    }
   }

   return true;
  }


  /*
   * Data
   */

  function parse_data_1_0($csv)
  {
   if(!$csv->next())
   {
    $this->e->add($this->type." API data header not provided", "error");
    return false;
   }

   if(!isset($csv->v[1])||!$csv->v[1])
   {
    $this->e->add($this->type." API no module set for data request", "error");
    return false;
   }

   if(!isset($csv->v[2])||!$csv->v[2])
   {
    $this->e->add($this->type." API no action set for data request", "error");
    return false;
   }

   if(!isset($csv->v[3])||!$csv->v[3])
   {
    $this->e->add($this->type." API no extra action set for data request", "error");
    return false;
   }

   $module=$csv->v[1];
   $action=$csv->v[2];
   $extra=$csv->v[3];

   $options=Array("skipCheckTables" => true);
   if(isset($csv->v[4])&&$csv->v[4])
    $options["format"]=$csv->v[4];
   else
    $options["format"]="api";

   if(isset($csv->v[5])&&$csv->v[5])
    $options["db"]=$csv->v[5];

   $module=explode(":", $module);
   $d=Data::cache($module[0], $action, $extra, $options);

   if(isset($module[1])) //*:* referenced module
   {
    if(!isset($d->reference[$module[1]]))
    {
     $this->e->add($this->type." API unknown referenced module: \"".$module[1]."\"", "error");
     return false;
    }

    if($d->reference[$module[1]]["type"]!="*:*")
    {
     $this->e->add($this->type." API referenced module: \"".$module[1]."\" isn't \"*:*\" type", "error");
     return false;
    }

    $d=&$d->reference[$module[1]]["data"];
    $d->setAction($action);
    $d->setExtra($extra);
   }

   $d->clearValues(); //ensure no default values are set

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   if(isset($csv->v[6])&&($count=$csv->v[6])&&strtolower($count)=="true")
    $count=true;
   else
    $count=false;

   $show=Array();
   if($csv->next()) //fields header
   {
    foreach($csv->v as $field)
    {
     if(!$field)
      continue;

     $show[]=$field;
    }
   }

   if($show)
   {
    $d->setShowFields($show);

    //values (request fields header
    $row=0;
    while($csv->next())
    {
     if(!$csv->v||(sizeof($csv->v)==1&&!$csv->v[0])) //empty line
      break; //done with value rows

     foreach($csv->v as $k => $v)
     {
      if(!isset($show[$k]))
      {
       $this->e->add($this->type." API can't set value for unknown field #".$k, "error");
       return false;
      }

      $field=$show[$k];

      if($d->field[$field]["type"]=="filedata")
       $v=base64_decode($v);

      if(!$d->setValue($field, $v, $row))
      {
       $this->e->add($this->type." API unable to set value for field: \"".$field."\"", "error");
       return false;
      }
     }

     if($setFields=$d->check("valuetype", "set")) //set OR operant for last field
     {
      if(!$d->setOperant($setFields[sizeof($setFields)-1], "OR", $row))
      {
       $this->e->add($this->type." API unable to set operant for field: \"".$setFields[sizeof($setFields)-1]."\"", "error");
       return false;
      }
     }

     $row++;
    }
   }

   $sort=Array();
   while($csv->next()) //more field found
   {
    if($csv->v[0]=="sort") //sort fields
    {
     $l=sizeof($csv->v);
     for($c=1;$c<$l;$c++)
     {
      if(!$csv->v[$c])
       continue;

      $sort[]=$csv->v[$c];
     }
    }
    else if($csv->v[0]=="mutilation") //mutilation
    {
     $field=$csv->v[1];
     $type=$csv->v[2];
     $args=array_slice($csv->v, 3);

     $d->addMutilation($field, $type, $args);
    }
   }

   if($sort)
    $d->sortOrder($sort);

   if(!$this->addData($d, $count))
    return false;

   return true;
  }

  function format_data_1_0($nr=0)
  {
   $csv=new Format_CSV();
   if(!$csv->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   $d=&$this->commands[$nr]["data"];

   //show fields
   $show=Array();
   if($d->action=="database"&&$d->extra=="info")
   {
    foreach($d->check("use", "info") as  $field)
     $show[]=$field;
   }
   else
   {
    foreach(array_keys($d->field) as $field)
     if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
      if(!$d->check("properties", "control", $field))
       $show[]=$field;
   }

   if($this->mode=="server")
    $fields=$show;
   else //client
   {
    if($d->action=="database"&&$d->extra=="info")
     $fields=Array();
    else //data
     $fields=$d->check("valuetype", "set");
   }

   if(!$csv->addLine("data", $d->module, $d->action, $d->extra, $d->dataformat, "", ($this->commands[$nr]["count"]?"true":"")))
   {
    $this->e->add($csv->e);
    return false;
   }

   foreach($fields as $field) //merge set + show fields
    if(!in_array($field, $show))
     $show[]=$field;

   if(!$csv->addLine($show))
   {
    $this->e->add($csv->e);
    return false;
   }

   $d->setShowFields($fields); //ensure Data::next set correct values

   $d->reset();
   while($d->next())
   {
    $tmp=Array();
    foreach($show as $field)
    {
     if(!in_array($field, $fields)) //show field
      $tmp[]=false; //don't set value
     else if(is_array($d->v[$field])) //set field
     {
      if(isset($d->reference[$field]))
      {
       foreach($d->reference[$field]["data"]->check("use", "show") as $k2)
       {
        if(!isset($d->v[$field][$k2]))
         continue;

        if(is_array($d->v[$field][$k2]))
         $tmp[]=implode($csv->subSplit, $d->v[$field][$k2]);
        else
         $tmp[]=$d->v[$field][$k2];
       }
      }
      else
       $tmp[]=implode($csv->subSplit, $d->v[$field]);
     }
     else if($d->field[$field]["type"]=="filedata") //set file data inside tag
      $tmp[]=base64_encode($d->v[$field]);
     else
      $tmp[]=$d->v[$field];
    }

    if(!$csv->addLine($tmp))
    {
     $this->e->add($csv->e);
     return false;
    }
   }

   //sorting
   if($d->extra!="info"&&$d->extra!="item")
    $sort=$d->sortOrder;
   else
    $sort=Array();

   //mutilations
   $mutilations=Array();
   foreach(array_keys($d->field) as $field)
    if($d->field[$field]["mutilation"])
     $mutilations[$field]=$d->field[$field]["mutilation"];

   if($sort||$mutilations)
   {
    if(!$csv->addLine()) //empty
    {
     $this->e->add($csv->e);
     return false;
    }

    if($sort)
    {
     array_unshift($sort, "sort");
     if(!$csv->addLine($sort)) //add sort fields
     {
      $this->e->add($csv->e);
      return false;
     }
    }

    foreach($mutilations as $field => $mutilation)
    {
     foreach($mutilation as $type => $args)
     {
      $row=Array("mutilation", $field, $type);

      if($args)
       $row=array_merge($row, $args);

      if(!$csv->addLine($row)) //add mutilation
      {
       $this->e->add($csv->e);
       return false;
      }
     }
    }
   }


   return $csv->output(false).$csv->linebreak;
  }


  /*
   * Language
   */

  function parse_language_1_0($csv)
  {
   global $_config;

   if(!$csv->next())
   {
    $this->e->add($this->type." API language header not provided", "error");
    return false;
   }

   if(!isset($csv->v[1])||!$csv->v[1])
   {
    $this->e->add($this->type." API no set for language request", "error");
    return false;
   }

   $set=$csv->v[1];

   if(!isset($csv->v[2])||!$csv->v[2])
    $lang=$_config->get("language");
   else
    $lang=$csv->v[2];

   if($this->mode=="server")
   {
    if(!$this->addLanguage($set, $lang))
     return false;
   }
   else //client
   {
    if(!$csv->next())
    {
     $this->e->add($this->type." API language translation header not provided", "error");
     return false;
    }

    if(!isset($csv->v[0])||!$csv->v[0])
    {
     $this->e->add($this->type." API original language not provided", "error");
     return false;
    }

    if(!isset($csv->v[1])||!$csv->v[1])
    {
     $this->e->add($this->type." API translation language not provided", "error");
     return false;
    }

    if($csv->v[1]!=$lang)
    {
     $this->e->add($this->type." API translation language differs from set language", "error");
     return false;
    }

    $l=new Language();
    $l->setOriginal($csv->v[0]); //set original language

    while($csv->next())
     if(!$l->setTranslation($set, $l->original, $lang, $csv->v[0], $csv->v[1]))
      return false;

    if(!$this->addLanguage($set, $lang, $l))
     return false;
   }

   return true;
  }

  function format_language_1_0($nr=0)
  {
   $csv=new Format_CSV();
   if(!$csv->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   if($this->mode=="server")
   {
    $l=&$this->commands[$nr]["language"];

    if(!$csv->addLine(
     "language",
     $this->commands[$nr]["set"],
     $this->commands[$nr]["lang"]
    ))
    {
     $this->e->add($csv->e);
     return false;
    }

    if(!$csv->addLine(
     $l->original,
     $this->commands[$nr]["lang"]
    ))
    {
     $this->e->add($csv->e);
     return false;
    }

    foreach($l->translations["raw"][$l->original][$this->commands[$nr]["lang"]][$this->commands[$nr]["set"]] as $original => $translated)
    {
     if(!$csv->addLine(
      $original,
      $translated
     ))
     {
      $this->e->add($csv->e);
      return false;
     }
    }
   }
   else //client
   {
    if(!$csv->addLine(
     "language",
     $this->commands[$nr]["set"],
     $this->commands[$nr]["lang"]
    ))
    {
     $this->e->add($csv->e);
     return false;
    }
   }

   return $csv->output(false).$csv->linebreak;
  }


  /*
   * Modules
   */

  function parse_modules_1_0($csv)
  {
   if(!$csv->next())
   {
    $this->e->add($this->type." API modules header not provided", "error");
    return false;
   }

   $all=false;
   $modules=Array();
   if($this->mode=="server")
   {
    if(isset($csv->v[1])&&$csv->v[1]=="all")
     $all=true;

    while($csv->next())
     $modules[]=$csv->v[0];
   }
   else //client
   {
    if(isset($csv->v[1])&&$csv->v[1]) //module definition
    {
     //load module (empty)
     $d=Data::cache("empty", false, false, Array("skipCheckTables" => true));

     if(!($d->module=$csv->v[1]))
     {
      $this->e->add($this->type." API module name not provided in header", "error");
      return false;
     }

     if(!($d->title=$csv->v[2]))
     {
      $this->e->add($this->type." API module title not provided in header", "error");
      return false;
     }

     $type=false;
     while($csv->next())
     {
      if(!$csv->v[0]) //linebreak
      {
       $type=false; //start new section
       continue;
      }
      else if(!$type)
      {
       if(in_array($csv->v[0], Array("field", "reference"))) //no section set, line should start with known marker
       {
        $type=$csv->v[0];
        continue;
       }
       else //unknown section
       {
        $this->e->add($this->type." API unknown module section: \"".$csv->v[0]."\"", "error");
        return false;
       }
      }

      switch($type)
      {
       case "field":
       {
        if(sizeof($csv->v)<3)
        {
         $this->e->add($this->type." API module ".$type." section requires at least 3 values", "error");
         return false;
        }

        if(isset($d->field[$csv->v[0]])) //fields may already exist (e.g. verify password is automatically added)
         continue;

        if(!$d->addField(
         $csv->v[0], //name
         $csv->v[1], //type
         $csv->v[2], //use
         (isset($csv->v[3])&&$csv->v[3]!==""?$csv->v[3]:false), //properties
         (isset($csv->v[4])&&$csv->v[4]!==""?(int)$csv->v[4]:false), //min
         (isset($csv->v[5])&&$csv->v[5]!==""?(int)$csv->v[5]:false), //max
         (isset($csv->v[6])&&$csv->v[6]!==""?$csv->v[6]:false), //default
         (isset($csv->v[7])&&$csv->v[7]!==""?$csv->v[7]:false), //title
         (isset($csv->v[8])&&$csv->v[8]!==""?$csv->v[8]:false) //text
        ))
        {
         $this->e->add($this->type." API unable to add field: \"".$csv->v[0]."\"", "error");
         $this->e->add($d->e);
         return false;
        }

        break;
       }
       case "reference":
       {
        if(sizeof($csv->v)!=5)
        {
         $this->e->add($this->type." API module ".$type." section requires 5 values", "error");
         return false;
        }

        if($csv->v[0]!=$csv->v[4]&&isset($d->field[$csv->v[0]])) //reference field may already exist (added above)
        {
         $tmp=$d->field[$csv->v[0]]; //store field
         unset($d->field[$csv->v[0]]); //temporarily remove field
        }
        else
         $tmp=false;

        if(!$d->addReference(
         $csv->v[0], //name
         $csv->v[1], //module
         $csv->v[2], //to
         $csv->v[3], //type
         $csv->v[4] //from
        ))
        {
         $this->e->add($this->type." API unable to add reference: \"".$csv->v[0]."\"", "error");
         $this->e->add($d->e);
         return false;
        }

        if($tmp)
         $d->field[$csv->v[0]]=$tmp; //restore field

        break;
       }
       default:
       {
        $this->e->add($this->type." API unknown module section: \"".$type."\"", "error");
        return false;
       }
      }
     }

     if(!$this->mergeModules($d))
      return false;
    }
    else //module name
    {
     while($csv->next())
      $modules[]=$csv->v[0];
    }

    if(!$modules) //modules already added above
     return true;
   }

   if(!$this->addModules($modules, $all))
    return false;

   return true;
  }

  function format_modules_1_0($nr=0)
  {
   $csv=new Format_CSV();
   if(!$csv->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   if($this->mode=="server")
   {
    if(!is_object($this->commands[$nr]["modules"][0])) //modules list
    {
     if(!$csv->addLine("modules"))
     {
      $this->e->add($csv->e);
      return false;
     }

     foreach($this->commands[$nr]["modules"] as $module)
     {
      if(!$csv->addLine($module))
      {
       $this->e->add($csv->e);
       return false;
      }
     }
    }
    else //get module definition for all request modules
    {
     $l=sizeof($this->commands[$nr]["modules"]);
     for($c=0;$c<$l;$c++)
     {
      $d=&$this->commands[$nr]["modules"][$c];

      if(!$csv->addLine("modules", $d->module, $d->title))
      {
       $this->e->add($csv->e);
       return false;
      }

      if(!$csv->addLine("field", "type", "use", "properties", "min", "max", "defaultvalue", "title", "text"))
      {
       $this->e->add($csv->e);
       return false;
      }

      foreach($d->field as $k => $v)
      {
       if($d->check("properties", "control", $k))
        continue;

       if(!$csv->addLine(
        $k,
        $v["type"],
        $v["use"],
        $v["properties"],
        $v["min"],
        $v["max"],
        (is_array($d->field[$k]["original"]["defaultvalue"])?implode(";", $d->field[$k]["original"]["defaultvalue"]):$d->field[$k]["original"]["defaultvalue"]),
        $v["title"],
        $v["text"]
       ))
       {
        $this->e->add($csv->e);
        return false;
       }
      }

      if(!$csv->addLine())
      {
       $this->e->add($csv->e);
       return false;
      }

      if(!$csv->addLine("reference", "module", "to", "type", "from"))
      {
       $this->e->add($csv->e);
       return false;
      }

      foreach($d->reference as $k => $v)
      {
       if(!$csv->addLine(
        $k,
        $v["module"],
        $v["to"],
        $v["type"],
        $v["from"]
       ))
       {
        $this->e->add($csv->e);
        return false;
       }
      }

      if(!$csv->addLine())
      {
       $this->e->add($csv->e);
       return false;
      }
     }
    }
   }
   else //client
   {
    if(!$csv->addLine("modules"))
    {
     $this->e->add($csv->e);
     return false;
    }

    if($this->commands[$nr]["modules"])
    {
     foreach($this->commands[$nr]["modules"] as $module)
     {
      if(!$csv->addLine((is_object($module)?$module->module:$module)))
      {
       $this->e->add($csv->e);
       return false;
      }
     }
    }
   }

   return $csv->output(false).$csv->linebreak;
  }


  /*
   * Projects
   */

  function parse_projects_1_0($csv)
  {
   if(!$csv->next())
   {
    $this->e->add($this->type." API projects header not provided", "error");
    return false;
   }

   $activate=false;
   $projects=Array();

   if($this->mode=="server")
   {
    if(isset($csv->v[1])&&$csv->v[1])
     $activate=$csv->v[1];
   }
   else //client
   {
    while($csv->next())
    {
     $projects[]=$csv->v[0];

     if(isset($csv->v[1])&&$csv->v[1])
      $activate=$csv->v[0];
    }
   }

   if(!$this->addProjects($activate, $projects))
    return false;

   return true;
  }

  function format_projects_1_0($nr=0)
  {
   $csv=new Format_CSV();
   if(!$csv->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   if($this->mode=="server")
   {
    if(!$csv->addLine("projects", "current"))
    {
     $this->e->add($csv->e);
     return false;
    }

    foreach($this->commands[$nr]["projects"] as $project)
    {
     if(!$csv->addLine($project, ($project==$this->commands[$nr]["activate"]?"true":"")))
     {
      $this->e->add($csv->e);
      return false;
     }
    }
   }
   else //client
   {
    if(!$this->commands[$nr]["activate"])
    {
     if(!$csv->addLine("projects"))
     {
      $this->e->add($csv->e);
      return false;
     }
    }
    else
    {
     if(!$csv->addLine("projects", $this->commands[$nr]["activate"]))
     {
      $this->e->add($csv->e);
      return false;
     }
    }
   }

   return $csv->output(false).$csv->linebreak;
  }


  /*
   * Errors
   */

  function parse_errors_1_0($csv)
  {
   $this->e->clear(); //remove all existing errors (so the error object only contains remote errors)

   if(!$csv->next())
   {
    $this->e->add($this->type." API errors header not provided", "error");
    return false;
   }

   if($csv->v!=Array("errors", "Level", "Extra"))
   {
    $this->e->add($this->type." API errors header incorrect", "error");
    return false;
   }

   //parse errors
   while($csv->next())
    $this->e->add($csv->v[0], $csv->v[1], $csv->v[2]);

   return true;
  }

  function format_errors_1_0()
  {
   $csv=new Format_CSV();
   if(!$csv->separators($this->csvSeparators[0], $this->csvSeparators[1], $this->csvSeparators[2]))
   {
    $this->e->add($tmp->e);
    return false;
   }

   $csv->addLine("errors", "Level", "Extra");
   for($c=0;$c<$this->e->getErrors();$c++)
    $csv->addLine($this->e->getError($c), $this->e->getLevel($c), $this->e->getExtra($c));

   return $csv->output(false).$csv->linebreak;
  }

 }
?>
