<?

 /*****************************************************************************
 * Filename: include/api/data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["api"]."api.php");

 class API_Data extends API
 {
  function __construct($mode=false, $config=false)
  {
   parent::__construct($mode, $config);

   $this->version=JAS::versionS();
  }


  function error()
  {
   if($this->output=$this->format_errors())
    return true;

   return parent::error();
  }


  function output($headers=true)
  {
   $this->output=serialize($this->output); //serialize for outputting
   $output=parent::output($headers); //get serialize version
   $this->output=unserialize($this->output); //restore unserialized output

   return $output;
  }

  function parse($serial)
  {
   if(!$serial)
   {
    $this->e->add($this->type." API no parse ".($this->mode=="client"?"response":"request")." provided", "error");
    return false;
   }

   if($this->mode=="server")
   {
    if($version=$this->getHeaderVersion()) //header version
    {
     if($version!=$this->version) //version mismatch
     {
      $this->e->add($this->type." API header version: \"".$version."\" doesn't match version: \"".$this->version."\"", "error");
      return false;
     }
    }
   }
   else //client
   {
    if(!($version=$this->getHeaderVersion())) //no header version
     $this->e->add($this->type." API header version not set, using version: \"".$this->version."\"", "warning");
    else if($this->version!=$version) //version mismatch
    {
     $this->e->add($this->type." API header version: \"".$version."\" doesn't match version: \"".$this->version."\"", "error");
     return false;
    }
   }

   if(($serial=@unserialize($serial))===false)
   {
    $this->e->add($this->type." API unable to parse serial", "error");
    return false;
   }

   if(!is_array($serial)&&!is_object($serial))
   {
    $this->e->add($this->type." API serial should be an array or an object", "error");
    return false;
   }

   if(is_object($serial)||isset($serial["type"])) //single request
    $serial=Array($serial); //convert to multi request

   $l=sizeof($serial);
   for($c=0;$c<$l;$c++)
   {
    if(is_object($serial[$c]))
    {
     if(get_class($serial[$c])=="Data")
      $function="parse_data";
     else if(get_class($serial[$c])=="Language")
      $function="parse_language";
     else if(get_class($serial[$c])=="Errors")
      $function="parse_error";
     else
     {
      $this->e->add($this->type." API serial should be a Data, Language or Error object", "error");
      return false;
     }

     if(!in_array($function, get_class_methods($this)))
     {
      $this->e->add($this->type." API unsupported parse ".get_class($serial[$c])." ".($this->mode=="client"?"response":"request")." for version: \"".$this->version."\"", "error");
      return false;
     }
    }
    else //array
    {
     if(!isset($serial[$c]["type"]))
     {
      $this->e->add($this->type." API unsupported parse ".($this->mode=="client"?"response":"request")." type not set for version: \"".$this->version."\"", "error");
      return false;
     }

     $function="parse_".$serial[$c]["type"];
     if(!in_array($function, get_class_methods($this)))
     {
      $this->e->add($this->type." API unsupported parse ".$serial[$c]["type"]." ".($this->mode=="client"?"response":"request")." for version: \"".$this->version."\"", "error");
      return false;
     }
    }

    if(!$this->$function($serial[$c]))
     return false;
   }

   return true;
  }


  function format($nr=false)
  {
   if($nr===false) //all commands
   {
    $this->output=Array();

    $success=true;
    $l=sizeof($this->commands);

    for($c=0;$c<$l;$c++)
     if(!$this->format($c))
      $success=false;

    return $success;
   }

   //format command
   $function="format_".$this->commands[$nr]["type"];
   if(!in_array($function, get_class_methods($this)))
   {
    $this->e->add($this->type." API unsupported format ".$this->commands[$nr]["type"]." ".($this->mode=="client"?"request":"response")." for version: \"".$this->version."\"", "error");
    return false;
   }

   if(!($output=$this->$function($nr)))
    return false;

   if(sizeof($this->commands)>1) //multi
    $this->output[]=$output;
   else //single
    $this->output=$output;

   return true;
  }


  /*
   * Data
   */

  function parse_data(&$serial)
  {
   //init proper Data object
   $d=Data::cache($serial->module, $serial->action, $serial->extra, ($serial->dataformat?Array("format" => $serial->dataformat):false));
   $d->vmax=$serial->vmax;
   $d->sortOrder=$serial->sortOrder;

   //copy field values to new Data object
   foreach(array_keys($d->field) as $field)
    if(isset($serial->field[$field]))
     foreach($serial->field[$field] as $k => $v)
      $d->field[$field][$k]=$v;

   if(!$this->addData($d, (isset($serial->_count)&&$serial->_count)))
    return false;

   return true;
  }

  function format_data($nr=0)
  {
   $d=clone $this->commands[$nr]["data"];

   $send=Array("module", "action", "extra", "dataformat", "vmax", "sortOrder", "field"); //send keys
   foreach($d as $key => $value) //remove all but send key
    if(!in_array($key, $send))
     unset($d->$key);

   $send=Array("use", "properties", "value", "valuetype", "operant", "operator", "mutilation"); //send field keys
   foreach(array_keys($d->field) as $field) //clean field array
    foreach(array_keys($d->field[$field]) as $key)
     if(!in_array($key, $send))
      unset($d->field[$field][$key]);

   $d->moduleClassName=false; //no class name so sleep/wakeup doesn't break
   $d->_count=$this->commands[$nr]["count"]; //add count flag

   return $d;
  }


  /*
   * Language
   */

  function parse_language($serial)
  {
   global $_config;

   if($this->mode=="server")
   {
    if(!isset($serial["set"]))
    {
     $this->e->add($this->type." API no set for language request", "error");
     return false;
    }

    $set=$serial["set"];

    if(isset($serial["language"]))
     $lang=$serial["language"];
    else
     $lang=$_config->get("language");

    if(!$this->addLanguage($set, $lang))
     return false;
   }
   else //client
   {
    if(!$this->addLanguage(false, $serial->language, $serial))
     return false;
   }

   return true;
  }

  function format_language($nr=0)
  {
   if($this->mode=="server")
    $serial=&$this->commands[$nr]["language"];
   else //client
   {
    $serial=Array(
     "type" => $this->commands[$nr]["type"],
     "set" => $this->commands[$nr]["set"],
     "language" => $this->commands[$nr]["lang"],
    );
   }

   return $serial;
  }


  /*
   * Modules
   */

  function parse_modules($serial)
  {
   $all=false;

   if(isset($serial["modules"])&&is_array($serial["modules"]))
    $modules=$serial["modules"];
   else
   {
    $modules=Array();

    if(isset($serial["all"])&&$serial["all"])
     $all=$serial["all"];
   }

   if(!$this->addModules($modules, $all))
    return false;

   return true;
  }

  function format_modules($nr=0)
  {
   $serial=Array(
    "type" => $this->commands[$nr]["type"],
    "modules" => Array(),
   );

   if($this->mode=="server")
    $serial["modules"]=$this->commands[$nr]["modules"];
   else //client
   {
    foreach($this->commands[$nr]["modules"] as $module)
    {
     if(is_object($module)) //data object
      $serial["modules"][]=$module->module;
     else //string
      $serial["modules"][]=$module;
    }
   }

   return $serial;
  }


  /*
   * Projects
   */

  function parse_projects($serial)
  {
   $activate=false;
   $projects=Array();

   if($this->mode=="server")
   {
    if(isset($serial["project"])&&$serial["project"])
     $activate=$serial["project"];
   }
   else //client
   {
    $projects=$serial["projects"];

    if(isset($serial["current"])&&$serial["current"])
     $activate=$serial["current"];
   }

   if(!$this->addProjects($activate, $projects))
    return false;

   return true;
  }

  function format_projects($nr=0)
  {
   $serial=Array(
    "type" => $this->commands[$nr]["type"],
   );

   if($this->mode=="server")
   {
    $serial["projects"]=$this->commands[$nr]["projects"];
    $serial["current"]=$this->commands[$nr]["activate"];
   }
   else //client
   {
    if($this->commands[$nr]["activate"])
     $serial["project"]=$this->commands[$nr]["activate"];
   }

   return $serial;
  }


  /*
   * Errors
   */

  function parse_error($serial)
  {
   $this->e->clear(); //remove all existing errors (so the error object only contains remote errors)

   if(!$this->e->add($serial))
    return false;

   return true;
  }

  function format_errors()
  {
   return $this->e;
  }

 }
?>
