<?

 /*****************************************************************************
 * Filename: include/cli.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");

 class CLI
 {
  function __construct()
  {
   global $_path;

   $this->e=new Errors();
   $this->e->l->read("cli");

   $this->ssh=Array();
   $this->ssh["ssh"]="ssh";
   $this->ssh["scp"]="scp";
   $this->ssh["keygen"]="ssh-keygen";
   $this->ssh["keytype"]="rsa"; //dsa or rsa

   $this->ssh["strict_host_key_checking"]=false;
   $this->ssh["host"]=false;
   $this->ssh["port"]=false;
   $this->ssh["user"]=false;

   $this->clear();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  function clear()
  {
   $this->return=false;
   $this->output=Array();
   $this->lastline=false;

   $this->execCMD=false;
   $this->execSSH=false;

   $this->process=false;
   $this->pipes=Array();

   $this->stdin="";
   $this->stdout="";
   $this->stderr="";
  }

  function input($stdin)
  {
   $this->stdin=$stdin;
  }

  function iexec($cmd, $ssh=true)
  {
   if(!$this->process)
   {
    $this->clear();

    if(!$cmd)
    {
     $this->e->add("Empty command given", "error");
     return false;
    }

    $this->execCMD=$cmd;
    $this->execSSH=$ssh;

    if($this->ssh["host"]&&$ssh)
     $exec=$this->sshCommand("ssh")." ".($this->ssh["user"]?$this->ssh["user"]."@":"").$this->ssh["host"]." \"".$cmd."\"";
    else
     $exec=$cmd;

    $descriptors[0]=Array("pipe", "r"); //stdin
    $descriptors[1]=Array("pipe", "w"); //stdout
    $descriptors[2]=Array("pipe", "w"); //stderr

    if(($this->process=proc_open("cd \"".@getcwd()."\" ; ".$exec." ; echo \"--EXIT:$?\" 1>&2", $descriptors, $this->pipes))===false)
    {
     $this->e->add("Unable to open process: \"".$this->execCMD."\"", "error");
     return false;
    }

    stream_set_blocking($this->pipes[0], 1);
    stream_set_blocking($this->pipes[1], 1);
    stream_set_blocking($this->pipes[2], 1);
   }

   if($this->stdin)
   {
    if(!fwrite($this->pipes[0], $this->stdin))
     $this->e->add("Unable to write to process: \"".$this->execCMD."\"", "warning");

    $this->stdin="";
   }

   $read=Array($this->pipes[1], $this->pipes[2]);
   $write=$except=Array();
   if($tmp=stream_select($read, $write, $except, NULL))
   {
    for($c=0;$c<2;$c++)
    {
     if(isset($read[$c])&&($output=fread($read[$c], 1024)))
     {
      if($this->pipes[1]==$read[$c])
       $this->stdout.=$output;
      else if($this->pipes[2]==$read[$c])
      {
       if(strstr($output, "--EXIT:"))
       {
        list($output, $return)=explode("--EXIT:", $output);
        $this->return=rtrim($return);
       }

       $this->stderr.=$output;
      }

      foreach(explode("\n", $output) as $line)
      {
       if(isset($this->output["unfinished"]))
       {
        $line=$this->output["unfinished"].$line;
        unset($this->output["unfinished"]);
       }

       $this->output[]=$line;
      }

      $lastline=end($this->output);

      if(!$lastline) //last line has proper line ending
      {
       array_pop($this->output); //remove bogus empty line
       $lastline=end($this->output);
      }
      else // half line
       $this->output["unfinished"]=array_pop($this->output); //safe for adding later

      $this->lastline=$lastline;
     }
    }
   }

   if($this->return===false)
    return true;

   fclose($this->pipes[0]);
   fclose($this->pipes[1]);
   fclose($this->pipes[2]);

   proc_close($this->process);
   $this->process=false;

   if(isset($this->output["unfinished"]))
   {
    $line=$this->output["unfinished"];
    unset($this->output["unfinished"]);
    $this->output[]=$line;
    $lastline=end($this->output);
   }

   return $this->_postExec();
  }

  function exec($cmd, $ssh=true)
  {
   if($this->process)
   {
    $this->e->add("Unable to execute, interactive process already running", "error");
    return false;
   }

   $this->clear();

   if(!$cmd)
   {
    $this->e->add("Empty command given", "error");
    return false;
   }

   $this->execCMD=$cmd;
   $this->execSSH=$ssh;

   if($this->ssh["host"]&&$ssh)
    $exec=$this->sshCommand("ssh")." ".($this->ssh["user"]?$this->ssh["user"]."@":"").$this->ssh["host"]." \"".str_replace("\"", "\\\"", $cmd)."\"";
   else
    $exec=$cmd;

   $this->lastline=exec($exec." 2>&1 ", $this->output, $this->return);

   if($this->_postExec()===false)
    return false;

   return true;
  }

  function _postExec()
  {
   if($this->ssh["host"]&&$this->execSSH)
   {
    if($this->output&&preg_match("/^Warning: (Permanently added \'\w+\'.*)$/", $this->output[0], $m))
    {
     $this->e->add($m[1], "warning");
     array_shift($this->output);
    }
   }

   switch($this->return)
   {
    case 0:
    {
     //cmd ok
     break;
    }
    case 127:
    {
     $this->e->add("Command: \"".preg_replace("/^([^\s]+).*$/", "\\1", $this->execCMD)."\" not found", "error");
     return false;
    }
    case 255:
    {
     if($this->ssh["host"]&&$this->execSSH)
     {
      if(stristr($this->lastline, "Permission denied"))
      {
       $this->e->add("SSH permission denied, make sure you've installed the public key: \"".$this->ssh["public_key"]."\" on host: \"".$this->ssh["host"]."\"", "error");
       return false;
      }

      if(stristr($this->lastline, "Could not resolve hostname"))
      {
       $this->e->add("SSH host: \"".$this->ssh["host"]."\" not found", "error");
       return false;
      }

      if(stristr($this->lastline, "Connection timed out"))
      {
       $this->e->add("SSH host: \"".$this->ssh["host"]."\" timed out", "error");
       return false;
      }
     }
     //non ssh, go to default
    }
    default:
    {
     $this->e->add("Command: \"".$this->execCMD."\" returned error code: \"".$this->return."\"", "warning", $this->lastline);
     break;
    }
   }

   return $this->return; //done
  }

  function home()
  {
   if(!$this->exec("echo ~/"))
    return false;

   return $this->output[0];
  }

  function whoami()
  {
   if(!$this->exec("whoami"))
    return false;

   return $this->output[0];
  }

  function ssh($host, $user=false, $port=false, $strict_host_key_checking=false, $timeout=false)
  {
   $return=false;
   $output=Array();
   exec($this->ssh["ssh"]." 2>&1", $output, $return);

   if($return==127) //test ssh
   {
    $this->e->add("SSH not available, command: \"".$this->ssh["ssh"]."\" not found");
    return false;
   }

   if(!isset($this->ssh["path"]))
    if(!$this->sshFiles())
     return false;

   if(!file_exists($this->ssh["private_key"])||!file_exists($this->ssh["public_key"]))
    if(!$this->sshSetup())
     return false;

   if(!preg_match("/^([a-z0-9_\-\.]+)$/", $host))
   {
    $this->e->add("Illegal hostname: \"".$host."\"");
    return false;
   }

   if($user&&!preg_match("/^([a-z0-9_]+)$/", $user))
   {
    $this->e->add("Illegal username: \"".$user."\"");
    return false;
   }

   if($port&&!($port>0))
   {
    $this->e->add("Illegal port number: \"".$port."\"");
    return false;
   }

   $this->ssh["host"]=$host;
   $this->ssh["user"]=$user;
   $this->ssh["port"]=($port?(int)$port:false);
   $this->ssh["strict_host_key_checking"]=($strict_host_key_checking?true:false);
   $this->ssh["timeout"]=($timeout?$timeout:10);

   return true;
  }

  function scpR($from, $to=false)
  {
   return $this->scp($from, $to, false);
  }
  function scpL($from, $to=false)
  {
   return $this->scp($from, $to, true);
  }
  function scp($from, $to, $toRemote)
  {
   if(!$this->ssh["host"])
   {
    $this->e->add("Unable to scp, host not set", "error");
    return false;
   }

   if($toRemote&&!file_exists($from))
   {
    $this->e->add("Local file: \"".$from."\" doesn't exist");
    return false;
   }

   if(!$to)
    $to=".";

   if($toRemote)
    $cmd=$this->sshCommand("scp")." '".$from."' ".($this->ssh["user"]?$this->ssh["user"]."@":"").$this->ssh["host"].":'".$to."'";
   else
    $cmd=$this->sshCommand("scp")." ".($this->ssh["user"]?$this->ssh["user"]."@":"").$this->ssh["host"].":'".$from."' '".$to."'";

   if(!$this->exec($cmd, false)||$this->return!=0)
    return false;

   return true;
  }


  function sshFiles($path=false, $private_key=false, $public_key=false, $known_hosts=false)
  {
   global $_path;

   if(!$path)
    $this->ssh["path"]=$_path["project"]["include"].".ssh/";
   else
    $this->ssh["path"]=$path;

   $this->ssh["private_key"]=($private_key?$private_key:$this->ssh["path"]."id_".$this->ssh["keytype"]);
   $this->ssh["public_key"]=($public_key?$public_key:$this->ssh["private_key"].".pub");
   $this->ssh["known_hosts"]=($known_hosts?$known_hosts:$this->ssh["path"]."known_hosts");

   if(!file_exists($this->ssh["path"])&&!@mkdir($this->ssh["path"]))
   {
    $this->e->add("Couldn't create ssh directory: \"".$this->ssh["path"]."\"");
    return false;
   }

   if(!$this->sshFixPermissions())
    return false;

   return true;
  }

  function sshFixPermissions()
  {
   if(!@chmod($this->ssh["path"], 0700))
   {
    $this->e->add("Couldn't change permissions for ssh directory: \"".$this->ssh["path"]."\"");
    return false;
   }

   if(file_exists($this->ssh["private_key"])&&!chmod($this->ssh["private_key"], 0400))
   {
    $this->e->add("Couldn't change permissions for private key: \"".$this->ssh["private_key"]."\"");
    return false;
   }
   if(file_exists($this->ssh["public_key"])&&!@chmod($this->ssh["public_key"], 0400))
   {
    $this->e->add("Couldn't change permissions for public key: \"".$this->ssh["public_key"]."\"");
    return false;
   }
   if(file_exists($this->ssh["known_hosts"])&&!@chmod($this->ssh["known_hosts"], 0600))
   {
    $this->e->add("Couldn't change permissions for known hosts: \"".$this->ssh["known_hosts"]."\"");
    return false;
   }

   return true;
  }

  function sshSetup()
  {
   if(!isset($this->ssh["path"]))
    if(!$this->sshFiles())
     return false;

   if(file_exists($this->ssh["private_key"])||file_exists($this->ssh["public_key"]))
   {
    $this->e->add("Private/public key pair already exists in: \"".$this->ssh["path"]."\"");
    return false;
   }

   if(!$this->exec($this->ssh["keygen"]." -t ".$this->ssh["keytype"]." -f ".$this->ssh["private_key"]." -N \"\" ", false))
   {
    $this->e->add("Couldn't generate SSH keys, please set up keys manually");
    return false;
   }

   if($this->ssh["public_key"]!=$this->ssh["private_key"].".pub")
   {
    if(!rename($this->ssh["private_key"].".pub", $this->ssh["public_key"]))
    {
     $this->e->add("Couldn't rename public key file to: \"".$this->ssh["public_key"]."\"");
     return false;
    }
   }

   if(!$this->sshFixPermissions())
    return false;

   $this->e->add("SSH keys set up", "note");
   return true;
  }


  function sshCommand($cmd)
  {
   $options="";
   $options.=" -o PubkeyAuthentication=yes";
   $options.=" -o PasswordAuthentication=no";
   $options.=" -o ConnectTimeout=".$this->ssh["timeout"];
   $options.=" -o StrictHostKeyChecking=".($this->ssh["strict_host_key_checking"]?"yes":"no");
   $options.=" -o UserKnownHostsFile='".$this->ssh["known_hosts"]."'";
   $options.=" -i '".$this->ssh["private_key"]."'";

   if($this->ssh["port"])
   {
    if($cmd=="scp")
     $options.=" -P ".$this->ssh["port"];
    else
     $options.=" -p ".$this->ssh["port"];
   }

   return $this->ssh[$cmd].$options;
  }

 }

?>
