<?

 /*****************************************************************************
 * Filename: include/cms.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."form.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."captcha.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."fs.php");

 class CMS
 {
  function __construct($config=false)
  {
   global $_config;

   $this->e=new Errors();

   $this->l=&$this->e->l;

   $this->multiple=false;//multiple modules in upcoming post ? (change field names)
   $this->javascriptRights=false; //javascript rights function already printed?

   $this->referenceManipulation=true;

   //get CMS config
   $this->config=$_config->merge("CMS", $config);

   $this->data=false;
   $this->form=false;

   $this->vars=Array();
   if(isset($this->config["defaultFormVars"]))
    foreach(explode("&amp;", $this->config["defaultFormVars"]) as $var)
     if(strstr($var, "=")&&list($k, $v)=explode("=", $var))
      $this->addVar($k, $v);
  }

  function __clone()
  {
   //save original
   $tmpE=&$this->e;
   $tmpL=&$this->e->l;

   //restore original language (dereference), so it's cloned correctly
   unset($this->e->l);
   $this->e->l=$tmpL;

   //clone object
   unset($this->e);
   $this->e=clone $tmpE;

   //restore original
   unset($tmpE->l);
   $tmpE->l=&$tmpL;

   //update references
   unset($this->l);
   $this->l=&$this->e->l;
  }


  function setData(&$data)
  {
   $this->data=&$data;
   $this->l->read($this->data->l);
   $this->l->read("cms"); //ensure module translations don't overwrite
  }

  function setForm(&$form)
  {
   $this->form=&$form;
  }

  function setDataForm(&$data, &$form)
  {
   $this->setData($data);
   $this->setForm($form);
  }

  function setReturnURL($url)
  {
   $this->config["returnURL"]=strip_tags($url);
  }

  function addVar($name, $value)
  {
   if(isset($this->vars[$name]))
    $this->e->add("Overwriting variable: \"".$name."\"", "warning");

   $this->vars[$name]=$value;
  }



  function frontForm($cmsFile=false)
  {
   $r=$this->formStart($cmsFile);

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Module")."</div>\n<div class=\"input\">";
   $r.=$this->form->selectStart(false, "m", "form", false, "onChange=\"this.form.submit();\"");
   $r.=$this->form->selectOption("");
   foreach(JAS::modules() as $m)
   {
    if($m==$this->data->module)
     $r.=$this->form->selectOption($m, false, "SELECTED");
    else
     $r.=$this->form->selectOption($m);
   }
   $r.=$this->form->selectEnd();

   $r.="</div>\n";
   $r.="</div>\n";

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Quick selector")."</div>\n<div class=\"input\">";

   $r.=$this->form->dataSelect($this->data->module, "nr", $this->data->getValue("nr"), "form", false, "onChange=\"this.form.submit();\"")."\n";

   $r.="</div>\n";
   $r.="</div>\n";

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Options")."</div>\n<div class=\"input\">";

   $r.=$this->form->selectStart(false, "a", "form", false, "onChange=\"this.form.submit();\"");
   $r.=$this->form->selectOption("");
   foreach($this->data->actionList as $m)
    $r.=$this->form->selectOption($m);
   $r.=$this->form->selectEnd()."</div>\n";

   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   $r.="</div>\n";
   $r.="</div>\n";

   $r.=$this->formEnd()."\n";

   return $r;
  }



  function simpleForm($cmsFile=false, $extra=false)
  {
   if(!($name=$this->data->getID()))
    return false;

   if(isset($_REQUEST[$name]))
    $values=$_REQUEST[$name];
   else
    $values=false;

   $r=$this->formStart($cmsFile);

   $r.="<div class=\"input\">";
   $r.=$this->form->select("multiple", $name, $this->data, $values, "form", false, $extra." style=\"height: 100px;\"");
   $r.="</div>\n";

   $r.="<div class=\"input\">";
   if($this->data->r("add"))
    $r.=$this->form->input("submit", "", $this->l->t("Add"), "submit", false, "onclick=\"this.form.a.value='add';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("copy"))
    $r.=$this->form->input("submit", "", $this->l->t("Copy"), "submit", false, "onclick=\"this.form.a.value='copy';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("edit"))
    $r.=$this->form->input("submit", "", $this->l->t("Edit"), "submit", false, "onclick=\"this.form.a.value='edit';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("delete"))
    $r.=$this->form->input("submit", "", $this->l->t("Delete"), "submit", false, "onclick=\"this.form.a.value='delete';\" style=\"width: 25%;\"")."\n";
   $r.="</div>\n";

   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   $r.=$this->form->input("hidden", "m", $this->data->module)."\n";
   $r.=$this->form->input("hidden", "a", "view")."\n";
   $r.=$this->form->input("hidden", "e", "item")."\n";

   $r.=$this->formEnd()."\n";

   return $r;
  }



  function form($cmsFile=false)
  {
   $r="";

   if(!$this->formDataSetup())
    return false;

   //submit
   if($this->config["handleSubmit"])
   {
    if($this->formHandleSubmit()) //submit successful
    {
     if(!request("cmsContinue")) //no more cms action?
     {
      if(!isset($this->config["showErrors"])||$this->config["showErrors"]) //show notices
       $r.=$this->formShowErrors();

      if($this->config["reloadPage"]) //reload page
       $r.=$this->handleReloadPage();

      return $r;
     }
    }
   }

   //reference submit
   if($this->referenceManipulation)
    $this->formHandleReferenceManipulation();

   //form header
   if($this->config["showTitles"])
    $r.="<h1>".$this->l->t($this->data->title)."</h1>\n";

   if($this->config["showText"])
    $r.=$this->formShowText();

   //render form
   $form=$this->formStart($cmsFile);

   if($this->config["showFields"])
    $form.=$this->formShowFields();

   if($this->config["showSubmit"])
    $form.=$this->formShowSubmit();

   if($this->config["showCancel"])
    $form.=$this->formShowCancel();

   $form.=$this->formEnd();

   //move data errors to cms error object
   if($this->data->e->any())
   {
    $this->e->add($this->data->e);
    $this->data->e->clear();
   }

   //show errors
   if(!isset($this->config["showErrors"])||$this->config["showErrors"])
    $r.=$this->formShowErrors();

   $r.=$form;

   return $r;
  }


  function formDataSetup()
  {
   if($this->data==false)
    $this->data=Data::cache(false, false, false, Array("format" => "form"));

   $this->data->dataformat="form";

   if(!$this->data->validAction())
    return false;

   return true;
  }


  function formStart($cmsFile=false)
  {
   global $_path;

   if(!$cmsFile)
    $cmsFile=$_path["HTML"]["url"];

   if(!$this->form)
    $this->form=new Form();

   return $this->form->start($cmsFile, false, "cms")."\n\n";
  }


  function formShowText()
  {
   global $_config;

   $r="";

   if(!request("s")) //only show first time
   {
    $r.="<div class=\"text\">\n";

    switch($this->data->action)
    {
     case "view":
     {
      $r.=$this->l->t("Viewing data, select an option below");
      break;
     }

     case "add":
     case "copy":
     {
      $r.=$this->l->t("Fill in the fields below and press \"Add\" to insert the new data");
      break;
     }

     case "edit":
     {
      $r.=$this->l->t("Modify the fields below and press \"Update\" to save the changes");
      break;
     }

     case "move":
     {
      $r.=$this->l->t("Select a category and press \"Move\" to save the new category");
      break;
     }

     case "delete":
     {
      if($this->data->module=="categories"&&isset($this->data->moduleObject->autoDeleteContent)&&($this->data->moduleObject->autoDeleteContent||$this->data->moduleObject->autoDeleteSubcategories))
      {
       if($this->data->moduleObject->autoDeleteContent&&$this->data->moduleObject->autoDeleteSubcategories)
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->l->t($this->data->title)."\" and it's content and subcategories?");
       else if($this->data->moduleObject->autoDeleteContent)
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->l->t($this->data->title)."\" and it's content?");
       else
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->l->t($this->data->title)."\" and it's subcategories?");
      }
      else
       $r.=$this->l->t("Are you sure you wish to delete this data?");

      break;
     }

     case "undelete":
     {
      $r.=$this->l->t("Are you sure you wish to undelete this data?");
      break;
     }

     case "order":
     {
      $r.=$this->l->t("Are you sure you wish to order this data ".$this->data->extra."?");
      break;
     }

     case "rights":
     {
      $r.=$this->l->t("Select the group / user, their new rights and press \"Update\" to save the new rights");
      break;
     }

     case "owner":
     {
      $r.=$this->l->t("Select an user and press \"Submit\" to save the new owner");
      break;
     }

     case "user":
     {
      switch($this->data->extra)
      {
       case "login":
       {
        $r.=$this->l->t("Please enter your username and password and press \"Login\"");
        break;
       }
       case "logout":
       {
        $r.=$this->l->t("Are you sure you wish to logout?");
        break;
       }
       case "password":
       {
        if($_config->userAnonymous())
        {
         $this->e->add("Can't change password of \"anonymous\" user", "hack");
         return false;
        }

        $r.=$this->l->t("Fill in your new password and verify it, then press \"Update\" to change it");
        break;
       }
      }
      break;
     }

     case "lost":
     {
      $r.=$this->l->t("Select a category and press \"Submit\" to save the new category");
      break;
     }

     default: return false;
    }

    $r.="</div>\n";
   }

   return $r;
  }

  function formHandleSubmit()
  {
   if(request("s"))
   {
    if($this->data->submit())
    {
     if(request("cmsContinue"))//oh you want more ?
     {
      if($this->data->action=="add")//Clean up, after adding
      {
       if(request("cmsContinue")=="add")
       {
        foreach($this->data->check("use", "form") as $k)
         if(!$this->data->check("reference", "categories", $k))
          $this->data->setDefaultValue($k);
       }
       else if(request("cmsContinue")=="edit")
       {
        $this->data->setAction("edit");
        $this->data->setValue("a", "edit");
       }
      }
     }

     $this->e->add($this->data->e);
     $this->data->e->clear();
     return true;
    }
    else
    {
     $this->e->add($this->data->e);
     $this->data->e->clear();
    }
   }

   return false;
  }


  function handleReloadPage()
  {
   global $_jses;

   $hs=new History();

   if($hs->getTotal()&&($page=$hs->previous())) //history in use
    $returnURL=$page;
   else if($this->config["returnURL"])
    $returnURL=$this->config["returnURL"];
   else
    return "";

   return "<script type=\"text/javascript\">setTimeout(\"this.location='".$returnURL."'\", ".$this->config["reloadTimeout"].");</script>\n";
  }


  function formHandleReferenceManipulation()
  {
   if($this->data->requestVar)
    $ref_name=request($this->data->requestVar, "_refmanip");
   else
    $ref_name=request("_refmanip");

   if(is_array($ref_name))
   {
    $ref_action=strtolower(reset($ref_name));
    $ref_name=array_keys($ref_name)[0];
   }
   else
   {
    $ref_name=false;
    $ref_action=false;
   }

   if($ref_action&&$ref_name) //reference manipulation
   {
    $name=$ref_name;
    $this->showReference[$name]=false;

    $subd=Data::cache($this->data->reference[$name]["module"], ($ref_action=="edit"?"edit":"add"), "item", Array("request" => $this->data->requestVar."SubCMS"));

    if(request($subd->requestVar, "s")) //submit
    {
     if($subd->submit())
     {
      $this->showReference[$name]=true;

      if($subd->action=="add")
      {
       if($this->data->reference[$name]["type"]=="0:*"||$this->data->reference[$name]["type"]=="1:*")
        $this->data->setValue($name, array_merge($this->data->getValue($name), Array($subd->getValue($subd->getID()))));
       else if($this->data->reference[$name]["type"]=="*:*")
        $this->data->setValue($name, array_merge($this->data->getValue($name), Array($subd->getValue($this->data->reference[$name]["to"]))));
       else
        $this->data->setValue($name, $subd->getValue($subd->getID()));
      }
     }
     else
     {
      $this->e->add($subd->e);
      return false;
     }
    }
   }

   return true;
  }


  function formShowErrors()
  {
   $r="";

   if($this->e->any())
   {
    $r.=$this->e->show();
    $this->e->clear();
   }

   return $r;
  }


  function formShowFields()
  {
   global $_config;

   //get data
   $get=true;
   switch($this->data->action)
   {
    case "add":
    case "order":
    case "user":
    {
     $get=false;
     break;
    }
    case "copy":
    case "edit":
    {
     if(request("s")||request("dontgetdata"))
      $get=false;

     break;
    }
    default: break;
   }

   if($get)
   {
    $catnr=false;
    if($this->data->extra=="list"&&(request("s")||request("reload_trigger"))) //list action page shown for second time
    {
     if($this->data->action=="move")
      $catnr=$this->data->getValue($this->data->getRightsReference()); //store current selected category

     //reget catnr from first entry
     $id=$this->data->getValue($this->data->getID());
     $action=$this->data->action;

     $this->data->setAction("view");
     $this->data->setExtra("item");

     $this->data->clearValues();
     $this->data->setValue($this->data->getID(), $id);

     $this->data->setShowFields($this->data->getRightsReference());
     $this->data->get();

     $this->data->setAction($action);
     $this->data->setExtra("list");
    }

    if($this->data->extra!="item"&&$this->data->extra!="andfound") //only get titles for list/all
     $this->data->setShowFields($this->data->check("properties", "title"));

    if(!$this->data->get()||$this->data->e->any("warning"))
    {
     $this->config["showSubmit"]=false; //don't show submit
     return false;
    }

    if($catnr!==false)
     $this->data->setValue($this->data->getRightsReference(), $catnr); //restore current selected category
   }


   //determine fields+groups
   $groups=true;
   $fields=Array();

   //control fields
   foreach($this->data->check("properties", "control") as $field)
   {
    if($field=="r"||strpos($field, "aggregation_")===0||strpos($field, "show_")===0) //ignore r, aggregation and show fields
     $fields[$field]="ignore";
    else
     $fields[$field]="hidden";
   }

   //identifier field(s)
   foreach($this->data->check("properties", "identifier") as $field)
    if(!isset($fields[$field]))
     $fields[$field]="hidden";

   //title/text/form fields
   switch($this->data->action)
   {
    case "view":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     if($this->data->extra=="item") //only show other fields if one item is shown
      foreach($this->data->check("use", "form") as $field)
       if(!isset($fields[$field]))
        $fields[$field]="fake";

     break;
    }

    case "add":
    {
     foreach($this->data->check("use", "form") as $field)
     {
      if($this->data->check("properties", "identifier", $field)&&$this->data->check("type", "text", $field)) //text identifier
       $fields[$field]=""; //overwrite hidden identifier

      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "noUpdate", $field))
       $fields[$field]="fake";
      else
       $fields[$field]="";
     }

     break;
    }
    case "copy":
    {
     foreach($this->data->check("use", "form") as $field)
     {
      if($this->data->check("properties", "identifier", $field)&&$this->data->check("type", "text", $field)) //text identifier
       $fields[$field]=""; //overwrite hidden identifier

      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "noUpdate", $field))
       $fields[$field]="fake";
      else
       $fields[$field]="";
     }

     break;
    }

    case "edit":
    {
     foreach($this->data->check("use", "form") as $field)
     {
      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;
      if($this->data->check("properties", "rightsReference", $field)) //don't show category reference
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "noUpdate", $field))
       $fields[$field]="fake";
      else
       $fields[$field]="";
     }

     break;
    }

    case "move":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     foreach($this->data->check("use", "form") as $field)
     {
      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "rightsReference", $field))
       $fields[$field]="";
      else if($this->data->extra=="item") //only show other fields if one item is shown
       $fields[$field]="disabled";
     }

     break;
    }

    case "delete":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     if($this->data->extra=="item") //only show other fields if one item is shown
      foreach($this->data->check("use", "form") as $field)
       if(!isset($fields[$field]))
        $fields[$field]="disabled";

     break;
    }
    case "undelete":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     if($this->data->extra=="item") //only show other fields if one item is shown
      foreach($this->data->check("use", "form") as $field)
       if(!isset($fields[$field]))
        $fields[$field]="disabled";

     break;
    }

    case "order":
    {
     break;
    }

    case "rights":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     foreach($this->data->check("use", "form") as $field)
     {
      if(isset($fields[$field]))
       continue;
      if($field=="usernr"||$field=="groupnr"||$field=="rights")
       continue;
      if($this->data->check("properties", "rightsReference", $field)) //don't show category reference
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->extra=="item") //only show other fields if one item is shown
       $fields[$field]="disabled";
     }
     break;
    }

    case "owner":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     foreach($this->data->check("use", "form") as $field)
     {
      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "rightsReference", $field)) //don't show category reference
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "owner", $field))
       $fields[$field]="";
      else if($this->data->extra=="item") //only show other fields if one item is shown
       $fields[$field]="disabled";
     }
     break;
    }

    case "user":
    {
     $groups=false;

     if($this->data->extra=="login")
     {
      foreach($this->data->check("use", "login") as $field)
       $fields[$field]="";
     }
     else if($this->data->extra=="logout")
     {
      foreach($this->data->check("use", "logout") as $field)
       $fields[$field]="";
     }
     else if($this->data->extra=="password")
     {
      if($_config->userAnonymous())
       $this->config["showSubmit"]=false; //don't show submit
      else
       foreach($this->data->check("properties", "password") as $field)
        $fields[$field]="";
     }
     else //fail
     {
      $this->e->add("Unknown ".$this->data->action." action option \"".$this->data->extra."\"", "error");
      return false;
     }

     break;
    }


    case "lost":
    {
     foreach($this->data->check("properties", "title") as $field)
      if(!isset($fields[$field]))
       $fields[$field]="fake";

     foreach($this->data->check("use", "form") as $field)
     {
      if(isset($fields[$field]))
       continue;
      if($this->data->check("properties", "owner", $field)) //don't show owner
       continue;

      if($this->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else if($this->data->check("properties", "rightsReference", $field))
       $fields[$field]="";
     }

     break;
    }


    case "database":
    {
     $groups=false;

     if($this->data->extra=="info")
     {
      $this->config["showSubmit"]=false; //don't show submit

      foreach($this->data->check("use", "info") as $field)
       $fields[$field]="fake";
     }
     else //create/alter/maintenance
     {
     }

     break;
    }

    default: break;
   }

   //get groups
   if($groups)
   {
    $groups=Array();
    foreach($this->data->getGroups() as $key => $group)
    {
     foreach($this->data->getGroupFields($key) as $field)
     {
      if(isset($fields[$field]))
      {
       if(!isset($groups[$group]))
        $groups[$group]=Array();

       $groups[$group][]=$field;
      }
     }
    }
   }

   //render fields
   $r="";

   if($groups) //group fields
   {
    $r.="<div class=\"groups ".$this->data->module."\">";

    foreach($groups as $group => $groupFields)
    {
     $tmp="";
     $visible=false;

     //render fields
     foreach($fields as $field => $type)
     {
      if($type=="ignore")
       continue;

      if(in_array($field, $groupFields))
      {
       if(!$this->data->check("properties", "owner", $field)&&!$this->data->check("properties", "rightsReference", $field))
       {
        if($this->data->extra=="item"||$this->data->check("properties", "control", $field)) //only show first entry
         $tmp.=$this->field($field, $type);
        else //show all entries
         for($entry=0;$entry<$this->data->vmax;$entry++)
          $tmp.=$this->field($field, $type, $entry);

        if($type!="hidden")
         $visible=true;
       }
      }
     }

     if($visible)
     {
      $r.="<div class=\"group ".preg_replace("/[^a-z]*/i", "", $group)."\">";
      $r.="<div class=\"groupHeader\">".$this->l->t($group)."</div>";
      $r.="<div class=\"groupBody\">".$tmp."</div>";
      $r.="<div class=\"groupFooter\"></div>";
      $r.="</div>";
     }
     else //no visible fields
      $r.=$tmp;
    }

    $r.="</div>";
   }
   else //no groups
   {
    //render fields
    foreach($fields as $field => $type)
    {
      if($type=="ignore")
       continue;

     if(!$this->data->check("properties", "owner", $field)&&!$this->data->check("properties", "rightsReference", $field))
     {
      if($this->data->extra=="item"||$this->data->check("properties", "control", $field)) //only show first entry
       $r.=$this->field($field, $type);
      else //show all entries
       for($entry=0;$entry<$this->data->vmax;$entry++)
        $r.=$this->field($field, $type, $entry);
     }
    }
   }

   //owner field(s)
   foreach($this->data->check("properties", "owner") as $field)
    if(isset($fields[$field]))
     $r.=$this->field($field, $fields[$field]);

   //category reference field(s)
   foreach($this->data->check("properties", "rightsReference") as $field)
    if(isset($fields[$field]))
     $r.=$this->field($field, $fields[$field]);

   if($this->data->action=="rights")
   {
    $r.=$this->field("usernr");
    $r.=$this->field("groupnr");
    $r.=$this->field("rights");
   }

   //add vars
   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   return $r;
  }

  function formShowSubmit()
  {
   $r="";

   if($this->data->requestVar)
    $fieldname=$this->data->requestVar."[s]";
   else
    $fieldname="s";

   if($this->config["useStyling"])
   {
    $r.="<div class=\"field\">\n";
    $r.="<div class=\"title\">\n";
    $r.=$this->l->t("Submit");
    $r.="</div>\n";

    $r.="<div class=\"input\">\n";
   }

   switch($this->data->action)
   {
    case "add":
    case "copy":
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Add"), "submit")."\n";
     $r.=$this->form->input("hidden", "cmsContinue", 0)."\n";

     if($this->data->action=="copy")
      $r.=$this->form->input("hidden", "dontgetdata", 1)."\n";

     if($this->config["showSubmitAdditional"])
     {
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Add and update"), "submit", false, "onclick=\"this.form.cmsContinue.value='edit'; \"")."\n";
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Add another"), "submit", false, "onclick=\"this.form.cmsContinue.value='add'; \"")."\n";
     }
    break;

    case "edit";
    case "rights";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Update"), "submit")."\n";
     $r.=$this->form->input("hidden", "cmsContinue", 0)."\n";
     $r.=$this->form->input("hidden", "dontgetdata", 1)."\n";

     if($this->config["showSubmitAdditional"])
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Update and continue"), "submit", false, "onclick=\"this.form.cmsContinue.value=1; \"")."\n";
    break;

    case "move";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Move"), "submit")."\n";
    break;

    case "delete";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Delete"), "submit")."\n";
    break;

    case "undelete";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Undelete"), "submit")."\n";
    break;

    case "user":
    {
     switch($this->data->extra)
     {
      case "login";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Login"), "submit")."\n";
      break;

      case "logout";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Logout"), "submit")."\n";
      break;

      case "password";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Update"), "submit")."\n";
      break;
     }
     break;
    }

    case "view";
     $this->data->next(); //set rights for checking

     $fieldname=$this->data->requestVar."['a']";
     $fieldname2=$this->data->requestVar."['e']";

     if($this->data->r("copy"))
      $r.=$this->form->input("submit", "", $this->l->t("Copy"), "submit", false, "onclick=\"this.form".$fieldname.".value='copy';\"")."\n";
     if($this->data->r("edit"))
      $r.=$this->form->input("submit", "", $this->l->t("Edit"), "submit", false, "onclick=\"this.form".$fieldname.".value='edit';\"")."\n";
     if($this->data->r("move"))
      $r.=$this->form->input("submit", "", $this->l->t("Move"), "submit", false, "onclick=\"this.form".$fieldname.".value='move';\"")."\n";
     if($this->data->r("delete"))
      $r.=$this->form->input("submit", "", $this->l->t("Delete"), "submit", false, "onclick=\"this.form".$fieldname.".value='delete';\"")."\n";
     if($this->data->module=="categories"&&$this->data->r("rights"))
      $r.=$this->form->input("submit", "", $this->l->t("Rights"), "submit", false, "onclick=\"this.form".$fieldname.".value='rights';\"")."\n";
    break;


    default:
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Submit"), "submit")."\n";
    break;
   }

   if($this->config["useStyling"])
   {
    $r.="</div>\n";
    $r.="</div>\n";
   }

   return $r;
  }


  function formShowCancel()
  {
   $r="";

   if($this->config["useStyling"])
   {
    $r.="<div class=\"field\">\n";
    $r.="<div class=\"title\">\n";
    $r.=$this->l->t("Cancel");
    $r.="</div>\n<div class=\"input\">\n";
   }

   $hs=new History();

   if($hs->getTotal()&&($page=$hs->previous())) //history in use
    $r.=$this->form->input("button", "", $this->l->t("Back"), "submit", false, "onclick=\"document.location='".$page."'\"")."\n";
   else //no history, use browser history
    $r.=$this->form->input("button", "", $this->l->t("Back"), "submit", false, "onclick=\"history.back();\"")."\n";

   if($this->config["useStyling"])
   {
    $r.="</div>\n";
    $r.="</div>\n";
   }

   return $r;
  }


  function formEnd()
  {
   return $this->form->end()."\n\n";
  }


  function field($names, $type=false, $nr=false)
  {
   global $_path;

   $r="";
   $d=&$this->data;

   if(!is_array($names))
    $names=Array($names);

   $v=&$d->field[$names[0]];

   //Show non hidden fields, type show overwrites anyway
   if($this->config["useStyling"]&&($type=="show"||(!$d->check("properties", "hidden", $names[0])&&$type!="hidden")))
   {
    switch($v["type"])
    {
     case "boolean":
      $r.="<div class=\"field\">";

      if($type=="fake")
       $r.="<div class=\"title\">".$this->l->t($v["title"])."</div>\n";
      else
       $r.="<div class=\"title\">&nbsp;</div>\n";

      if($this->config["showTips"]&&$v["text"]!="")
       $r.="<div class=\"text\">".$this->l->t($v["text"])."</div>";

      $r.="<div class=\"input\">\n";

      foreach($names as $name)
       $r.=$this->fieldInput($name, $type, $nr);

      $r.="</div>\n";
      $r.="</div>\n";
     break;

     default:
      $r.="<div class=\"field\">";
      $r.="<div class=\"title\">";

      if($v["min"]>0)
      {
       if($this->config["requiredIndicatorFile"])
       {
        if(file_exists($_path["project"]["images"].$this->config["requiredIndicatorFile"]))
         $r.="<img src=\"".$_path["HTML"]["prefix"].$_path["project"]["images"].$this->config["requiredIndicatorFile"]."\" alt=\"".$this->l->t("Required")."\" /> ";
        else
         $r.=$this->config["requiredIndicatorFile"];
       }
       else
        $r.=" * ";
      }

      if($v["title"])
       $r.=$this->l->t($v["title"]);
      else
       $r.="&nbsp; ";

      $r.="</div>\n";

      if($this->config["showTips"]&&$v["text"]!="")
       $r.="<div class=\"text\">".$this->l->t($v["text"])."</div>";

      if(($type=="fake"&&isset($d->reference[$names[0]]))||(isset($this->showReference)&&isset($this->showReference[$names[0]])&&!$this->showReference[$names[0]]))
       $r.="<div class=\"input reference\">\n";
      else
       $r.="<div class=\"input\">\n";

      $cmsOptions=false;
      foreach($names as $name)
       $r.=$this->fieldInput($name, $type, $nr, $cmsOptions);

      $r.="</div>\n";

      if($cmsOptions!==false&&$type!="disabled") //strict check, "" adds empty options container
       if($this->config["cmsOptions"]||$cmsOptions) //don't show javascript cms options if configuration is disabled
        $r.="<div class=\"options\">".$cmsOptions."</div>";

      $r.="</div>\n";
     break;
    }
   }
   else //no fancy showing
   {
    foreach($names as $name)
     $r.=$this->fieldInput($name, $type, $nr);
   }

   return $r;
  }


  function fieldInput($name, $type=false, $nr=false, &$cmsOptions=false)
  {
   global $_config, $_path;

   $r="";
   $d=&$this->data;
   $f=&$this->form;

   $v=&$d->field[$name];

   if($d->requestVar)
   {
    if($nr===false||$name=="m"||$name=="a"||$name=="e"){$fieldname=$d->requestVar."[".$name."]"; $nr=0;}
    else{$fieldname=$d->requestVar."[".$name."]"."[".$nr."]";}
   }
   else
   {
    if($nr===false||$name=="m"||$name=="a"||$name=="e"){$fieldname=$name; $nr=0;}
    else{$fieldname=$name."[".$nr."]";}
   }

   if($type=="show"||(!$d->check("properties", "hidden", $name)&&$type!="hidden"))
   {
    if($this->config["anchoring"])
     $r.="<a id=\"".$fieldname."\"></a>";

    if($type!="fake")
    {
     $extra="";
     if($d->check("properties", "reload", $name))
     {
      if($this->config["anchoring"])
       $extra.="onchange=\"this.form.action=this.form.action.replace(/\#.*$/, ''); this.form.action+='#".$fieldname."'; this.form.reload_trigger.value='".$name."'; this.form.submit();\" ";
      else
       $extra.="onchange=\"this.form.reload_trigger.value='".$name."'; this.form.submit();\" ";
     }

     if($type=="disabled"||$d->check("properties", "disabled", $name))
      $extra.="disabled ";

     //show references
     if(isset($d->reference[$name]))
     {
      $class="form ".$d->reference[$name]["module"]." ".$name;

      if($d->reference[$name]["type"]=="1:1")
      {
       $subd=Data::cache($d->reference[$name]["module"], $this->data->action, $this->data->extra, Array("request" => md5($name)));

       if(($subd->action=="copy"||$subd->action=="edit")&&!request("s")&&!request("dontgetdata"))//get data the first time
       {
        $subd->setValue($d->reference[$name]["to"], $d->getValue($d->reference[$name]["from"], $nr), $nr);
        $subd->get();
       }

       $subcms=new CMS();
       $subcms->setDataForm($subd, $this->form);

       foreach($subd->check("use", "form") as $field)
        if($subd->check("use", "sql", $field)&&!$subd->check("properties", "owner", $field)&&!$subd->check("properties", "rightsReference", $field)&&!$subd->check("properties", "identifier", $field))
         $r.=$subcms->field($field, $type);

       unset($subcms, $subd);
      }
      else //*:0, *:1, *:*, 0:* and 1:*
      {
       if($this->data->requestVar)
        $ref_name=request($this->data->requestVar, "_refmanip");
       else
        $ref_name=request("_refmanip");

       if(is_array($ref_name))
       {
        $ref_action=strtolower(reset($ref_name));
        $ref_name=array_keys($ref_name)[0];
       }
       else
       {
        $ref_name=false;
        $ref_action=false;
       }

       if($this->referenceManipulation&&$ref_action&&$ref_name==$name) //reference manipulation
       {
        if(isset($this->showReference)&&isset($this->showReference[$name])&&!$this->showReference[$name])
        {
         $subd=Data::cache($d->reference[$name]["module"], ($ref_action=="edit"?"edit":"add"), "item", Array("request" => $this->data->requestVar."SubCMS"));

         if($subd->action=="edit")
         {
          if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
           $subd->setValue($subd->getID(), end($d->getValue($ref_name)));
          else
           $subd->setValue($subd->getID(), $d->getValue($ref_name));

          if(!request($subd->requestVar, "dontgetdata"))//get data the first time
           if(!$subd->get()||$subd->e->any("warning"))
            $r.=$subd->e->show();
         }

         $subcms=new CMS();
         $subcms->referenceManipulation=false;
         $subcms->setDataForm($subd, $this->form);

         $r.=$subcms->formShowFields();

         if($this->data->requestVar)
          $this->addVar($this->data->requestVar."[_refmanip][".$name."]", $subd->action);
         else
          $this->addVar("_refmanip[".$name."]", $subd->action);

         if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
         {
          foreach(array_keys($v["value"][$nr]) as $k)
           $r.=$f->input("hidden", $fieldname."[".$nr."][".$k."]", $v["value"][$nr][$k]);
         }
         else
          $r.=$f->input("hidden", $fieldname, $v["value"][$nr]);

         $r.=$f->input("hidden", $subd->requestVar."[dontgetdata]", 1)."\n";

         $r.="<div class=\"field\">\n";
         $r.="<div class=\"title\">\n";
         $r.="</div>\n<div class=\"input\">\n";

         if($subd->action=="add")
          $r.=$f->input("submit", $subd->requestVar."[s]", $subcms->l->t("Add"), "submit")."\n";
         else
          $r.=$f->input("submit", $subd->requestVar."[s]", $subcms->l->t("Update"), "submit")."\n";

         $r.=$f->input("button", "", $subcms->l->t("Back"), "submit", false, "onclick=\"history.back();\"")."\n";

         $r.="</div>\n";
         $r.="</div>\n";
        }

        unset($subcms, $subd);
       }

       if(!isset($this->showReference)||!isset($this->showReference[$name])||$this->showReference[$name])
       {
        if($this->referenceManipulation)
        {
         if($this->data->requestVar)
          $_refmanip=$this->data->requestVar."[_refmanip][".$name."]";
         else
          $_refmanip="_refmanip[".$name."]";
        }

        if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
        {
         $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

         if(isset($d->listOptions[$name])&&$d->listOptions[$name])
         {
          if(isset($this->config["tomanyReferenceStyle"])&&$this->config["tomanyReferenceStyle"]=="list")
           $r.=$f->listview(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
          else
           $r.=$f->select(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
         }
         else
         {
          if(isset($this->config["tomanyReferenceStyle"])&&$this->config["tomanyReferenceStyle"]=="list")
           $r.=$f->dataListview($d->reference[$name]["module"], $fieldname."[".$nr."]", $v["value"][$nr], $class, false, $extra, true)."\n";
          else
           $r.=$f->dataSelect($d->reference[$name]["module"], $fieldname."[".$nr."]", $v["value"][$nr], $class, false, $extra, true)."\n";
         }

         $cmsOptions=""; //enable cmsOptions
         if($this->referenceManipulation&&$this->config["tomanyReferenceManip"]&&!strstr($extra, "disabled "))
         {
          $cmsOptions.=$f->input("submit", $_refmanip, "Add");
          $cmsOptions.=$f->input("submit", $_refmanip, "Edit");
         }
        }
        else //*:0 and *:1
        {
         if(isset($d->listOptions[$name])&&$d->listOptions[$name])
         {
          if(isset($this->config["manytoReferenceStyle"])&&$this->config["manytoReferenceStyle"]=="list")
           $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
          else
           $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
         }
         else
         {
          if(isset($this->config["manytoReferenceStyle"])&&$this->config["manytoReferenceStyle"]=="list")
           $r.=$f->dataListview($d->reference[$name]["module"], $fieldname, $v["value"][$nr], $class, false, $extra, false, ($d->reference[$name]["type"]=="*:0"))."\n";
          else
           $r.=$f->dataSelect($d->reference[$name]["module"], $fieldname, $v["value"][$nr], $class, false, $extra, false, ($d->reference[$name]["type"]=="*:0"))."\n";
         }

         $cmsOptions=""; //enable cmsOptions
         if($this->referenceManipulation&&$this->config["manytoReferenceManip"]&&!strstr($extra, "disabled "))
         {
          $cmsOptions.=$f->input("submit", $_refmanip, "Add");
          $cmsOptions.=$f->input("submit", $_refmanip, "Edit");
         }
        }
       }
      }
     }
     else
     {
      $class="form ".$v["type"]." ".$name;

      if($d->check("properties", "options", $name))
       $cmsOptions=""; //enable cmsOptions

     //show fields
      switch($v["type"])
      {
       case "list":
       {
        if(isset($this->config["listStyle"])&&$this->config["listStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }
       case "multilist":
       {
        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["multilistStyle"])&&$this->config["multilistStyle"]=="list")
         $r.=$f->listview(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "module":
       {
        //Set module to upper lying category module by default
        if(!$v["value"][$nr]&&!request("s")&&$d->module=="categories")
        {
         $d2=Data::cache("categories", "view", "item");
         $d2->setValue("nr", $d->getValue("catnr"));
         $d2->get();
         $v["value"][$nr]=$d2->getValue("module", $nr);
        }

        if(isset($this->config["moduleStyle"])&&$this->config["moduleStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }
       case "modules":
       {
        //Set module to upper lying category module by default
        if(!$v["value"][$nr]&&!request("s")&&$d->module=="categories")
        {
         $d2=Data::cache("categories", "view", "item");
         $d2->setValue("nr", $d->getValue("catnr"));
         $d2->get();
         $v["value"][$nr]=$d2->getValue("module", $nr);
        }

        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["modulesStyle"])&&$this->config["modulesStyle"]=="list")
         $r.=$f->listview(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(true, $fieldname."[".$nr."]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localfile":
       {
        $options=$d->listOptions[$name];
        array_unshift($options, "");

        if(isset($this->config["localfileStyle"])&&$this->config["localfileStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localfiles":
       {
        $options=$d->listOptions[$name];

        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["localfilesStyle"])&&$this->config["localfilesStyle"]=="list")
         $r.=$f->listview(true, $fieldname."[".$nr."]", $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(true, $fieldname."[".$nr."]", $options, $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localdirectory":
       {
        $options=$d->listOptions[$name];
        array_unshift($options, "");

        if(isset($this->config["localfileStyle"])&&$this->config["localfileStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "plugin":
       {
        if(isset($this->config["pluginStyle"])&&$this->config["pluginStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "project":
       {
        if(isset($this->config["projectStyle"])&&$this->config["projectStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "action":
       {
        $options=$d->actionList;
        array_unshift($options, "");

        if(isset($this->config["actionStyle"])&&$this->config["actionStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "extra":
       {
        $options=Array("");
        foreach($d->extraList as $actions)
         foreach($actions as $extra)
          if(!in_array($extra, $options))
           $options[]=$extra;

        if(isset($this->config["extraStyle"])&&$this->config["extraStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "rights":
       {
        $r.=$f->rights($fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }

       case "html":
       case "text":
       {
        if($v["max"]>=256)
        {
         if($v["type"]=="html"&&$d->check("properties", "wysiwyg", $name))
          $r.=$f->wysiwyg($fieldname, $v["value"][$nr], $class, false, $extra)."\n";
         else
         {
          $r.=$f->input("textarea", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

          if(isset($d->listOptions[$name])&&$d->listOptions[$name])
           $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");
         }

         break;
        }

        //don't break; use logic below
       }

       case "url":
       case "title":
       case "hostname":
       case "domain":
       case "email":
       case "phone":
       case "regexp":
       case "search":
       case "filename":
       case "alphabetic":
       case "ip":
       case "ipv4":
       case "ipv6":
       {
        if($v["max"]>0)
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra."maxlength=\"".$v["max"]."\"")."\n";
        else
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        if(isset($d->listOptions[$name])&&$d->listOptions[$name])
         $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");

        break;
       }

       case "nr":
       {
        if($d->check("properties", "dropdown", $name)&&$v["max"]>0)
        {
         $r.=$f->selectStart(false, $fieldname, $class, false, $extra)."\n";

         for($c=$v["min"];$c<=$v["max"];$c++)
         {
          if($c==$v["value"][$nr])
           $r.=$f->selectOption($c, false, "selected")."\n";
          else
           $r.=$f->selectOption($c)."\n";
         }
         $r.=$f->selectEnd()."\n";
        }
        else
        {
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

         if(isset($d->listOptions[$name])&&$d->listOptions[$name])
          $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");
        }

        break;
       }

       case "float":
       case "price":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        if(isset($d->listOptions[$name])&&$d->listOptions[$name])
         $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");

        break;
       }


       case "md5":
       case "password":
       {
        if(
         !preg_match("/^verify.+$/", $name)&& //non verify field
         ( //add, copy, edit, change password
          $d->action=="add"||
          $d->action=="copy"||
          $d->action=="edit"||
          ($d->action=="user"&&$d->extra=="password")
         )
        )
        {
         $cmsOptions=""; //enable cmsOptions

         //generate password requirements
         $placeholder=Array();
         if($v["min"])
          $placeholder[]=$v["min"]." long";
         if($_config->get("passwordupper"))
          $placeholder[]=$_config->get("passwordupper")." upper";
         if($_config->get("passwordnumbers"))
          $placeholder[]=$_config->get("passwordnumbers")." numbers";
         if($_config->get("passwordspecials"))
          $placeholder[]=$_config->get("passwordspecials")." special";

         if($placeholder)
          $placeholder=" placeholder=\"".implode(", ", $placeholder)."\"";
         else
          $placeholder="";
        }
        else
         $placeholder="";

        $r.=$f->input("password", $fieldname, (isset($v["value"][$nr])?$v["value"][$nr]:""), $class, false, $extra.$placeholder)."\n";
        break;
       }

       case "captcha":
       {
        $captcha=new Captcha();

        if(request("captchaRefresh"))
         $captcha->random();

        $r.=$this->l->t("Please enter the following ".$captcha->length." characters in the field below")."<br/>";
        $r.="<img src=\"".$_path["HTML"]["root"]."captcha.php?".rand(0,32768)."\" alt=\"Captcha\" />\n"; //random hack for IE (6+8) caching problem...
        $r.=$f->input("hidden", "captchaRefresh", 0)."<a href=\"#\" onclick=\"this.previousSibling.value=1; this.previousSibling.form.submit(); return false;\" title=\"".$this->l->t("New image")."\">".$this->l->t("New image")."</a><br/>\n";
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        unset($captcha);
        break;
       }


       case "boolean":
       {
        //IE onchange triggered onblur after onclick: reload fix: onclick triggers onchange
        $r.=$f->input("hidden", $fieldname, 0, false, false, $extra);
        $r.=$f->checkbox($v["value"][$nr], $this->l->t($v["title"]), $fieldname, 1, $class, false, $extra.($d->check("properties", "reload", $name)?"onclick=\"this.onchange();\"":""))."\n";
        break;
       }

       case "enum":
       {
        if(array_type($d->listOptions[$name])=="indexed")
        {
         foreach($d->listOptions[$name] as $k)
         {
          if($v["value"][$nr]===false)
           $v["value"][$nr]=$k;

          $r.=$f->radio(($v["value"][$nr]==$k), $k, $fieldname, $k, $class, false, $extra)."\n";
         }
        }
        else //associative
        {
         foreach($d->listOptions[$name] as $k => $t)
         {
          if($v["value"][$nr]===false)
           $v["value"][$nr]=$k;

          $r.=$f->radio(($v["value"][$nr]==$k), $t, $fieldname, $k, $class, false, $extra)."\n";
         }
        }

        break;
       }


       case "file":
       case "image":
       {
        $r.=$f->input("file", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }

       case "date":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra." placeholder=\"".$v["format"][$d->dataformat]["dateformat"]."\"")." ";

        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["dateformat"]."</a>\n";

        break;
       }

       case "time":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)." ";

        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["timeformat"]."</a>\n";

        break;
       }

       case "datetime":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra." placeholder=\"".$v["format"][$d->dataformat]["datetimeformat"]."\"")." ";

        if($this->config["showTimezone"])
         $r.=$this->l->t("Timezone").": ".$v["format"][$d->dataformat]["timezone"].($this->config["showDateformat"]?", ":"");
        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["datetimeformat"]."</a>\n";

        break;
       }


       case "version":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       case "label":
       {
        if(isset($v["value"][$nr]))
         $r.=$v["value"][$nr]."\n";
        break;
       }
       case "submit":
       {
        $r.=$f->input("submit", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       default:
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr), $class)."\n";
        break;
       }
      }
     }
    }
    else //fake fields
    {
     if(isset($d->reference[$name]))
     {
      $dr=Data::cache($d->reference[$name]["module"], "view", "item");
      $dr->setValue("nr", $v["value"][$nr]);
      $dr->get();

      $cms=new CMS();
      $cms->setDataForm($dr, $this->form);

      foreach($dr->check("use", "form") as $name2)
       if($dr->check("use", "show", $name2))
        if(!isset($dr->reference[$name2]))
         if(!$dr->check("properties", "default", $name2))
          for($nr2=0;$nr2<$dr->vmax;$nr2++)
           $r.=$cms->field($name2, "fake", $nr2);

      unset($cms, $dr);
     }
     else
     {
      $class="form ".$v["type"]." ".$name;
      $extra="disabled";

      switch($v["type"])
      {
       case "boolean":
        $r.=$f->input("fake", $fieldname, ($v["value"][$nr]?$this->l->t("Yes"):$this->l->t("No")))."\n";
       break;


       case "nr":
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr))."\n";
        break;
       }


       case "image":
       {
        $r.="<img src=\"".$_path["HTML"]["root"]."images/?m=".$d->module."&amp;field=".$name."&amp;nr=".$d->getValue("nr", $nr)."\" style=\"width: 100px;\" alt=\"".$this->l->t($v["title"])."\">";
        break;
       }

       case "file":
       {
        $r.="<a href=\"".$_path["HTML"]["root"]."files/?m=".$d->module."&amp;field=".$name."&amp;nr=".$d->getValue("nr", $nr)."\" title=\"Download ".$d->getValue($name, $nr)."\">".$d->getValue($name, $nr)."</a>";
        break;

       }

       case "multilist":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }

       case "modules":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }

       case "localfiles":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }


       case "rights":
       {
        $r.=$f->rights($fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       case "label":
       {
        if(isset($v["value"][$nr]))
         $r.=$v["value"][$nr]."\n";
        break;
       }
       case "submit":
       {
        $r.=$f->input("submit", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       default:
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr))."\n";
        break;
       }
      }
     }
    }
   }
   else
   {
    if($name=="reload_trigger")
     $r.=$f->input("hidden", $fieldname, "")."\n";
    else
    {
     $value=$d->getValue($name, $nr);

     if(!is_array($value))
      $r.=$f->input("hidden", $fieldname, $value)."\n";
     else
      foreach($value as $k => $v)
       $r.=$f->input("hidden", $fieldname."[0][".$k."]", $v)."\n";
    }
   }

   return $r;
  }

 }
?>
