<?

 /*****************************************************************************
 * Filename: include/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");

 $_config=new Config();

 class Config
 {
  function __construct()
  {
   $this->e=new Errors();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }


  function set($key, $value=NULL)
  {
   global $_session;

   if(!$key)
    $key="config";
   else
    $key="config.".$key;

   return $_session->set($key, $value);
  }

  function get($key)
  {
   global $_session;

   if(!$key)
    $key="config";
   else
    $key="config.".$key;

   return $_session->get($key);
  }


  function userAnonymous()
  {
   if($this->get("usernr")==$this->get("anonnr"))
    return true;

   return false;
  }
  function userRoot()
  {
   if($this->get("usernr"))
    if($this->get("usernr")==$this->get("rootnr"))
     return true;

   return false;
  }
  function userBackend()
  {
   if($this->get("userbackend")||$this->userRoot())
    return true;

   return false;
  }

  function su($user=false)
  {
   if($user===true) //become root
   {
    if($this->get("suusernr")!==NULL) //already another user
     return false;

    $this->set("suusernr", $this->get("usernr")); //store current user
    $this->set("usernr", $this->get("rootnr"));
   }
   else if($user===false) //revert to original user
   {
    if($this->get("suusernr")===NULL) //not another user
     return false;

    $this->set("usernr", $this->get("suusernr")); //restore current user
    $this->set("suusernr"); //unset suusernr
   }
   else //become user
   {
    if(!$this->userRoot()) //must be root
     return false;

    if($this->get("suusernr")!==NULL) //already another user
     return false;

    $d=Data::cache($this->get("usermodule"), "view", "all");
    $d->setShowFields("nr");
    $d->clearValues();

    if($d->e->any()) //error initializing module
     return false;

    if(preg_match("/^\d+$/", $user)) //id
     $d->setValue($d->getID(), $user);
    else //username
    {
     if(!($userField=$d->check("properties", "username"))) //no username field
      return false;

     if(sizeof($userField)!=1) //multiple username fields, not supported
      return false;

     $userField=$userField[0];
     $d->setValue($userField, $user);
    }

    if(!$d->get()||$d->vmax!=1||!$d->next()) //no/multiple users found
     return false;

    $this->set("suusernr", $this->get("usernr")); //store current user
    $this->set("usernr", $d->v("id"));
   }

   return true;
  }


  function reload()
  {
   $this->set("reload", true);
   return $this->load();
  }

  function load()
  {
   global $_session, $_path;

   //load config if not yet done, on reload or if previous load failed
   if($this->get(false)&&!$this->get("reload")&&!$this->get("failed"))
   {
    //update script timezone
    if($this->get("timezone")&&$this->get("timezone")!=date_default_timezone_get())
     date_default_timezone_set($this->get("timezone"));

    return true;
   }

   $config=Array();

   if($this->get("usernr")) //keep user logged in on reload and failures
   {
    if($this->get("userbackend"))
     $config=Array("usernr" => $this->get("usernr"), "groupnrs" => $this->get("groupnrs"), "userbackend" => $this->get("userbackend"));
    else
     $config=Array("usernr" => $this->get("usernr"), "groupnrs" => $this->get("groupnrs"));
   }
   else //normal load, get everything
    $config=Array();

   $config["language"]="en";//set default language for language.php

   $customConfigModules=Array();


   //general config
   $dc=Data::cache("config", "view", "all");
   $customConfigModules[]=$dc->module;

   if(!$dc->get()||!$dc->next()) //problem in loading config, make fake config
   {
    foreach($dc->check("use", "show") as $k)
     if($k!="r")
      if(!isset($config[$k])||$k=="language") //only set fields (except language) that aren't set yet, ie: usernr will not be overwritten on reload and failures
       $config[$k]=$dc->field[$k]["defaultvalue"];

    $config["failed"]=true; //keep reloading config till successful
    $config["useModRewrite"]=false; //make sure modRewrite is handled properly
    $config["language"]=strtolower($this->get("language"));
    $config["title"]="Project configuration failure";

    $this->set(false, $config);
    return false;
   }
   else
   {
    foreach($dc->check("use", "show") as $k)
     if($k!="r")
      if(!isset($config[$k])||$k=="language") //only set fields (except language) that aren't set yet, ie: usernr will not be overwritten on reload and failures
       $config[$k]=$dc->v[$k];

    $config["language"]=strtolower($config["language"]);

    $this->set(false, $config); //set preliminary config so subsequent feature have a base config

    //load group rights
    if(!isset($config["groupnrs"]))
    {
     $users=Data::cache($config["usermodule"], "view", "item");
     $config["groupnrs"]=$users->moduleObject->getGroups($config["usernr"]);
     unset($users);
    }
   }


   //Modules configs, multiple rows
   $config["modules"]=Array();
   $dc=Data::cache("configModules", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$dc->module;

   if($dc->tableExists()&&$dc->get())
   {
    while($dc->next())
    {
     $config["modules"][$dc->v["module"]]=Array();
     foreach($dc->check("use", "show") as $k)
     {
      if($k!="r")
      {
       if($dc->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$dc->v[$k]==-1)) //empty, use config
        $config["modules"][$dc->v["module"]][$k]=$config[$k];
       else
        $config["modules"][$dc->v["module"]][$k]=$dc->v[$k];
      }
     }
    }
   }


   //Module fields configs, multiple rows and are placed under applicable module
   $dc=Data::cache("configFields", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$dc->module;

   if($dc->tableExists()&&$dc->get())
   {
    while($dc->next())
    {
     if(!isset($config["modules"][$dc->v["module"]]))
      $config["modules"][$dc->v["module"]]=Array("field" => Array());

     foreach($dc->check("use", "show") as $k)
     {
      if($k!="r")
      {
       if($dc->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$dc->v[$k]==-1)) //empty, use config/moduleConfig
       {
        if(isset($config["modules"][$dc->v["module"]][$k]))
         $config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=&$config["modules"][$dc->v["module"]][$k];
        else
         $config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=&$config[$k];
       }
       else
        $config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=$dc->v[$k];
      }
     }
    }
   }


   //Repositories config, multiple rows
   $config["repositories"]=Array();
   $dc=Data::cache("configRepositories", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$dc->module;

   if($dc->tableExists()&&$dc->get())
   {
    while($dc->next())
    {
     $config["repositories"][$dc->v["code"]]=Array();
     foreach($dc->check("use", "show") as $k)
     {
      if($k!="r"&&$k!="code")
       $config["repositories"][$dc->v["code"]][$k]=$dc->v[$k];
     }
    }
   }

   if(!$config["repositories"]) //no repositories configured, use default
   {
    $code=$dc->getDefaultValue("code");
    $config["repositories"][$code]=Array();
    foreach($dc->check("use", "show") as $k)
    {
     if($k!="r"&&$k!="code")
      $config["repositories"][$code][$k]=$dc->getDefaultValue($k);
    }
   }


   //JAS backend sites config, multiple rows
   if(in_array("configSites", JAS::modules()))
   {
    $config["sites"]=Array();
    $dc=Data::cache("configSites", "view", "all", Array("skipCheckTables" => true));
    $customConfigModules[]=$dc->module;

    if($dc->tableExists()&&$dc->get())
    {
     while($dc->next())
     {
      $config["sites"][$dc->v["domain"]]=Array();
      foreach($dc->check("use", "show") as $k)
      {
       if($k!="r"&&$k!="domain")
        $config["sites"][$dc->v["domain"]][$k]=$dc->v[$k];
      }
     }
    }
   }


   //Other "generic" configs
   foreach(JAS::modules() as $module)
   {
    if(strpos($module, "config")===0&&!in_array($module, $customConfigModules)) //should start with config prefix and shouldn't be already gathered above
    {
     $dc=Data::cache($module, "view", "all", Array("skipCheckTables" => true));
     $key=strtolower(str_replace("config", "", $dc->module));

     if(isset($config[$key]))
     {
      $this->e->add("Config key: \"".$key."\" for module: \"".$dc->module."\" already defined by another config module", "module", "Please rename: \"".$dc->module."\"");
      continue;
     }

     if($dc->tableExists()&&$dc->get()&&$dc->next())
     {
      $config[$key]=Array();
      foreach($dc->check("use", "show") as $k)
       if($k!="r")
        $config[$key][$k]=$dc->v[$k];
     }
    }
   }

   //store config
   $this->set(false, $config);

   //set whether mod rewrite should actually be used
   switch($config["modRewrite"])
   {
    case "Yes":
    {
     $this->set("useModRewrite", true); //this will get you into trouble if mod rewrite is not active
     break;
    }
    case "No":
    {
     $this->set("useModRewrite", false);
     break;
    }
    default:
    case "Auto":
    {
     $this->set("useModRewrite", $_session->get("modRewriteActive"));
     break;
    }
   }

   //update script timezone
   if($this->get("timezone")&&$this->get("timezone")!=date_default_timezone_get())
    date_default_timezone_set($this->get("timezone"));

   return true;
  }


  function merge($key, $config=false)
  {
   //check if config key exists
   $module="config".$key;
   if(!in_array($module, JAS::modules()))
   {
    $this->e->add($key." config merge failed: unknown config type", "system");
    return false;
   }

   if(!$config||!is_array($config)) //no /illegal config parameter
    $config=Array(); //use empty config

   $var=strtolower($key);

   //fill in the blanks in the config array
   if(!($configured=$this->get($var))||!is_array($configured)) //config key not set: use default values
   {
    $d=Data::cache($module, "view", "item", Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true)); //don't fully init object, only default values needed

    if($d->e->any())
     $this->e->add($d->e);

    $configured=Array();
    foreach($d->check("use", "show") as $field)
     $configured[$field]=$d->getDefaultValue($field);
   }

   return array_merge($configured, $config); //merge config (configured overwritten with provided values)
  }

 }
?>
