<?

 /*****************************************************************************
 * Filename: include/form.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");

/******************************************************************
 FORM class

 functions:
 + start:      Returns html form opening tage
 + input:      Returns input field of given type
 - end:        Returns html form closing tage


******************************************************************/

 class Form
 {

  function __construct()
  {
   $this->l=new Language();
   $this->l->read("form");
   $this->l->read("cms");

   $this->form=0;

   $this->listValuesOrdering=true; //order list based on ordering used in values (listview/select)
  }

  function __clone()
  {
   //clone objects
   $this->l=clone $this->l;
  }


/******************************************************************
 Start
 Returns the start tag of a form

 + action:  what file open
 - name:    form name, default: frm
 - id:      id of form
 - extra:   extra options like javascript

******************************************************************/
  function start($action=false, $method=false, $class=false, $id=false, $extra=false)
  {
   global $_jses, $_path;

   if(!$action){$action=$_path["HTML"]["url"];}
   if(!$method){$method="post";}

   if($class){$class=" class=\"$class\"";}
   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}

   $r="<form action=\"".$action."\" method=\"".$method."\" enctype=\"multipart/form-data\"".$class.$id.$extra.">";
   $r.="<div class=\"invisible\">";

   return $r;
  }


/******************************************************************
 Input
 Returns the input of a specified type

 + type:    type of input field: text, password, hidden
 - name:    input field name
 - value:   value of input field
 - class:   css class reference, default: form
 - id:      id of form
 - extra:   extra options like javascript
 - text:    extra text

******************************************************************/
  function input($type="hidden", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   $value=htmlentities($value, ENT_COMPAT|ENT_XHTML);

   if($type=="hidden")
    $class="";
   else
   {
    if(!$class)
     $class="form";

    $class.=" ".$type;
    $class=" class=\"$class\"";
   }

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}

   if($type=="radio"||$type=="checkbox")
   {
    $extra=str_ireplace(Array("  ", " checked", " disabled"), Array(" ", " checked=\"checked\"", " disabled=\"disabled\""), $extra);
    $extra=" style=\"width: 15px; height: 15px;\"".$extra;
   }
   else if($type=="hidden"||$type=="fake")
    $extra=str_ireplace(Array("  ", " checked", " disabled"), Array(" ", "", ""), $extra);
   else
    $extra=str_ireplace(Array("  ", " checked", " disabled"), Array(" ", "", " disabled=\"disabled\""), $extra);

   $r="";
   switch($type)
   {
    case "textarea":
     $r.="<textarea name=\"".$name."\"".$class.$id.$extra." cols=\"\" rows=\"\">";
     $r.=$value;
     $r.="</textarea>";
    break;

    case "fake":
     if($value==""){$value="&nbsp; ";}
     $r.="<p".$class.$id.$extra.">";
     $r.=$value;
     $r.="</p>";
    break;

    default:
     $r.="<input type=\"".$type."\" name=\"".$name."\" value=\"".$value."\"".$class.$id.$extra."/>";
    break;
   }

   if(strpos($extra, " disabled")!==false) //input disabled
   {
    //add hidden name/value to ensure values are submitted
    if($type!="radio"&&$type!="checkbox") //normal field
     $r.=$this->input("hidden", $name, $value);
    else if(strpos($extra, " checked")!==false) //checkbox/radio only if checked
     $r.=$this->input("hidden", $name, $value);
   }

   return $r;
  }

  function button($name="", $value=false, $class=false, $id=false, $extra=false)
  {
   if(!$class)
    $class="form";

   $class.=" button";
   $class=" class=\"$class\"";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}
   $extra=str_ireplace(Array("  ", " submit", " reset", " checked", " disabled"), Array(" ", " type=\"submit\"", " type=\"reset\"", "", " disabled=\"disabled\""), $extra);

   if(!preg_match("/ type=\"[^\"]+\"/", $extra))
    $extra=" type=\"button\"".$extra;

   $value2=trim(strip_tags($value));

   $r="<button".($name?" name=\"".$name."\"":"").($value2?" value=\"".$value2."\"":"").$class.$id.$extra.">";
   $r.=$value;
   $r.="</button>";

   return $r;
  }

  function checkbox($checked=false, $text="", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   if($checked)
    $extra.=" checked";

   if(!$id)
    $id="checkbox".rand(0,32768);

   $r="";
   if($text!="")
    $r.="<label for=\"".$id."\">";

   $r.=$this->input("checkbox", $name, $value, $class, $id, $extra);

   if($text!="")
    $r.=" ".$text."</label>";

   return $r;
  }

  function radio($checked=false, $text="", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   if($checked)
    $extra.=" checked";

   if(!$id)
    $id="radio".rand(0,32768);

   $r="";
   if($text!="")
    $r.="<label for=\"".$id."\">";

   $r.=$this->input("radio", $name, $value, $class, $id, $extra);

   if($text!="")
    $r.=" ".$text."</label>";

   return $r;
  }

  function selectStart($multiple=false, $name="", $class=false, $id=false, $extra=false)
  {
   if(!$class)
    $class="form";

   if($multiple)
   {
    $extra.=" multiple";
    $class.=" multi";
   }
   else
    $class.=" select";

   $class=" class=\"$class\"";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" ".$extra;}
   $extra=str_ireplace(Array("  ", " checked", " disabled", " multiple"), Array(" ", "", " disabled=\"disabled\"", " multiple=\"multiple\""), $extra);

   if(strpos($extra, "multiple=\"multiple\"")!==false)
    $name.="[]";

   return "<select name=\"".$name."\"".$class.$id.$extra.">";
  }
  function selectOption($value, $text=false, $selected=false, $extra=false)
  {
   if($selected)
    $extra.=" selected";

   if(!$text){$text=$value;}
   if($extra){$extra=" ".$extra;}
   $extra=str_ireplace(Array("  ", " selected", " disabled", " checked"), Array(" ", " selected=\"selected\"", " disabled=\"disabled\"", " selected=\"selected\""), $extra);

   $value=" value=\"".$value."\"";

   return "<option".$value.$extra.">".$text."</option>";
  }
  function selectEnd()
  {
   return "</select>";
  }

  function select($multiple=false, $name="", $list=false, $values=false, $class=false, $id=false, $extra=false, $addEmptyOption=false)
  {
   if(!is_array($values))
    $values=Array($values);

   $array_type=array_type($list);

   //create normal list from data object
   if($array_type=="data")
   {
    $tmp=Array();
    if($list->get())
    {
     while($list->next())
     {
      if(isset($list->v["treedepth"])&&$list->v["treedepth"])
       $depth=str_repeat("&nbsp; ", ($list->v["treedepth"]-1)*3)."|_ ";
      else
       $depth="";

      $tmp[$list->v("id")]=$depth.$list->v("title");
     }
    }

    $array_type="associative";

    $list=$tmp;
    unset($tmp);
   }

   if($this->listValuesOrdering)
    $list=$this->orderListValues($list, $values);

   $r=$this->selectStart($multiple, $name, $class, $id, $extra);

   if($addEmptyOption)
   {
    $r.=$this->selectOption("", "");

    if(in_array("", $values))
     $selected[]="";
   }

   $selected=Array();
   switch($array_type)
   {
    case "indexed":
    {
     foreach($list as $v)
     {
      if(in_array($v, $values))
      {
       $selected[]=$v;
       $r.=$this->selectOption($v, $v, true);
      }
      else
       $r.=$this->selectOption($v, $v);
     }

     break;
    }
    case "associative":
    {
     foreach($list as $k => $v)
     {
      if(in_array($k, $values))
      {
       $selected[]=$k;
       $r.=$this->selectOption($k, $v, true);
      }
      else
       $r.=$this->selectOption($k, $v);
     }

     break;
    }
    default: break;
   }

   $r.=$this->selectEnd();

   if(strpos($extra, "disabled")!==false) //select disabled
   {
    if($multiple||strpos($extra, "multiple")!==false)
    {
     foreach($selected as $value)
      $r.=$this->input("hidden", $name."[]", $value);
    }
    else //single value
    {
     if($selected)
      $r.=$this->input("hidden", $name, $selected[0]);
    }
   }

   return $r;
  }

  function listview($multiple=false, $name="", $list=false, $values=false, $class=false, $id=false, $extra=false, $addEmptyOption=false)
  {
   if(!is_array($values))
    $values=Array($values);

   $array_type=array_type($list);

   //create normal list from data object
   if($array_type=="data")
   {
    $tmp=Array();
    if($list->get())
    {
     while($list->next())
     {
      if(isset($list->v["treedepth"])&&$list->v["treedepth"])
       $depth=str_repeat("&nbsp; ", ($list->v["treedepth"]-1)*3)."|_ ";
      else
       $depth="";

      $tmp[$list->v("id")]=$depth.$list->v("title");
     }
    }

    $array_type="associative";

    $list=$tmp;
    unset($tmp);
   }

   if($this->listValuesOrdering)
    $list=$this->orderListValues($list, $values);

   if(!$class)
    $class="form";

   if($id){$id=" id=\"$id\"";}

   if($multiple||strpos($extra, "multiple")!==false)
   {
    $type="checkbox";
    $class.=" multilistview ".$name;
    $name.="[]";
   }
   else
   {
    $type="radio";
    $class.=" listview ".$name;
   }

   $class=" class=\"$class\"";

   if($extra){$extra=" $extra";}
   $extra=str_ireplace(Array("  ", " checked", " disabled", " multiple"), Array(" ", "", " disabled=\"disabled\"", ""), $extra);

   $r="<div".$class.$id.$extra.">";

   $disabled=(strpos($extra, "disabled")!==false?"disabled":false); //listview disabled

   if($addEmptyOption)
    $r.=$this->$type((in_array("", $values)), "", $name, "", false, false, $disabled)."<br/>";

   switch(array_type($list))
   {
    case "indexed":
    {
     foreach($list as $v)
      $r.=$this->$type((in_array($v, $values)), $v, $name, $v, false, false, $disabled)."<br/>";

     break;
    }
    case "associative":
    {
     foreach($list as $k => $v)
      $r.=$this->$type((in_array($k, $values)), $v, $name, $k, false, false, $disabled)."<br/>";

     break;
    }
    default: break;
   }

   $r.="</div>";
   return $r;
  }

  function dataSelect($module="categories", $name="catnr", $values=false, $class=false, $id=false, $extra=false, $multi=false, $addEmptyOption=false)
  {
   $d=Data::cache($module, "view", "all");
   $d->clearValues();

   if(($treeReference=$d->getSelfReference())&&!$d->extends)
   {
    $d->setAction("tree");
    $d->setExtra("down");

    $d->setValue($treeReference, 0);
    $d->setShowFields(array_merge($d->check("properties", "title"), Array("treedepth", $treeReference)));
   }
   else
    $d->setShowFields($d->check("properties", "title"));

   return $this->select(($multi?true:false), $name, $d, $values, $class, $id, $extra, $addEmptyOption);
  }

  function dataListview($module="categories", $name="catnr", $values=false, $class=false, $id=false, $extra=false, $multi=false, $addEmptyOption=false)
  {
   $d=Data::cache($module, "view", "all");
   $d->clearValues();

   if(($treeReference=$d->getSelfReference())&&!$d->extends)
   {
    $d->setAction("tree");
    $d->setExtra("down");

    $d->setValue($treeReference, 0);
    $d->setShowFields(array_merge($d->check("properties", "title"), Array("treedepth", $treeReference)));
   }
   else
    $d->setShowFields($d->check("properties", "title"));

   return $this->listview(($multi?true:false), $name, $d, $values, $class, $id, $extra, $addEmptyOption);
  }

  function orderListValues($list, $values)
  {
   if(sizeof($values)<2) //no ordered values
    return $list; //don't bother

   $newlist=Array();

   switch(array_type($list))
   {
    case "indexed":
    {
     foreach($list as $item)
     {
      if(in_array($item, $newlist)) //item already in list
       continue; //skip

      if(!in_array($item, $values)) //not in values
       $newlist[]=$item; //add in current order
      else //in values
      {
       //ensure all previous values are in list
       foreach($values as $value)
       {
        if($value==$item) //current value
        {
         $newlist[]=$item; //add to list
         break; //done
        }
        else if(in_array($value, $list)&&!in_array($value, $newlist)) //earlier valid value, not in list yet
         $newlist[]=$value; //add to list
       }
      }
     }

     break;
    }
    case "associative":
    {
     foreach($list as $key => $item)
     {
      if(isset($newlist[$key])) //item already in list
       continue; //skip

      if(!in_array($key, $values)) //not in values
       $newlist[$key]=$item; //add in current order
      else //in values
      {
       //ensure all previous values are in list
       foreach($values as $value)
       {
        if($value==$key) //current value
        {
         $newlist[$key]=$item; //add to list
         break; //done
        }
        else if(isset($list[$value])&&!isset($newlist[$value])) //earlier valid value, not in list yet
         $newlist[$value]=$list[$value]; //add to list
       }
      }
     }

     break;
    }
    default: break;
   }

   return $newlist;
  }

  function wysiwyg($name, $value, $class=false, $id=false, $extra=false)
  {
   global $_path, $_jses;

   if(!$class)
    $class="form";

   $class.=" wysiwyg";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" ".$extra;}

   $r="";
   $r.=$this->input("textarea", $name, $value, $class, $id, $extra)."\n";

   $r.="<script type=\"text/javascript\">

    function checkWYSIWYG()
    {
     tmp=document.getElementsByTagName('textarea');
     for(c=0;c<tmp.length;c++)
     {
      if(tmp[c].className.match('wysiwyg'))
      {
       if(typeof(initWYSIWYG)=='undefined')
        tmp[c].parentNode.innerHTML+='WYSIWYG script not found on page, make sure you\'ve included wysiwyg.js<br/>';
       if(typeof(onload)=='undefined')
        tmp[c].parentNode.innerHTML+='Onload script not found on page, make sure you\'ve included onload.js<br/>';
       if(typeof(setGetElementFunctions)=='undefined')
        tmp[c].parentNode.innerHTML+='GetElements script not found on page, make sure you\'ve included getelements.js<br/>';
       if(typeof(browser)=='undefined'||typeof(rootpath)=='undefined')
        tmp[c].parentNode.innerHTML+='JAS::JavaScript() not loaded, needed for browser and path detection<br/>';
      }
     }
    }

    checkWYSIWYG();

   </script>\n";

   return $r;
  }

  function rights($name, $value, $class=false, $id=false, $extra=false)
  {
   $keys=Array();
   $d=Data::cache("empty"); //need for rights

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" ".$extra;}

   $r="<table class=\"rights".($class?" ".$class:"")."\"".$id.">";

   foreach($d->rights as $type => $rights)
   {
    $r.="<tr class=\"".$type."\">";
    $r.="<td class=\"type\">".$this->l->t(ucfirst($type))."</td>";

    foreach($rights as $right => $key)
    {
     if(!in_array($key, $keys))
     {
      $r.="<td>".$this->input("hidden", $name."[".$key."]", 0).$this->checkbox((isset($value[$key])&&$value[$key]==1), $this->l->t($right), $name."[".$key."]", "1", $class, false, $extra)."</td>";
      $keys[]=$key;
     }
    }

    $r.="</tr>\n";
   }

   $r.="</table>\n";

   return $r;
  }



/******************************************************************
 End
 Returns the end tag of a form

******************************************************************/
  function end()
  {
   $this->form++;
   return "</div></form>";
  }

 }
?>
