<?

 /*****************************************************************************
 * Filename: include/format/format.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."errors.php");

 class Format
 {
  function __construct()
  {
   $this->fs=new FS();
   $this->e=new Errors();

   $this->clear();

   $this->encoding="ISO-8859-1";
   $this->contentType="text/plain";

   if(get_class($this)=="Format")
    $this->type=false;
   else if(preg_match("/^Format_([A-Za-z]+)$/", get_class($this), $m))
   {
    $this->type=$m[1];
    unset($m);
   }
   else
   {
    $this->e->add("Unable to determine Format type from: \"".get_class($this)."\"", "system");
    return;
   }
  }

  function __clone()
  {
   //clone objects
   $this->fs=clone $this->fs;
   $this->e=clone $this->e;
  }

  function clear()
  {
   $this->output="";
   $this->original="";
   $this->filename=false;

   $this->vc=0;
   $this->vmax=0;
   $this->v=false;
   $this->data=Array();
  }

  function filename($filename)
  {
   $this->filename=$filename;
   return true;
  }


  /*
   * Iterate
   */

  function reset()
  {
   $this->vc=0;
   $this->v=false;
  }

  function next()
  {
   //set all values to the v variable
   if(!$this->value($this->vc))
   {
    $this->vc=0; //restart
    return false;
   }

   $this->vc++; //go to next entry
   return true;
  }

  function previous()
  {
   //set all values to the v variable
   if(!$this->value($this->vmax-($this->vc+1))) //vc from end
   {
    $this->vc=0; //restart
    return false;
   }

   $this->vc++; //go to "next" entry
   return true;
  }

  function first()
  {
   //set all values to the v variable
   if(!$this->value(0))
    return false;

   return true;
  }

  function last()
  {
   //set all values to the v variable
   if(!$this->value($this->vmax-1))
    return false;

   return true;
  }

  function search($value)
  {
   //search if the value exists
   for($c=$this->vc;$c<$this->vmax;$c++)
   {
    if(stristr($this->data[$c], $value))
    {
     //set all values to the v variable
     if(!$this->value($c))
      return false;

     $this->vc=$c+1; //go to next entry
     return true;
    }
   }

   $this->reset();
   return false;
  }

  function value($nr)
  {
//illegal entry
   if($nr<0||$nr>=$this->vmax)
   {
    $this->v=false;
    return false;
   }

   $this->v=$this->data[$nr];
   return true;
  }



  /*
   * Read / Parse
   */

  function read($file, $parse=true)
  {
   $this->clear();

   if(($data=$this->fs->read($file))===false)
   {
    $this->e->add($this->fs->e);
    return false;
   }

   $this->output=$data;
   $this->original=$data;
   $this->filename=$file;

   if($parse)
    return $this->parse($data);

   return true;
  }

  function parse($data)
  {
   $this->output=$data;

   if(!$this->original)
    $this->original=$data;

   $this->vmax=0;
   $this->reset();
   $this->data=Array();

   if(!$data)
   {
    $this->e->add("No data to parse", "error", $this->filename);
    return false;
   }

   $this->data=explode("\n", $data);
   $this->vmax=sizeof($this->data);

   return true;
  }

  function reparse($output=false)
  {
   if($output)
   {
    if(!$this->output)
    {
     $this->e->add("Reparse failed, no data set", "error", $this->filename);
     return false;
    }

    return $this->parse($this->output);
   }

   if(!$this->original)
   {
    $this->e->add("Reparse failed, no original data set", "error", $this->filename);
    return false;
   }

   return $this->parse($this->original);
  }


  /*
   * Write / Output
   */

  function output($headers=false)
  {
   if($headers)
   {
    $h=new Headers();
    if($this->filename)
     $h->file($this->filename, $this->contentType, strlen($this->output));
    else
    {
     $h->type($this->contentType);
     $h->size(strlen($this->output));
    }
   }

   return $this->output;
  }

  function write($file=false)
  {
   if($file)
    $this->filename=$file;

   if(!$this->filename)
   {
    $this->e->add($this->type." Format write failed, no filename given", "system");
    return false;
   }

   if($this->fs->write($this->filename, $this->output(), "overwrite")===false)
   {
    $this->e->add($this->fs->e);
    return false;
   }

   return true;
  }
 }

?>
