<?

 /*****************************************************************************
 * Filename: include/format/rss.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["format"]."xml.php");

 class Format_RSS extends Format
 {
  function __construct($format=false)
  {
   parent::__construct();

   $this->dt=new DT();
   $this->xml=new Format_XML();

   if(!$format)
    $format="atom-1.0";

   $this->setFormat($format);
   $this->setInfo("JAS RSS feed", "JAS example RSS feed", "https://jas.junerules.com/", "https://jas.junerules.com/api/rss/");
  }


  /*
   * Format
   */

  function setFormat($format)
  {
   global $_path;

   $format=strtolower($format);

   if(!($formats=$this->fs->files($_path["format"]."rss/", false, true)))
   {
    $this->e->add("Unable to read RSS formats from: \"".$_path["format"]."rss/\"", "system");
    return false;
   }

   $formats=str_replace(".xml", "", $formats);

   if(!in_array($format, $formats))
   {
    $this->e->add("Unknown RSS format: \"".$format."\"");
    return false;
   }

   $this->format=$format;
   return true;
  }

  function getFormat()
  {
   return $this->format;
  }


  /*
   * Info
   */

  function setInfo($title, $description, $link, $url=false, $updated=false, $generated=false)
  {
   if(!$url&&isset($_SERVER["HTTP_HOST"]))
    $url=(isset($_SERVER["HTTPS"])&&$_SERVER["HTTPS"]?"https":"http")."://".$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];

   if($link&&!preg_match(REGEXP_DATA_URL, $link))
   {
    $this->e->add("Set RSS info failed, incorrect link: \"".$link."\"", "error");
    return false;
   }

   if($url&&!preg_match(REGEXP_DATA_URL, $url))
   {
    $this->e->add("Set RSS info failed, incorrect URL: \"".$url."\"", "error");
    return false;
   }

   if($updated&&!$this->dt->dateExtract($updated, "Y-m-d H:i:s"))
   {
    $this->e->add("Set RSS info failed, incorrect last updatedate: \"".$updated."\"", "error", "Use date format: Y-m-d H:i:s");
    return false;
   }

   if($generated&&!$this->dt->dateExtract($generated, "Y-m-d H:i:s"))
   {
    $this->e->add("Set RSS info failed, incorrect generated date: \"".$generated."\"", "error", "Use date format: Y-m-d H:i:s");
    return false;
   }

   $this->info=Array(
    "title" => $title,
    "description" => $description,
    "link" => $link,
    "url" => $url,
    "updated" => $updated,
    "generated" => $generated,
   );

   return true;
  }

  function getInfo($key=false)
  {
   if(!$key)
    return $this->info;

   if(!isset($this->info[$key]))
    return false;

   return $this->info[$key];
  }


  /*
   * Entries
   */

  function add($title, $description, $link, $updated, $id=false)
  {
   if(!preg_match(REGEXP_DATA_URL, $link))
   {
    $this->e->add("Add RSS entry failed, incorrect link: \"".$link."\"", "error");
    return false;
   }

   if($updated&&!$this->dt->dateExtract($updated, "Y-m-d H:i:s"))
   {
    $this->e->add("Add RSS entry failed, incorrect date: \"".$updated."\"", "error", "Use date format: Y-m-d H:i:s");
    return false;
   }

   if(!$id) //no id
    $id=md5($title.$updated);

   $values=Array(
    "title" => $title,
    "description" => $description,
    "link" => $link,
    "updated" => $updated,
    "id" => $id,
   );

   //look up sort position based on date
   $position=false;
   foreach($this->data as $k => $v)
   {
    if($v["updated"]<$updated) //entry before new entry
    {
     $position=$k;
     break;
    }
   }

   if($position===false) //new entry oldest
    $this->data[]=$values;
   else //insert new entry in array
    array_splice($this->data, $position, 0, Array($values));

   $this->vmax++;
   return true;
  }


  /*
   * Parse
   */

  function parse($data)
  {
   if(!$this->xml->parse($data))
    return false;

   if($this->xml->getAttribute("feed", "xmlns")=="http://www.w3.org/2005/Atom")
    $this->format="atom-1.0";
   else if($this->xml->getAttribute("rss", "version")=="2.0")
    $this->format="rss-2.0";
   else
   {
    $this->e->add("Parse RSS format can't be determined", "error");
    return false;
   }

   $call="parse_".preg_replace("/[^a-z0-9]/", "_", $this->format);
   if(!method_exists($this, $call))
   {
    $this->e->add("Parse RSS format: \"".$this->format."\" not supported", "system");
    return false;
   }

   if(!$this->$call())
    return false;

   return true;
  }

  function parse_atom_1_0()
  {
   //info
   if(!($title=html_entity_decode($this->xml->getInner("feed/title"), ENT_QUOTES|ENT_XML1)))
   {
    $this->e->add("Parse RSS failed, no feed title", "error");
    return false;
   }

   if(!($description=html_entity_decode($this->xml->getInner("feed/subtitle"), ENT_QUOTES|ENT_XML1)))
    $description=false;

   $link=$url=false;
   if($this->xml->getAttribute("feed/link", "rel")=="self") //url
   {
    if(!($url=$this->xml->getAttribute("feed/link", "href"))||!preg_match(REGEXP_DATA_URL, $url))
     $url=false;
   }
   else //link
   {
    if(!($link=$this->xml->getAttribute("feed/link", "href"))||!preg_match(REGEXP_DATA_URL, $link))
     $link=false;
   }
   if($this->xml->getAttribute("feed/link[1]", "rel")=="self") //url
   {
    if(!$url)
     if(!($url=$this->xml->getAttribute("feed/link[1]", "href"))||!preg_match(REGEXP_DATA_URL, $url))
      $url=false;
   }
   else //link
   {
    if(!$link)
     if(!($link=$this->xml->getAttribute("feed/link[1]", "href"))||!preg_match(REGEXP_DATA_URL, $link))
      $link=false;
   }


   if($link&&!preg_match(REGEXP_DATA_URL, $link))
    $link=false;
   if($url&&!preg_match(REGEXP_DATA_URL, $url))
    $url=false;

   if(!($updated=$this->xml->getInner("feed/updated")))
   {
    $this->e->add("Parse RSS failed, no feed last update time", "error");
    return false;
   }

   if(!($updated=$this->dt->dateConvert($updated, "c", "Y-m-d H:i:s")))
   {
    $this->e->add("Parse RSS failed, feed last update time format incorrect", "error");
    return false;
   }

   if(!$this->setInfo($title, $description, $link, $url, $updated, false))
    return false;

   //get base URL
   if($url&&preg_match("!^http(s)?://!", $url))
    $url=preg_replace("!^(http(s)://[^/]+).*$!", "\\1", $url);
   else if(preg_match("!^http(s)?://!", $link))
    $url=preg_replace("!^(http(s)://[^/]+).*$!", "\\1", $link);

   //entries
   $this->xml->reset();
   while($this->xml->search("entry"))
   {
    if($entry=$this->xml->getInnerXML($this->xml->vc-1))
    {
     if(!($title=html_entity_decode($entry->getInner("title"), ENT_QUOTES|ENT_XML1)))
     {
      $this->e->add("Parse RSS failed, no entry title", "error");
      return false;
     }

     if(!($description=$entry->getInner("content")))
     {
      if(!($description=$entry->getInner("summary")))
      {
       $this->e->add("Parse RSS failed, no entry description", "error");
       return false;
      }
     }
     $description=html_entity_decode($description, ENT_QUOTES|ENT_XML1);

     if(!($link=$entry->getAttribute("link", "href")))
     {
      $this->e->add("Parse RSS failed, no entry link", "error");
      return false;
     }

     if($link[0]=="/") //partial url
      $link=$url.$link; //append base URL

     if(!($updated=$entry->getInner("updated")))
     {
      $this->e->add("Parse RSS failed, no entry last update time", "error");
      return false;
     }

     if(!($updated=$this->dt->dateConvert($updated, "c", "Y-m-d H:i:s")))
     {
      $this->e->add("Parse RSS failed, entry last update time format incorrect", "error");
      return false;
     }

     if(!($id=html_entity_decode($entry->getInner("id"), ENT_QUOTES|ENT_XML1)))
      $id=false;

     if(!$this->add($title, $description, $link, $updated, $id))
      return false;
    }
   }

   return true;
  }

  function parse_rss_2_0()
  {
   //info
   if(!($title=html_entity_decode($this->xml->getInner("rss/channel/title"), ENT_QUOTES|ENT_XML1)))
   {
    $this->e->add("Parse RSS failed, no feed title", "error");
    return false;
   }

   if(!($description=html_entity_decode($this->xml->getInner("rss/channel/description"), ENT_QUOTES|ENT_XML1)))
    $description=false;
   else if(strpos($description, "<![CDATA[")!==false) //HTML content
    $description=preg_replace("/<\!\[CDATA\[(.+)\]]>/sm", "\\1", $description);
   else
    $description=html_entity_decode($description, ENT_QUOTES|ENT_XML1);

   if(!($link=$this->xml->getInner("rss/channel/link"))||!preg_match(REGEXP_DATA_URL, $link))
    $link=false;
   else if(strpos($link, "<![CDATA[")!==false) //HTML content
    $link=preg_replace("/<\!\[CDATA\[(.+)\]]>/sm", "\\1", $link);

   if(!($updated=$this->xml->getInner("rss/channel/pubDate")))
    $updated=false;
   else if(!($updated=$this->dt->dateConvert($updated, "r", "Y-m-d H:i:s")))
   {
    $this->e->add("Parse RSS failed, feed last update time format incorrect", "error");
    return false;
   }

   if(!($generated=$this->xml->getInner("rss/channel/lastBuildDate")))
    $generated=false;
   else if(!($generated=$this->dt->dateConvert($generated, "r", "Y-m-d H:i:s")))
   {
    $this->e->add("Parse RSS failed, feed generated time format incorrect", "error");
    return false;
   }

   if(!$this->setInfo($title, $description, $link, false, $updated, $generated))
    return false;

   //get base URL
   if(preg_match("!^http(s)?://!", $link))
    $url=preg_replace("!^(http(s)://[^/]+).*$!", "\\1", $link);

   //entries
   $this->xml->reset();
   while($this->xml->search("item"))
   {
    if($entry=$this->xml->getInnerXML($this->xml->vc-1))
    {
     if(!($title=html_entity_decode($entry->getInner("title"), ENT_QUOTES|ENT_XML1)))
     {
      $this->e->add("Parse RSS failed, no entry title", "error");
      return false;
     }

     if(!($description=$entry->getInner("content:encoded")))
     {
      if(!($description=$entry->getInner("description")))
      {
       $this->e->add("Parse RSS failed, no entry description", "error");
       return false;
      }
     }

     if(strpos($description, "<![CDATA[")!==false) //HTML content
      $description=preg_replace("/<\!\[CDATA\[(.+)\]]>/sm", "\\1", $description);
     else
      $description=html_entity_decode($description, ENT_QUOTES|ENT_XML1);

     if(!($link=$entry->getInner("link")))
     {
      $this->e->add("Parse RSS failed, no entry link", "error");
      return false;
     }
     else if(strpos($link, "<![CDATA[")!==false) //HTML content
      $link=preg_replace("/<\!\[CDATA\[(.+)\]]>/sm", "\\1", $link);

     if($link[0]=="/") //partial url
      $link=$url.$link; //append base URL

     if(!($updated=$entry->getInner("pubDate")))
      $updated=false;
     else if(!($updated=$this->dt->dateConvert($updated, "r", "Y-m-d H:i:s")))
     {
      $this->e->add("Parse RSS failed, entry last update time format incorrect", "error");
      return false;
     }

     if(!($id=html_entity_decode($entry->getInner("guid"), ENT_QUOTES|ENT_XML1)))
      $id=false;

     if(!$this->add($title, $description, $link, $updated, $id))
      return false;
    }
   }

   return true;
  }


  /*
   * Output
   */

  function output($headers=false)
  {
   global $_path;

   if(!($xml=$this->fs->read($_path["format"]."rss/".$this->format.".xml")))
   {
    $this->e->add("Unable to read RSS format template: \"".$_path["format"]."rss/".$this->format.".xml\"", "error");
    return false;
   }

   if(!$this->xml->parse($xml))
    return false;

   $call="output_".preg_replace("/[^a-z0-9]/", "_", $this->format);
   if(!method_exists($this, $call))
   {
    $this->e->add("Output RSS format: \"".$this->format."\" not supported", "system");
    return false;
   }

   //get last updated
   if($this->info["updated"]) //last update overwrite
    $updated=$this->info["updated"];
   else if(isset($this->data[0])) //get last updated from newest entry
    $updated=$this->data[0]["updated"];
   else //use current date time
    $updated=date("Y-m-d H:i:s");

   if(!$this->$call($updated))
    return false;

   return $this->xml->output($headers);
  }

  function output_atom_1_0($updated)
  {
   //info
   if(!$this->xml->setInside("feed/title", $this->info["title"]))
    return false;

   if(!$this->xml->setInside("feed/subtitle", $this->info["description"]))
    return false;

   if(!$this->xml->setAttribute("feed/link", "href", $this->info["link"]))
    return false;

   if(!$this->xml->setAttribute("feed/link[1]", "href", $this->info["url"]))
    return false;

   if(!$this->xml->setInside("feed/updated", $this->dt->dateConvert($updated, "Y-m-d H:i:s", "c")))
    return false;

   //entries
   $this->reset();
   while($this->previous()) //reverse order for setInsideCopy
   {
    $values=Array(
     "title" => Array(htmlentities($this->v["title"], ENT_COMPAT|ENT_XML1)),
     "summary" => Array(htmlentities(strip_tags($this->v["description"]), ENT_COMPAT|ENT_XML1)),
     "content" => Array(htmlentities($this->v["description"], ENT_COMPAT|ENT_XML1)),
     "updated" => Array($this->dt->dateConvert($this->v["updated"], "Y-m-d H:i:s", "c")),
     "id" => Array(htmlentities($this->v["id"], ENT_COMPAT|ENT_XML1)),
    );

    if(!$this->xml->setInsideCopy("feed/entry", $values))
     return false;

    if(!$this->xml->setAttribute("feed/entry/link", "href", $this->v["link"]))
     return false;
   }

   if(!$this->xml->setReplace("feed/entry[".$this->vmax."]", false)) //remove example template item
    return false;

   return true;
  }

  function output_rss_2_0($updated)
  {
   //info
   if(!$this->xml->setInside("rss/channel/title", $this->info["title"]))
    return false;

   if(!$this->xml->setInside("rss/channel/description", $this->info["description"]))
    return false;

   if(!$this->xml->setInside("rss/channel/link", $this->info["link"]))
    return false;

   if(!$this->xml->setInside("rss/channel/pubDate", $this->dt->dateConvert($updated, "Y-m-d H:i:s", "r")))
    return false;

   if(!$this->xml->setInside("rss/channel/lastBuildDate", date("r")))
    return false;

   //entries
   $this->reset();
   while($this->previous()) //reverse order for setInsideCopy
   {
    $values=Array(
     "title" => Array(htmlentities($this->v["title"], ENT_COMPAT|ENT_XML1)),
     "description" => Array(htmlentities($this->v["description"], ENT_COMPAT|ENT_XML1)),
     "link" => Array($this->v["link"]),
     "pubDate" => Array($this->dt->dateConvert($this->v["updated"], "Y-m-d H:i:s", "r")),
     "guid" => Array(htmlentities($this->v["id"], ENT_COMPAT|ENT_XML1)),
    );

    if(!$this->xml->setInsideCopy("rss/channel/item", $values))
     return false;
   }

   if(!$this->xml->setReplace("rss/channel/item[".$this->vmax."]", false)) //remove example template item
    return false;

   return true;
  }

 }

?>
