<?

 /*****************************************************************************
 * Filename: include/headers.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."mime.php");
 require_once($_path["include"]."jas.php");

 class Headers
 {
  function __construct()
  {
   $this->e=new Errors();
   $this->e->l->read("headers");

   $this->mime=false;
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  static function __callStatic($method, $arguments)
  {
   $object=new self();
   $object->e->logging(false); //disable logging for static calls

   if(substr($method, -1)=="S") //method must have a S suffix
   {
    $method=substr($method, 0, -1); //remove S suffix
    if(method_exists($object, $method)) //check if dynamic method exists
     return call_user_func_array(Array($object, $method), $arguments);
   }

   $bt=debug_backtrace();
   print "<br/>\n<b>Fatal error</b>:  Call to undefined method ".get_class($object)."::".$method." in <b>".$bt[0]["file"]."</b> on line <b>".$bt[0]["line"]."</b><br />\n";
   exit;
  }


  function request($server=false)
  {
   if(!$server&&function_exists("getallheaders"))
    return getallheaders();

   $headers=Array();
   foreach($_SERVER as $k => $v)
   {
    if(substr($k, 0, 5)=="HTTP_") //HTTP header
    {
     $k=substr($k, 5); //get header name: HTTP_CACHE_CONTROL => CACHE_CONTROL
     $k=str_replace(" ", "-", ucwords(strtolower(str_replace("_", " ", $k)))); //transform CACHE_CONTROL => Cache-Control
     $headers[$k]=$v;
    }
   }

   return $headers;
  }


  function set($key, $value)
  {
   if(headers_sent())
   {
    $this->e->add("Can't add HTTP header: \"".$key."\", cache already sent", "error", $value);
    return false;
   }

   header($key.": ".$value, true);
   return true;
  }


  function get($key=false)
  {
   //get all headers
   $headers=Array();
   foreach(headers_list() as $header)
   {
    list($k, $v)=explode(":", $header, 2);
    $headers[$k]=ltrim($v, " ");
   }

   if(!$key) //no key search
    return $headers; //return associative header list

   if(isset($headers[$key]))
    return $headers[$key];

   $keyl=ucfirst(strtolower($key)); //try lower case key
   if(isset($headers[$keyl]))
    return $headers[$keyl];

   $this->e->add("Can't get HTTP header: \"".$key."\", no such header", "error");
   return false;
  }


  function delete($key=false)
  {
   if(!function_exists("header_remove"))
   {
    $this->e->add("Can't delete HTTP header: \"".$key."\", PHP 5.3 or higher is required", "error");
    return false;
   }

   //remove all headers
   if($key===false)
   {
    header_remove();
    return true;
   }

   //remove header by key
   foreach(headers_list() as $header) //get all headers
   {
    if(strpos($header, $key.":")!==false) //key matches
    {
     header_remove($key);
     return true;
    }
   }

   $this->e->add("Can't delete HTTP header: \"".$key."\", no such header", "error");
   return false;
  }


  function basic()
  {
   //send basic headers
   if(!$this->cache()) //caching not allowed
    return false;

   return true;
  }


  function cache($method=false, $store=true, $expires=false, $delta=false)
  {
   //set default method
   if(!$method)
   {
    if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
    {
     $method="private"; //default is private to ensure files can be saved to disk
     $store=false; //don't store so caching is still effectively disabled
    }
    else //non HTTPS or non IE
     $method="no-cache"; //no caching
   }

   if(!in_array($method, Array("private", "public", "no-cache")))
   {
    $this->e->add("Unknown header method: \"".$method."\"", "error");
    return false;
   }

   //cache method
   $control=$method;

   if($method=="no-cache")
    $control.=", must-revalidate";

   //store cache
   if(!$store)
    $control.=", no-store";

   //expiry time
   if($expires===false) //no expiry time given
   {
    $expires=0; //beginning of time
    $delta=false; //no delta
   }

   if($delta) //expiry is a delta compared to current time
    $expires+=time(); //make expiry delta an exact date time

   //add max-age (delta) control directive
   $control.=", max-age=".($expires-time());

   //proper expiry time format
   $expires=gmdate("D, d, M Y H:i:s T", $expires);

   //send headers
   if(!$this->set("Cache-Control", $control)) //http 1.1
    return false;

   if(!$this->set("Pragma", $method)) //http 1.0
    return false;

   if(!$this->set("Expires", $expires)) //proxies
    return false;

   return true;
  }


  function filename($filename, $disposition=true)
  {
   if(!$filename)
   {
    $this->e->add("Won't send filename HTTP header", "error", "No filename given");
    return false;
   }

   if($disposition===true)
    $disposition="attachment";

   $disposition.=($disposition?"; ":"")."filename=\"".$filename."\"";

   if(!$this->set("Content-Disposition", $disposition)) //filename
    return false;

   return true;
  }


  function type($type)
  {
   if(!preg_match("!^[A-Z0-9]+/.+$!i", $type))
   {
    $this->e->add("Won't send type HTTP header", "error", "Invalid content type: \"".$type."\"");
    return false;
   }

   if(!$this->set("Content-Type", $type)) //mime type
    return false;

   return true;
  }


  function size($size)
  {
   if(!preg_match("/^\d+$/", $size))
   {
    $this->e->add("Won't send size HTTP header", "error", "Size: \"".$size."\" isn't a positive number");
    return false;
   }

   if(!$this->set("Content-Length", $size)) //file size
    return false;

   return true;
  }


  function file($filename, $type=false, $size=false, $attachment=true)
  {
   if(!$filename)
   {
    $this->e->add("Won't send file HTTP header", "error", "No filename given");
    return false;
   }

   //lookup file type
   if($type===true)
   {
    if(!$this->mime) //cache Mime database
     $this->mime=new Mime();

    //lookup mime type for filename
    $type=$this->mime->filename($filename);
   }

   if(!$this->filename($filename, $attachment)) //filename
    return false;

   if($type)
    if(!$this->type($type)) //mime type
     return false;

   if($size!==false)
    if(!$this->size($size)) //file size
     return false;

   return true;
  }


  function redirect($url)
  {
   if(!$this->set("Location", $url))
    return false;

   exit;
  }


  function server($jas=false, $apache=false, $php=false, $ssl=false, $custom=false)
  {
   //lookup jas version
   if($jas===true)
    $jas="JAS/".JAS::versionS();

   //lookup apache version
   if($apache===true)
   {
    if(exec("httpd -v", $output)&&preg_match("!(Apache/[0-9\.]+)(\s\([^\)]+\))!", $output[0], $m))
     $apache=$m[0];
    else if(preg_match("!(Apache/[0-9\.]+)(\s\([^\)]+\))!", apache_get_version(), $m))
     $apache=$m[0];
    else
    {
     $this->e->add("Server HTTP header failed", "warning", "Unable to get Apache version");
     $apache=false;
    }
   }

   //lookup php version
   if($php===true)
   {
    if(preg_match("!([0-9\.]+)!", phpversion(), $m))
     $php="PHP/".$m[0];
    else if(preg_match("!(PHP/[0-9\.]+)!", apache_get_version(), $m))
     $php=$m[0];
    else
    {
     $this->e->add("Server HTTP header failed", "warning", "Unable to get PHP version");
     $php=false;
    }
   }

   //lookup ssl version
   if($ssl===true)
   {
    if(preg_match("!(OpenSSL) ([0-9a-z\.]+)!", OPENSSL_VERSION_TEXT, $m))
     $ssl=$m[1]."/".$m[2];
    else if(preg_match("!(OpenSSL/[0-9a-z\.]+)!", apache_get_version(), $m))
     $ssl=$m[0];
    else
    {
     $this->e->add("Server HTTP header failed", "warning", "Unable to get OpenSSL version");
     $ssl=false;
    }
   }

   $server="";

   if($jas)
    $server.=($server?" ":"").$jas;
   if($apache)
    $server.=($server?" ":"").$apache;
   if($ssl)
    $server.=($server?" ":"").$ssl;
   if($php)
    $server.=($server?" ":"").$php;
   if($custom)
    $server.=($server?" ":"").$custom;

   if(!$this->set("Server", $server))
    return false;

   return true;
  }


 }
?>
