<?

 /*****************************************************************************
 * Filename: include/jas.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."functions.php");

 class JAS
 {
  function __construct()
  {
   $this->e=new Errors();
   $this->e->l->read("JAS");
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  static function __callStatic($method, $arguments)
  {
   $object=new self();
   $object->e->logging(false); //disable logging for static calls

   if(substr($method, -1)=="S") //method must have a S suffix
   {
    $method=substr($method, 0, -1); //remove S suffix
    if(method_exists($object, $method)) //check if dynamic method exists
     return call_user_func_array(Array($object, $method), $arguments);
   }

   $bt=debug_backtrace();
   print "<br/>\n<b>Fatal error</b>:  Call to undefined method ".get_class($object)."::".$method." in <b>".$bt[0]["file"]."</b> on line <b>".$bt[0]["line"]."</b><br />\n";
   exit;
  }


  static function URL()
  {
   return "https://jas.junerules.com/";
  }

  function JavaScript()
  {
   global $_session, $_path;

   $script="  <script type=\"text/javascript\"><!--\n";
   $script.="   var browser=\"".$_session->get("browser")."\";\n";
   $script.="   var path=\"".$_path["HTML"]["prefix"].$_path["project"]["root"]."\";\n";
   $script.="   var rootpath=\"".$_path["HTML"]["prefix"].$_path["root"]."\";\n";
   $script.="  --></script>";

   return $script;
  }

  function changelogVersion()
  {
   global $_path, $_session;

   $changelogFile=$_path["include"]."changelog.txt";

   if(!FS::existsS($changelogFile, "file"))
   {
    $this->e->add("Can't detect JAS version", "error", "Changelog file: \"".$changelogFile."\" not found");
    return false;
   }

   if(!($changelog=FS::readS($changelogFile)))
   {
    $this->e->add("Can't detect JAS version", "error", "Can't read changelog file: \"".$changelogFile."\"");
    return false;
   }

   if(!preg_match("/\nVersion ([0-9\.]+)[ \-]*([A-Z0-9]+)?/i", $changelog, $version))
   {
    $this->e->add("Can't detect JAS version info", "error", "Can't obtain version from changelog file: \"".$changelogFile."\"");
    return false;
   }

   $development=(preg_match("/\nDevelopment version/", $changelog));

   if(!preg_match("/\n\t(\d{4}-\d{2}-\d{2})/", $changelog, $updated))
   {
    $this->e->add("Can't detect JAS version info", "error", "Can't get last updated from changelog file: \"".$changelogFile."\"");
    return false;
   }

   $modified=FS::getTimeS($changelogFile, "modified");


   $_session->set("jas.version", $version[1]);
   $_session->set("jas.tag", (isset($version[2])?$version[2]:""));
   $_session->set("jas.development", $development);

   $_session->set("jas.updated", DT::dateConvertS($updated[1], "Y-m-d"));
   $_session->set("jas.modified", $modified);

   return true;
  }

  function changelogUpdated()
  {
   global $_path, $_session;

   if(!$_session->get("jas.modified")) //version information not cached
    return true;

   $changelogFile=$_path["include"]."changelog.txt";

   if(FS::existsS($changelogFile, "file"))
    if(FS::getTimeS($changelogFile, "modified")!=$_session->get("jas.modified"))
     return true;

   return false;
  }

  function changelog($html=false)
  {
   global $_path;

   $changelogFile=$_path["include"]."changelog.txt";

   if(!FS::existsS($changelogFile, "file"))
   {
    $this->e->add("Can't open JAS changelog", "error", "Changelog file: \"".$changelogFile."\" not found");
    return false;
   }

   if(!($changelog=FS::readS($changelogFile)))
   {
    $this->e->add("Can't read JAS changelog", "error", "Changelog file: \"".$changelogFile."\"");
    return false;
   }

   if($html) //html mode (otherwise raw text will be used)
    $changelog=$this->changelogFormat($changelog);

   return $changelog;
  }

  function changelogFormat($changelog)
  {
   $changelog=Mutilate::htmlEncodeS(trim($changelog));
   $changelog=preg_replace("!(^|\n)(Development version)\n!", "<h3>\\2</h3>", $changelog);
   $changelog=preg_replace("!(^|\n)(Version [\d\.+[A-Za-z\-]*)\n!", "<h3>\\2</h3>", $changelog);
   $changelog=preg_replace("!(\d+\-\d+\-\d+)\n!", "<h4>\\1</h4>", $changelog);
   $changelog=str_replace("\n", "<br/>\n", $changelog);
   $changelog=str_replace("\t*", "<div class=\"hilite\">*</div>", $changelog);

   return $changelog;
  }

  function version($type=false)
  {
   global $_session;

   if(!$type)
    $type="version";

   if($this->changelogUpdated()) //version information not cached
    if(!$this->changelogVersion())
     return false;

   if(!$_session->get("jas.".$type))
   {
    $this->e->add("JAS version type: \"".$type."\" not supported", "error");
    return false;
   }

   return $_session->get("jas.".$type);
  }


  function license($html=false, $type=false)
  {
   global $_path;

   if(strtoupper($type)=="GPL")
    $licenseFile=$_path["root"]."GPL.txt";
   else if(strtoupper($type)=="LGPL")
    $licenseFile=$_path["root"]."LGPL.txt";
   else
    $licenseFile=$_path["root"]."LICENSE.txt";

   if(!FS::existsS($licenseFile, "file"))
   {
    $this->e->add("Can't open JAS license", "error", "License file: \"".$licenseFile."\" not found");
    return false;
   }

   if(!($license=FS::readS($licenseFile)))
   {
    $this->e->add("Can't read JAS license", "error", "License file: \"".$licenseFile."\"");
    return false;
   }

   if($html) //html mode (otherwise raw text will be used)
    $license=$this->licenseFormat($license);

   return $license;
  }

  function licenseFormat($license)
  {
   $license=Mutilate::htmlEncodeS(trim($license));
   $license=preg_replace("!([a-z]+\:\/\/[^\s<>&]+)!", "<a href=\"\\1\" title=\"\\1\" rel=\"external\">\\1</a>", $license);
   $license=preg_replace("!\n\s*(\d+\.\s.+)\n!i", "\n\n<h3>\\1</h3>", $license);
   $license=str_replace("\n", "<br/>\n", $license);
   $license=str_replace("\t*", "<div class=\"hilite\">*</div>", $license);

   return $license;
  }


  /*
   * Upgrade
   */

  function upgrade($file, $new=false, $changed=false, $delete=false)
  {
   global $_path, $_session, $_config;

   if(!$_session->inBackend())
   {
    $this->e->add("Upgrade must be ran from backend", "error");
    return false;
   }

   $excludes=Array( //exclude files + directories from being upgraded / deleted
    $_path["root"]."docs/",
    $_path["root"]."test/",
    $_path["projects"]."jas/include/db.conf.php",
    $_path["projects"]."jas/include/data/",
    $_path["projects"]."jas/include/plugins/status.txt",
   );

   $required=Array( //required files in release
    $_path["root"]."index.php",
    $_path["root"]."include.php",
    $_path["script"]."http.js",
    $_path["include"]."data.php",
    $_path["include"]."storage/mysql.php",
    $_path["project"]["root"]."index.php",
    $_path["project"]["include"]."db.conf.php",
   );

   $fs=new FS();
   $zip=new Archive();
   if(!$zip->open($file))
   {
    $this->e->add($zip->e);
    return false;
   }

   //reroot
   $reroot=false;
   foreach($zip->getFiles() as $zipfile)
   {
    if(($tmp=strpos($zipfile, "include.php"))!==false)
    {
     $reroot=substr($zipfile, 0, $tmp);
     break;
    }
   }

   if($reroot===false)
   {
    $this->e->add("Unable to find \"include.php\" in release: \"".$file."\"", "error");
    return false;
   }

   if($reroot&&!$zip->rerootFiles($reroot))
   {
    $this->e->add($zip->e);
    return false;
   }

   //check required files
   foreach($zip->getFiles() as $zipfile)
   {
    if(($tmp=array_search($zipfile, $required))!==false) //required file found
    {
     unset($required[$tmp]); //remove from required list

     if(!$required) //no more required ites
      break; //done
    }
   }

   if($required)
   {
    $this->e->add("Required files missing in release: \"".$file."\"", "error");
    return false;
   }

   //get file status
   $files=Array();
   foreach($zip->getFiles() as $zipfile)
   {
    //exclude files and directories
    $skip=false;
    foreach($excludes as $exclude)
    {
     if(strpos($zipfile, $exclude)!==false)
     {
      $skip=true;
      break;
     }
    }

    if($skip)
     continue;

    //gather file info
    $files[$zipfile]=Array(
     "size" => $zip->getSize($zipfile),
     "type" => $zip->getType($zipfile),
     "status" => "new",
    );

    //determine state for existing files
    if($fs->exists($_path["root"].$zipfile))
    {
     //check if file is not a directory and vice versa
     if($files[$zipfile]["type"]=="directory")
     {
      if(!$fs->exists($_path["root"].$zipfile, "directory")) //not a directory
      {
       $files[$zipfile]["status"]="directory";
       continue;
      }

      $files[$zipfile]["status"]="unchanged";
     }
     else //file
     {
      if($fs->exists($_path["root"].$zipfile, "directory")) //not a file
      {
       $files[$zipfile]["status"]="file";
       continue;
      }

      if($fs->size($_path["root"].$zipfile)!=$files[$zipfile]["size"])
       $files[$zipfile]["status"]="changed";
      else if($fs->read($_path["root"].$zipfile)!=$zip->getContents($zipfile))
       $files[$zipfile]["status"]="changed";
      else
       $files[$zipfile]["status"]="unchanged";
     }
    }
   }

   //clean
   if(!($clean=$_path["root"]))
    $clean="./";

   foreach($fs->files($clean, true, true) as $localfile)
   {
    if(is_dir($localfile)) //skip directories
     continue;

    if(preg_match("!^project/jas/cache/(releases|plugins)/.*\.zip$!", $localfile)) //skip releases
     continue;

    if(strpos($localfile, "project/")===0) //skip non JAS projects
     if(strpos($localfile, "project/jas/")===false&&strpos($localfile, "project/index.php")===false)
      continue;

    //exclude files and directories
    $skip=false;
    foreach($excludes as $exclude)
    {
     if(strpos($localfile, $exclude)!==false)
     {
      $skip=true;
      break;
     }
    }

    if($skip)
     continue;

    if(!isset($files[$localfile]))
    {
     $files[$localfile]=Array(
      "size" => $fs->size($localfile),
      "type" => "file",
      "status" => "delete",
     );
    }
   }

   ksort($files);


   //upgrade
   if($new||$changed||$delete)
   {
    if(!$_config->userRoot())
    {
     $this->e->add("Upgrade must be ran by root user", "error");
     return false;
    }

    foreach($files as $upgradefile => $info)
    {
     if($info["status"]=="delete"&&$delete) //delete file
     {
      if(!$fs->delete($upgradefile)) //create file
      {
       $this->e->add("Can't delete file \"".$upgradefile."\" for release: \"".$file."\"", "error");
       return false;
      }

      $files[$upgradefile]["status"]="deleted";
     }
     else if($info["status"]=="new"&&$new) //new file
     {
      if($info["type"]=="directory")
      {
       if(!$fs->exists($upgradefile))
       {
        if(!$fs->mkdir($upgradefile, true)) //create directories
        {
         $this->e->add("Can't create directory \"".$upgradefile."\" for release: \"".$file."\"", "error");
         return false;
        }
       }
      }
      else //file
      {
       if(($tmp=strrpos($upgradefile, "/"))!==false) //check if directory exists
       {
        $directory=substr($upgradefile, 0, $tmp+1);

        if(!$fs->exists($directory))
        {
         if(!$fs->mkdir($directory, true)) //create directories
         {
          $this->e->add("Can't create directory \"".$directory."\" for release: \"".$file."\"", "error");
          return false;
         }
        }
       }

       if(!$fs->write($upgradefile, $zip->getContents($upgradefile))) //create file
       {
        $this->e->add("Can't create new file \"".$upgradefile."\" for release: \"".$file."\"", "error");
        return false;
       }
      }

      $files[$upgradefile]["status"]="installed";
     }
     else if($info["status"]=="changed"&&$changed) //changed file
     {
      if(!$fs->write($upgradefile, $zip->getContents($upgradefile), "overwrite")) //overwite file
      {
       $this->e->add("Can't overwite file \"".$upgradefile."\" for release: \"".$file."\"", "error");
       return false;
      }

      $files[$upgradefile]["status"]="upgraded";
     }
    }
   }

   return $files;
  }


  /*
   * Modules list
   */

  static function modules($moduleType=false, $plugins=false, $tableExists=false)
  {
   global $_path, $_modules, $_session, $_config;

   if(!$moduleType)
    $moduleType="all";
   if(!$plugins)
    $plugins="all";

   //cache modules list for current page load
   if($plugins=="all") //only if modules for all plugins are gathered
   {
    if(!isset($_modules[$_session->get("project")]))
     $_modules[$_session->get("project")]=Array();

    if(!$tableExists)
    {
     if(!isset($_modules[$_session->get("project")]["_modules"])) //abuse _modules register in _modules cache
      $_modules[$_session->get("project")]["_modules"]=Array();

     $cache=&$_modules[$_session->get("project")]["_modules"];
    }
    else //table exists
    {
     if(!isset($_modules[$_session->get("project")]["_modulesExists"])) //abuse _modulesExists register in _modules cache
      $_modules[$_session->get("project")]["_modulesExists"]=Array();

     $cache=&$_modules[$_session->get("project")]["_modulesExists"];
    }

    if(isset($cache[$moduleType])&&$cache[$moduleType]) //cache already exists
     return $cache[$moduleType]; //return cached list
   }

   //gather modules
   $modules=Array();

   $dirs=Array();
   if(isset($_path["project"]["modules"]))
    $dirs["project"]=$_path["project"]["modules"];

   if(isset($_path["modules"]))
    $dirs["system"]=$_path["modules"];

   foreach($dirs as $type => $dir)
   {
    if($moduleType=="all"||$moduleType==$type)
    {
     if(is_dir($dir))
     {
      $readdir=opendir($dir);
      while($file=readdir($readdir))
      {
       if(preg_match("/^([a-z]{1,25})\.php$/i", $file))
       {
        $file=substr($file, 0, -4);
        if(!in_array($file, $modules))
         array_push($modules, $file);
       }
      }
     }
    }
   }

   //gather plugin(s) modules
   if($moduleType=="all"||$moduleType=="plugins")
   {
    if($plugins=="all")
     $plugins=JAS::plugins();
    else if(in_array($plugins, JAS::plugins()))
     $plugins=Array($plugins);
    else
     $plugins=false;

    if($plugins)
    {
     if($_config->get("logging"))
     {
      $logging=$_config->get("logging");
      $_config->set("logging", false); //disable logging
     }

     foreach($plugins as $p)
     {
      $plugin=new Plugin($p);
      $dir=$plugin->path["modules"];

      if($plugin->isEnabled()&&is_dir($dir))
      {
       $readdir=opendir($dir);
       while($file=readdir($readdir))
       {
        if(preg_match("/^([a-z]{1,25})\.php$/i", $file))
        {
         $module=substr($file, 0, -4);
         $module=$plugin->name."_".$module;

         if(!in_array($module, $modules))
          array_push($modules, $module);
        }
       }
      }
     }

     if(isset($logging))
      $_config->set("logging", $logging);
    }
   }

   //remove modules that don't have a database table
   if($tableExists)
   {
    $tmp=Array(); //temp modules list
    $d=Data::cache("empty"); //init empty module to prevent endless loop (doesn't have a module(s) fields with an "exists" property)

    foreach($modules as $module)
    {
     $d->module=$module; //dirty hack to speed up

     if($d->tableExists()) //table exists...
      $tmp[]=$module; //add module to module list
    }

    $modules=$tmp;
   }

   //sort modules
   sort($modules);

   //cache module list
   if(isset($cache))
    $cache[$moduleType]=$modules;

   return $modules;
  }


  /*
   * Plugins list
   */

  static function plugins($pluginType=false)
  {
   global $_path;

   if(!$pluginType)
    $pluginType="all";

   //gather plugins
   $plugins=Array();

   $dirs=Array();
   if(isset($_path["project"]["plugins"]))
    $dirs["project"]=$_path["project"]["plugins"];

   if(isset($_path["plugins"]))
    $dirs["system"]=$_path["plugins"];

   foreach($dirs as $type => $dir)
   {
    if($pluginType=="all"||$pluginType==$type)
    {
     if(is_dir($dir))
     {
      if($readdir=@opendir($dir))
      {
       while($file=readdir($readdir))
       {
        if(is_dir($dir.$file)&&!preg_match("/^\./", $file))
        {
         if(!in_array($file, $plugins))
          array_push($plugins, $file);
        }
       }
      }
     }
    }
   }

   sort($plugins);
   return $plugins;
  }


  /*
   * Projects list
   */

  static function projects()
  {
   global $_path;

   //gather projects
   $projects=Array();
   $dh=opendir($_path["projects"]);
   while($project=readdir($dh))
   {
    if(preg_match("/^([a-z0-9]{1,25})$/i", $project))
     $projects[]=$project;
   }

   sort($projects);
   return $projects;
  }
 }
?>
