<?

 /*****************************************************************************
 * Filename: include/modules/configAPI.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_configAPI extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->types=Array(
    "viewItem" => "View item",
    "viewList" => "View list",
    "viewAll" => "View all",
    "viewDeleted" => "View deleted",
    "treeUp" => "Tree up",
    "treeDown" => "Tree down",
    "searching" => "Search",

    "add" => "Add",
    "edit" => "Edit",
    "move" => "Move",
    "delete" => "Delete",
    "undelete" => "Undelete",

    "databaseInfo" => "Database info",

    "userLoginLogout" => "User login/logout",
    "userPassword" => "User password change",

    "language" => "Language",
    "modules" => "Modules listing",
    "projects" => "Projects listing",
   );

   $this->typesModules=Array(
    "viewItem",
    "viewList",
    "viewAll",
    "viewDeleted",
    "treeUp",
    "treeDown",
    "searching",

    "add",
    "edit",
    "move",
    "delete",
    "undelete",

    "databaseInfo",
   );

   $this->data->title="API configuration";

   $this->data->defaultFields("id;added;updated");
   $this->data->setShowFields(); //clear show fields

   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("faketitle", "text", "show", "title;hidden", 0, false, $this->data->title, "Title");

   //APIs
   $this->data->addField("csv", "boolean", "all", "", false, false, false, "CSV API", "Enable the CSV API");
   $this->data->addField("data", "boolean", "all", "", false, false, false, "Data API", "Enable the Data API");
   $this->data->addField("xml", "boolean", "all", "", false, false, false, "XML API", "Enable the XML API");

   //Supported types
   foreach($this->types as $type => $text)
   {
    $this->data->addField($type."Label", "label", "form", "", false, false, false, $text);

    if(!in_array($type, $this->typesModules))
     $this->data->addField($type, "boolean", "all", "", false, false, false, $text." support", "Allow ".strtolower($text)." support");
    else
    {
     $this->data->addField($type, "boolean", "all", "reload", false, false, false, $text." support", "Allow ".strtolower($text)." support");
     $this->data->addField($type."AllModules", "boolean", "all", "reload", false, false, false, "All modules", "Allow ".strtolower($text)." support for all modules");
     $this->data->addField($type."Modules", "modules", "all", "", 0, 65536, false, "Modules", "Select the modules for which you would like ".strtolower($text)." support");
    }
   }

   $this->data->group("Read",
    "viewItem", "viewItemLabel", "viewItemModules", "viewItemAllModules",
    "viewList", "viewListLabel", "viewListModules", "viewListAllModules",
    "viewAll", "viewAllLabel", "viewAllModules", "viewAllAllModules",
    "viewDeleted", "viewDeletedLabel", "viewDeletedModules", "viewDeletedAllModules",
    "treeUp", "treeUpLabel", "treeUpModules", "treeUpAllModules",
    "treeDown", "treeDownLabel", "treeDownModules", "treeDownAllModules",
    "searching", "searchingLabel", "searchingModules", "searchingAllModules",
    "databaseInfo", "databaseInfoLabel", "databaseInfoModules", "databaseInfoAllModules"
   );

   $this->data->group("Write",
    "add", "addLabel", "addModules", "addAllModules",
    "edit", "editLabel", "editModules", "editAllModules",
    "move", "moveLabel", "moveModules", "moveAllModules",
    "delete", "deleteLabel", "deleteModules", "deleteAllModules",
    "undelete", "undeleteLabel", "undeleteModules", "undeleteAllModules"
   );

   $this->data->group("User",
    "userLoginLogout", "userLoginLogoutLabel", "userLoginLogoutModules", "userLoginLogoutAllModules",
    "userPassword", "userPasswordLabel", "userPasswordModules", "userPasswordAllModules"
   );

   $this->data->group("Other",
    "language", "languageLabel", "languageModules", "languageAllModules",
    "modules", "modulesLabel", "modulesModules", "modulesAllModules",
    "projects", "projectsLabel", "projectsModules", "projectsAllModules"
   );
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   //Only show module selection for types that are actually active
   foreach(array_keys($this->types) as $type)
   {
    if(!in_array($type, $this->typesModules))
     continue;

    if($this->data->getValue($type))
    {
     $this->data->resetField($type."AllModules");

     if($this->data->getValue($type."AllModules"))
      $this->data->field[$type."Modules"]["properties"].=";hidden";
     else
      $this->data->resetField($type."Modules");
    }
    else
    {
     $this->data->field[$type."Modules"]["properties"].=";hidden";
     $this->data->field[$type."AllModules"]["properties"].=";hidden";
    }
   }

   return true;
  }

  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    $this->data->setValue("faketitle", $this->data->title, $c);

   return true;
  }

 }

?>
