<?

 /*****************************************************************************
 * Filename: include/modules/users.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_users extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Users";

   $this->data->defaultFields("id;rights;added;updated");
   $this->data->field[$this->data->getID()]["properties"].=";owner"; //id is owner field

   $this->data->addField("username", "title", "all;login", "username;title", 4, 15, false, "Username");
   $this->data->addField("password", "password", "sql;form;login", "password", 7, false, false, "Password");

   $this->data->addField("backend", "boolean", "all", "", false, false, false, "Backend login", "If this user is allowed to log in to the JAS backend");

   $this->data->addReference("usersAdditionalGroups", "categories", "nr", "*:*", "nr");

   if(isset($this->data->reference["usersAdditionalGroups"]))  //reference initialized?
   {
    $this->data->field["usersAdditionalGroups"]["title"]="Additional user groups";
    $this->data->field["usersAdditionalGroups"]["text"]="Select additional rights groups for this user";

    //not required
    $this->data->field["usersAdditionalGroups"]["min"]=0;
    $this->data->field["usersAdditionalGroups"]["original"]["min"]=0;
   }

   $this->data->addActions("user");
   $this->data->sortOrder("username");

   $this->data->addUnique("username", "username");
  }


  function postInit()
  {
   if(!$this->filterAdditionalGroups())
    return false;

   return $this->customizeFields();
  }
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   global $_config;

   //hide backend field for none root users
   if($_config->userRoot())
    $this->data->resetField("backend");
   else
    $this->data->field["backend"]["properties"]=";hidden";

   return true;
  }

  function filterAdditionalGroups()
  {
   if($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit")
   {
    $d=Data::cache("categories", "view", "all");
    $d->setShowFields($d->check("properties", "title"));

    //only categories where users module is set
    $d->clearValues();
    $d->setValue("module", "users");
    $d->setOperator("module", "like");

    $categories=Array();
    if($d->get())
     while($d->next())
      $categories[$d->v("id")]=$d->v("title");

    if($categories)
    {
     $this->data->listOptions("usersAdditionalGroups", $categories);
     $this->data->resetField("usersAdditionalGroups");
    }
    else //no user categories found, don't use additional groups
     $this->data->field["usersAdditionalGroups"]["use"]="";
   }

   return true;
  }

  function addPreSubmit(){return $this->editPreSubmit();}
  function copyPreSubmit(){return $this->editPreSubmit();}
  function editPreSubmit()
  {
   global $_config;

   //make sure backend value is only set by root user
   if(!$_config->userRoot())
    $this->data->unsetValue("backend");

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->editPreSubmitErrorCheck();}
  function copyPreSubmitErrorCheck(){return $this->editPreSubmitErrorCheck();}
  function editPreSubmitErrorCheck()
  {
   global $_config;

   $success=true;

   //anonymous may not edit itself
   if($this->data->action=="edit"&&$_config->userAnonymous())
   {
    for($this->data->vc=0;$this->data->vc<$this->data->vmax;$this->data->vc++)
    {
     $v=$this->data->getValue("nr", $this->data->vc);

     if($v==$_config->get("usernr"))
     {
      $this->data->e->add("Can't edit anonymous user", "hack");
      $success=false;
     }
    }
   }

   //check additional groups
   if(isset($this->data->reference["usersAdditionalGroups"])&&($this->data->check("valuetype", "request", "usersAdditionalGroups")||$this->data->check("valuetype", "set", "usersAdditionalGroups"))) //check additional groups
   {
    //get current groups+categories
    $d=clone $this->data;
    if($this->data->action=="edit")
    {
     $d->setAction("view");
     $d->setExtra("item");

     $d->setShowFields("catnr", "usersAdditionalGroups");
     $d->get();

     $catnr=$d->getValue("catnr");
     $current=$d->getValue("usersAdditionalGroups");
    }
    else //add/copy (no current groups+categories)
    {
     $catnr=$d->getValue("catnr");
     $current=false;
    }

    if(!$storage=$d->storageObject())
     return false;

    for($this->data->vc=0;$this->data->vc<$this->data->vmax;$this->data->vc++)
    {
     if(!($new=$this->data->getValue("usersAdditionalGroups", $this->data->vc)))
      $new=Array();

     if($catnr&&in_array($catnr, $new)) //main category in additional groups
     {
      //remove category from additional groups
      $new=array_remove($new, $catnr);
      $this->data->setValue("usersAdditionalGroups", $new, $this->data->vc); //set new value
     }

     if($this->data->action=="edit"&&$current===$new) //additional groups not changed
      continue; //skip rights check

     //validate add/delete rights for new/old groups
     $d->setAction("add");
     $d->clearValues();

     foreach($new as $v)
     {
      if($current&&!in_array($v, $current)) //new group
      {
       $d->setValue("catnr", $v);
       if(!$storage->checkRights("add"))
        $success=false;
      }
     }

     if($current)
     {
      $d->setAction("delete");
      $d->clearValues();

      foreach($current as $v)
      {
       if(!in_array($v, $new)) //removed group
       {
        $d->setValue("nr", $this->data->getValue("nr", $this->data->vc));
        if(!$storage->checkRights("delete"))
         $success=false;
       }
      }
     }
    }

    if(!$success)
     $this->data->e->add($storage->e);
   }

   return $success;
  }


  function deletePreSubmitErrorCheck()
  {
   global $_config;

   for($this->data->vc=0;$this->data->vc<$this->data->vmax;$this->data->vc++)
   {
    $v=$this->data->getValue("nr", $this->data->vc);

    if($v==$_config->get("rootnr"))
     $this->data->e->add("Can't delete root user", "hack");
    else if($v==$_config->get("anonnr"))
     $this->data->e->add("Can't delete anonymous user", "error");
    else if($v==$_config->get("usernr"))
     $this->data->e->add("Can't delete own user", "error");
   }

   return true;
  }

  function deletePostSubmit()
  {
   //remove rights for user
   $du=Data::cache("userRights", "delete", "all");
   if(!($dus=$du->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //delete user rights
    $du->clearValues();
    $du->setValue("nr", $this->data->getValue("nr", $c));

    for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
    {
     if(!$dus->delete(true))
     {
      $this->data->e->add($dus->e);
      return false;
     }
    }
   }

   return true;
  }


  function userPostSubmitErrorCheck()
  {
   if(!$this->data->check("use", "show", "backend"))
    $this->data->setShowFields(array_merge($this->data->check("use", "show"), Array("backend")));

   if($this->data->e->any()&&$this->data->extra=="login")
   {
    $this->data->e->clear();
    $this->data->e->add("Login failed");
   }

   return true;
  }

  function userPostSubmit()
  {
   global $_config;

   switch($this->data->extra)
   {
    case "login":
    {
     $this->data->e->clear();
     if($this->data->next())
     {
      $_config->set("usernr", $this->data->v("id"));
      $_config->set("groupnrs", $this->getGroups($this->data->v("id")));
      $_config->set("userbackend", $this->data->v["backend"]);

      $this->data->e->add("Login successful", "note");
     }
     else
     {
      $this->data->e->add("Login failed", "error");
      return false;
     }

     break;
    }
    case "logout":
    {
     $_config->set("usernr", $_config->get("anonnr"));
     $_config->set("groupnrs", $this->getGroups($_config->get("anonnr")));
     $_config->set("userbackend");
     $_config->reload();

     $this->data->e->add("Logout successful", "note");

     break;
    }
    default: break; //password, do nothing
   }

   return true;
  }


  function getGroups($usernr)
  {
   global $_config;
   $groups=Array();

   $d=Data::cache($this->data->module, "view", "item");
   $d->setValue("nr", $usernr);

   if(isset($d->reference["usersAdditionalGroups"]))
    $d->setShowFields("catnr", "usersAdditionalGroups");
   else
    $d->setShowFields("catnr");

   //sudo get groups
   $_config->su(true);
   if(!$d->get())
   {
    $_config->su();
    return $groups;
   }
   $_config->su();
   unset($tmp);

   if($d->next())
   {
    if(isset($d->v["usersAdditionalGroups"]))
     $groups=array_merge(Array($d->v["catnr"]), $d->v["usersAdditionalGroups"]);
    else
     $groups=Array($d->v["catnr"]);
   }

   return $groups;
  }

 }
?>
