<?

 /*****************************************************************************
 * Filename: include/plugins/lister/modules/pluginConfigs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["modules"]."dataset.php");

 class Plugin_Module_lister_pluginConfigs extends System_Module_dataset
 {
  function __construct(&$data)
  {
   $data->l->read("module_dataset");

   parent::__construct($data);

   unset($this->data->field["nr"]);
   unset($this->data->field["catnr"]);
   unset($this->data->field["owner"]);
   unset($this->data->field["updated"]);

   if($this->data->module!="pluginConfigs")
    $this->extend("pluginConfigs");

   $this->data->title="Lister plugin configuration";

   $this->data->addField("listtype", "enum", "all", "reload", false, false, "Item", "List type");
   $this->data->enumOptions("listtype", "Item", "List", "Tree", "All", "Data set", "Search", "Dynamic");

   $this->data->addField("static", "boolean", "all", "reload", false, false, false, "Static item/list/tree");

   $this->data->addField("listcatnr", "nr", "all", "", false, false, 0, "List category");

   $this->data->addField("itemnr", "nr", "all", "reload", false, false, 0, "Item");

   $this->data->addField("itempage", "boolean", "all", "reload", false, false, false, "Use item page");
   $this->data->addField("itempagenr", "nr", "all", "", false, false, false, "Item page");
   $this->data->addReference("itempagenr", "categories", "nr", "*:0");

   $this->data->addField("cmspage", "boolean", "all", "reload", false, false, false, "Use CMS page");
   $this->data->addField("cmspagenr", "nr", "all", "", false, false, false, "CMS page");
   $this->data->addReference("cmspagenr", "categories", "nr", "*:0");

   $this->data->addField("singletemplate", "boolean", "all", "", false, false, false, "Single item template", "Single results use item template");


   $this->data->addField("orderbymanual", "boolean", "all", "", false, false, false, "Allow user based sorting");


   $this->data->addField("pagebrowser", "boolean", "all", "reload", false, false, false, "Show page browser");

   $this->data->addField("listitems", "nr", "all", "", 1, 100, 10, "Items per page");
   $this->data->addField("groupfield", "list", "all", "reload", 0, 255, false, "Group items by field");
   $this->data->addField("showfields", "multilist", "all", "", 0, 65536, false, "Show fields for _fields tag");

   $this->data->addField("templatepagebrowser", "localfile", "all", "recursive", false, 255, "", "Alternative page browser template");
   $this->data->addField("templateitem", "localfile", "all", "recursive", false, 255, "", "Item template");
   $this->data->addField("templatelist", "localfile", "all", "recursive", false, 255, "", "List template");


   //delete dataset groups
   $this->data->deleteGroup($this->data->getGroupID("Data set"));
   $this->data->deleteGroup($this->data->getGroupID("Filters"));
   $this->data->deleteGroup($this->data->getGroupID("Aggregation"));
   $this->data->deleteGroup($this->data->getGroupID("Sorting"));

   //add new groups in proper order
   $this->data->group("List", "module", "listtype", "static", "itemnr", "listcatnr", "itempage", "itempagenr", "cmspage", "cmspagenr", "singletemplate");
   $this->data->group("Show", "groupfield", "showfields");

   $fields=Array();
   for($c=1;$c<=$this->filters;$c++)
   {
    $fields[]="filter_field_".$c;
    $fields[]="filter_operator_".$c;
    $fields[]="filter_value_".$c;
    $fields[]="filter_type_".$c;
   }

   $this->data->group("Filters", $fields);

   $this->data->group("Sorting", "orderby1", "orderbysort1", "orderby2", "orderbysort2", "orderby3", "orderbysort3", "orderbymanual");

   $this->data->group("Page browser", "pagebrowser", "listitems", "templatepagebrowser");
   $this->data->group("Alternative templates", "templateitem", "templatelist");
  }

  function postInit()
  {
   global $_path;

   $this->data->setFileDir("templatepagebrowser", $_path["project"]["templates"]);
   $this->data->setFileExtensions("templatepagebrowser", "html", "htm");
   $this->data->setFileDir("templateitem", $_path["project"]["templates"]);
   $this->data->setFileExtensions("templateitem", "html", "htm");
   $this->data->setFileDir("templatelist", $_path["project"]["templates"]);
   $this->data->setFileExtensions("templatelist", "html", "htm");

   return $this->customizeFields();
  }
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   //always use view all extra action
   $this->data->setValue("extra", "All");

   if($this->data->getValue("groupfield"))
    $this->data->setValue("aggregation_field", $this->data->getValue("groupfield"));

   if(!parent::customizeFields())
    return false;

   //make sure irrelevant dataset fields aren't used
   $this->data->field["extra"]["use"]="";
   $this->data->field["name"]["use"]="show";

   $this->data->field["aggregation_field"]["use"]="";
   $this->data->field["aggregation_type"]["use"]="";
   $this->data->field["aggregation_method"]["use"]="";
   $this->data->field["aggregation_interval"]["use"]="";

   //hide filter fields
   if(
    $this->data->getValue("listtype")=="Item"||
    $this->data->getValue("listtype")=="List"||
    $this->data->getValue("listtype")=="Tree"||
    $this->data->getValue("listtype")=="Data set"||
    $this->data->getValue("listtype")=="Search"
   )
   {
    for($c=1;$c<=$this->filters;$c++)
    {
     $this->data->field["filter_field_".$c]["properties"].=";hidden";
     $this->data->field["filter_operator_".$c]["properties"].=";hidden";
     $this->data->field["filter_value_".$c]["properties"].=";hidden";
     $this->data->field["filter_type_".$c]["properties"].=";hidden";
    }
   }

   if($this->data->getValue("listtype")=="Data set")
   {
    $this->data->field["module"]["min"]=0;
    $this->data->field["module"]["properties"].=";hidden";

    $this->data->resetField("static");
    $this->data->resetField("listcatnr");
    $this->data->resetField("itemnr");

    $this->data->resetField("itempage");
    $this->data->resetField("itempagenr");

    $this->data->resetField("cmspage");
    $this->data->resetField("cmspagenr");

    $this->data->resetField("singletemplate");

    $this->data->resetField("orderby1");
    $this->data->resetField("orderbysort1");
    $this->data->resetField("orderby2");
    $this->data->resetField("orderbysort2");
    $this->data->resetField("orderby3");
    $this->data->resetField("orderbysort3");

    $this->data->resetField("pagebrowser");
    $this->data->resetField("templatepagebrowser");
    $this->data->resetField("listitems");

    $this->data->resetField("templateitem");
    $this->data->resetField("templatelist");


    if(!$this->data->getValue("cmspage"))
     $this->data->field["cmspagenr"]["properties"].=";hidden";
    else
     $this->data->resetField("cmspagenr");

    $this->data->field["groupfield"]["properties"].=";hidden";

    $this->data->field["orderby1"]["properties"].=";hidden";
    $this->data->field["orderbysort1"]["properties"].=";hidden";
    $this->data->field["orderby2"]["properties"].=";hidden";
    $this->data->field["orderbysort2"]["properties"].=";hidden";
    $this->data->field["orderby3"]["properties"].=";hidden";
    $this->data->field["orderbysort3"]["properties"].=";hidden";

    $this->data->unsetValue("orderby1");
    $this->data->unsetValue("orderby2");
    $this->data->unsetValue("orderby3");

    $this->data->field["itemnr"]["min"]=1;
    $this->data->field["itemnr"]["title"]="Data set";

    $tmp=Data::cache("dataset", "view", "item");

    //add reference to dataset
    if(!isset($this->data->reference["itemnr"]))
     $this->data->addReference("itemnr", $tmp->module, $tmp->getID(), "*:1");

    //get dataset module show fields
    $tmp->setValue("nr", $this->data->getValue("itemnr"));
    if($tmp->get()&&$tmp->next())
    {
     $tmp=$tmp->moduleObject->getDataObject();

     $fields=Array();
     foreach($tmp->check("use", "show") as $field)
     {
      if(
       !$tmp->check("properties", "order", $field)&&
       !$tmp->check("properties", "control", $field)&&
       !$tmp->getReferences($field)
      )
       $fields[$field]=$tmp->field[$field]["title"];
     }

     $this->data->listOptions("showfields", $fields);
    }

    unset($tmp);
   }
   else if(($module=$this->data->getValue("module"))&&!is_array($module)&&$module!=$this->data->module)
   {
    $this->data->resetField("module");

    $this->data->resetField("static");
    $this->data->resetField("listcatnr");
    $this->data->resetField("itemnr");

    $this->data->resetField("itempage");
    $this->data->resetField("itempagenr");

    $this->data->resetField("cmspage");
    $this->data->resetField("cmspagenr");

    $this->data->resetField("singletemplate");

    $this->data->resetField("orderby1");
    $this->data->resetField("orderbysort1");
    $this->data->resetField("orderby2");
    $this->data->resetField("orderbysort2");
    $this->data->resetField("orderby3");
    $this->data->resetField("orderbysort3");

    $this->data->resetField("pagebrowser");
    $this->data->resetField("templatepagebrowser");
    $this->data->resetField("listitems");

    $this->data->resetField("templateitem");
    $this->data->resetField("templatelist");

    $tmp=Data::cache($module);


    //get module sort fields
    $fields=Array("" => "");
    foreach(array_keys($tmp->field) as $k)
     if($tmp->check("use", "sql", $k)||$tmp->check("use", "show", $k))
      if($k!="r")
       $fields[$k]=$tmp->field[$k]["title"];

    $fields[$k]=$tmp->field[$k]["title"];

    $this->data->listOptions("orderby1", $fields);
    $this->data->listOptions("orderby2", $fields);
    $this->data->listOptions("orderby3", $fields);


    //get module show fields
    $fields=Array();
    foreach($tmp->check("use", "show") as $field)
    {
     if(
      !$tmp->check("properties", "order", $field)&&
      !$tmp->check("properties", "control", $field)&&
      !$tmp->getReferences($field)
     )
      $fields[$field]=$tmp->field[$field]["title"];
    }

    $this->data->listOptions("groupfield", array_merge(Array("" => ""), $fields));
    $this->data->listOptions("showfields", $fields);


    if(!$this->data->getValue("cmspage"))
     $this->data->field["cmspagenr"]["properties"].=";hidden";
    else
     $this->data->resetField("cmspagenr");
   }
   else //no dataset nor module selected, disable most fields
   {
    $this->data->resetField("module");

    $this->data->field["static"]["properties"].=";hidden";

    $this->data->field["listcatnr"]["properties"].=";hidden";
    $this->data->field["itemnr"]["properties"].=";hidden";

    $this->data->field["itempage"]["properties"].=";hidden";
    $this->data->field["itempagenr"]["properties"].=";hidden";

    $this->data->field["cmspage"]["properties"].=";hidden";
    $this->data->field["cmspagenr"]["properties"].=";hidden";

    $this->data->field["singletemplate"]["properties"].=";hidden";

    $this->data->field["orderby1"]["properties"].=";hidden";
    $this->data->field["orderbysort1"]["properties"].=";hidden";
    $this->data->field["orderby2"]["properties"].=";hidden";
    $this->data->field["orderbysort2"]["properties"].=";hidden";
    $this->data->field["orderby3"]["properties"].=";hidden";
    $this->data->field["orderbysort3"]["properties"].=";hidden";

    $this->data->field["pagebrowser"]["properties"].=";hidden";
    $this->data->field["templatepagebrowser"]["properties"].=";hidden";
    $this->data->field["listitems"]["properties"].=";hidden";

    $this->data->field["templateitem"]["properties"].=";hidden";
    $this->data->field["templatelist"]["properties"].=";hidden";

    return true; //don't bother checking the rest
   }

   if($this->data->getValue("listtype")=="Item")
   {
    $this->data->field["listcatnr"]["properties"].=";hidden";

    $this->data->field["itempage"]["properties"].=";hidden";
    $this->data->field["itempagenr"]["properties"].=";hidden";

    $this->data->field["singletemplate"]["properties"].=";hidden";

    $this->data->field["orderby1"]["properties"].=";hidden";
    $this->data->field["orderbysort1"]["properties"].=";hidden";
    $this->data->field["orderby2"]["properties"].=";hidden";
    $this->data->field["orderbysort2"]["properties"].=";hidden";
    $this->data->field["orderby3"]["properties"].=";hidden";
    $this->data->field["orderbysort3"]["properties"].=";hidden";
    $this->data->field["orderbymanual"]["properties"].=";hidden";

    $this->data->field["pagebrowser"]["properties"].=";hidden";
    $this->data->field["templatepagebrowser"]["properties"].=";hidden";
    $this->data->field["listitems"]["properties"].=";hidden";
    $this->data->field["listitems"]["min"]=0;
    $this->data->field["listitems"]["defaultvalue"]=0;

    $this->data->field["templatelist"]["properties"].=";hidden";

    if($this->data->getValue("static"))
    {
     $this->data->resetField("itemnr");
     $this->data->field["itemnr"]["min"]=1;
     $tmp=Data::cache($module);
     $this->data->addReference("itemnr", $tmp->module, $tmp->getID(), "*:1");
     unset($tmp);
    }
    else //dynamic nr filtering
     $this->data->field["itemnr"]["properties"].=";hidden";
   }
   else //List, All, Search, Dynamic (Item, List, All, Search)
   {
    if($this->data->getValue("listtype")!="Data set")
     $this->data->field["itemnr"]["properties"].=";hidden";

    if($this->data->getValue("listtype")=="List")
    {
     $this->data->addReference("listcatnr", "categories", "nr", "*:0");
     $this->data->field["listcatnr"]["title"]="List category";


     if($this->data->getValue("static"))
      $this->data->resetField("listcatnr");
     else
      $this->data->field["listcatnr"]["properties"].=";hidden";
    }
    else if($this->data->getValue("listtype")=="Tree")
    {
     $tmp=Data::cache($module);

     $this->data->field["singletemplate"]["properties"].=";hidden";

     $this->data->setValue("orderby1", "");
     $this->data->field["orderby1"]["properties"].=";hidden";
     $this->data->field["orderbysort1"]["properties"].=";hidden";
     $this->data->field["orderby2"]["properties"].=";hidden";
     $this->data->field["orderbysort2"]["properties"].=";hidden";
     $this->data->field["orderby3"]["properties"].=";hidden";
     $this->data->field["orderbysort3"]["properties"].=";hidden";
     $this->data->field["orderbymanual"]["properties"].=";hidden";

     $this->data->setValue("pagebrowser", false);
     $this->data->field["pagebrowser"]["properties"].=";hidden";
     $this->data->field["templatepagebrowser"]["properties"].=";hidden";
     $this->data->field["listitems"]["properties"].=";hidden";


     if(!$tmp->validAction("tree"))
     {
      $this->data->field["static"]["properties"].=";hidden";

      $this->data->field["listcatnr"]["properties"].=";hidden";
      $this->data->field["itemnr"]["properties"].=";hidden";

      $this->data->field["itempage"]["properties"].=";hidden";
      $this->data->field["itempagenr"]["properties"].=";hidden";

      $this->data->field["cmspage"]["properties"].=";hidden";
      $this->data->field["cmspagenr"]["properties"].=";hidden";

      $this->data->field["templateitem"]["properties"].=";hidden";
      $this->data->field["templatelist"]["properties"].=";hidden";

      $this->data->e->add("Module: \"".$tmp->module."\" doesn't have tree capabilities", "error");
      return false;
     }


     $this->data->addReference("listcatnr", $this->data->getValue("module"), $tmp->getID(), "*:0");
     $this->data->field["listcatnr"]["title"]=$tmp->title." tree";


     if($this->data->getValue("static"))
      $this->data->resetField("listcatnr");
     else
      $this->data->field["listcatnr"]["properties"].=";hidden";
    }
    else
    {
     $this->data->field["static"]["properties"].=";hidden";
     $this->data->field["listcatnr"]["properties"].=";hidden";
    }


    if(!$this->data->getValue("itempage"))
     $this->data->field["itempagenr"]["properties"].=";hidden";
    else
     $this->data->resetField("itempagenr");


    $this->data->resetField("listitems");

    if($this->data->getValue("listtype")!="Data set") //sorting done in dataset
    {
     if(!$this->data->getValue("orderby1"))
     {
      $this->data->field["orderbysort1"]["properties"].=";hidden";

      $this->data->field["orderby2"]["properties"].=";hidden";
      $this->data->field["orderbysort2"]["properties"].=";hidden";
      $this->data->field["orderby3"]["properties"].=";hidden";
      $this->data->field["orderbysort3"]["properties"].=";hidden";
     }
     else
     {
      $this->data->resetField("orderbysort1");

      if(!$this->data->getValue("orderby2"))
      {
       $this->data->field["orderbysort2"]["properties"].=";hidden";

       $this->data->field["orderby3"]["properties"].=";hidden";
       $this->data->field["orderbysort3"]["properties"].=";hidden";
      }
      else
      {
       $this->data->resetField("orderbysort2");

       if(!$this->data->getValue("orderby3"))
        $this->data->field["orderbysort3"]["properties"].=";hidden";
       else
        $this->data->resetField("orderbysort3");
      }
     }
    }

    if(!$this->data->getValue("pagebrowser"))
    {
     $this->data->field["templatepagebrowser"]["properties"].=";hidden";

     //move listitems to List group
     $this->data->field["listitems"]["min"]=0;
     $this->data->field["listitems"]["defaultvalue"]=0;
     $this->data->field["listitems"]["group"]=$this->data->getGroupID("Show");

     if($this->data->getValue("listtype")=="Tree")
      $this->data->field["listitems"]["title"]="Tree depth";
     else
      $this->data->field["listitems"]["title"]="Items limit";
    }
    else
    {
     $this->data->resetField("templatepagebrowser");

     $this->data->resetField("listitems");
     $this->data->field["listitems"]["group"]=$this->data->getGroupID("Page browser");
     $this->data->field["listitems"]["title"]="Items per page";
    }


    if($this->data->getValue("listtype")!="Dynamic")
     $this->data->field["templateitem"]["properties"].=";hidden";
   }

   return true;
  }

 }
?>
