<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."fs.php");

 define("UT_OK", 1, true);
 define("UT_FAILED", 0, true);
 define("UT_PARTIALLYFAILED", -1, true);
 define("UT_NOTESTS", -2, true);
 define("UT_SKIPPED", -3, true);

 class UT
 {
  function __construct()
  {
   $this->e=new Errors();

   $this->tests=false;
   $this->supportFunctions=false;

   $this->baseFunctions=get_class_methods("UT");
   $this->parentFunctions=get_class_methods(get_parent_class($this));

   $this->results=false;
  }

  function _initTests()
  {
   $files=Array();
   $this->tests=Array();
   $this->supportFunctions=Array();
   foreach(get_class_methods($this) as $function)
   {
    if(in_array($function, $this->baseFunctions)) //ignore base functions
     continue;

    $refMet=new ReflectionMethod($this, $function); //check who declares the method
    if($refMet->getDeclaringClass()->getName()!=get_class($this)) //if not current it's not a test
     continue;

    $file=$refMet->getFileName();
    if(!isset($files[$file]))
     $files[$file]=FS::readLinesS($file);

    $info=Array(
     "file" => $refMet->getFileName(),
     "code" => ($files[$file]?implode("\n", array_slice($files[$file], $refMet->getStartLine()-1, $refMet->getEndLine()-$refMet->getStartLine()+1)):""),
     "start" => $refMet->getStartLine(),
     "end" => $refMet->getEndLine(),
     "unit" => get_class($this),
     "test" => $function,
    );

    $info["ifs"]=substr_count($info["code"], " if(");

    if(preg_match("/^_/", $function))
     $this->supportFunctions[$function]=$info;
    else
     $this->tests[$function]=$info;
   }
  }

  function _run()
  {
   global $_config;

   if($this->tests===false)
    $this->_initTests();

   if(!($tests=func_get_args()))
    $tests=$this->_getTests();

   $this->results=Array(
    "status" => UT_NOTESTS,
    "tests" => Array(),
   );

   if($_config->get("logging"))
   {
    $logging=$_config->get("logging");
    $_config->set("logging", false);
   }
   else
    $logging=false;

   foreach($tests as $test)
   {
    if(isset($this->tests[$test]))
    {
     $this->e=new Errors();
     $this->e->l->read("plugin_unit_testing");

     $tmp=Array();
     $tmp["time"]=microtime(true);
     $tmp["status"]=$this->$test();
     $tmp["time"]=microtime(true)-$tmp["time"];

     switch($tmp["status"])
     {
      case UT_OK:
      {
       if($this->results["status"]===UT_NOTESTS)
        $this->results["status"]=UT_OK;
       else if($this->results["status"]===UT_FAILED)
        $this->results["status"]=UT_PARTIALLYFAILED;

       break;
      }
      case UT_FAILED:
      {
       $this->e->add("Test failed: ".implode(" / ", $this->_getUnitPath())." / ".str_replace("_", " ", $test), "error");

       if($this->results["status"]===UT_NOTESTS)
        $this->results["status"]=UT_FAILED;
       else if($this->results["status"]===UT_OK)
        $this->results["status"]=UT_PARTIALLYFAILED;

       break;
      }
      case UT_SKIPPED:
      {
       $this->e->add("Test skipped: ".implode(" / ", $this->_getUnitPath())." / ".str_replace("_", " ", $test), "warning");

       break;
      }
      default:
      {
       $this->e->add("Test has unknown result: ".implode(" / ", $this->_getUnitPath())." / ".str_replace("_", " ", $test), "error");

       $this->results["status"]=UT_FAILED;
       break;
      }
     }

     $tmp["e"]=$this->e;

     $this->results["tests"][$test]=$tmp;
    }
   }

   if($logging)
    $_config->set("logging", true);

   return $this->results;
  }

  function _getUnitPath()
  {
   $tmp=get_class($this);
   $path[]=$tmp;

   while($tmp=get_parent_class($tmp))
   {
    if($tmp=="UT")
     break;

    $path[]=str_replace("_", " ", str_replace("UT_", "", $tmp));
   }

   return $path;
  }

  function _getUnitName()
  {
   return str_replace("_", " ", str_replace("UT_", "", get_class($this)));
  }

  function _getParentUnitName()
  {
   return str_replace("_", " ", str_replace("UT_", "", get_parent_class($this)));
  }

  function _getTestInfo($test)
  {
   if($this->tests===false)
    $this->_initTests();

   if(!isset($this->tests[$test]))
    return false;

   return $this->tests[$test];
  }

  function _getTests()
  {
   if($this->tests===false)
    $this->_initTests();

   return array_keys($this->tests);
  }

  function _getSupportFunctions()
  {
   if($this->supportFunctions===false)
    $this->_initTests();

   return array_keys($this->supportFunctions);
  }
 }

?>
