<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/api.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."data.php");
 require_once($_path["api"]."api.php");
 require_once($_path["api"]."xml.php");

 class UT_API extends UT_Classes
 {
  function _config($enabled)
  {
   //enable/disable all
   $config=Data::cache("configAPI", "view", "item", Array("skipCheckTables" => true));
   foreach($config->check("type", "boolean") as $field)
    $config->setValue($field, $enabled);

   $config->next();
   return $config->v;
  }


  function Constructor()
  {
   $object=new API();

   if(!is_object($object)||get_class($object)!="API")
   {
    $this->e->add("Constructor doesn't return a API object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->type)
   {
    $this->e->add("API type shouldn't be set");
    return UT_FAILED;
   }

   if($object->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($object->encoding!="ISO-8859-1")
   {
    $this->e->add("API enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->contentType!="text/plain")
   {
    $this->e->add("API content type isn't text/plain");
    return UT_FAILED;
   }

   if($object->filename)
   {
    $this->e->add("API filename shouldn't be set");
    return UT_FAILED;
   }

   if(!isset($object->config))
   {
    $this->e->add("API configuration not set");
    return UT_FAILED;
   }

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   //merge config
   $api=new API_XML("server", $this->_config(true)); //API enabled

   if(!$api->config["xml"])
   {
    $this->e->add("API XML should be enabled");
    return UT_FAILED;
   }

   $api=new API_XML("server", $this->_config(false)); //API disabled

   if($api->config["xml"])
   {
    $this->e->add("API XML should be disabled");
    return UT_FAILED;
   }


   $object=new API("failed");

   if(!is_object($object)||get_class($object)!="API")
   {
    $this->e->add("Constructor doesn't return a API object", "error");
    return UT_FAILED;
   }

   if($object->mode)
   {
    $this->e->add("API mode shouldn't be set");
    return UT_FAILED;
   }

   if(!$object->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new API();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("API object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->mode="different";
   if($orig->mode==$copy->mode)
   {
    $this->e->add("API object copy also modified original \"mode\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("API object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $api=new API("server");

   $api->output="test";
   $api->filename="test";
   $api->commands=Array("test");
   $api->clear();

   if($api->output)
   {
    $this->e->add("API clear should have cleared output");
    return UT_FAILED;
   }

   if($api->filename)
   {
    $this->e->add("API clear should have cleared filename");
    return UT_FAILED;
   }

   if($api->commands)
   {
    $this->e->add("API clear should have cleared commands");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Error()
  {
   $api=new API("server");

   if(!$api->e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   if(!$api->error())
   {
    $this->e->add("API error should be successful");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->output!="error: Error\n")
   {
    $this->e->add("API output doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Filename()
  {
   $api=new API("server");

   if($api->filename)
   {
    $this->e->add("API filename shouldn't be set");
    return UT_FAILED;
   }

   if(!$api->filename("api.txt"))
   {
    $this->e->add("API filename should be successful");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->filename!="api.txt")
   {
    $this->e->add("API filename doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetRequestResponseHeaders()
  {
   $api=new API("server");

   if(!$api->setRequestResponseHeaders())
   {
    $this->e->add("Unable to unset request/response headers");
    return UT_FAILED;
   }

   if(!$api->setRequestResponseHeaders(Array(
    "Accept" => "text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/webp, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1",
    "Accept-Language" => "en-US,en;q=0.9",
    "Accept-Encoding" => "gzip, deflate",
    "X-JAS-API" => "1.0",
    "Cache-Control" => "no-cache",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if(!isset($api->requestResponseHeaders["X-JAS-API"])||$api->requestResponseHeaders["X-JAS-API"]!="1.0")
   {
    $this->e->add("X-JAS-API request/response header set incorrectly");
    return UT_FAILED;
   }

   if(!$api->setRequestResponseHeaders(Array(
    "Accept: text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/webp, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1",
    "Accept-Language: en-US,en;q=0.9",
    "Accept-Encoding: gzip, deflate",
    "X-JAS-API: Data-1.1",
    "Cache-Control: no-cache",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if(!isset($api->requestResponseHeaders["X-JAS-API"])||$api->requestResponseHeaders["X-JAS-API"]!="Data-1.1")
   {
    $this->e->add("X-JAS-API request/response header set incorrectly");
    return UT_FAILED;
   }

   if(!$api->setRequestResponseHeaders(
"Accept: text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/webp, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1
Accept-Language: en-US,en;q=0.9
Accept-Encoding: gzip, deflate
X-JAS-API: 1.2
Cache-Control: no-cache"
   ))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if(!isset($api->requestResponseHeaders["X-JAS-API"])||$api->requestResponseHeaders["X-JAS-API"]!="1.2")
   {
    $this->e->add("X-JAS-API request/response header set incorrectly");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetHeaderVersion()
  {
   $api=new API("server");

   if($version=$api->getHeaderVersion())
   {
    $this->e->add("API header version: ".$version." shouldn't be set");
    return UT_FAILED;
   }

   if(!$api->setRequestResponseHeaders(Array(
    "Accept" => "text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/webp, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1",
    "Accept-Language" => "en-US,en;q=0.9",
    "Accept-Encoding" => "gzip, deflate",
    "X-JAS-API" => "1.0",
    "Cache-Control" => "no-cache",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if(!($version=$api->getHeaderVersion()))
   {
    $this->e->add("API header version should be set");
    return UT_FAILED;
   }

   if($version!="1.0")
   {
    $this->e->add("API header version doesn't match");
    return UT_FAILED;
   }


   $api->type="Data";
   if(!$api->setRequestResponseHeaders(Array(
    "Accept" => "text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/webp, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1",
    "Accept-Language" => "en-US,en;q=0.9",
    "Accept-Encoding" => "gzip, deflate",
    "X-JAS-API" => "Data-1.1",
    "Cache-Control" => "no-cache",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if(!($version=$api->getHeaderVersion()))
   {
    $this->e->add("API header version should be set");
    return UT_FAILED;
   }

   if($version!="1.1")
   {
    $this->e->add("API header version doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $api=new API("server");
   $api->filename("api.txt");
   $api->output="test";

   if($api->output(false)!="test")
   {
    $this->e->add("API output should have been returned");
    return UT_FAILED;
   }

   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_OK;
   }

   $this->_storeHeaders();

   if($api->output()!="test")
   {
    $this->e->add("API output should have been returned");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("X-JAS-API: ".$api->version, headers_list()))
   {
    $this->e->add("API output should have set X-JAS-API header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $found=false;
   foreach(headers_list() as $header)
   {
    if(strstr($header, "Content-type: ".$api->contentType))
    {
     $found=true;
     break;
    }
   }

   if(!$found)
   {
    $this->e->add("API output should have set content type header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("Content-Length: 4", headers_list()))
   {
    $this->e->add("API output should have set content length header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("Content-Disposition: attachment; filename=\"api.txt\"", headers_list()))
   {
    $this->e->add("API output should have set filename header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }

  function Parse()
  {
   $api=new API("server");

   if($api->parse("test"))
   {
    $this->e->add("API parse should have failed");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format()
  {
   $api=new API("server");

   if($api->format())
   {
    $this->e->add("API format should have failed");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Enabled()
  {
   $api=new API_XML("server", $this->_config(true)); //API enabled

   if(!$api->enabled())
   {
    $this->e->add("API should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->enabled("data", "categories", "view", "all"))
   {
    $this->e->add("API data view/all should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->enabled("data", "users", "delete", "item"))
   {
    $this->e->add("API data delete/item should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->enabled("language"))
   {
    $this->e->add("API language should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->enabled("modules"))
   {
    $this->e->add("API modules should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->enabled("projects"))
   {
    $this->e->add("API projects should be enabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $api=new API_XML("server", $this->_config(false)); //API disabled

   if($api->enabled())
   {
    $this->e->add("API should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->enabled("data", "categories", "view", "all"))
   {
    $this->e->add("API data view/all should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->enabled("data", "users", "delete", "item"))
   {
    $this->e->add("API data delete/item should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->enabled("language"))
   {
    $this->e->add("API language should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->enabled("modules"))
   {
    $this->e->add("API modules should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->enabled("projects"))
   {
    $this->e->add("API projects should be disabled");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddData()
  {
   $api=new API("server");

   $tmp["type"]="data";
   $tmp["data"]=Data::cache("categories", "view", "all", Array("db" => "session"));
   $tmp["count"]=false;

   if(!$api->addData($tmp["data"], $tmp["count"]))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API added data command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="data";
   $tmp["data"]=Data::cache("users", "delete", "item", Array("db" => "session"));
   $tmp["count"]=false;

   if(!$api->addData($tmp["data"], $tmp["count"]))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[1]))
   {
    $this->e->add("API added data command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[1], $tmp))
    return UT_FAILED;

   return UT_OK;
  }

  function SetData()
  {
   $api=new API("server");

   $tmp["type"]="data";
   $tmp["data"]=Data::cache("categories", "view", "all", Array("db" => "session"));
   $tmp["count"]=false;

   if(!$api->setData(0, $tmp["data"], $tmp["count"]))
   {
    $this->e->add("API set data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set data command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="data";
   $tmp["data"]=Data::cache("users", "delete", "item", Array("db" => "session"));
   $tmp["count"]=false;

   if(!$api->setData(0, $tmp["data"], $tmp["count"]))
   {
    $this->e->add("API set data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set data command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;

   return UT_OK;
  }


  function AddLanguage()
  {
   $api=new API("server");

   $tmp["type"]="language";
   $tmp["language"]=new Language();
   $tmp["lang"]="nl";
   $tmp["set"]="modules";

   if(!$api->addLanguage($tmp["set"], $tmp["lang"]))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API added language command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="language";
   $tmp["language"]=new Language();
   $tmp["lang"]="en";
   $tmp["set"]="data";

   if(!$api->addLanguage($tmp["set"], $tmp["lang"], $tmp["language"]))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[1]))
   {
    $this->e->add("API added language command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[1], $tmp))
    return UT_FAILED;

   return UT_OK;
  }

  function SetLanguage()
  {
   $api=new API("server");

   $tmp["type"]="language";
   $tmp["language"]=new Language();
   $tmp["lang"]="nl";
   $tmp["set"]="modules";

   if(!$api->setLanguage(0, $tmp["set"], $tmp["lang"]))
   {
    $this->e->add("API set language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set language command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="language";
   $tmp["language"]=new Language();
   $tmp["lang"]="en";
   $tmp["set"]="data";

   if(!$api->setLanguage(0, $tmp["set"], $tmp["lang"], $tmp["language"]))
   {
    $this->e->add("API set language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set language command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;

   return UT_OK;
  }


  function AddModules()
  {
   $api=new API("server");

   $tmp=Array();
   $tmp["all"]=false;
   $tmp["type"]="modules";
   $tmp["modules"]=Array();

   if(!$api->addModules())
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API added modules command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["all"]=true;
   $tmp["type"]="modules";
   $tmp["modules"]=Array("categories", "users");

   if(!$api->addModules($tmp["modules"], true))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[1]))
   {
    $this->e->add("API added modules command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[1], $tmp))
    return UT_FAILED;

   return UT_OK;
  }

  function SetModules()
  {
   $api=new API("server");

   $tmp=Array();
   $tmp["all"]=false;
   $tmp["type"]="modules";
   $tmp["modules"]=Array();

   if(!$api->setModules(0))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set modules command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["all"]=true;
   $tmp["type"]="modules";
   $tmp["modules"]=Array("categories", "users");

   if(!$api->setModules(0, $tmp["modules"], true))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set modules command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;

   return UT_OK;
  }


  function MergeModules()
  {
   $api=new API("server");

   $d=Data::cache("categories");

   //new entry
   $tmp=Array();
   $tmp["all"]=false;
   $tmp["type"]="modules";
   $tmp["modules"]=Array(&$d);

   if(!$api->mergeModules($d))
   {
    $this->e->add("API merge modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API merge modules command not found");
    return UT_FAILED;
   }

   if(sizeof($api->commands)>1)
   {
    $this->e->add("API merge modules to many commands");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   //new in existing modules command
   $tmp["all"]=true;
   $tmp["type"]="modules";
   $tmp["modules"]=Array("users");

   if(!$api->setModules(0, $tmp["modules"], true))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->mergeModules($d))
   {
    $this->e->add("API merge modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set modules command not found");
    return UT_FAILED;
   }

   if(sizeof($api->commands)>1)
   {
    $this->e->add("API merge modules to many commands");
    return UT_FAILED;
   }

   $tmp["modules"][1]=&$d;
   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   //overwrite module name
   $tmp["all"]=false;
   $tmp["type"]="modules";
   $tmp["modules"]=Array("categories", "users");

   if(!$api->setModules(0, $tmp["modules"]))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->mergeModules($d))
   {
    $this->e->add("API merge modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set modules command not found");
    return UT_FAILED;
   }

   if(sizeof($api->commands)>1)
   {
    $this->e->add("API merge modules to many commands");
    return UT_FAILED;
   }

   $tmp["modules"][0]=&$d;
   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   //overwrite data object
   $tmp["all"]=false;
   $tmp["type"]="modules";
   $tmp["modules"]=Array("users", &$d);

   if(!$api->setModules(0, $tmp["modules"]))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->mergeModules($d))
   {
    $this->e->add("API merge modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set modules command not found");
    return UT_FAILED;
   }

   if(sizeof($api->commands)>1)
   {
    $this->e->add("API merge modules to many commands");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;

   return UT_OK;
  }


  function CompareModules()
  {
   $api=new API("server");

   $d1=Data::cache("categories");
   $d2=Data::cache("categories");

   if(!$api->compareModules($d1, $d2))
   {
    $this->e->add("API compare modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $d2->title="Changed";
   if($api->compareModules($d1, $d2))
   {
    $this->e->add("API compare modules should");
    return UT_FAILED;
   }

   $d2=Data::cache("users");
   if($api->compareModules($d1, $d2))
   {
    $this->e->add("API compare modules should");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function AddProjects()
  {
   $api=new API("server");

   $tmp["type"]="projects";
   $tmp["projects"]=Array();
   $tmp["activate"]=false;

   if(!$api->addProjects($tmp["activate"]))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API added projects command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="projects";
   $tmp["projects"]=Array();
   $tmp["activate"]="jas";

   if(!$api->addProjects($tmp["activate"], $tmp["projects"]))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[1]))
   {
    $this->e->add("API added projects command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[1], $tmp))
    return UT_FAILED;

   return UT_OK;
  }

  function SetProjects()
  {
   $api=new API("server");

   $tmp["type"]="projects";
   $tmp["projects"]=Array();
   $tmp["activate"]=false;

   if(!$api->setProjects(0, $tmp["activate"]))
   {
    $this->e->add("API set projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set projects command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;


   $tmp["type"]="projects";
   $tmp["projects"]=Array();
   $tmp["activate"]="jas";

   if(!$api->setProjects(0, $tmp["activate"], $tmp["projects"]))
   {
    $this->e->add("API set projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API set projects command not found");
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0], $tmp))
    return UT_FAILED;

   return UT_OK;
  }

  function DeleteCommand()
  {
   $api=new API("server");

   if(!$api->setModules(0))
   {
    $this->e->add("API set modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->setProjects(1))
   {
    $this->e->add("API set projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->setLanguage(2, false, false))
   {
    $this->e->add("API set language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->deleteCommand(1))
   {
    $this->e->add("API delete command failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("API first commands expected to be modules type");
    return UT_FAILED;
   }

   if($api->commands[1]["type"]!="language")
   {
    $this->e->add("API second commands expected to be language type");
    return UT_FAILED;
   }

   if(!$api->deleteCommand(1))
   {
    $this->e->add("API delete command failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("API first commands expected to be modules type");
    return UT_FAILED;
   }

   if(!$api->deleteCommand(0))
   {
    $this->e->add("API delete command failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->commands)
   {
    $this->e->add("API commands should be empty");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Command()
  {
   $api=new API_XML("server", $this->_config(true)); //API enabled

   if(!$api->addModules())
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->addProjects())
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!isset($api->commands[0]))
   {
    $this->e->add("API command not found");
    return UT_FAILED;
   }

   if(!isset($api->commands[1]))
   {
    $this->e->add("API command not found");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("API not a modules command");
    return UT_FAILED;
   }

   if($api->commands[1]["type"]!="projects")
   {
    $this->e->add("API not a projects command");
    return UT_FAILED;
   }

   if(!$api->commands[0]["modules"])
   {
    $this->e->add("API command no modules gathered");
    return UT_FAILED;
   }

   if(!$api->commands[1]["projects"])
   {
    $this->e->add("API command no projects gathered");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Command_Data()
  {
   global $_session;

   $api=new API("server");

   $d=Data::cache("categories", "view", "item", Array("db" => "session"));
   $d->setValue("nr", 1);

   if(!$api->addData(clone $d, false))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$d->get(0, false))
   {
    $this->e->add("Data get failed");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$api->command_data(0))
   {
    $this->e->add("API command data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0]["data"], $d))
    return UT_FAILED;


   $api->clear();
   $_session->set("sessionStorage.data.unit_testing_testmodule"); //ensure no values in DB

   $d=Data::cache("unit_testing_testmodule", "add", "item", Array("db" => "session"));
   $d->setValue("name", "test");
   $d->setValue("catnr", 1);

   if(!$api->addData(clone $d, false))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$d->submit())
   {
    $this->e->add("Data submit failed");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $_session->set("sessionStorage.data.unit_testing_testmodule"); //ensure no values in DB

   if(!$api->command_data(0))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API command data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0]["data"], $d))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    return UT_FAILED;
   }

   $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
   return UT_OK;
  }

  function Command_Language()
  {
   $api=new API("server");

   $l=new Language();

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$l->read("modules", "nl"))
   {
    $this->e->add("Language read failed");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$api->command_language(0))
   {
    $this->e->add("API command language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0]["language"], $l))
    return UT_FAILED;

   return UT_OK;
  }

  function Command_Modules()
  {
   $api=new API("server");

   $modules=JAS::modules();

   if(!$api->addModules(Array()))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->command_modules(0))
   {
    $this->e->add("API command modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   //remove *:* modules
   $justModules=Array();
   foreach($api->commands[0]["modules"] as $module)
    if(!strpos($module, ":"))
     $justModules[]=$module;

   if(!$this->_compare($justModules, $modules))
    return UT_FAILED;

   //initialized modules
   $api->clear();
   $modules=Array(Data::cache("categories", "view", "item", Array("skipCheckTables" => true)));

   if(!$api->addModules(Array("categories")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->command_modules(0))
   {
    $this->e->add("API command modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0]["modules"], $modules))
    return UT_FAILED;

   return UT_OK;
  }

  function Command_Projects()
  {
   $api=new API("server");

   $projects=JAS::projects();

   if(!$api->addProjects())
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->command_projects(0))
   {
    $this->e->add("API command projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($api->commands[0]["projects"], $projects))
    return UT_FAILED;

   return UT_OK;
  }

 }

?>
