<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/mutilate.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."storage/storage.php");

 class UT_Base_Data extends UT_Classes
 {
  function _init()
  {
   global $_path;

   $this->sqlValueTests=Array(
    "html" => Array(
     "normal" => Array(
      "text" => "<html><head><title>Title</title></head></html>",
      "html" => "<html><head><title>Title</title></head></html>",
      "sql" => "<html><head><title>Title</title></head></html>",
     ),
     "script" => Array(
      "text" => "<html><head><script>somefunction();</script></head></html>",
      "html" => "<html><head><script>somefunction();</script></head></html>",
      "sql" => "<html><head></head></html>",
     ),
    ),
    "text" => Array(
     "normal" => Array(
      "text" => "Some text",
      "html" => "Some text",
      "sql" => "Some text",
     ),
     "entities" => Array(
      "text" => "Some text &amp; &quot;more&apos;",
      "html" => "Some text &amp; &quot;more&apos;",
      "sql" => "Some text & \"more'",
     ),
    ),
   );


   $this->setValuesTests=Array(
    "html" => Array(
     "normal" => Array(
      "text" => "<html><head><title>Title</title></head></html>",
      "html" => "<html><head><title>Title</title></head></html>",
      "sql" => "<html><head><title>Title</title></head></html>",
     ),
     "url" => Array(
      "text" => "<html><body><a href=\"http://example\">Example</a></body></html>",
      "html" => "<html><body><a href=\"http://example\">Example</a></body></html>",
      "sql" => "<html><body><a href=\"http://example\">Example</a></body></html>",
     ),
     "files url" => Array(
      "text" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/something/\">Example</a></body></html>",
      "html" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/something/\">Example</a></body></html>",
      "sql" => "<html><body><a href=\"/files/something/\">Example</a></body></html>",
     ),
     "images url" => Array(
      "text" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/something/\"/></body></html>",
      "html" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/something/\"/></body></html>",
      "sql" => "<html><body><img src=\"images/something/\"/></body></html>",
     ),
     "files url args" => Array(
      "text" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/?m=files&nr=123\">Example</a></body></html>",
      "html" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/?m=files&nr=123\">Example</a></body></html>",
      "sql" => "<html><body><a href=\"/jas/files/?m=files&nr=123\">Example</a></body></html>",
     ),
     "images url args" => Array(
      "text" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/?m=images&nr=123\"/></body></html>",
      "html" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/?m=images&nr=123\"/></body></html>",
      "sql" => "<html><body><img src=\"/images?m=images&nr=123\"/></body></html>",
     ),
     "url files local" => Array(
      "text" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/something/\">Example</a></body></html>",
      "html" => "<html><body><a href=\"".$_path["HTML"]["root"]."files/something/\">Example</a></body></html>",
      "sql" => "<html><body><a href=\"http://".$_SERVER["HTTP_HOST"]."/files/something/\">Example</a></body></html>",
     ),
     "images url local" => Array(
      "text" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/something/\"/></body></html>",
      "html" => "<html><body><img src=\"".$_path["HTML"]["root"]."images/something/\"/></body></html>",
      "sql" => "<html><body><img src=\"https://".$_SERVER["HTTP_HOST"]."/images/something/\"/></body></html>",
     ),
     "files url remote" => Array(
      "text" => "<html><body><a href=\"http://example/files/something/\">Example</a></body></html>",
      "html" => "<html><body><a href=\"http://example/files/something/\">Example</a></body></html>",
      "sql" => "<html><body><a href=\"http://example/files/something/\">Example</a></body></html>",
     ),
     "images url remote" => Array(
      "text" => "<html><body><img src=\"https://example/images/something/\"/></body></html>",
      "html" => "<html><body><img src=\"https://example/images/something/\"/></body></html>",
      "sql" => "<html><body><img src=\"https://example/images/something/\"/></body></html>",
     ),
    ),
    "text" => Array(
     "normal" => Array(
      "text" => "Some text",
      "html" => "Some text",
      "sql" => "Some text",
     ),
     "entities" => Array(
      "html" => "Some text &amp; &quot;more'",
      "text" => "Some text & \"more'",
      "sql" => "Some text & \"more'",
     ),
    ),
   );
  }

  function Constructor()
  {
   $object=new BaseData();

   if(!is_object($object)||get_class($object)!="BaseData")
   {
    $this->e->add("Constructor doesn't return a BaseData object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new BaseData();
   $orig->init();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("BaseData object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->format=Array();
   if($orig->format==$copy->format)
   {
    $this->e->add("BaseData object copy also modified original \"format\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("BaseData object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("BaseData object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("BaseData object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("BaseData object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   //error
   $orig=new BaseData();
   $orig->init();
   $orig->e->add("test 1");
   $orig->e->add("test 2");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("BaseData object copy doesn't match original", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Title()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));

   if($d->title()!="Category")
   {
    $this->e->add("Module title doesn't match", "error");
    return UT_FAILED;
   }

   if($d->title("nr")!="#")
   {
    $this->e->add("Module nr field title doesn't match", "error");
    return UT_FAILED;
   }

   if($d->title("name")!="Name")
   {
    $this->e->add("Module name field title doesn't match", "error");
    return UT_FAILED;
   }

   if($d->title(false, true)!=$d->l->t("Category"))
   {
    $this->e->add("Module title doesn't match", "error");
    return UT_FAILED;
   }

   if($d->title("nr", true)!="#")
   {
    $this->e->add("Module nr field title doesn't match", "error");
    return UT_FAILED;
   }

   if($d->title("name", true)!=$d->l->t("Name"))
   {
    $this->e->add("Module name field title doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function MapValue()
  {
   global $_config;

   $d=new Data("empty");

//usernr
   $d->addField("usernr", "nr");
   $d->field["usernr"]["value"]=Array("usernr", "-1", "usernr");

   if(!$d->mapValue("usernr"))
   {
    $this->e->add("Map value failed for usernr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["usernr"]["value"]!=Array($_config->get("usernr"), "-1", "usernr"))
   {
    $this->e->add("Map value incorrect for usernr, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("usernr", 1))
   {
    $this->e->add("Map value failed for usernr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["usernr"]["value"]!=Array($_config->get("usernr"), "-1", "usernr"))
   {
    $this->e->add("Map value incorrect for usernr, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("usernr", 2))
   {
    $this->e->add("Map value failed for usernr, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["usernr"]["value"]!=Array($_config->get("usernr"), "-1", $_config->get("usernr")))
   {
    $this->e->add("Map value incorrect for usernr, entry: 2", "error");
    return UT_FAILED;
   }


//rootnr
   $d->addField("rootnr", "nr");
   $d->field["rootnr"]["value"]=Array("rootnr", "-1", "rootnr");

   if(!$d->mapValue("rootnr"))
   {
    $this->e->add("Map value failed for rootnr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["rootnr"]["value"]!=Array($_config->get("rootnr"), "-1", "rootnr"))
   {
    $this->e->add("Map value incorrect for rootnr, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("rootnr", 1))
   {
    $this->e->add("Map value failed for rootnr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["rootnr"]["value"]!=Array($_config->get("rootnr"), "-1", "rootnr"))
   {
    $this->e->add("Map value incorrect for rootnr, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("rootnr", 2))
   {
    $this->e->add("Map value failed for rootnr, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["rootnr"]["value"]!=Array($_config->get("rootnr"), "-1", $_config->get("rootnr")))
   {
    $this->e->add("Map value incorrect for rootnr, entry: 2", "error");
    return UT_FAILED;
   }


//ip
   $d->addField("ip", "ip");
   $d->field["ip"]["value"]=Array("ip", "255.255.255.255", "ip");

   if(!$d->mapValue("ip"))
   {
    $this->e->add("Map value failed for ip, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["ip"]["value"]!=Array($_SERVER["REMOTE_ADDR"], "255.255.255.255", "ip"))
   {
    $this->e->add("Map value incorrect for ip, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("ip", 1))
   {
    $this->e->add("Map value failed for ip, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["ip"]["value"]!=Array($_SERVER["REMOTE_ADDR"], "255.255.255.255", "ip"))
   {
    $this->e->add("Map value incorrect for ip, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("ip", 2))
   {
    $this->e->add("Map value failed for ip, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["ip"]["value"]!=Array($_SERVER["REMOTE_ADDR"], "255.255.255.255", $_SERVER["REMOTE_ADDR"]))
   {
    $this->e->add("Map value incorrect for ip, entry: 2", "error");
    return UT_FAILED;
   }


//date
   $d->addField("date", "date");
   $date=date($d->field["date"]["format"][$d->dataformat]["dateformat"], 0);
   $d->field["date"]["value"]=Array("now", $date, "now-86400");

   if(!$d->mapValue("date"))
   {
    $this->e->add("Map value failed for date, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["value"]!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), $date, "now-86400"))
   {
    $this->e->add("Map value incorrect for date, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("date", 1))
   {
    $this->e->add("Map value failed for date, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["value"]!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), $date, "now-86400"))
   {
    $this->e->add("Map value incorrect for date, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("date", 2))
   {
    $this->e->add("Map value failed for date, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["value"]!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), $date, date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400)))
   {
    $this->e->add("Map value incorrect for date, entry: 2", "error");
    return UT_FAILED;
   }


//datetime
   $d->addField("datetime", "datetime");
   $datetime=date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], 0);
   $d->field["datetime"]["value"]=Array("now", $datetime, "now-86520");

   if(!$d->mapValue("datetime"))
   {
    $this->e->add("Map value failed for datetime, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["datetime"]["value"]!=Array(date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"]), $datetime, "now-86520"))
   {
    $this->e->add("Map value incorrect for datetime, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("datetime", 1))
   {
    $this->e->add("Map value failed for datetime, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["datetime"]["value"]!=Array(date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"]), $datetime, "now-86520"))
   {
    $this->e->add("Map value incorrect for datetime, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("datetime", 2))
   {
    $this->e->add("Map value failed for datetime, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["datetime"]["value"]!=Array(date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"]), $datetime, date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], time()-86520)))
   {
    $this->e->add("Map value incorrect for datetime, entry: 2", "error");
    return UT_FAILED;
   }


//time
   $d->addField("time", "time");
   $time=date($d->field["time"]["format"][$d->dataformat]["timeformat"], 0);
   $d->field["time"]["value"]=Array("now", $time, "now-120");

   if(!$d->mapValue("time"))
   {
    $this->e->add("Map value failed for time, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array(date($d->field["time"]["format"][$d->dataformat]["timeformat"]), $time, "now-120"))
   {
    $this->e->add("Map value incorrect for time, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("time", 1))
   {
    $this->e->add("Map value failed for time, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array(date($d->field["time"]["format"][$d->dataformat]["timeformat"]), $time, "now-120"))
   {
    $this->e->add("Map value incorrect for time, entry: 1", "error");
    return UT_FAILED;
   }

   if(!$d->mapValue("time", 2))
   {
    $this->e->add("Map value failed for time, entry: 2", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array(date($d->field["time"]["format"][$d->dataformat]["timeformat"]), $time, date($d->field["time"]["format"][$d->dataformat]["timeformat"], time()-120)))
   {
    $this->e->add("Map value incorrect for time, entry: 2", "error");
    return UT_FAILED;
   }


//failures
   if($d->mapValue("doesntexist")!==false)
   {
    $this->e->add("Map value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   $d->addField("data", "data");
   if($d->mapValue("data")!==false)
   {
    $this->e->add("Map value should have failed for data", "error");
    return UT_FAILED;
   }

   $d->addField("new", "nr");
   if($d->mapValue("new", 123)!==false)
   {
    $this->e->add("Map value should have failed for new, entry: 123", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MapValues()
  {
   global $_config;

   $d=new Data("empty");

//usernr
   $d->addField("usernr", "nr");
   $d->field["usernr"]["value"]=Array("usernr", "-1", "usernr");

   if(!$d->mapValues("usernr"))
   {
    $this->e->add("Map values failed for usernr", "error");
    return UT_FAILED;
   }
   if($d->field["usernr"]["value"]!=Array($_config->get("usernr"), "-1", $_config->get("usernr")))
   {
    $this->e->add("Map values incorrect for usernr", "error");
    return UT_FAILED;
   }


//rootnr
   $d->addField("rootnr", "nr");
   $d->field["rootnr"]["value"]=Array("rootnr", "-1", "rootnr");

   if(!$d->mapValues("rootnr"))
   {
    $this->e->add("Map values failed for rootnr", "error");
    return UT_FAILED;
   }
   if($d->field["rootnr"]["value"]!=Array($_config->get("rootnr"), "-1", $_config->get("rootnr")))
   {
    $this->e->add("Map values incorrect for rootnr", "error");
    return UT_FAILED;
   }


//ip
   $d->addField("ip", "ip");
   $d->field["ip"]["value"]=Array("ip", "255.255.255.255", "ip");

   if(!$d->mapValues("ip"))
   {
    $this->e->add("Map values failed for ip", "error");
    return UT_FAILED;
   }
   if($d->field["ip"]["value"]!=Array($_SERVER["REMOTE_ADDR"], "255.255.255.255", $_SERVER["REMOTE_ADDR"]))
   {
    $this->e->add("Map values incorrect for ip", "error");
    return UT_FAILED;
   }


//date
   $d->addField("date", "date");
   $date=date($d->field["date"]["format"][$d->dataformat]["dateformat"], 0);
   $d->field["date"]["value"]=Array("now", $date, "now-86400");

   if(!$d->mapValues("date"))
   {
    $this->e->add("Map values failed for date", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["value"]!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), $date, date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400)))
   {
    $this->e->add("Map values incorrect for date", "error");
    return UT_FAILED;
   }


//datetime
   $d->addField("datetime", "datetime");
   $datetime=date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], 0);
   $d->field["datetime"]["value"]=Array("now", $datetime, "now-86520");

   if(!$d->mapValues("datetime"))
   {
    $this->e->add("Map values failed for datetime", "error");
    return UT_FAILED;
   }
   if($d->field["datetime"]["value"]!=Array(date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"]), $datetime, date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], time()-86520)))
   {
    $this->e->add("Map values incorrect for datetime", "error");
    return UT_FAILED;
   }


//time
   $d->addField("time", "time");
   $time=date($d->field["time"]["format"][$d->dataformat]["timeformat"], 0);
   $d->field["time"]["value"]=Array("now", $time, "now-120");

   if(!$d->mapValues("time"))
   {
    $this->e->add("Map values failed for time", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array(date($d->field["time"]["format"][$d->dataformat]["timeformat"]), $time, date($d->field["time"]["format"][$d->dataformat]["timeformat"], time()-120)))
   {
    $this->e->add("Map values incorrect for time", "error");
    return UT_FAILED;
   }


//all
   $d->field["usernr"]["value"]=Array("usernr", "-1", "usernr");
   $d->field["rootnr"]["value"]=Array("rootnr", "-1", "rootnr");

   $d->field["ip"]["value"]=Array("ip", "255.255.255.255", "ip");

   $date=date($d->field["date"]["format"][$d->dataformat]["dateformat"], 0);
   $d->field["date"]["value"]=Array("now", $date, "now-86400");
   $datetime=date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], 0);
   $d->field["datetime"]["value"]=Array("now", $datetime, "now-86520");
   $time=date($d->field["time"]["format"][$d->dataformat]["timeformat"], 0);
   $d->field["time"]["value"]=Array("now", $time, "now-120");

   if(!$d->mapValues())
   {
    $this->e->add("Map values failed", "error");
    return UT_FAILED;
   }

   if($d->field["usernr"]["value"]!=Array($_config->get("usernr"), "-1", $_config->get("usernr")))
   {
    $this->e->add("Map values incorrect for usernr", "error");
    return UT_FAILED;
   }
   if($d->field["rootnr"]["value"]!=Array($_config->get("rootnr"), "-1", $_config->get("rootnr")))
   {
    $this->e->add("Map values incorrect for rootnr", "error");
    return UT_FAILED;
   }

   if($d->field["ip"]["value"]!=Array($_SERVER["REMOTE_ADDR"], "255.255.255.255", $_SERVER["REMOTE_ADDR"]))
   {
    $this->e->add("Map values incorrect for ip", "error");
    return UT_FAILED;
   }

   if($d->field["date"]["value"]!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), $date, date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400)))
   {
    $this->e->add("Map values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->field["datetime"]["value"]!=Array(date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"]), $datetime, date($d->field["datetime"]["format"][$d->dataformat]["datetimeformat"], time()-86520)))
   {
    $this->e->add("Map values incorrect for datetime", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array(date($d->field["time"]["format"][$d->dataformat]["timeformat"]), $time, date($d->field["time"]["format"][$d->dataformat]["timeformat"], time()-120)))
   {
    $this->e->add("Map values incorrect for time", "error");
    return UT_FAILED;
   }


//failures
   if($d->mapValues("doesntexist")!==false)
   {
    $this->e->add("Map values should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   $d->addField("data", "data");
   if($d->mapValues("data")!==false)
   {
    $this->e->add("Map values should have failed for data", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->setValue("nr", 123))
   {
    $this->e->add("Set value failed for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(123))
   {
    $this->e->add("Set value incorrect for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="set")
   {
    $this->e->add("Set value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant))
   {
    $this->e->add("Set value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Set value vmax incorrect", "error");
    return UT_FAILED;
   }

   if(!$d->setValue("nr", "usernr", 1))
   {
    $this->e->add("Set value failed for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(123, $_config->get("usernr")))
   {
    $this->e->add("Set value incorrect for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="set")
   {
    $this->e->add("Set value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant, $d->defaultOperant))
   {
    $this->e->add("Set value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=2)
   {
    $this->e->add("Set value vmax incorrect", "error");
    return UT_FAILED;
   }

//failures
   if($d->setValue("doesntexist", 123)!==false)
   {
    $this->e->add("Set value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function UnsetValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->unsetValue("nr"))
   {
    $this->e->add("Unset value failed for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(false))
   {
    $this->e->add("Unset value incorrect for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="none")
   {
    $this->e->add("Unset value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant))
   {
    $this->e->add("Unset value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Unset value vmax incorrect", "error");
    return UT_FAILED;
   }

   if(!$d->unsetValue("nr", 1))
   {
    $this->e->add("Unset value failed for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(false, false))
   {
    $this->e->add("Unset value incorrect for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="none")
   {
    $this->e->add("Unset value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant, $d->defaultOperant))
   {
    $this->e->add("Unset value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Unset value vmax incorrect", "error");
    return UT_FAILED;
   }

//failures
   if($d->unsetValue("doesntexist")!==false)
   {
    $this->e->add("Unset value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetDefaultValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr", false, false, false, false, 123);

   if(!$d->setDefaultValue("nr"))
   {
    $this->e->add("Set default value failed for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(123))
   {
    $this->e->add("Set default value incorrect for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="default")
   {
    $this->e->add("Set default value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant))
   {
    $this->e->add("Set default value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Set default value vmax incorrect", "error");
    return UT_FAILED;
   }

   $d->field["nr"]["defaultvalue"]="rootnr";
   if(!$d->setDefaultValue("nr", 1))
   {
    $this->e->add("Set default value failed for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array(123, $_config->get("rootnr")))
   {
    $this->e->add("Set default value incorrect for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="default")
   {
    $this->e->add("Set default value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array($d->defaultOperant, $d->defaultOperant))
   {
    $this->e->add("Set default value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=2)
   {
    $this->e->add("Set default value vmax incorrect", "error");
    return UT_FAILED;
   }

//failures
   if($d->setDefaultValue("doesntexist")!==false)
   {
    $this->e->add("Set default value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetDefaultValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr", false, false, false, false, 123);

   if($d->setDefaultValue("nr")!=123)
   {
    $this->e->add("Get default value failed for nr", "error");
    return UT_FAILED;
   }

   $d->field["nr"]["defaultvalue"]="rootnr";
   if($d->setDefaultValue("nr")!="rootnr")
   {
    $this->e->add("Get default value failed for nr", "error");
    return UT_FAILED;
   }

//failures
   if($d->setDefaultValue("doesntexist")!==false)
   {
    $this->e->add("Set default value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->setValue("nr", 123))
   {
    $this->e->add("Set value failed for nr, entry: 0", "error");
    return UT_FAILED;
   }
   if($d->getValue("nr")!=123)
   {
    $this->e->add("Get value incorrect for nr, entry: 0", "error");
    return UT_FAILED;
   }

   if(!$d->setValue("nr", "usernr", 1))
   {
    $this->e->add("Set value failed for nr, entry: 1", "error");
    return UT_FAILED;
   }
   if($d->getValue("nr", 1)!=$_config->get("usernr"))
   {
    $this->e->add("Get value incorrect for nr, entry: 1", "error");
    return UT_FAILED;
   }

//failures
   if($d->getValue("doesntexist")!==false)
   {
    $this->e->add("Get value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetValues()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->setValue("nr", 123))
   {
    $this->e->add("Set value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }

   if(!$d->setValue("nr", "usernr", 1))
   {
    $this->e->add("Set value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, $_config->get("usernr")))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }

//failures
   if($d->getValues("doesntexist")!==false)
   {
    $this->e->add("Get values should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetValueType()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if($d->getValueType("nr")!="none")
   {
    $this->e->add("Get value type incorrect for nr", "error");
    return UT_FAILED;
   }

   if(!$d->setValue("nr", 123))
   {
    $this->e->add("Set value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->getValueType("nr")!="set")
   {
    $this->e->add("Get value type incorrect for nr", "error");
    return UT_FAILED;
   }

   if(!$d->setDefaultValue("nr"))
   {
    $this->e->add("Set default value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->getValueType("nr")!="default")
   {
    $this->e->add("Get value type incorrect for nr", "error");
    return UT_FAILED;
   }

   if(!$d->unsetValue("nr"))
   {
    $this->e->add("Unset value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->getValueType("nr")!="none")
   {
    $this->e->add("Get value type incorrect for nr", "error");
    return UT_FAILED;
   }

//failures
   if($d->getValueType("doesntexist")!==false)
   {
    $this->e->add("Get value type should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SearchValue()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->setValue("nr", 123))
   {
    $this->e->add("Set value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->searchValue("nr", 123)!==0)
   {
    $this->e->add("Search value incorrect for nr", "error");
    return UT_FAILED;
   }

   if(!$d->setValue("nr", "usernr", 1))
   {
    $this->e->add("Set value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->searchValue("nr", $_config->get("usernr"))!==1)
   {
    $this->e->add("Search value incorrect for nr", "error");
    return UT_FAILED;
   }

//failures
   if($d->searchValue("doesntexist", 1)!==false)
   {
    $this->e->add("Search value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   $d->addField("new", "nr");
   if($d->searchValue("new", 123)!==false)
   {
    $this->e->add("Search value should have failed for non existing value", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ClearValues()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr");

   if(!$d->clearValues("nr"))
   {
    $this->e->add("Clear value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array())
   {
    $this->e->add("Clear value incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="none")
   {
    $this->e->add("Clear value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array())
   {
    $this->e->add("Clear value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Clear value vmax incorrect", "error");
    return UT_FAILED;
   }

   $d->addField("date", "date");
   $d->addField("time", "time");

   if(!$d->clearValues())
   {
    $this->e->add("Clear value failed for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["value"]!=Array())
   {
    $this->e->add("Clear value incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["valuetype"]!="none")
   {
    $this->e->add("Clear value valuetype incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["nr"]["operant"]!=Array())
   {
    $this->e->add("Clear value defaultOperant incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["value"]!=Array())
   {
    $this->e->add("Clear value incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["valuetype"]!="none")
   {
    $this->e->add("Clear value valuetype incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->field["date"]["operant"]!=Array())
   {
    $this->e->add("Clear value defaultOperant incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["value"]!=Array())
   {
    $this->e->add("Clear value incorrect for time", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["valuetype"]!="none")
   {
    $this->e->add("Clear value valuetype incorrect for time", "error");
    return UT_FAILED;
   }
   if($d->field["time"]["operant"]!=Array())
   {
    $this->e->add("Clear value defaultOperant incorrect for time", "error");
    return UT_FAILED;
   }
   if($d->vmax!=0)
   {
    $this->e->add("Clear value vmax incorrect", "error");
    return UT_FAILED;
   }

//failures
   if($d->unsetValue("doesntexist")!==false)
   {
    $this->e->add("Clear value should have failed for doesntexist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function AddEntry()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr", "sql", false, false, false, 123);
   $d->addField("float", "float", "", false, false, false, 123.456);
   $d->addField("date", "date", "all", false, false, false, "now");
   $d->addField("text", "text", "show", false, false, false, "some text");

   if(!$d->clearValues())
   {
    $this->e->add("Clear value failed", "error");
    return UT_FAILED;
   }

   if(!$d->addEntry())
   {
    $this->e->add("Added entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"])))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Add entry vmax incorrect", "error");
    return UT_FAILED;
   }


   if(!$d->addEntry())
   {
    $this->e->add("Added entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"])))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text", "some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=2)
   {
    $this->e->add("Add entry vmax incorrect", "error");
    return UT_FAILED;
   }


   $d->field["nr"]["defaultvalue"]="rootnr";
   $d->field["date"]["defaultvalue"]="now-86400";
   $d->field["text"]["defaultvalue"]="some other text";

   if(!$d->addEntry(1))
   {
    $this->e->add("Added entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, $_config->get("rootnr"), 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400), date($d->field["date"]["format"][$d->dataformat]["dateformat"])))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text", "some other text", "some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=3)
   {
    $this->e->add("Add entry vmax incorrect", "error");
    return UT_FAILED;
   }


   $d->field["nr"]["defaultvalue"]="456";
   $d->field["date"]["defaultvalue"]="now+86400";
   $d->field["text"]["defaultvalue"]="some more text";

   if(!$d->addEntry(3))
   {
    $this->e->add("Added entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, $_config->get("rootnr"), 123, 456))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400), date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()+86400)))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text", "some other text", "some text", "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=4)
   {
    $this->e->add("Add entry vmax incorrect", "error");
    return UT_FAILED;
   }


   $d->field["nr"]["defaultvalue"]="789";
   $d->field["date"]["defaultvalue"]="now+172800";
   $d->field["text"]["defaultvalue"]="some less text";
   $d->setValue("float", 123.456, 1);

   if(!$d->addEntry(0))
   {
    $this->e->add("Added entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(789, 123, $_config->get("rootnr"), 123, 456))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array(1 => 123.456))
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()+172800), date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()-86400), date($d->field["date"]["format"][$d->dataformat]["dateformat"]), date($d->field["date"]["format"][$d->dataformat]["dateformat"], time()+86400)))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some less text", "some text", "some other text", "some text", "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=5)
   {
    $this->e->add("Add entry vmax incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CopyEntry()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr", "sql", false, false, false, 123);
   $d->addField("float", "float", "");
   $d->addField("date", "date", "all", false, false, false, "now");
   $d->addField("text", "text", "show", false, false, false, "some text");

   if(!$d->clearValues("float"))
   {
    $this->e->add("Clear value failed for float", "error");
    return UT_FAILED;
   }

   if(!$d->copyEntry(0))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("now", "now"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text", "some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=2)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }


   $d->setValue("nr", 456, 1);
   $d->setValue("date", "23-10-2015", 1);
   $d->setValue("text", "some more text", 1);

   if(!$d->copyEntry(1, 0))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(456, 123, 456))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("23-10-2015", "now", "23-10-2015"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some more text", "some text", "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=3)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }


   $d->setValue("float", 123.456, 1);

   if(!$d->copyEntry(1, 3))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(456, 123, 456, 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array(1 => 123.456))
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("23-10-2015", "now", "23-10-2015", "now"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some more text", "some text", "some more text", "some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=4)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }


   unset($d->field["nr"]["value"][2]);

   if(!$d->copyEntry(2, 123))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(0 => 456, 1 => 123, 3 => 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array(1 => 123.456))
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("23-10-2015", "now", "23-10-2015", "now", "23-10-2015"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some more text", "some text", "some more text", "some text", "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=5)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }

//no entries to copy
   unset($d->field["r"]["value"][2]);
   unset($d->field["nr"]["value"][2]);
   unset($d->field["date"]["value"][2]);
   unset($d->field["text"]["value"][2]);

   if($d->copyEntry(2))
   {
    $this->e->add("Copy entry should have failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(0 => 456, 1 => 123, 3 => 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array(1 => 123.456))
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(0 => "23-10-2015", 1 => "now", 3 => "now", 4 => "23-10-2015"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array(0 => "some more text", 1 => "some text", 3 => "some text", 4 => "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=5)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }


   if($d->copyEntry(123))
   {
    $this->e->add("Copy entry should have failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(0 => 456, 1 => 123, 3 => 123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array(1 => 123.456))
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array(0 => "23-10-2015", 1 => "now", 3 => "now", 4 => "23-10-2015"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array(0 => "some more text", 1 => "some text", 3 => "some text", 4 => "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=5)
   {
    $this->e->add("Copy entry vmax incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function RemoveEntry()
  {
   global $_config;

   $d=new Data("empty");

   $d->addField("nr", "nr", "sql", false, false, false, 123);
   $d->addField("float", "float", "");
   $d->addField("date", "date", "all", false, false, false, "now");
   $d->addField("text", "text", "show", false, false, false, "some text");

   if(!$d->copyEntry(0))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }

   $d->setValue("nr", 456, 1);
   $d->setValue("date", "23-10-2015", 1);
   $d->setValue("text", "some more text", 1);

   if(!$d->copyEntry(1, 0))
   {
    $this->e->add("Copy entry failed", "error");
    return UT_FAILED;
   }

   if(!$d->removeEntry(0))
   {
    $this->e->add("Remove entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123, 456))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("now", "23-10-2015"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text", "some more text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=2)
   {
    $this->e->add("Remove entry vmax incorrect", "error");
    return UT_FAILED;
   }


   if(!$d->removeEntry(1))
   {
    $this->e->add("Remove entry failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("now"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Remove entry vmax incorrect", "error");
    return UT_FAILED;
   }


   if($d->removeEntry(123))
   {
    $this->e->add("Remove entry should have failed", "error");
    return UT_FAILED;
   }
   if($d->getValues("nr")!=Array(123))
   {
    $this->e->add("Get values incorrect for nr", "error");
    return UT_FAILED;
   }
   if($d->getValues("float")!=Array())
   {
    $this->e->add("Get value incorrect for float", "error");
    return UT_FAILED;
   }
   if($d->getValues("date")!=Array("now"))
   {
    $this->e->add("Get values incorrect for date", "error");
    return UT_FAILED;
   }
   if($d->getValues("text")!=Array("some text"))
   {
    $this->e->add("Get values incorrect for text", "error");
    return UT_FAILED;
   }
   if($d->vmax!=1)
   {
    $this->e->add("Remove entry vmax incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Values()
  {
   $d=new Data("empty");

   if($d->values()!=1)
   {
    $this->e->add("Values number incorrect", "error");
    return UT_FAILED;
   }

   $d->clearValues();
   if($d->values()!=0)
   {
    $this->e->add("Values number incorrect", "error");
    return UT_FAILED;
   }

   $d->addEntry();
   $d->addEntry();
   if($d->values()!=2)
   {
    $this->e->add("Values number incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function RegExp_Project()
  {
   foreach(Array("project", "project2", "longprojectnamebutstillok", "1234567890") as $project)
   {
    if(!preg_match(REGEXP_DATA_PROJECT, $project))
    {
     $this->e->add("Project regexp shouldn't have failed on: \"".$project."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("project name", "%&34237y347", "waytoolonglongprojectnameiguess", "") as $project)
   {
    if(preg_match(REGEXP_DATA_PROJECT, $project))
    {
     $this->e->add("Project regexp should have failed on: \"".$project."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Module()
  {
   foreach(Array("module", "longmodulenamebutstillok") as $module)
   {
    if(!preg_match(REGEXP_DATA_MODULE, $module))
    {
     $this->e->add("Module regexp shouldn't have failed on: \"".$module."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("module name", "1234567890", "%&34237y347", "waytoolonglongmodulenameiguess", "") as $module)
   {
    if(preg_match(REGEXP_DATA_MODULE, $module))
    {
     $this->e->add("Module regexp should have failed on: \"".$module."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Plugin()
  {
   foreach(Array("plugin", "plugin2", "plugin_3", "longpluginnamebutstillok", "1234567890", ".tmp") as $plugin)
   {
    if(!preg_match(REGEXP_DATA_PLUGIN, $plugin))
    {
     $this->e->add("Plugin regexp shouldn't have failed on: \"".$plugin."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("plugin name", "%&34237y347", "waytoolonglongpluginnameiguess", "") as $plugin)
   {
    if(preg_match(REGEXP_DATA_PLUGIN, $plugin))
    {
     $this->e->add("Plugin regexp should have failed on: \"".$plugin."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Plugin_Module()
  {
   foreach(Array("plugin_module", "plugin2_module", "plugin_3_module", "longpluginnamebutstillok_longmodulenamebutstillok", "1234567890_module") as $module)
   {
    if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $module))
    {
     $this->e->add("Plugin module regexp shouldn't have failed on: \"".$module."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("plugin name_module name", "%&34237y347_%&34237y347", "waytoolonglongpluginnameiguess_waytoolonglongmodulenameiguess", "plugin_module2", "module", "_", "") as $module)
   {
    if(preg_match(REGEXP_DATA_PLUGIN_MODULE, $module))
    {
     $this->e->add("Plugin module regexp should have failed on: \"".$module."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Action()
  {
   foreach(Array("action", "longactionnamebutstillok") as $action)
   {
    if(!preg_match(REGEXP_DATA_ACTION, $action))
    {
     $this->e->add("Action regexp shouldn't have failed on: \"".$action."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("action name", "1234567890", "%&34237y347", "waytoolonglongactionnameiguess", "") as $action)
   {
    if(preg_match(REGEXP_DATA_ACTION, $action))
    {
     $this->e->add("Action regexp should have failed on: \"".$action."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Extra()
  {
   foreach(Array("extra", "longextranamebutstillok") as $extra)
   {
    if(!preg_match(REGEXP_DATA_EXTRA, $extra))
    {
     $this->e->add("Extra regexp shouldn't have failed on: \"".$extra."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("extra name", "1234567890", "%&34237y347", "waytoolonglongextranameiguess", "") as $extra)
   {
    if(preg_match(REGEXP_DATA_EXTRA, $extra))
    {
     $this->e->add("Extra regexp should have failed on: \"".$extra."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Fieldname()
  {
   foreach(Array("fieldname", "fieldname2", "fieldname_3", "longfieldnamenamebutstillokatleastforthepurposeyes", "1234567890") as $fieldname)
   {
    if(!preg_match(REGEXP_DATA_FIELDNAME, $fieldname))
    {
     $this->e->add("Fieldname regexp shouldn't have failed on: \"".$fieldname."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("fieldname name", "%&34237y347", "waytoolonglongfieldnamenameiguessbutweretryingitanyway", "") as $fieldname)
   {
    if(preg_match(REGEXP_DATA_FIELDNAME, $fieldname))
    {
     $this->e->add("Fieldname regexp should have failed on: \"".$fieldname."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Enumoption()
  {
   foreach(Array("enumoption", "enum.option 2", "enum-option_3", "longenumoptionnamebutstillokatleastforthepurpose", "1234567890", "") as $enumoption)
   {
    if(!preg_match(REGEXP_DATA_ENUMOPTION, $enumoption))
    {
     $this->e->add("Enumoption regexp shouldn't have failed on: \"".$enumoption."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("enumoption%name", "%&34237y347", "waytoolonglongenumoptionnameiguessbutweretryingitanyway") as $enumoption)
   {
    if(preg_match(REGEXP_DATA_ENUMOPTION, $enumoption))
    {
     $this->e->add("Enumoption regexp should have failed on: \"".$enumoption."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Listoption()
  {
   foreach(Array("listoption", "list.option 2", "list-option_3", "longlistoptionnamebutstillokatleastforthepurpose", "1234567890", "") as $listoption)
   {
    if(!preg_match(REGEXP_DATA_LISTOPTION, $listoption))
    {
     $this->e->add("Listoption regexp shouldn't have failed on: \"".$listoption."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("listoption\"name", "list;option") as $listoption)
   {
    if(preg_match(REGEXP_DATA_LISTOPTION, $listoption))
    {
     $this->e->add("Listoption regexp should have failed on: \"".$listoption."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Hostname()
  {
   foreach(Array("hostname", "host2.name", "host-name3.com", "host1_name.com") as $hostname)
   {
    if(!preg_match(REGEXP_DATA_HOSTNAME, $hostname))
    {
     $this->e->add("Hostname regexp shouldn't have failed on: \"".$hostname."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("http://hostname", "%&34237y347", "") as $hostname)
   {
    if(preg_match(REGEXP_DATA_HOSTNAME, $hostname))
    {
     $this->e->add("Hostname regexp should have failed on: \"".$hostname."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Domain()
  {
   foreach(Array("domain.com", "domain2.com", "domain-name3.com", "domain1_name.com", "domain.name.com") as $domain)
   {
    if(!preg_match(REGEXP_DATA_DOMAIN, $domain))
    {
     $this->e->add("Domain regexp shouldn't have failed on: \"".$domain."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("http://domain", "%&34237y347", "domain", "domain.", ".domain", "") as $domain)
   {
    if(preg_match(REGEXP_DATA_DOMAIN, $domain))
    {
     $this->e->add("Domain regexp should have failed on: \"".$domain."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_Email()
  {
   foreach(Array("email@address.com", "email2.address@domain.name.com", "email3-address@domain-name3.com", "email1_address@domain_name.com") as $email)
   {
    if(!preg_match(REGEXP_DATA_EMAIL, $email))
    {
     $this->e->add("Email regexp shouldn't have failed on: \"".$email."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("http://email@address.com", "%&34237y347", "email@address", "@address.com", "email@", "") as $email)
   {
    if(preg_match(REGEXP_DATA_EMAIL, $email))
    {
     $this->e->add("Email regexp should have failed on: \"".$email."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_URL()
  {
   //use URLs from Mutilations unit tests
   $ut=new UT_Mutilate();
   $ut->_init();

   foreach($ut->urlTests as $url)
   {
    if(preg_match("!^\w+://!", $url["encoded"])) //only full urls
    {
     if(!preg_match(REGEXP_DATA_URL, $url["encoded"]))
     {
      $this->e->add("URL regexp shouldn't have failed on: \"".$url["encoded"]."\"", "error");
      return UT_FAILED;
     }
    }
    else //half urls
    {
     if(preg_match(REGEXP_DATA_URL, $url["encoded"]))
     {
      $this->e->add("URL regexp should have failed on: \"".$url["encoded"]."\"", "error");
      return UT_FAILED;
     }
    }
   }

   foreach(Array("%&34237y347", "htp:/url", ".url", "") as $url)
   {
    if(preg_match(REGEXP_DATA_URL, $url))
    {
     $this->e->add("URL regexp should have failed on: \"".$url."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_IPV4()
  {
   foreach(Array("0.0.0.0", "1.2.3.4", "123.234.123.234", "192.168.0.1", "127.0.0.1", "255.255.255.255") as $IPV4)
   {
    if(!preg_match(REGEXP_DATA_IPV4, $IPV4))
    {
     $this->e->add("IPV4 regexp shouldn't have failed on: \"".$IPV4."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("1.2.3.", "1234.5.6.7", "1.2.3.4.5", "") as $IPV4)
   {
    if(preg_match(REGEXP_DATA_IPV4, $IPV4))
    {
     $this->e->add("IPV4 regexp should have failed on: \"".$IPV4."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_IPV6()
  {
   foreach(Array("0::0", "d3ad:8e3f::1", "1234:5678:90ab:cdef:1234:5678:90ab:cdef", "0000:0000:0000:0000:0000:0000:0000:0001") as $IPV6)
   {
    if(!preg_match(REGEXP_DATA_IPV6, $IPV6))
    {
     $this->e->add("IPV6 regexp shouldn't have failed on: \"".$IPV6."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("1::", "12345678::abcdef", "1234:5678:90ab:cdef:1234:5678:90ab:cdef:1234:5678:90ab:cdef:1234:5678:90ab:cdef", "") as $IPV6)
   {
    if(preg_match(REGEXP_DATA_IPV6, $IPV6))
    {
     $this->e->add("IPV6 regexp should have failed on: \"".$IPV6."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function RegExp_IP()
  {
   foreach(Array("0.0.0.0", "1.2.3.4", "123.234.123.234", "192.168.0.1", "127.0.0.1", "255.255.255.255", "0::0", "d3ad:8e3f::1", "1234:5678:90ab:cdef:1234:5678:90ab:cdef", "0000:0000:0000:0000:0000:0000:0000:0001") as $IP)
   {
    if(!preg_match(REGEXP_DATA_IP, $IP))
    {
     $this->e->add("IP regexp shouldn't have failed on: \"".$IP."\"", "error");
     return UT_FAILED;
    }
   }

   foreach(Array("1.2.3.", "1234.5.6.7", "1.2.3.4.5", "1::", "12345678::abcdef", "1234:5678:90ab:cdef:1234:5678:90ab:cdef:1234:5678:90ab:cdef:1234:5678:90ab:cdef", "") as $IP)
   {
    if(preg_match(REGEXP_DATA_IP, $IP))
    {
     $this->e->add("IP regexp should have failed on: \"".$IP."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

 }



 class UT_Data extends UT_Base_Data
 {
  var $_module="unit_testing_testmodule";
  var $_db="default";
  var $_aggregationSetup=false;
  var $_jasPath=false;
  var $_utEnabled=false;
  var $_aggregationValues=Array(
    Array(
     "nr" => 1,
     "catnr" => 1,
     "name" => "A",
     "updated" => "01:00:00 01-01-2001",
    ),
    Array(
     "nr" => 2,
     "catnr" => 1,
     "name" => "B",
     "updated" => "02:00:00 01-01-2001",
    ),
    Array(
     "nr" => 3,
     "catnr" => 1,
     "name" => "C",
     "updated" => "03:00:00 01-01-2001",
    ),
    Array(
     "nr" => 4,
     "catnr" => 1,
     "name" => "D",
     "updated" => "04:00:00 01-01-2001",
    ),
    Array(
     "nr" => 5,
     "catnr" => 2,
     "name" => "A",
     "updated" => "01:00:00 01-01-2002",
    ),
    Array(
     "nr" => 6,
     "catnr" => 2,
     "name" => "B",
     "updated" => "01:30:00 01-01-2002",
    ),
    Array(
     "nr" => 7,
     "catnr" => 2,
     "name" => "C",
     "updated" => "02:00:00 01-01-2002",
    ),
    Array(
     "nr" => 8,
     "catnr" => 2,
     "name" => "D",
     "updated" => "02:30:00 01-01-2002",
    ),
  );


  function _setupAggregation()
  {
   global $_path, $_config;

   if($this->_aggregationSetup) //already setup
    return true; //done

   $this->_jasPath=$_path;

   //create unit_testing_testmodule table
   $d=Data::cache($this->_module, "database", "create", Array("db" => $this->_db, "skipCheckTables" => true));
   $d->clearValues();
   $d->e->clear();

   if(get_class($d->storageObject())!="System_Storage_mysql") //must by MySQL
   {
    $this->e->add("Setup aggregation failed, MySQL Storage project needed", "error");
    return false;
   }

   //create table
   $_config->su(true);
   if(!$d->tableExists()&&!$d->submit()) //create table
   {
    $_config->su();

    $this->e->add($d->e);
    return false;
   }
   $_config->su();


   $d->clearValues();
   $d->setAction("delete");
   $d->setExtra("all");

   if(!$d->get()) //get ids
   {
    $this->e->add($d->e);
    return false;
   }

   //delete entries
   if($d->vmax)
   {
    $_config->su(true);
    if(!$d->submit()) //create table
    {
     $_config->su();

     $this->e->add($d->e);
     return false;
    }
    $_config->su();
   }

   $d->clearValues();
   $d->setAction("add");
   $d->setExtra("item");

   $d->field["nr"]["properties"]=str_replace("identifier;", "", $d->field["nr"]["properties"]); //remove nr forced identifier
   $d->field["updated"]["properties"]=str_replace("forceUpdated;", "", $d->field["updated"]["properties"]); //remove updated forced updated
   $d->field["catnr"]["properties"]=str_replace("rightsReference;", "", $d->field["catnr"]["properties"]); //remove rights reference
   unset($d->reference["catnr"]); //remove catnr reference

   foreach($this->_aggregationValues as $c => $values)
    foreach($values as $key => $value)
     $d->setValue($key, $value, $c);

   //submit values
   $_config->su(true);
   if(!$d->submit()) //create table
   {
    $_config->su();

    $this->e->add($d->e);
    return false;
   }
   $_config->su();


   $this->_aggregationSetup=true;
   return true;
  }

  function _desetupAggregation()
  {
   global $_config;

   if(!$this->_aggregationSetup) //not setup
    return true; //done

   //clear values
   $d=Data::cache($this->_module, "delete", "all", Array("db" => $this->_db));
   $d->clearValues();
   $d->e->clear();

   $_config->su(true);
   if($d->get())
   {
    //submit values
    $d->submit(); //create
    $_config->su();
   }
   $_config->su();

   //desetup done
   $this->_aggregationSetup=false;

   return true;
  }




  function Constructor()
  {
   $object=new Data();

   if(!is_object($object)||get_class($object)!="Data")
   {
    $this->e->add("Constructor doesn't return a Data object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->l)||get_class($object->l)!="Language")
   {
    $this->e->add("Object: \"l\" isn't an Language object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Data("categories");
   $copy=clone $orig;

   $orig->moduleObject->data=false; //remove recursive dependency
   $copy->moduleObject->data=false;
   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Data object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $orig->moduleObject->data=false; //remove recursive dependency
   $copy->moduleObject->data=false;
   $orig->extends["notes"]->moduleObject->data=false;
   $copy->extends["notes"]->moduleObject->data=false;
   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Data object copy doesn't match original", "error");
    return UT_FAILED;
   }

   //error
   $orig=new Data("categories");
   $orig->e->add("test 1");
   $orig->e->add("test 2");
   $copy=clone $orig;

   $orig->moduleObject->data=false; //remove recursive dependency
   $copy->moduleObject->data=false;
   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Data object copy doesn't match original", "error");
    return UT_FAILED;
   }

   //dataformat
   $orig=new Data("categories");
   $copy=clone $orig;

   $copy->dataformat=-1;
   if($orig->dataformat==$copy->dataformat)
   {
    $this->e->add("Data object copy also modified original \"dataformat\" variable", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->dataformat=-1;
   if($orig->dataformat==$copy->dataformat)
   {
    $this->e->add("Data object copy also modified original \"dataformat\" variable", "error");
    return UT_FAILED;
   }

   //moduleObject
   $orig=new Data("categories");
   $copy=clone $orig;

   $orig->moduleObject->data=-2;
   $copy->moduleObject->data=-1;
   if($orig->moduleObject->data==$copy->moduleObject->data)
   {
    $this->e->add("Data object copy also modified original \"ModuleObject\" object", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $orig->moduleObject->data=-2;
   $copy->moduleObject->data=-1;
   if($orig->moduleObject->data==$copy->moduleObject->data)
   {
    $this->e->add("Data object copy also modified original \"ModuleObject\" object", "error");
    return UT_FAILED;
   }

   //fields
   $orig=new Data("users");
   $copy=clone $orig;

   $copy->field["nr"]["value"]=-1;
   if($orig->field==$copy->field)
   {
    $this->e->add("Data object copy also modified original \"nr field value\" variable", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->field["file"]["value"]=-1;
   if($orig->field==$copy->field)
   {
    $this->e->add("Data object copy also modified original \"file field value\" variable", "error");
    return UT_FAILED;
   }

   //references
   $orig=new Data("users");
   $copy=clone $orig;

   $copy->reference["catnr"]["from"]=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"catnr reference from\" variable", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->reference["catnr"]["from"]=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"catnr reference from\" variable", "error");
    return UT_FAILED;
   }

   //reference data
   $orig=new Data("users");
   $orig->initReferenceData("catnr");
   $copy=clone $orig;

   $copy->reference["catnr"]["data"]->dataformat=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"catnr reference data\" object", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $orig->initReferenceData("catnr");
   $copy=clone $orig;

   $copy->reference["catnr"]["data"]->dataformat=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"catnr reference data\" object", "error");
    return UT_FAILED;
   }

   //reference m2m
   $orig=new Data("users");
   $copy=clone $orig;

   $copy->reference["usersAdditionalGroups"]["data"]->dataformat=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"usersAdditionalGroups reference data\" object", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->reference["manytomany"]["data"]->dataformat=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"manytomany reference data\" object", "error");
    return UT_FAILED;
   }

   //reference manytoone referenced field check
   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->field["manytoone"]["value"]=-1;
   if($orig->field["manytoone"]==$copy->field["manytoone"])
   {
    $this->e->add("Data object copy also modified original \"manytoone\" field", "error");
    return UT_FAILED;
   }

   if($copy->field["manytoone"]["value"]!=$copy->field["manyto1"]["value"])
   {
    $this->e->add("Data object copy didn't modify \"manyto1\" referenced field value", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->field["manyto1"]["value"]=-1;
   if($orig->field["manyto1"]==$copy->field["manyto1"])
   {
    $this->e->add("Data object copy also modified original \"manyto1\" field", "error");
    return UT_FAILED;
   }

   //reference onetomany
   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->reference["onetomany"]["from"]=-1;
   if($orig->reference==$copy->reference)
   {
    $this->e->add("Data object copy also modified original \"onetomany reference from\" variable", "error");
    return UT_FAILED;
   }

   //plugin
   $orig=new Data("unit_testing_testmodule", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->plugin->class=Array();
   if($orig->plugin==$copy->plugin)
   {
    $this->e->add("Data object copy also modified original \"plugin\" object", "error");
    return UT_FAILED;
   }

   $orig=new Data("unit_testing_testcopy", false, false, Array("skipCheckTables" => true)); //extends
   $copy=clone $orig;

   $copy->plugin->class=Array();
   if($orig->plugin==$copy->plugin)
   {
    $this->e->add("Data object copy also modified original \"plugin\" object", "error");
    return UT_FAILED;
   }

   //extend




   return UT_Base_Data::Copy_Constructor(); //run parent Copy_Constructor tests
  }

  function Cache()
  {
   $object=Data::cache();

   if(!is_object($object)||get_class($object)!="Data")
   {
    $this->e->add("Cache constructor doesn't return a Data object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Serialize()
  {
   global $_path;

   $this->_jasPath=$_path;

   $d=Data::cache();
   $d->initReferenceData("owner");
   $d->initReferenceData("catnr");

   $object=unserialize(serialize($d));

   //fix d
   $d->moduleObject=unserialize($d->moduleObject);
   $d->moduleObject->data=&$d;


   if(!is_object($object)||get_class($object)!="Data")
   {
    $this->e->add("Deserialized object isn't a Data object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Deserialized object \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!$object->moduleObject||get_class($d->moduleObject)!=get_class($object->moduleObject))
   {
    $this->e->add("Deserialized module object doesn't match", "error");
    return UT_FAILED;
   }

   if(!$object->reference["catnr"]["data"]||get_class($d->reference["catnr"]["data"])!=get_class($object->reference["catnr"]["data"]))
   {
    $this->e->add("Deserialized object reference \"catnr\" doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ViewItem()
  {
   global $_config;

   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View item should check identifier field for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", 1);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", "asdasdasd");

   if($d->get())
   {
    $this->e->add("View item should fail identifier field check for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", 1);
   $d->setValue("catnr", "asdasdasd");

   if(!$d->get())
   {
    $this->e->add("View item not should fail category field check for module: \"".$d->module."\"", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d=Data::cache($_config->get("usermodule"), "view", "item", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View item should check identifier field for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", 1);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d=Data::cache("empty", "view", "item", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View item should fail identifier field check for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ViewList()
  {
   global $_config;

   $d=Data::cache("categories", "view", "list", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View list should check category field for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("catnr", 1);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("catnr", "asdasdasd");

   if($d->get())
   {
    $this->e->add("View list should fail categories field check for module: \"".$d->module."\"", "error");
    //return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", "asdasdasd");
   $d->setValue("catnr", 1);

   if(!$d->get())
   {
    $this->e->add("View list not should fail identifier field check for module: \"".$d->module."\"", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d=Data::cache($_config->get("usermodule"), "view", "list", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View list should check category field for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->setValue("catnr", 1);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d=Data::cache("empty", "view", "list", Array("db" => "stub"));
   $d->clearValues();

   if($d->get())
   {
    $this->e->add("View list should fail category field check for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ViewAll()
  {
   global $_config;

   $d=Data::cache("categories", "view", "all", Array("db" => "stub"));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", 1);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", "asdasdasd");

   if($d->get())
   {
    $this->e->add("View all should fail identifier field check for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d->e->clear();
   $d->clearValues();
   $d->setValue("nr", "asdasdasd");
   $d->setValue("catnr", 1);

   if($d->get())
   {
    $this->e->add("View all should fail identifier field check for module: \"".$d->module."\"", "error");
    return UT_FAILED;
   }

   $d=Data::cache($_config->get("usermodule"), "view", "all", Array("db" => "stub"));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d=Data::cache("empty", "view", "all", Array("db" => "stub"));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function StorageObject()
  {
   //stub storage
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_stub")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_stub object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //xml storage
   $d=Data::cache("categories", "view", "item", Array("db" => "xml"));
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //mime storage
   $d=Data::cache("categories", "view", "item", Array("db" => "mime"));
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //custom storage
   $d=Data::cache("categories", "view", "item", Array("db" => "xml://test/db.xml"));
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //custom storage
   $d=Data::cache("categories", "view", "item", Array("db" => "lala://host/db"));
   $storage=$d->storageObject();

   if($storage)
   {
    $this->e->add("Object: \"storage\" should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function StorageSwitch()
  {
   //stub storage
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_stub")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_stub object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //xml storage
   $d->storageSwitch("xml");
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //mime storage
   $d->storageSwitch("mime");
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //custom storage
   $d->storageSwitch("xml://test/db.xml");
   $storage=$d->storageObject();

   if(!is_object($storage)||get_class($storage)!="System_Storage_xml")
   {
    $this->e->add("Object: \"storage\" isn't an System_Storage_xml object", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //custom storage
   $d->storageSwitch("lala://host/db");
   $storage=$d->storageObject();

   if($storage)
   {
    $this->e->add("Object: \"storage\" should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Aggregation()
  {
   if(!$this->_setupAggregation())
   {
    $this->_aggregationSetup=true; //force desetup
    $this->_desetupAggregation();
    return UT_SKIPPED;
   }

   $d=Data::cache($this->_module, "view", "all", Array("db" => $this->_db));
   $d->setShowFields("nr", "catnr", "name", "updated");
   $d->sortOrder("nr+");
   $d->clearValues();

   $d->field["nr"]["properties"]=str_replace("identifier;", "", $d->field["nr"]["properties"]); //remove nr forced identifier
   $d->field["catnr"]["properties"]=str_replace("rightsReference;", "", $d->field["catnr"]["properties"]); //remove rights reference
   unset($d->reference["catnr"]); //remove catnr reference



//aggregate by nr
   $d->setValue("aggregation_field", "nr");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=1||
    $d->field["nr"]["value"][1]!=2||
    $d->field["nr"]["value"][2]!=3||
    $d->field["nr"]["value"][3]!=4||
    $d->field["nr"]["value"][4]!=5||
    $d->field["nr"]["value"][5]!=6||
    $d->field["nr"]["value"][6]!=7||
    $d->field["nr"]["value"][7]!=8
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by nr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=1||
    $d->field["catnr"]["value"][2]!=1||
    $d->field["catnr"]["value"][3]!=1||
    $d->field["catnr"]["value"][4]!=2||
    $d->field["catnr"]["value"][5]!=2||
    $d->field["catnr"]["value"][6]!=2||
    $d->field["catnr"]["value"][7]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by nr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["name"]["value"][0]!="A"||
    $d->field["name"]["value"][1]!="B"||
    $d->field["name"]["value"][2]!="C"||
    $d->field["name"]["value"][3]!="D"||
    $d->field["name"]["value"][4]!="A"||
    $d->field["name"]["value"][5]!="B"||
    $d->field["name"]["value"][6]!="C"||
    $d->field["name"]["value"][7]!="D"
   )
   {
    $this->e->add("Name field doesn't match expected output", "error", "aggregate by nr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["updated"]["value"][0]!="01:00:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="02:00:00 01-01-2001"||
    $d->field["updated"]["value"][2]!="03:00:00 01-01-2001"||
    $d->field["updated"]["value"][3]!="04:00:00 01-01-2001"||
    $d->field["updated"]["value"][4]!="01:00:00 01-01-2002"||
    $d->field["updated"]["value"][5]!="01:30:00 01-01-2002"||
    $d->field["updated"]["value"][6]!="02:00:00 01-01-2002"||
    $d->field["updated"]["value"][7]!="02:30:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by nr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by name
   $d->setValue("aggregation_field", "name");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["name"]["value"][0]!="A"||
    $d->field["name"]["value"][1]!="B"||
    $d->field["name"]["value"][2]!="C"||
    $d->field["name"]["value"][3]!="D"
   )
   {
    $this->e->add("Name field doesn't match expected output", "error", "aggregate by name");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by name rows
   $d->setValue("aggregation_field", "name");
   if(!$d->get(false, true))
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["show_count"]["value"][0]!=4
   )
   {
    $this->e->add("Show count field doesn't match expected output", "error", "aggregate by name rows");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by catnr average
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "average");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=2.5||
    $d->field["nr"]["value"][1]!=6.5
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by catnr average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by catnr average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="02:30:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="01:45:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by catnr average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by catnr sum
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "sum");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=10||
    $d->field["nr"]["value"][1]!=26
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by catnr sum");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by catnr sum");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by catnr count
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "count");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=4||
    $d->field["nr"]["value"][1]!=4
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by catnr count");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by catnr count");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by catnr min
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "min");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=1||
    $d->field["nr"]["value"][1]!=5
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by catnr min");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by catnr min");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="01:00:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="01:00:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by catnr min");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by catnr max
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "max");
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=4||
    $d->field["nr"]["value"][1]!=8
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by catnr max");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by catnr max");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="04:00:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="02:30:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by catnr max");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by name average
   $d->setValue("aggregation_field", "name");
   $d->setValue("aggregation_type", "average");
   $d->setValue("aggregation_interval", 0);
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=3||
    $d->field["nr"]["value"][1]!=4||
    $d->field["nr"]["value"][2]!=5||
    $d->field["nr"]["value"][3]!=6
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by name average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1.5||
    $d->field["catnr"]["value"][1]!=1.5||
    $d->field["catnr"]["value"][2]!=1.5||
    $d->field["catnr"]["value"][3]!=1.5
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by name average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="14:00:00 02-07-2001"||
    $d->field["updated"]["value"][1]!="14:45:00 02-07-2001"||
    $d->field["updated"]["value"][2]!="15:30:00 02-07-2001"||
    $d->field["updated"]["value"][3]!="16:15:00 02-07-2001"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by name average");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by updated sum interval after
   $d->setValue("aggregation_field", "updated");
   $d->setValue("aggregation_type", "sum");
   $d->setValue("aggregation_method", "after");
   $d->setValue("aggregation_interval", 60*60*4); //4 hours
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=10||
    $d->field["nr"]["value"][1]!=26
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by updated sum interval after");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=4||
    $d->field["catnr"]["value"][1]!=8
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by updated sum interval after");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="01:00:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="01:00:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by updated sum interval after");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by nr average interval around
   $d->setValue("aggregation_field", "nr");
   $d->setValue("aggregation_type", "average");
   $d->setValue("aggregation_method", "around");
   $d->setValue("aggregation_interval", 2);
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=2||
    $d->field["nr"]["value"][1]!=4||
    $d->field["nr"]["value"][2]!=6||
    $d->field["nr"]["value"][3]!=8
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by nr average interval around");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=1||
    $d->field["catnr"]["value"][1]!=1||
    $d->field["catnr"]["value"][2]!=2||
    $d->field["catnr"]["value"][3]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by nr average interval around");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="01:30:00 01-01-2001"||
    $d->field["updated"]["value"][1]!="03:30:00 01-01-2001"||
    $d->field["updated"]["value"][2]!="01:15:00 01-01-2002"||
    $d->field["updated"]["value"][3]!="02:15:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by nr average interval around");
    $this->_desetupAggregation();
    return UT_FAILED;
   }



//aggregate by nr average interval around where catnr
   $d->setValue("aggregation_field", "nr");
   $d->setValue("aggregation_type", "average");
   $d->setValue("aggregation_method", "around");
   $d->setValue("aggregation_interval", 2);
   $d->setValue("catnr", 2);
   if(!$d->get())
   {
    $this->e->add($d->e);
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["nr"]["value"][0]!=6||
    $d->field["nr"]["value"][1]!=8
   )
   {
    $this->e->add("NR field doesn't match expected output", "error", "aggregate by nr average interval around where catnr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }

   if(
    $d->field["catnr"]["value"][0]!=2||
    $d->field["catnr"]["value"][1]!=2
   )
   {
    $this->e->add("CatNR field doesn't match expected output", "error", "aggregate by nr average interval around where catnr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   if(
    $d->field["updated"]["value"][0]!="01:15:00 01-01-2002"||
    $d->field["updated"]["value"][1]!="02:15:00 01-01-2002"
   )
   {
    $this->e->add("Updated field doesn't match expected output", "error", "aggregate by nr average interval around where catnr");
    $this->_desetupAggregation();
    return UT_FAILED;
   }


   $this->_desetupAggregation();
   return UT_OK;
  }


  function GetReferenceDepth()
  {
   $d=new Data("categories");

//no show fields
   $d->setShowFields();
   if($d->getReferenceDepth()!==false)
   {
    $this->e->add("Data reference depth should be: false", "error", "no show fields");
    return UT_FAILED;
   }

//normal fields
   $d->setShowFields("nr", "name");
   if($d->getReferenceDepth()!==0)
   {
    $this->e->add("Data reference depth should be: 0", "error", "normal fields");
    return UT_FAILED;
   }

//*:1 no fields
   $d->setShowFields("catnr");
   if($d->getReferenceDepth()!==0)
   {
    $this->e->add("Data reference depth should be: 0", "error", "*:1 no fields");
    return UT_FAILED;
   }

//*:1 normal fields
   $d2=$d->getReferenceData("catnr");
   $d2->setShowFields("name");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "*:1 normal fields");
    return UT_FAILED;
   }

//*:1 *:1 no fields
   $d2->setShowFields("catnr");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "*:1 *:1 no fields");
    return UT_FAILED;
   }

   $d3=$d2->getReferenceData("catnr");
   $d3->setShowFields("name");
   if($d->getReferenceDepth()!==2)
   {
    $this->e->add("Data reference depth should be: 2", "error", "*:1 *:1 normal fields");
    return UT_FAILED;
   }


//1:*
   $d=new Data("categories");
   $d->addReference("below", "categories", "catnr", "1:*", "nr"); //add fake reference

//1:* no fields
   $d->setShowFields("below");
   if($d->getReferenceDepth()!==0)
   {
    $this->e->add("Data reference depth should be: 0", "error", "1:* no fields");
    return UT_FAILED;
   }

//1:* normal fields
   $d2=$d->getReferenceData("below");
   $d2->addReference("below", "categories", "catnr", "1:*", "nr"); //add fake reference
   $d2->setShowFields("name");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "1:* normal fields");
    return UT_FAILED;
   }

//1:* 1:* no fields
   $d2->setShowFields("below");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "1:* 1:* no fields");
    return UT_FAILED;
   }

//1:* 1:* normal fields
   $d3=$d2->getReferenceData("below");
   $d3->addReference("below", "categories", "catnr", "1:*", "nr"); //add fake reference
   $d3->setShowFields("name");
   if($d->getReferenceDepth()!==2)
   {
    $this->e->add("Data reference depth should be: 2", "error", "1:* 1:* normal fields");
    return UT_FAILED;
   }


//*:*
   $d=new Data("categories");
   $d->addReference("cross", "categories", "nr", "*:*", "nr"); //add fake reference


//*:* no fields
   $d->setShowFields("cross");
   if($d->getReferenceDepth()!==0)
   {
    $this->e->add("Data reference depth should be: 0", "error", "*:* no fields");
    return UT_FAILED;
   }

//*:* normal fields
   $d2=$d->getReferenceData("cross");
   $d2=$d2->getReferenceData("categories_2");
   $d2->setShowFields("name");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "*:* normal fields");
    return UT_FAILED;
   }

//*:* *:* no fields
   $d2=$d->getReferenceData("cross");
   $d2=$d2->getReferenceData("categories_2");
   $d2->addReference("cross", "categories", "nr", "*:*", "nr"); //add fake reference
   $d2->setShowFields("cross");
   if($d->getReferenceDepth()!==1)
   {
    $this->e->add("Data reference depth should be: 1", "error", "*:* *:* no fields");
    return UT_FAILED;
   }


//*:* *:* normal fields
   $d3=$d2->getReferenceData("cross");
   $d3=$d3->getReferenceData("categories_2");
   $d3->setShowFields("name");
   if($d->getReferenceDepth()!==2)
   {
    $this->e->add("Data reference depth should be: 2", "error", "*:* *:* normal fields");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetDataFormat()
  {
   $d=new Data(false, false, false, Array("format" => "html"));

//check constructor
   if($d->dataformat!="html")
   {
    $this->e->add("Data format not set to: \"html\"");
    return UT_FAILED;
   }

//normal set (sql)
   if(!$d->setDataFormat("sql"))
   {
    $this->e->add("Unable to set data format to: \"sql\"");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->dataformat!="sql")
   {
    $this->e->add("Data format not set to: \"sql\"");
    return UT_FAILED;
   }

//normal set (html)
   if(!$d->setDataFormat("html"))
   {
    $this->e->add("Unable to set data format to: \"html\"");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->dataformat!="html")
   {
    $this->e->add("Data format not set to: \"html\"");
    return UT_FAILED;
   }

//illegal format
   if($d->setDataFormat("asdasd"))
   {
    $this->e->add("Shouldn't be able to set data format to: \"asdasd\"");
    return UT_FAILED;
   }

   if($d->dataformat!="html")
   {
    $this->e->add("Data format should still be set to: \"html\"");
    return UT_FAILED;
   }

//references
   $d=new Data("categories", false, false, Array("format" => "html"));
   $d2=$d->getReferenceData("catnr");
   $d3=$d->getReferenceData("catnr");
   $d4=$d->getReferenceData("owner");

//check constructor
   if($d->dataformat!="html"||$d2->dataformat!="html"||$d3->dataformat!="html"||$d4->dataformat!="html")
   {
    $this->e->add("Data format not set to: \"html\"");
    return UT_FAILED;
   }

//normal set (sql)
   if(!$d->setDataFormat("form"))
   {
    $this->e->add("Unable to set data format to: \"form\"");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->dataformat!="form"||$d2->dataformat!="form"||$d3->dataformat!="form"||$d4->dataformat!="form")
   {
    $this->e->add("Data format not set to: \"form\"");
    return UT_FAILED;
   }

//illegal format
   if($d->setDataFormat("asdasd"))
   {
    $this->e->add("Shouldn't be able to set data format to: \"asdasd\"");
    return UT_FAILED;
   }

   if($d->dataformat!="form"||$d2->dataformat!="form"||$d3->dataformat!="form"||$d4->dataformat!="form")
   {
    $this->e->add("Data format should still be set to: \"form\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetUseFields()
  {
   $d=new Data("users");

//no fields
   $showFields=Array();
   if(!$d->setUseFields("show"))
   {
    $this->e->add("Unable to set use fields", "error", "no fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields");
     return UT_FAILED;
    }
   }

//exiting (single) field
   $formFields=Array("password");
   if(!$d->setUseFields("form", "password"))
   {
    $this->e->add("Unable to set use fields", "error", "exiting (single) field");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing (single) field");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing (single) field");
     return UT_FAILED;
    }
   }


//exiting fields
   $formFields=Array("nr", "catnr");
   if(!$d->setUseFields("form", $formFields))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing field");
     return UT_FAILED;
    }
   }


//reference fields
   $showFields=Array("usersAdditionalGroups");
   if(!$d->setUseFields("show", $showFields))
   {
    $this->e->add("Unable to set use fields", "error", "existing fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr";
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields");
     return UT_FAILED;
    }
   }


//no fields add mode
   $showFields=Array("nr", "catnr");
   if(!$d->setUseFields("show", $showFields))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }
   if(!$d->setUseFields("show", false, true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields add mode");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "no fields add mode");
     return UT_FAILED;
    }
   }


//exiting fields add mode
   $formFields=Array("nr", "catnr");
   if(!$d->setUseFields("form", $formFields))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }
   $formFields[]="password";
   if(!$d->setUseFields("form", "password", true))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields add mode");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields add mode");
     return UT_FAILED;
    }
   }


//reference fields add mode
   $showFields=Array("catnr");
   if(!$d->setUseFields("show", $showFields))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }
   $showFields[]="usersAdditionalGroups";
   if(!$d->setUseFields("show", "usersAdditionalGroups", true))
   {
    $this->e->add("Unable to set use fields", "error", "reference fields add mode");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields add mode");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields add mode");
     return UT_FAILED;
    }
   }


//no fields recursive
   $d2=$d->getReferenceData("catnr");

   $showFields=Array();
   if(!$d->setUseFields("show", false, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "no fields recursive");
     return UT_FAILED;
    }
   }

   foreach($d2->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d2->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d2->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "no fields recursive");
     return UT_FAILED;
    }
   }


//exiting fields recursive
   $formFields=Array("nr", "catnr");
   if(!$d->setUseFields("form", $formFields, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields recursive");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields recursive");
     return UT_FAILED;
    }
   }

   foreach($d2->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d2->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields recursive");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d2->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields recursive");
     return UT_FAILED;
    }
   }


//reference fields recursive
   $d2=$d->getReferenceData("usersAdditionalGroups");
   $oldGroupsShowFields=$d2->check("use", "show");
   $d3=$d2->getReferenceData("categories");

   $showFields=Array("usersAdditionalGroups", "catnr");
   if(!$d->setUseFields("show", $showFields, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "reference fields recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields recursive");
     return UT_FAILED;
    }
   }

   if(array_diff($oldGroupsShowFields, $d2->check("use", "show"))||array_diff($d2->check("use", "show"), $oldGroupsShowFields))
   {
    $this->e->add("UsersAdditionalGroups show fields shouldn't have changed", "error", "reference fields recursive"); //usersAdditionalGroups is a *:* table which shouldn't be touched
    return UT_FAILED;
   }

   foreach($d3->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d3->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields recursive");
     return UT_FAILED;
    }
   }

   foreach(Array("catnr") as $field)
   {
    if(!$d3->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields recursive");
     return UT_FAILED;
    }
   }


//no fields add recursive
   $showFields=Array("nr", "catnr");
   if(!$d->setUseFields("show", $showFields, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d2=$d->getReferenceData("catnr");

   if(!$d->setUseFields("show", false, true , true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "no fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($d2->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d2->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d2->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "no fields add recursive");
     return UT_FAILED;
    }
   }


//exiting fields add recursive
   $formFields=Array("nr");
   if(!$d->setUseFields("form", $formFields, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $formFields[]="catnr";
   if(!$d->setUseFields("form", "catnr", true, true))
   {
    $this->e->add("Unable to set use fields", "error", "exiting fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($d2->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d2->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d2->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields add recursive");
     return UT_FAILED;
    }
   }


//reference fields add recursive
   $showFields=Array("catnr");
   if(!$d->setUseFields("show", $showFields, false, true))
   {
    $this->e->add("Unable to set use fields", "error", "no fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d2=$d->getReferenceData("usersAdditionalGroups");
   $oldGroupsShowFields=$d2->check("use", "show");
   $d3=$d2->getReferenceData("categories");

   $showFields[]="usersAdditionalGroups";
   if(!$d->setUseFields("show", "usersAdditionalGroups", true, true))
   {
    $this->e->add("Unable to set use fields", "error", "reference fields add recursive");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields add recursive");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields add recursive");
     return UT_FAILED;
    }
   }

   if(array_diff($oldGroupsShowFields, $d2->check("use", "show"))||array_diff($d2->check("use", "show"), $oldGroupsShowFields))
   {
    $this->e->add("UsersAdditionalGroups show fields shouldn't have changed", "error", "reference fields add recursive"); //usersAdditionalGroups is a *:* table which shouldn't be touched
    return UT_FAILED;
   }

   foreach($d3->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d3->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields add recursive");
     return UT_FAILED;
    }
   }

   foreach(Array("catnr") as $field)
   {
    if(!$d3->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields add recursive");
     return UT_FAILED;
    }
   }


//illegal use argument: asdasd
   if($d->setUseFields("asdasd"))
   {
    $this->e->add("Shouldn't be able to set use fields", "error", "illegal use argument: asdasd");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetShowFields()
  {
   $d=new Data("users");

//no fields
   $showFields=Array();
   if(!$d->setShowFields())
   {
    $this->e->add("Unable to set show fields", "error", "no fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "no fields");
     return UT_FAILED;
    }
   }


//exiting fields
   $showFields=Array("nr", "catnr");
   if(!$d->setShowFields("nr", "catnr"))
   {
    $this->e->add("Unable to set show fields", "error", "exiting fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "existing fields");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "existing fields");
     return UT_FAILED;
    }
   }


//exiting fields array
   $showFields=Array("nr", "catnr");
   if(!$d->setShowFields($showFields))
   {
    $this->e->add("Unable to set show fields", "error", "exiting fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "existing fields array");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "existing fields array");
     return UT_FAILED;
    }
   }


//reference fields
   $showFields=Array("usersAdditionalGroups");
   if(!$d->setShowFields("usersAdditionalGroups"))
   {
    $this->e->add("Unable to set show fields", "error", "existing fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields");
     return UT_FAILED;
    }
   }


//reference fields array
   $showFields=Array("usersAdditionalGroups");
   if(!$d->setShowFields($showFields))
   {
    $this->e->add("Unable to set show fields", "error", "reference fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $showFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "show") as $field)
   {
    if(!in_array($field, $showFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a show field", "error", "reference fields array");
     return UT_FAILED;
    }
   }

   foreach($showFields as $field)
   {
    if(!$d->check("use", "show", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a show field", "error", "reference fields array");
     return UT_FAILED;
    }
   }


   return UT_OK;
  }


  function SetFormFields()
  {
   $d=new Data("users");

//no fields
   $formFields=Array();
   if(!$d->setFormFields())
   {
    $this->e->add("Unable to set form fields", "error", "no fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "no fields");
     return UT_FAILED;
    }
   }


//exiting fields
   $formFields=Array("nr", "catnr");
   if(!$d->setFormFields("nr", "catnr"))
   {
    $this->e->add("Unable to set form fields", "error", "exiting fields");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields");
     return UT_FAILED;
    }
   }


//exiting fields array
   $formFields=Array("nr", "catnr");
   if(!$d->setFormFields($formFields))
   {
    $this->e->add("Unable to set form fields", "error", "exiting fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "existing fields array");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "existing fields array");
     return UT_FAILED;
    }
   }


//reference fields
   $formFields=Array("usersAdditionalGroups");
   if(!$d->setFormFields("usersAdditionalGroups"))
   {
    $this->e->add("Unable to set form fields", "error", "existing fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $formFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "reference fields");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "reference fields");
     return UT_FAILED;
    }
   }


//reference fields array
   $formFields=Array("usersAdditionalGroups");
   if(!$d->setFormFields($formFields))
   {
    $this->e->add("Unable to set form fields", "error", "reference fields array");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $formFields[]="nr"; //nr should be forced by usersAdditionalGroups (from) reference
   foreach($d->check("use", "form") as $field)
   {
    if(!in_array($field, $formFields)&&!$d->check("properties", "control", $field))
    {
     $this->e->add("Field: \"".$field."\" shouldn't be set as a form field", "error", "reference fields array");
     return UT_FAILED;
    }
   }

   foreach($formFields as $field)
   {
    if(!$d->check("use", "form", $field))
    {
     $this->e->add("Field: \"".$field."\" should be set as a form field", "error", "reference fields");
     return UT_FAILED;
    }
   }


   return UT_OK;
  }


  function SQLValue()
  {
   $this->_init();

   foreach(Array("text", "html") as $format)
   {
    $d=new Data("unit_testing_mutilate", "view", "all", Array("format" => $format));

    foreach($this->sqlValueTests as $field => $test)
    {
     foreach($test as $key => $v)
     {
      $d->setValue($field, $v[$format]);

      if(!$this->_compare($v["sql"], $d->sqlValue($field)))
      {
       $this->e->add("SQLValue \"".$field."\" field \"".$key."\" test \"".$format."\" output doesn't match", "error");
       return UT_FAILED;
      }
     }
    }
   }

   return UT_OK;
  }


  function SetValues()
  {
   $this->_init();

   foreach(Array("text", "html") as $format)
   {
    $d=new Data("unit_testing_mutilate", "view", "all", Array("format" => $format));
    $storage=new Storage($d, "storage");

    foreach($this->setValuesTests as $field => $test)
    {
     foreach($test as $key => $v)
     {
      $storage->clear();
      $storage->results=Array(Array($field => $v["sql"]));
      if(!$d->setValues($storage))
      {
       $this->e->add("SetValues \"".$field."\" field \"".$key."\" test failed", "error");
       return UT_FAILED;
      }

      if(!$this->_compare($v[$format], $d->getValue($field)))
      {
       $this->e->add("SetValues \"".$field."\" field \"".$key."\" test \"".$format."\" output doesn't match", "error");
       return UT_FAILED;
      }
     }
    }
   }

   return UT_OK;
  }


  function Reset()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);

   if($d->vmax!==1)
   {
    $this->e->add("Vmax should be set to 1", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   $d->next();

   if($d->vc!==1)
   {
    $this->e->add("VC should be set to 1", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   $d->reset();

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Next()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->next())
   {
    $this->e->add("Next shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==1)
   {
    $this->e->add("VC should be set to 1", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   if(!$d->next())
   {
    $this->e->add("Next shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==2)
   {
    $this->e->add("VC should be set to 2", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 2, "name" => "Test 2"), $d->v))
    return UT_FAILED;

   if($d->next())
   {
    $this->e->add("Next should have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Previous()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->previous())
   {
    $this->e->add("Previous shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==1)
   {
    $this->e->add("VC should be set to 1", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 2, "name" => "Test 2"), $d->v))
    return UT_FAILED;

   if(!$d->previous())
   {
    $this->e->add("Previous shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==2)
   {
    $this->e->add("VC should be set to 2", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   if($d->previous())
   {
    $this->e->add("Previous should have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function First()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->first())
   {
    $this->e->add("First shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   return UT_OK;
  }

  function Last()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->last())
   {
    $this->e->add("Last shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 2, "name" => "Test 2"), $d->v))
    return UT_FAILED;

   return UT_OK;
  }

  function Search()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->search("name", "Test 1"))
   {
    $this->e->add("Search shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==1)
   {
    $this->e->add("VC should be set to 1", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   if(!$d->search("name", "Test 2"))
   {
    $this->e->add("Search shouldn't have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==2)
   {
    $this->e->add("VC should be set to 2", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 2, "name" => "Test 2"), $d->v))
    return UT_FAILED;

   if($d->search("name", "Test 2"))
   {
    $this->e->add("Search should have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if($d->search("name", "Unknown entry"))
   {
    $this->e->add("Search should have failed", "error");
    return UT_FAILED;
   }

   if($d->vc!==0)
   {
    $this->e->add("VC should be set to 0", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Value()
  {
   $d=Data::cache("categories", "view", "item", Array("db" => "stub"));
   $d->setShowFields("nr", "name");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Test 1", 0);
   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Test 2", 1);

   if($d->vmax!==2)
   {
    $this->e->add("Vmax should be set to 2", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if(!$d->value(0))
   {
    $this->e->add("Value shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 1, "name" => "Test 1"), $d->v))
    return UT_FAILED;

   if(!$d->value(1))
   {
    $this->e->add("Value shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("nr" => 2, "name" => "Test 2"), $d->v))
    return UT_FAILED;

   if($d->value(2))
   {
    $this->e->add("Value should have failed", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   if($d->value(-1))
   {
    $this->e->add("Value should have failed", "error");
    return UT_FAILED;
   }

   if($d->v!==Array())
   {
    $this->e->add("V should be set an Array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
