<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/format_csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/format.php");
 require_once($_path["format"]."csv.php");

 class UT_Format_CSV extends UT_Format
 {
  function _init()
  {
   global $_path;

   $this->testfileI=$_path["plugins"]."unit_testing/misc/indexed.csv";
   $this->testfileA=$_path["plugins"]."unit_testing/misc/associative.csv";
   $this->outputfile=$_path["plugins"]."unit_testing/misc/output.csv";

   $this->valuesI=Array(
    Array("1", "test1", "Lorem Ipsum?"),
    Array("2", "test2", "Ipsum?"),
    Array("3", "test3", "Lorem?"),
   );

   $this->valuesA=Array(
    Array("id" => 1, "title" => "test1", "text" => "Lorem Ipsum?"),
    Array("id" => 2, "title" => "test2", "text" => "Ipsum?"),
    Array("id" => 3, "title" => "test3", "text" => "Lorem?"),
   );
  }

  function Constructor()
  {
   $object=new Format_CSV();

   if(!is_object($object)||get_class($object)!="Format_CSV")
   {
    $this->e->add("Constructor doesn't return a Format_CSV object", "error");
    return UT_FAILED;
   }

   if($object->split!=";")
   {
    $this->e->add("Split character isn't: \";\"", "error");
    return UT_FAILED;
   }
   if($object->quote!="\"")
   {
    $this->e->add("Quote character isn't: \"\"\"", "error");
    return UT_FAILED;
   }
   if($object->linebreak!="\r\n")
   {
    $this->e->add("Line break character isn't: \"\\r\\n\"", "error");
    return UT_FAILED;
   }
   if($object->subSplit!="\r\n")
   {
    $this->e->add("Sub split character isn't: \"\\r\\n\"", "error");
    return UT_FAILED;
   }
   if($object->headerLine!==false)
   {
    $this->e->add("Header line isn't: \"false\"", "error");
    return UT_FAILED;
   }

   if(!is_object($object->fs)||get_class($object->fs)!="FS")
   {
    $this->e->add("Object: \"e\" isn't a FS object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->filename!="")
   {
    $this->e->add("Default filename isn't empty", "error");
    return UT_FAILED;
   }

   if($object->vc!==0)
   {
    $this->e->add("Value counter should be at 0", "error");
    return UT_FAILED;
   }
   if($object->vmax!==0)
   {
    $this->e->add("Value max counter should be at 0", "error");
    return UT_FAILED;
   }
   if($object->v)
   {
    $this->e->add("Values array isn't empty", "error");
    return UT_FAILED;
   }

   if(!is_array($object->data))
   {
    $this->e->add("Grid isn't an array", "error");
    return UT_FAILED;
   }
   if($object->data)
   {
    $this->e->add("Grid isn't empty", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Format_CSV();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Format_CSV object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->contentType=-1;
   if($orig->contentType==$copy->contentType)
   {
    $this->e->add("Format_CSV object copy also modified original \"contentType\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Format_CSV object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_Format::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function Separators()
  {
   $csv=new Format_CSV();

//default
   if($csv->split!=";")
   {
    $this->e->add("Split character isn't: \";\"", "error");
    return UT_FAILED;
   }
   if($csv->quote!="\"")
   {
    $this->e->add("Quote character isn't: \"\"\"", "error");
    return UT_FAILED;
   }
   if($csv->linebreak!="\r\n")
   {
    $this->e->add("Line break character isn't: \"\\r\\n\"", "error");
    return UT_FAILED;
   }
   if($csv->subSplit!="\r\n")
   {
    $this->e->add("Sub split character isn't: \"\\r\\n\"", "error");
    return UT_FAILED;
   }

//custom
   if(!$csv->separators(",", "", "\n"))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->split!=",")
   {
    $this->e->add("Split character isn't: \",\"", "error");
    return UT_FAILED;
   }
   if($csv->quote!="")
   {
    $this->e->add("Quote character isn't: \"\"", "error");
    return UT_FAILED;
   }
   if($csv->linebreak!="\n")
   {
    $this->e->add("Line break character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }
   if($csv->subSplit!="; ")
   {
    $this->e->add("Sub split character isn't: \"; \"", "error");
    return UT_FAILED;
   }

//line endings
   if(!$csv->separators(false, false, "\n"))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->split!=";")
   {
    $this->e->add("Split character isn't: \";\"", "error");
    return UT_FAILED;
   }
   if($csv->quote!="\"")
   {
    $this->e->add("Quote character isn't: \"\"\"", "error");
    return UT_FAILED;
   }
   if($csv->linebreak!="\n")
   {
    $this->e->add("Line break character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }
   if($csv->subSplit!="\n")
   {
    $this->e->add("Sub split character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }

//custom
   if(!$csv->separators(";", "'", "\n", " - "))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->subSplit!=" - ")
   {
    $this->e->add("Sub split character isn't: \" - \"", "error");
    return UT_FAILED;
   }

   if($csv->separators(",", ",", ","))
   {
    $this->e->add("Split, quote and linebreak character shouldn't be allowed to be the same", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Header()
  {
   $csv=new Format_CSV();

   if(!$csv->header(true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->headerLine!==true)
   {
    $this->e->add("Header line isn't: \"true\"", "error");
    return UT_FAILED;
   }

   if(!$csv->header(false))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->headerLine!==false)
   {
    $this->e->add("Header line isn't: \"false\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckLine()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   if(!$csv->checkLine($this->valuesI[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //force first line check
   $csv->data[0]=$this->valuesI[1];
   $csv->vmax++;

   if(!$csv->checkLine($this->valuesI[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }


   $csv->e->clear();

   $tmp=$this->valuesI[1];
   array_pop($tmp); //remove line value

   if(!$csv->checkLine($tmp)) //too little (indexed) values only creates a (non-fatal) warning
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->e->getErrors()!=1||$csv->e->getLevel(0)!="warning")
   {
    $this->e->add("Line should have too little values warning", "error");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckLineHeader()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true); //Associative check

   if(!$csv->checkLine($this->valuesA[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //force first line check
   $csv->data[0]=$this->valuesA[1];
   $csv->vmax++;

   if(!$csv->checkLine($this->valuesA[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   $tmp=$this->valuesA[1];
   array_pop($tmp); //remove line value

   if($csv->checkLine($tmp))
   {
    $this->e->add("Line should have too little values", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   $tmp["Lorem"]="Impsum"; //same size

   if($csv->checkLine($tmp))
   {
    $this->e->add("Line should have incorrect keys", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function AddLine()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if($csv->data!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Add line should increased vmax", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine
   if($csv->addLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function AddLineHeader()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if($csv->data!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Add line should increased vmax", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine
   if($csv->addLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function EditLine()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->editLine(1, $this->valuesI[2]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data[1]!=$this->valuesI[2])
   {
    $this->e->add("Edit line should have updated line: \"1\"", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Edit line shouldn't have changed vmax", "error");
    return UT_FAILED;
   }

   if($csv->editLine(1241, $this->valuesI[2]))
   {
    $this->e->add("Edit line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //reset line 1
   if(!$csv->editLine(1, $this->valuesI[1]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine
   if($csv->editLine(0, $this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function EditLineHeader()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->editLine(1, $this->valuesA[2]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data[1]!=$this->valuesA[2])
   {
    $this->e->add("Edit line should have updated line: \"1\"", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Edit line shouldn't have changed vmax", "error");
    return UT_FAILED;
   }

   if($csv->editLine(1241, $this->valuesA[2]))
   {
    $this->e->add("Edit line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //reset line 1
   if(!$csv->editLine(1, $this->valuesA[1]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine
   if($csv->editLine(0, $this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function DeleteLine()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->deleteLine(1))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)!=2)
   {
    $this->e->add("Delete line should decreased the number of lines in the grid", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=2)
   {
    $this->e->add("Delete line should decreased vmax", "error");
    return UT_FAILED;
   }

   if($csv->data[0]!=$this->valuesI[0])
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->data[1]!=$this->valuesI[2])
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->deleteLine(1241, $this->valuesI[2]))
   {
    $this->e->add("Delete line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function Reset()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   $csv->reset();
   if($csv->vc!=0)
   {
    $this->e->add("Reset should set vc to 0", "error");
    return UT_FAILED;
   }

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->next())
   {
    $this->e->add("Next value failed", "error");
    return UT_FAILED;
   }

   $csv->reset();
   if($csv->vc!=0)
   {
    $this->e->add("Reset should set vc to 0", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Next()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   while($csv->next())
   {
    if($csv->v!=$this->valuesI[$c])
    {
     $this->e->add("Next value doesn't match line", "error");
     return UT_FAILED;
    }

    if($csv->vc!=++$c)
    {
     $this->e->add("Next value should increased vc", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function NextHeader()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   while($csv->next())
   {
    if($csv->v!=$this->valuesA[$c])
    {
     $this->e->add("Next value doesn't match line", "error");
     return UT_FAILED;
    }

    if($csv->vc!=++$c)
    {
     $this->e->add("Next value should increased vc", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function Previous()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   $vmax=sizeof($this->valuesI);
   while($csv->previous())
   {
    if($csv->v!=$this->valuesI[$vmax-($c+1)])
    {
     $this->e->add("Previous value doesn't match line", "error");
     return UT_FAILED;
    }

    if($csv->vc!=++$c)
    {
     $this->e->add("Previous value should increased vc", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function PreviousHeader()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   $vmax=sizeof($this->valuesA);
   while($csv->previous())
   {
    if($csv->v!=$this->valuesA[$vmax-($c+1)])
    {
     $this->e->add("Next value doesn't match line", "error");
     return UT_FAILED;
    }

    if($csv->vc!=++$c)
    {
     $this->e->add("Next value should increased vc", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function Search()
  {
   $this->_init();

   $csv=new Format_CSV();
   $csv->header(true);

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->search("id", 2))
   {
    $this->e->add("Search value failed", "error");
    return UT_FAILED;
   }

   if($csv->vc!=2)
   {
    $this->e->add("Reset should set vc to 2", "error");
    return UT_FAILED;
   }

   if($csv->v["id"]!=2)
   {
    $this->e->add("Reset should set v to correct entry", "error");
    return UT_FAILED;
   }

   if($csv->search("id", 2)) //should hit end of list
   {
    $this->e->add("Search value should fail", "error");
    return UT_FAILED;
   }

   if(!$csv->search("id", 1))
   {
    $this->e->add("Search value failed", "error");
    return UT_FAILED;
   }

   if($csv->vc!=1)
   {
    $this->e->add("Reset should set vc to 1", "error");
    return UT_FAILED;
   }

   if($csv->v["id"]!=1)
   {
    $this->e->add("Reset should set v to correct entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse()
  {
   $csv=new Format_CSV();
   $csv->separators(false, "");

   if(!$csv->parse("1;test 1"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;;"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1", "")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array()), $csv->data))
    return UT_FAILED;


   $csv->clear();
   if(!$csv->parse("1;test 1\r\n2;test 2"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;\r\n2;test 2"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;\r\n2;test 2;"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1;;\r\n2;test 2;;"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1", ""), Array("2", "test 2", "")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n2;test 2\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n2;test 2;\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n2;test 2\r\n\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2"), Array()), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("1;test 1\r\n\r\n2;test 2;\r\n\r\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array(), Array("2", "test 2"), Array()), $csv->data))
    return UT_FAILED;

   return UT_OK;
  }

  function ParseQuotes()
  {
   $csv=new Format_CSV();
   $csv->separators(";", "'", "\n");

   if(!$csv->parse("'1';'test 1'"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';'';"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1", "")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1")), $csv->data))
    return UT_FAILED;


   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n'2';'test 2'"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';\n'2';'test 2'"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';\n'2';'test 2';"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1';'';\n'2';'test 2';'';"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1", ""), Array("2", "test 2", "")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n'2';'test 2'\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n'2';'test 2';\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n'2';'test 2'\n\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n'2';'test 2';\n\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1'\n\n'2';'test 2';\n\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1"), Array("2", "test 2")), $csv->data))
    return UT_FAILED;


   $csv->clear();
   if(!$csv->parse("'1';'test 1\n'\n'2';'test 2\n';\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1\n"), Array("2", "test 2\n")), $csv->data))
    return UT_FAILED;

   $csv->clear();
   if(!$csv->parse("'1';'test 1\n\n'\n'2';'test 2\n\n';\n"))
   {
    $this->e->add("CSV parse failed");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("1", "test 1\n\n"), Array("2", "test 2\n\n")), $csv->data))
    return UT_FAILED;

   return UT_OK;
  }

  function Read()
  {
   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileI, false))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->read("nonexistingfile", false))
   {
    $this->e->add("Read shouldn't be able to read \"nonexistingfile\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ReadHeader()
  {
   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileA, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->data!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileI, false, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output="";
   foreach($csv->data as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if($csv->output(false)!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function OutputHeader()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileA, true, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $csv->filename("format.csv");

   $output=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, array_keys($csv->data[0])).$csv->quote.$csv->linebreak;
   foreach($csv->data as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if($csv->output(true)!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!strstr(Headers::getS("Content-Type"), $csv->contentType))
   {
    $this->e->add("Format output should have set content type header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("Content-Length: 93", headers_list()))
   {
    $this->e->add("Format output should have set content length header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("Content-Disposition: attachment; filename=\"format.csv\"", headers_list()))
   {
    $this->e->add("Format output should have set filename header");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }

  function Write()
  {
   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileI, false, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$csv->write($this->outputfile))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output="";
   foreach($csv->data as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if(file_get_contents($this->outputfile)!=$output)
   {
    $this->e->add("Output file doesn't match expected output", "error");
    @unlink($this->outputfile);
    return UT_FAILED;
   }

   @unlink($this->outputfile);
   return UT_OK;
  }

  function WriteHeader()
  {
   $this->_init();

   $csv=new Format_CSV();

   if(!$csv->read($this->testfileA, true, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$csv->write($this->outputfile))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, array_keys($csv->data[0])).$csv->quote.$csv->linebreak;
   foreach($csv->data as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if(file_get_contents($this->outputfile)!=$output)
   {
    $this->e->add("Output file doesn't match expected output", "error");
    @unlink($this->outputfile);
    return UT_FAILED;
   }

   @unlink($this->outputfile);
   return UT_OK;
  }

 }

?>
