<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/functions.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 class UT_Functions extends UT_Core
 {
  function FunctionsFileExists()
  {
   global $_path;

   if(!file_exists($_path["include"]."functions.php"))
    return UT_FAILED;

   require_once($_path["include"]."functions.php");
   return UT_OK;
  }

  function FunctionsExist()
  {
   global $_path;
   $functions=Array(
    "d",
    "request",
    "array_type",
   );

   $status=UT_OK;

   foreach($functions as $function)
   {
    if(!function_exists($function))
    {
     $this->e->add("Function: \"".$function."\" doesn't exist");
     $status=UT_FAILED;
    }
   }

   return $status;
  }

  function Debug()
  {
   global $_path;

   if(!function_exists("d"))
    return UT_FAILED;

   ob_start();
   d("test");
   $output=ob_get_clean();

   if($output!="<pre>string(4) \"test\"\n</pre>")
    return UT_FAILED;

   return UT_OK;
  }

  function Get_Argument_Names()
  {
   //1 arg
   if(($args=get_argument_names("array_type"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for array_type");
    return UT_FAILED;
   }

   if($args!=Array("array"))
   {
    $this->e->add("Argument names don't match for array_type");
    return UT_FAILED;
   }

   //multi args
   if(($args=get_argument_names("get_argument_names"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for get_argument_names");
    return UT_FAILED;
   }

   if($args!=Array("function", "object", "required"))
   {
    $this->e->add("Argument names don't match for get_argument_names");
    return UT_FAILED;
   }

   //no args
   if(($args=get_argument_names("d"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for d");
    return UT_FAILED;
   }

   if($args!=Array())
   {
    $this->e->add("Argument names don't match for d");
    return UT_FAILED;
   }

   //non-existent function
   if(($args=get_argument_names("lsaglkwjgljlsjsag"))!==false)
   {
    $this->e->add("Get argument names should have failed for lsaglkwjgljlsjsag");
    return UT_FAILED;
   }

   //class
   //1 arg
   if(($args=get_argument_names("setOptions", "Data"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for Data::setOptions");
    return UT_FAILED;
   }

   if($args!=Array("options"))
   {
    $this->e->add("Argument names don't match for Data::setOptions");
    return UT_FAILED;
   }

   //multi args
   if(($args=get_argument_names("postInit", "Data"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for Data::postInit");
    return UT_FAILED;
   }

   if($args!=Array("extendedModules", "initializedReferences"))
   {
    $this->e->add("Argument names don't match for Data::postInit");
    return UT_FAILED;
   }

   //no args
   if(($args=get_argument_names("clear", "Data"))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for Data::clear");
    return UT_FAILED;
   }

   if($args!=Array())
   {
    $this->e->add("Argument names don't match for Data::clear");
    return UT_FAILED;
   }

   //non-existent function
   if(($args=get_argument_names("lsaglkwjgljlsjsag", "Data"))!==false)
   {
    $this->e->add("Get argument names should have failed for Data::lsaglkwjgljlsjsag");
    return UT_FAILED;
   }

   //object
   $d=new Data();

   //1 arg
   if(($args=get_argument_names("setOptions", $d))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for \$Data->setOptions");
    return UT_FAILED;
   }

   if($args!=Array("options"))
   {
    $this->e->add("Argument names don't match for \$Data->setOptions");
    return UT_FAILED;
   }

   //multi args
   if(($args=get_argument_names("postInit", $d))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for \$Data->postInit");
    return UT_FAILED;
   }

   if($args!=Array("extendedModules", "initializedReferences"))
   {
    $this->e->add("Argument names don't match for \$Data->postInit");
    return UT_FAILED;
   }

   //no args
   if(($args=get_argument_names("clear", $d))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for \$Data->clear");
    return UT_FAILED;
   }

   if($args!=Array())
   {
    $this->e->add("Argument names don't match for \$Data->clear");
    return UT_FAILED;
   }

   //non-existent function
   //1 arg
   if(($args=get_argument_names("get_argument_names", false, true))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for get_argument_names");
    return UT_FAILED;
   }

   if($args!=Array("function"))
   {
    $this->e->add("Argument names don't match for array_type");
    return UT_FAILED;
   }

   //no args
   if(($args=get_argument_names("modules", "JAS", true))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for JAS::modules");
    return UT_FAILED;
   }

   if($args!=Array())
   {
    $this->e->add("Argument names don't match for JAS::modules");
    return UT_FAILED;
   }

   //multi args
   if(($args=get_argument_names("addField", $d, true))===false)
   {
    $this->e->add("Get argument names shouldn't have failed for \$Data->addField");
    return UT_FAILED;
   }

   if($args!=Array("name", "type"))
   {
    $this->e->add("Argument names don't match for \$Data->addField");
    return UT_FAILED;
   }

   //non-existent function
   if(($args=get_argument_names("lsaglkwjgljlsjsag", false, true))!==false)
   {
    $this->e->add("Get argument names should have failed for lsaglkwjgljlsjsag");
    return UT_FAILED;
   }



   return UT_OK;
  }

 }


 class UT_Array_Type_function extends UT_Functions
 {
  function Indexed()
  {
   $array=Array();
   $array[]="123";
   $array[]="546";
   $array[]="asdsa";
   $array[]="dfg";

   if(array_type($array)!=="indexed")
    return UT_FAILED;

   return UT_OK;
  }
  function Associative()
  {
   $array=Array();
   $array["asd"]="123";
   $array["354"]="546";
   $array["h"]="asdsa";
   $array["123"]="dfg";

   if(array_type($array)!=="associative")
    return UT_FAILED;

   return UT_OK;
  }
  function Data()
  {
   $array=new Data("categories");

   if(array_type($array)!=="data")
    return UT_FAILED;

   return UT_OK;
  }
  function Unknown()
  {
   $array=new Plugin("unit_testing");

   if(array_type($array)!=="unknown")
    return UT_FAILED;

   return UT_OK;
  }

 }


 class UT_Array_Remove_function extends UT_Functions
 {
  function Indexed()
  {
   //first
   $array=Array("123", "546", "asd", "dfg");
   $new=array_remove($array, 123);
   $array=Array("546", "asd", "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //last
   $array=Array("123", "546", "asd", "dfg");
   $new=array_remove($array, "dfg");
   $array=Array("123", "546", "asd");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //middle
   $array=Array("123", "546", "asd", "dfg");
   $new=array_remove($array, "asd");
   $array=Array("123", "546", "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   $array=Array("123", "546", "asd", "dfg");
   $new=array_remove($array, "546");
   $array=Array("123", "asd", "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //doesn't exist
   $array=Array("123", "546", "asd", "dfg");
   $new=array_remove($array, "asldfkslkfjlskadfasdf");
   $array=Array("123", "546", "asd", "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   return UT_OK;
  }
  function Associative()
  {
   //first
   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");
   $new=array_remove($array, 123);
   $array=Array("354" => "546", "h" => "asd", "123" => "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //last
   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");
   $new=array_remove($array, "dfg");
   $array=Array("asd" => "123", "354" => "546", "h" => "asd");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //middle
   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");
   $new=array_remove($array, "asd");
   $array=Array("asd" => "123", "354" => "546", "123" => "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");
   $new=array_remove($array, "546");
   $array=Array("asd" => "123", "h" => "asd", "123" => "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   //doesn't exist
   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");
   $new=array_remove($array, "asldfkslkfjlskadfasdf");
   $array=Array("asd" => "123", "354" => "546", "h" => "asd", "123" => "dfg");

   if(!$this->_compare($array, $new))
    return UT_FAILED;

   return UT_OK;
  }

 }
?>
