<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/mime.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."mime.php");

 class UT_Mime extends UT_Classes
 {
  function Constructor()
  {
   $object=new Mime();

   if(!is_object($object)||get_class($object)!="Mime")
   {
    $this->e->add("Constructor doesn't return a Mime object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Mime();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Mime object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->new="test";
   if(isset($orig->new))
   {
    $this->e->add("Mime object copy also modified original \"new\" variable", "error");
    return UT_FAILED;
   }

   $copy->d->format=Array();
   if($orig->d->format==$copy->d->format)
   {
    $this->e->add("Mime object copy also modified original \"Data\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Mime object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $orig->e->errors=-1;
   if($orig->e->errors!=$orig->d->e->errors)
   {
    $this->e->add("Mime object original didn't modify original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-2;
   if($copy->e->errors!=$copy->d->e->errors)
   {
    $this->e->add("Mime object copy didn't modify copy \"Error\" object", "error");
    return UT_FAILED;
   }

   //data
   $orig=new Mime();
   $orig->d->setValue("mime", "test 1", 0);
   $orig->d->setValue("mime", "test 2", 1);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Mime object copy doesn't match original", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ReadDB()
  {
   $mime=new Mime();
   $mime->d->clearValues();

   if(!$mime->readDB())
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if(!$mime->d->vmax)
   {
    $this->e->add("Mime database empty", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Filename()
  {
   $mime=new Mime();

   if(($type=$mime->filename("jas.txt"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/plain")
   {
    $this->e->add("Filename mime type doesn't match text/plain", "error");
    return UT_FAILED;
   }

   if(($type=$mime->filename("JAS.PDF"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="application/pdf")
   {
    $this->e->add("Filename mime type doesn't match application/pdf", "error");
    return UT_FAILED;
   }

   if(($type=$mime->filename("jas.ASDASDASDASDASDASDASD"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="application/octet-stream")
   {
    $this->e->add("Filename mime type doesn't match application/octet-stream", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Extension()
  {
   $mime=new Mime();

   if(($type=$mime->extension("html"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/html")
   {
    $this->e->add("Extension mime type doesn't match text/html", "error");
    return UT_FAILED;
   }

   if(($type=$mime->extension("JPEG"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="image/jpg")
   {
    $this->e->add("Extension mime type doesn't match image/jpg", "error");
    return UT_FAILED;
   }

   if(($type=$mime->extension("ASDASDASDASDASDASDASD"))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="application/octet-stream")
   {
    $this->e->add("Extension mime type doesn't match application/octet-stream", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Content()
  {
   global $_path;
   $mime=new Mime();

   $xmlFile=$_path["include"]."mime.xml";
   $pngFile=$_path["images"]."imagenotfound.png";

   $contents=file_get_contents($xmlFile);
   if(($type=$mime->content($contents))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/xml")
   {
    $this->e->add("Content mime type doesn't match text/xml", "error");
    return UT_FAILED;
   }

   $contents=file_get_contents($pngFile);
   if(($type=$mime->content($contents))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="image/png")
   {
    $this->e->add("Content mime type doesn't match image/png", "error");
    return UT_FAILED;
   }

   $content="Test content";
   if(($type=$mime->content($content))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/plain")
   {
    $this->e->add("Content mime type doesn't match text/plain", "error");
    return UT_FAILED;
   }

   $content=pack("c", "Test content");
   if(($type=$mime->content($content))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="application/octet-stream")
   {
    $this->e->add("Content mime type doesn't match application/octet-stream", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function File()
  {
   global $_path;
   $mime=new Mime();

   $xmlFile=$_path["include"]."mime.xml";
   $pngFile=$_path["images"]."imagenotfound.png";

   if(($type=$mime->file($xmlFile))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/xml")
   {
    $this->e->add("File mime type doesn't match text/xml", "error");
    return UT_FAILED;
   }
   if(($type=$mime->file($xmlFile, file_get_contents($xmlFile)))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="text/xml")
   {
    $this->e->add("File mime type doesn't match text/xml", "error");
    return UT_FAILED;
   }

   if(($type=$mime->file($pngFile))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="image/png")
   {
    $this->e->add("File mime type doesn't match image/png", "error");
    return UT_FAILED;
   }
   if(($type=$mime->file($pngFile, file_get_contents($pngFile)))===false)
   {
    $this->e->add($mime->e);
    return UT_FAILED;
   }
   if($type!="image/png")
   {
    $this->e->add("File mime type doesn't match image/png", "error");
    return UT_FAILED;
   }

   if($mime->file("ASDASDASDASDASDASDASD"))
   {
    $this->e->add("File mime type shouldn't be able to read: \"ASDASDASDASDASDASDASD\"", "error");
    return UT_FAILED;
   }
   return UT_OK;
  }

 }

?>
