<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/mp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 if(PHP_SAPI=="cli") //command line: setup environment for run below
 {
  ini_set("error_reporting", E_ALL & ~E_STRICT);

  $_path["root"]="../../../../";
  require($_path["root"]."include.php");
 }

 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."mp.php");

 declare(ticks=1); //XXX: redeclare (already done in mp.php)

 class TestSignals
 {
  function testSignal()
  {
   global $_TestSignals;

   if(!is_array($_TestSignals))
    $_TestSignals=Array();

   $_TestSignals[]=Array(
    "object" => &$this,
    "args" => func_get_args(),
   );
  }
 }

 class UT_MP extends UT_Classes
 {
  function Constructor()
  {
   $object=new MP();

   if(!is_object($object)||get_class($object)!="MP")
   {
    $this->e->add("Constructor doesn't return a Module object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->messageBusDir!="/tmp/")
   {
    $this->e->add("Message bus directory not set correctly", "error");
    return UT_FAILED;
   }

   if($object->messageBusPrefix!="jas_bus-")
   {
    $this->e->add("Message bus prefix not set correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new MP();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("MP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->messageBusDir="asd";
   if($orig->messageBusDir==$copy->messageBusDir)
   {
    $this->e->add("MP object copy also modified original \"messageBusDir\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("MP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $mp=new MP();
   $mp->pid=-123;
   $mp->parent=123;
   $mp->childs=Array(123, 456);
   $mp->sockets=Array(new Socket());
   $mp->signalHandlers=Array(Array("test"));

   $mp->clear();

   if($mp->pid!=posix_getpid())
   {
    $this->e->add("Clear should have set pid", "error");
    return UT_FAILED;
   }

   if($mp->parent)
   {
    $this->e->add("Clear should have unset parent", "error");
    return UT_FAILED;
   }

   if($mp->childs)
   {
    $this->e->add("Clear should have unset childs", "error");
    return UT_FAILED;
   }

   if($mp->sockets)
   {
    $this->e->add("Clear should have unset sockets", "error");
    return UT_FAILED;
   }

   if($mp->signalHandlers)
   {
    $this->e->add("Clear should have unset signalHandlers", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ConvertSignal()
  {
   $mp=new MP();

   if(PHP_SAPI!="cli")
   {
    if($mp->convertSignal("hup"))
    {
     $this->e->add("Convert signal should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   $signals=Array("hup", "int", "quit", "ill", "abrt", "fpe", "kill", "segv", "pipe", "alrm", "term", "usr1", "usr2", "chld", "cont", "stop");

   //convert string to int
   foreach($signals as $signal)
   {
    $sigNr=eval("return SIG".strtoupper($signal).";");
    if($mp->convertSignal($signal)!=$sigNr)
    {
     $this->e->add("Convert signal doesn't match expected output: ".$sigNr, "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   //convert int to string
   foreach($signals as $signal)
   {
    $sigNr=eval("return SIG".strtoupper($signal).";");
    if($mp->convertSignal($sigNr)!=$signal)
    {
     $this->e->add("Convert signal doesn't match expected output: ".$signal, "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function SetSignal()
  {
   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    if($mp->setSignal("hup", $ts, "testSignal"))
    {
     $this->e->add("Set signal should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   //catch signal (no handling)
   $handlers=Array();
   $handlers[SIGINT]=Array();

   if(!$mp->setSignal("interrupt"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$this->_compare($handlers, $mp->signalHandlers))
    return UT_FAILED;

   //method
   $handlers[SIGHUP][]=Array("object" => &$ts, "method" => "testSignal", "arguments" => NULL);

   if(!$mp->setSignal("hup", $ts, "testSignal"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$this->_compare($handlers, $mp->signalHandlers))
    return UT_FAILED;

   //method arguments
   $handlers[SIGHUP][]=Array("object" => &$ts, "method" => "testSignal", "arguments" => Array("test", 2, &$ts));
   if(!$mp->setSignal(SIGHUP, $ts, "testSignal", Array("test", 2, &$ts)))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$this->_compare($handlers, $mp->signalHandlers))
    return UT_FAILED;

   //sigsegv
   $handlers[SIGSEGV][]=Array("object" => &$ts, "method" => "testSignal", "arguments" => Array("segfault"));
   if(!$mp->setSignal("segfault", $ts, "testSignal", "segfault"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$this->_compare($handlers, $mp->signalHandlers))
    return UT_FAILED;

   //fail
   if($mp->setSignal("hup", $ts, "lhasdjflasdjlsajdf"))
   {
    $this->e->add("Set signal should have failed", "error");
    return UT_FAILED;
   }

   if($mp->setSignal("lhasdjflasdjlsajdf", $ts, "testSignal"))
   {
    $this->e->add("Set signal should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function UnsetSignal()
  {
   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    if($mp->unsetSignal("hup"))
    {
     $this->e->add("Unset signal should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   if(!$mp->setSignal("interrupt"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$mp->setSignal("hup", $ts, "testSignal"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$mp->setSignal(SIGHUP, $ts, "testSignal", Array("test", 2, &$ts)))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$mp->setSignal("segfault", $ts, "testSignal", "segfault"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(!$mp->unsetSignal("interrupt"))
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(isset($mp->signalHandlers[SIGINT]))
   {
    $this->e->add("Unset signal should have unset signal handlers", "error");
    return UT_FAILED;
   }

   if(!$mp->unsetSignal("hup"))
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(isset($mp->signalHandlers[SIGHUP]))
   {
    $this->e->add("Unset signal should have unset signal handlers", "error");
    return UT_FAILED;
   }

   if(!$mp->unsetSignal(SIGSEGV))
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   if(isset($mp->signalHandlers[SIGSEGV]))
   {
    $this->e->add("Unset signal should have unset signal handlers", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function HandleSignal()
  {
   global $_TestSignals;

   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   if(!$mp->setSignal("interrupt"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $_TestSignals=Array();
   $mp->handleSignal(SIGINT);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("hup", $ts, "testSignal"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array());

   $_TestSignals=Array();
   $mp->handleSignal(SIGHUP);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal(SIGHUP, $ts, "testSignal", Array("test", 2, &$ts)))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("test", 2, &$ts));

   $_TestSignals=Array();
   $mp->handleSignal(SIGHUP);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("segfault", $ts, "testSignal", "segfault"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   $mp->handleSignal(SIGHUP);
   $mp->handleSignal(SIGSEGV);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal("hup"))
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   $mp->handleSignal(SIGHUP);
   $mp->handleSignal(SIGSEGV);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal(SIGSEGV, true)) //ignore
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();

   $_TestSignals=Array();
   $mp->handleSignal(SIGHUP);
   $mp->handleSignal(SIGSEGV);

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   return UT_OK;
  }

  function HandleSignalSend()
  {
   global $_TestSignals;

   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   if(!$mp->setSignal("interrupt"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGINT))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("hup", $ts, "testSignal"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array());

   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGHUP))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal(SIGHUP, $ts, "testSignal", Array("test", 2, &$ts)))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("test", 2, &$ts));

   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGHUP))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("segfault", $ts, "testSignal", "segfault"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGHUP))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!posix_kill(posix_getpid(), SIGSEGV))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal("hup", true)) //ignore
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGHUP))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!posix_kill(posix_getpid(), SIGSEGV))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal(SIGSEGV, true)) //ignore
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();

   $_TestSignals=Array();
   if(!posix_kill(posix_getpid(), SIGHUP))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!posix_kill(posix_getpid(), SIGSEGV))
   {
    $this->e->add("Sending kill signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   $mp->unsetSignal(SIGHUP); //restore
   $mp->unsetSignal(SIGSEGV); //restore

   return UT_OK;
  }

  function SendSignal()
  {
   global $_TestSignals;

   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   if(!$mp->setSignal("interrupt"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGINT))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("hup", $ts, "testSignal"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array());

   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGHUP))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal(SIGHUP, $ts, "testSignal", Array("test", 2, &$ts)))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("test", 2, &$ts));

   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGHUP))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->setSignal("segfault", $ts, "testSignal", "segfault"))
   {
    $this->e->add("Set signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGHUP))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$mp->sendSignal(SIGSEGV))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal("hup", true)) //ignore
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();
   $TestSignals[]=Array("object" => &$ts, "args" => Array("segfault"));

   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGHUP))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$mp->sendSignal(SIGSEGV))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   if(!$mp->unsetSignal(SIGSEGV, true)) //ignore
   {
    $this->e->add("Unset signal shouldn't have failed", "error");
    $this->e->add($mp->e);
    return UT_FAILED;
   }

   $TestSignals=Array();

   $_TestSignals=Array();
   if(!$mp->sendSignal(SIGHUP))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$mp->sendSignal(SIGSEGV))
   {
    $this->e->add("Sending signal shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($TestSignals, $_TestSignals))
    return UT_FAILED;

   $mp->unsetSignal(SIGHUP); //restore
   $mp->unsetSignal(SIGSEGV); //restore

   return UT_OK;
  }

  function Fork()
  {
   $mp=new MP();

   if(PHP_SAPI!="cli")
   {
    if($mp->fork())
    {
     $this->e->add("Fork should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   $pid=posix_getpid();

   switch($mp->fork())
   {
    case "child":
    {
     if($mp->pid!=posix_getpid())
      exit(1);

     if($mp->parent!=$pid)
      exit(2);

     exit(254);
    }
    case "parent":
    {
     if($mp->pid!=posix_getpid())
     {
      $this->e->add("Fork parent pid doesn't match", "error");
      return UT_FAILED;
     }

     if(!$mp->childs)
     {
      $this->e->add("Fork parent doesn't have childs", "error");
      return UT_FAILED;
     }

     $status=false;
     pcntl_waitpid($mp->childs[0], $status);
     switch(pcntl_wexitstatus($status))
     {
      case 1:
      {
       $this->e->add("Fork child pid doesn't match", "error");
       return UT_FAILED;
      }
      case 2:
      {
       $this->e->add("Fork child parent pid doesn't match", "error");
       return UT_FAILED;
      }
      case 254:
       break; //success
      default:
      {
       $this->e->add("Fork child exit value doesn't match", "error");
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   $mp->clear();


   //message bus
   switch($mp->fork(true))
   {
    case "child":
    {
     if(!isset($mp->sockets[$mp->pid]))
      exit(1);

     if($mp->sockets[$mp->pid]->file!=$mp->messageBusDir.$mp->messageBusPrefix.$mp->pid)
      exit(1);

     exit(254);
    }
    case "parent":
    {
     $child=$mp->childs[0];
     if(!isset($mp->sockets[$child]))
     {
      $this->e->add("Fork parent message bus not setup", "error");
      return UT_FAILED;
     }

     if($mp->sockets[$child]->file!=$mp->messageBusDir.$mp->messageBusPrefix.$child)
     {
      $this->e->add("Fork parent message bus socket doesn't match", "error");
      return UT_FAILED;
     }

     $status=false;
     pcntl_waitpid($child, $status);
     switch(pcntl_wexitstatus($status))
     {
      case 1:
      {
       $this->e->add("Fork child message bus not setup", "error");
       return UT_FAILED;
      }
      case 1:
      {
       $this->e->add("Fork child message bus socket doesn't match", "error");
       return UT_FAILED;
      }
      case 254:
       break; //success
      default:
      {
       $this->e->add("Fork child exit value doesn't match", "error");
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function ReadWrite()
  {
   $mp=new MP();

   if(PHP_SAPI!="cli")
   {
    if($mp->read())
    {
     $this->e->add("Read should have failed", "error");
     return UT_FAILED;
    }

    if($mp->write("test"))
    {
     $this->e->add("Write should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   switch($mp->fork(true))
   {
    case "child":
    {
     if(!$mp->sockets[$mp->pid]->setTimeout(1))
      exit(1);

     if($mp->read("parent")!="ping")
      exit(2);

     if(!$mp->write("pong", "parent"))
      exit(3);

     exit(254);
    }
    case "parent":
    {
     $child=$mp->childs[0];

     if(!$mp->sockets[$child]->setTimeout(1))
     {
      $this->e->add("Read/Write parent set timeout failed", "error");
      $this->e->add($mp->sockets[$child]->e);
      return UT_FAILED;
     }

     if(!$mp->write("ping", 0))
     {
      $this->e->add("Read/Write parent write to child failed", "error");
      $this->e->add($mp->e);
      return UT_FAILED;
     }

     if($mp->read(0)!="pong")
     {
      $this->e->add("Read/Write parent read from child failed", "error");
      $this->e->add($mp->e);
      return UT_FAILED;
     }

     $status=false;
     pcntl_waitpid($child, $status);
     switch(pcntl_wexitstatus($status))
     {
      case 1:
      {
       $this->e->add("Read/Write child set read timeout failed", "error");
       return UT_FAILED;
      }
      case 2:
      {
       $this->e->add("Read/Write child read from parent failed", "error");
       return UT_FAILED;
      }
      case 3:
      {
       $this->e->add("Read/Write child write to parent failed", "error");
       return UT_FAILED;
      }
      case 254:
       break; //success
      default:
      {
       $this->e->add("Read/Write child exit value doesn't match", "error");
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Read/Write shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   $mp->clear();


   //multiple childs
   switch($mp->fork(true))
   {
    case "child":
    {
     if(!$mp->sockets[$mp->pid]->setTimeout(1))
      exit(1);

     if($mp->read("all")!=Array("parent" => "ping"))
      exit(2);

     if(!$mp->write("pong1", "parent"))
      exit(3);

     exit(254);
    }
    case "parent":
    {
     switch($mp->fork(true))
     {
      case "child":
      {
       if(!$mp->sockets[$mp->pid]->setTimeout(1))
        exit(1);

       if($mp->read("all")!=Array("parent" => "ping"))
        exit(2);

       usleep(10000); //ensure pong is sent later
       if(!$mp->write("pong2", "parent"))
        exit(3);

       exit(253);
      }
      case "parent":
      {
       foreach($mp->childs as $child)
       {
        if(!$mp->sockets[$child]->setTimeout(1))
        {
         $this->e->add("Read/Write parent set timeout failed", "error");
         $this->e->add($mp->sockets[$child]->e);
         return UT_FAILED;
        }
       }

       if(!$mp->write("ping", "childs"))
       {
        $this->e->add("Read/Write parent write to childs failed", "error");
        $this->e->add($mp->e);
        return UT_FAILED;
       }

       usleep(20000); //wait for pongs
       if($mp->read("childs")!=Array("pong1", "pong2"))
       {
        $this->e->add("Read/Write parent read from childs failed", "error");
        $this->e->add($mp->e);
        return UT_FAILED;
       }

       foreach($mp->childs as $child)
       {
        $status=false;
        pcntl_waitpid($child, $status);
        switch(pcntl_wexitstatus($status))
        {
         case 1:
         {
          $this->e->add("Read/Write child ".$child." set read timeout failed", "error");
          return UT_FAILED;
         }
         case 2:
         {
          $this->e->add("Read/Write child ".$child." read from parent failed", "error");
          return UT_FAILED;
         }
         case 3:
         {
          $this->e->add("Read/Write child ".$child." write to parent failed", "error");
          return UT_FAILED;
         }
         case 253:
         case 254:
          break; //success
         default:
         {
          $this->e->add("Read/Write child ".$child." exit value doesn't match", "error");
          return UT_FAILED;
         }
        }
       }

       break;
      }
      default: //false
      {
       $this->e->add("Read/Write 2 shouldn't have failed", "error");
       $this->e->add($mp->e);
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Read/Write 1 shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function Close()
  {
   $mp=new MP();

   if(PHP_SAPI!="cli")
   {
    if($mp->close())
    {
     $this->e->add("Close should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   switch($mp->fork(true))
   {
    case "child":
    {
     if(!$mp->close("parent"))
      exit(1);

     exit(254);
    }
    case "parent":
    {
     $child=$mp->childs[0];

     if(!$mp->close(0))
     {
      $this->e->add("Fork parent close failed", "error");
      $this->e->add($mp->e);
      return UT_FAILED;
     }

     $status=false;
     pcntl_waitpid($child, $status);
     switch(pcntl_wexitstatus($status))
     {
      case 1:
      {
       $this->e->add("Fork child close failed", "error");
       return UT_FAILED;
      }
      case 254:
       break; //success
      default:
      {
       $this->e->add("Fork child exit value doesn't match", "error");
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   $mp->clear();


   //multiple childs
   switch($mp->fork(true))
   {
    case "child":
    {
     if(!$mp->close("all"))
      exit(1);

     exit(254);
    }
    case "parent":
    {
     switch($mp->fork(true))
     {
      case "child":
      {
       if(!$mp->close("all"))
        exit(1);

       exit(253);
      }
      case "parent":
      {
       if(!$mp->close("childs"))
       {
        $this->e->add("Fork parent close failed", "error");
        $this->e->add($mp->e);
        return UT_FAILED;
       }

       foreach($mp->childs as $child)
       {
        $status=false;
        pcntl_waitpid($child, $status);
        switch(pcntl_wexitstatus($status))
        {
         case 1:
         {
          $this->e->add("Fork child ".$child." close failed", "error");
          return UT_FAILED;
         }
         case 253:
         case 254:
          break; //success
         default:
         {
          $this->e->add("Fork child ".$child." exit value doesn't match", "error");
          return UT_FAILED;
         }
        }
       }

       break;
      }
      default: //false
      {
       $this->e->add("Fork 2 shouldn't have failed", "error");
       $this->e->add($mp->e);
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork 1 shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function Wait()
  {
   $mp=new MP();

   if(PHP_SAPI!="cli")
   {
    if($mp->wait())
    {
     $this->e->add("Wait should have failed", "error");
     return UT_FAILED;
    }

    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }

   switch($mp->fork(true))
   {
    case "child":
    {
     exit(254);
    }
    case "parent":
    {
     $child=$mp->childs[0];

     if($mp->wait(0)!=254)
     {
      $this->e->add("Fork parent wait exit value doesn't match", "error");
      $this->e->add($mp->e);
      return UT_FAILED;
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   $mp->clear();


   //multiple childs
   switch($mp->fork(true))
   {
    case "child":
    {
     exit(254);
    }
    case "parent":
    {
     switch($mp->fork(true))
     {
      case "child":
      {
       usleep(10000); //ensure close is later
       exit(253);
      }
      case "parent":
      {
       if($mp->close("childs")!=Array(254, 253))
       {
        $this->e->add("Fork parent wait exit values doesn't match", "error");
        $this->e->add($mp->e);
        return UT_FAILED;
       }

       break;
      }
      default: //false
      {
       $this->e->add("Fork 2 shouldn't have failed", "error");
       $this->e->add($mp->e);
       return UT_FAILED;
      }
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork 1 shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function Destructor()
  {
   $mp=new MP();
   $ts=new TestSignals();

   if(PHP_SAPI!="cli")
   {
    $this->e->add("Please run tests from cli", "warning", " php./mp.php");
    return UT_SKIPPED;
   }


   switch($mp->fork(true))
   {
    case "child":
    {
     exit(254);
    }
    case "parent":
    {
     $child=$mp->childs[0];

     $sock=&$mp->sockets[$child]->socket;
     unset($mp);

     if($sock!==false)
     {
      $this->e->add("Socket should have been close");
      return UT_FAILED;
     }

     break;
    }
    default: //false
    {
     $this->e->add("Fork shouldn't have failed", "error");
     $this->e->add($mp->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }
 }


 if(PHP_SAPI=="cli") //command line: execute tests
 {
  $ut=new UT_MP();
  $ut->_initTests();
  $results=$ut->_run();

  foreach($results["tests"] as $test => $result)
  {
   print $test.": ";

   if($result["status"]==0)
    print "FAILURE";
   else if($result["status"]==-3)
    print "Skipped";

   print "\n";


   for($c=0;$c<$result["e"]->getErrors();$c++)
    print " ".$result["e"]->getLevel($c).": ".$result["e"]->getError($c)."\n";

   print "\n";

  }
 }
?>
