<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/socket.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."socket.php");

 class UT_Socket extends UT_Classes
 {
  function Constructor()
  {
   $object=new Socket();

   if(!is_object($object)||get_class($object)!="Socket")
   {
    $this->e->add("Constructor doesn't return a Socket object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!$object->ipv4)
   {
    $this->e->add("Object ipv4 not enabled", "error");
    return UT_FAILED;
   }

   if(!$object->ipv6)
   {
    $this->e->add("Object ipv6 not enabled", "error");
    return UT_FAILED;
   }

   if(!$object->blocking)
   {
    $this->e->add("Object blocking not enabled", "error");
    return UT_FAILED;
   }

   if($object->timeout[0]!==NULL)
   {
    $this->e->add("Object timeout seconds not null", "error");
    return UT_FAILED;
   }

   if($object->timeout[1]!==NULL)
   {
    $this->e->add("Object timeout microseconds not null", "error");
    return UT_FAILED;
   }

   if($object->backlog!=10)
   {
    $this->e->add("Object backlog doesn't match", "error");
    return UT_FAILED;
   }

   if($object->reread!=Array(
    "enabled" => false,
    "timeout" => 0.0001,
    "counter" => 0,
   ))
   {
    $this->e->add("Object reread options don't match", "error");
    return UT_FAILED;
   }

   if($object->readBytes!=1024)
   {
    $this->e->add("Object read bytes doesn't match", "error");
    return UT_FAILED;
   }

   if($object->readType!=PHP_BINARY_READ)
   {
    $this->e->add("Object read type doesn't match", "error");
    return UT_FAILED;
   }

   if($object->connectionTypes!=Array("tcp", "udp", "file"))
   {
    $this->e->add("Object connection types don't match", "error");
    return UT_FAILED;
   }

   if($object->socket)
   {
    $this->e->add("Object socket set", "error");
    return UT_FAILED;
   }

   if($object->socket6)
   {
    $this->e->add("Object socket6 set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Socket();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Socket object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->ipv4=false;
   if($orig->ipv4==$copy->ipv4)
   {
    $this->e->add("Socket object copy also modified original \"ipv4\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Socket object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   //connected
   $orig=new Socket();
   $orig->connect("localhost:80");
   $copy=clone $orig;

   if($this->_compare($orig, $copy))
   {
    $this->e->add("Socket object copy shouldn't match original", "error");
    return UT_FAILED;
   }

   if($copy->socket!==false)
   {
    $this->e->add("Socket object copy \"socket\" variable should be reset", "error");
    return UT_FAILED;
   }

   $copy->ipv4=false;
   if($orig->ipv4==$copy->ipv4)
   {
    $this->e->add("Socket object copy also modified original \"ipv4\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Socket object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->socket="resource";
   if($orig->socket==$copy->socket)
   {
    $this->e->add("Socket object copy also modified original \"socket\" variable", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $s=new Socket();
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   $keys=Array("mode", "file", "host", "port", "shost", "sport", "bytesSend", "bytesReceived", "connectionTime", "startTime", "errno", "error");
   foreach($keys as $key)
    $s->$key=$key;

   $s->clear();

   if($s->socket!==false)
   {
    $this->e->add("Clear left socket set", "error");
    return UT_FAILED;
   }

   foreach($keys as $key)
   {
    if($s->$key)
    {
     $this->e->add("Clear left ".$key." set", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function Error()
  {
   socket_clear_error();

   $s=new Socket();
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   if($s->error()!==0)
   {
    $this->e->add("Error should have returned no error", "error");
    return UT_FAILED;
   }

   if($s->errno!==0)
   {
    $this->e->add("Error should have set no errno", "error");
    return UT_FAILED;
   }

   if($s->error!=="")
   {
    $this->e->add("Error should have set no error", "error");
    return UT_FAILED;
   }


   //fail
   @socket_read($s->socket, 1);

   if($s->error()===0)
   {
    $this->e->add("Error should have returned an error", "error");
    return UT_FAILED;
   }

   if($s->errno===0)
   {
    $this->e->add("Error should have set errno", "error");
    return UT_FAILED;
   }

   if($s->error==="")
   {
    $this->e->add("Error should have set an error", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Inet()
  {
   $s=new Socket();

   if(!$s->inet(true, true))
   {
    $this->e->add("Inet shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->ipv4)
   {
    $this->e->add("Inet should have set ipv4", "error");
    return UT_FAILED;
   }

   if(!$s->ipv6)
   {
    $this->e->add("Inet should have set ipv6", "error");
    return UT_FAILED;
   }

   if(!$s->inet(true, false))
   {
    $this->e->add("Inet shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->ipv4)
   {
    $this->e->add("Inet should have set ipv4", "error");
    return UT_FAILED;
   }

   if($s->ipv6)
   {
    $this->e->add("Inet should have set ipv6", "error");
    return UT_FAILED;
   }

   if(!$s->inet(false, true))
   {
    $this->e->add("Inet shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->ipv4)
   {
    $this->e->add("Inet should have set ipv4", "error");
    return UT_FAILED;
   }

   if(!$s->ipv6)
   {
    $this->e->add("Inet should have set ipv6", "error");
    return UT_FAILED;
   }


   //no v4 and v6
   $s->inet(true, true);
   if($s->inet(false, false))
   {
    $this->e->add("Inet should have failed", "error");
    return UT_FAILED;
   }

   if(!$s->ipv4)
   {
    $this->e->add("Inet shouldn't have set ipv4", "error");
    return UT_FAILED;
   }

   if(!$s->ipv6)
   {
    $this->e->add("Inet shouldn't have set ipv6", "error");
    return UT_FAILED;
   }

   //active socket
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);
   if($s->inet(true, false))
   {
    $this->e->add("Inet should have failed", "error");
    return UT_FAILED;
   }

   if(!$s->ipv4)
   {
    $this->e->add("Inet shouldn't have set ipv4", "error");
    return UT_FAILED;
   }

   if(!$s->ipv6)
   {
    $this->e->add("Inet shouldn't have set ipv6", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Blocking()
  {
   $s=new Socket();

   if(!$s->setBlocking(true))
   {
    $this->e->add("Blocking shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->blocking)
   {
    $this->e->add("Blocking should have set blocking", "error");
    return UT_FAILED;
   }

   if(!$s->setBlocking(false))
   {
    $this->e->add("Blocking shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->blocking)
   {
    $this->e->add("Blocking should have set blocking", "error");
    return UT_FAILED;
   }

   //active socket
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   if(!$s->setBlocking(true))
   {
    $this->e->add("Blocking shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->blocking)
   {
    $this->e->add("Blocking should have set blocking", "error");
    return UT_FAILED;
   }

   if(!$s->setBlocking(false))
   {
    $this->e->add("Blocking shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->blocking)
   {
    $this->e->add("Blocking should have set blocking", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetTimeout()
  {
   $s=new Socket();

   if(!$s->setTimeout(123))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->timeout[0]!==123)
   {
    $this->e->add("SetTimeout should have set seconds", "error");
    return UT_FAILED;
   }

   if($s->timeout[1]!==0)
   {
    $this->e->add("SetTimeout should have set microseconds", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(1.1))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->timeout[0]!==1)
   {
    $this->e->add("SetTimeout should have set seconds", "error");
    return UT_FAILED;
   }

   if($s->timeout[1]!==100000)
   {
    $this->e->add("SetTimeout should have set microseconds", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(0.000001))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->timeout[0]!==0)
   {
    $this->e->add("SetTimeout should have set seconds", "error");
    return UT_FAILED;
   }

   if($s->timeout[1]!==1)
   {
    $this->e->add("SetTimeout should have set microseconds", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(0.0))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->timeout[0]!==0)
   {
    $this->e->add("SetTimeout should have set seconds", "error");
    return UT_FAILED;
   }

   if($s->timeout[1]!==0)
   {
    $this->e->add("SetTimeout should have set microseconds", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout())
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->timeout[0]!==NULL)
   {
    $this->e->add("SetTimeout should have unset seconds", "error");
    return UT_FAILED;
   }

   if($s->timeout[1]!==NULL)
   {
    $this->e->add("SetTimeout should have unset microseconds", "error");
    return UT_FAILED;
   }

   //fail
   if($s->setTimeout("asd"))
   {
    $this->e->add("SetTimeout should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetTimeout()
  {
   $s=new Socket();

   if(!$s->setTimeout(123))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getTimeout()!==123.0)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(1.1))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getTimeout()!==1.1)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(0.000001))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getTimeout()!==0.000001)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout(0.0))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getTimeout()!==0.0)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!$s->setTimeout())
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getTimeout()!==false)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   //fail
   if($s->setTimeout("asd"))
   {
    $this->e->add("SetTimeout should have failed", "error");
    return UT_FAILED;
   }

   if($s->getTimeout()!==false)
   {
    $this->e->add("GetTimeout doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetReRead()
  {
   $s=new Socket();

   if(!$s->setReRead(true))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["enabled"]!==true)
   {
    $this->e->add("SetReRead didn't set enabled flag to true", "error");
    return UT_FAILED;
   }

   if(!$s->setReRead(false))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["enabled"]!==false)
   {
    $this->e->add("SetReRead didn't set enabled flag to false", "error");
    return UT_FAILED;
   }


   if(!$s->setReRead(1))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["enabled"]!==true)
   {
    $this->e->add("SetReRead didn't set enabled flag to true", "error");
    return UT_FAILED;
   }

   if(!$s->setReRead(""))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["enabled"]!==false)
   {
    $this->e->add("SetReRead didn't set enabled flag to false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetReRead()
  {
   $s=new Socket();

   if(!$s->setReRead(true))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getReRead()!==true)
   {
    $this->e->add("GetReRead return true", "error");
    return UT_FAILED;
   }

   if(!$s->setReRead(false))
   {
    $this->e->add("SetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getReRead()!==false)
   {
    $this->e->add("GetReRead return false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ResetReRead()
  {
   $s=new Socket();

   if(!$s->resetReRead())
   {
    $this->e->add("ResetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["counter"]!==0)
   {
    $this->e->add("ResetReRead didn't reset counter0", "error");
    return UT_FAILED;
   }

   $s->reread["counter"]=123;

   if(!$s->resetReRead())
   {
    $this->e->add("ResetReRead shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->reread["counter"]!==0)
   {
    $this->e->add("ResetReRead didn't reset counter0", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function SetReadBytes()
  {
   $s=new Socket();

   if(!$s->setReadBytes(123))
   {
    $this->e->add("SetReadBytes shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->readBytes!=123)
   {
    $this->e->add("SetReadBytes should have set readBytes", "error");
    return UT_FAILED;
   }


   //fail
   if($s->setReadBytes("asd"))
   {
    $this->e->add("SetReadBytes should have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetReadType()
  {
   $s=new Socket();

   if(!$s->setReadType(PHP_BINARY_READ))
   {
    $this->e->add("SetReadType shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->readType!=PHP_BINARY_READ)
   {
    $this->e->add("SetReadType should have set readType", "error");
    return UT_FAILED;
   }

   if(!$s->setReadType("binary"))
   {
    $this->e->add("SetReadType shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->readType!=PHP_BINARY_READ)
   {
    $this->e->add("SetReadType should have set readType", "error");
    return UT_FAILED;
   }

   if(!$s->setReadType(PHP_NORMAL_READ))
   {
    $this->e->add("SetReadType shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->readType!=PHP_NORMAL_READ)
   {
    $this->e->add("SetReadType should have set readType", "error");
    return UT_FAILED;
   }

   if(!$s->setReadType("normal"))
   {
    $this->e->add("SetReadType shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->readType!=PHP_NORMAL_READ)
   {
    $this->e->add("SetReadType should have set readType", "error");
    return UT_FAILED;
   }


   //fail
   if($s->setReadType("asd"))
   {
    $this->e->add("SetReadType should have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetOption()
  {
   $s=new Socket();
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   if(!$s->setOption(SO_BROADCAST, 1))
   {
    $this->e->add("SetOption shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(@socket_get_option($s->socket, SOCK_STREAM, SO_BROADCAST)!==1)
   {
    $this->e->add("SetOption should have set broadcast", "error");
    return UT_FAILED;
   }


   //fail
   if($s->setOption("key", "value"))
   {
    $this->e->add("SetOption should have failed", "error");
    return UT_FAILED;
   }

   $s->close();

   if($s->setOption(SO_BROADCAST, 1))
   {
    $this->e->add("SetOption should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetOption()
  {
   $s=new Socket();
   $s->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   if(!$s->setOption(SO_BROADCAST, 1))
   {
    $this->e->add("SetOption shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->getOption(SO_BROADCAST)!==1)
   {
    $this->e->add("GetOption should have returned broadcast", "error");
    return UT_FAILED;
   }


   //fail
   if($s->getOption("key"))
   {
    $this->e->add("GetOption should have failed", "error");
    return UT_FAILED;
   }

   $s->close();

   if($s->getOption(SO_BROADCAST))
   {
    $this->e->add("GetOption should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function URL()
  {
   $s=new Socket();

   $connection=Array(
    "url" => "localhost",
    "protocol" => false,
    "user" => false,
    "pass" => false,
    "host" => "localhost",
    "port" => false,
    "path" => false,
    "values" => false,
   );
   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="tcp://127.0.0.1:21";
   $connection["protocol"]="tcp";
   $connection["host"]="127.0.0.1";
   $connection["port"]="21";
   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="http://hostname/";
   $connection["protocol"]="http";
   $connection["host"]="hostname";
   $connection["path"]="/";
   $connection["port"]=false;
   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="http://username:password@[fe80::1]:80";
   $connection["protocol"]="http";
   $connection["user"]="username";
   $connection["pass"]="password";
   $connection["host"]="fe80::1";
   $connection["path"]=false;
   $connection["port"]="80";
   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="http://localhost/dir/sub/file";
   $connection["protocol"]="http";
   $connection["user"]=false;
   $connection["pass"]=false;
   $connection["host"]="localhost";
   $connection["port"]=false;
   $connection["path"]="/dir/sub/file";

   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="http://localhost?key=value&t=1&t=2&test";
   $connection["protocol"]="http";
   $connection["host"]="localhost";
   $connection["path"]=false;
   $connection["values"]=Array(
    Array("key" => "key", "value" => "value"),
    Array("key" => "t",  "value" => "1"),
    Array("key" => "t",  "value" => "2"),
    Array("key" => "test", "value" => ""),
   );

   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="file://username@/dir/sub/file";
   $connection["protocol"]="file";
   $connection["user"]="username";
   $connection["host"]=false;
   $connection["path"]="/dir/sub/file";
   $connection["values"]=false;

   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   $connection["url"]="file://dir/sub/file?key=value&t[]=1&t[]=2&test";
   $connection["protocol"]="file";
   $connection["user"]=false;
   $connection["path"]="dir/sub/file";
   $connection["values"]=Array(
    Array("key" => "key", "value" => "value"),
    Array("key" => "t[]",  "value" => "1"),
    Array("key" => "t[]",  "value" => "2"),
    Array("key" => "test", "value" => ""),
   );

   if(!($c=$s->URL($connection["url"])))
   {
    $this->e->add("Socket URL shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($connection, $c))
    return UT_FAILED;


   return UT_OK;
  }


  function Connect()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);

   if(!$s->connect("tcp://localhost:80"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:80");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->mode!="connection")
   {
    $this->e->add("Connect set mode incorrectly");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $s->close();


   //no protocol (should assume first: tcp)
   if(!$s->connect("localhost:80"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:80");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->mode!="connection")
   {
    $this->e->add("Connect set mode incorrectly");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $s->close();



   //file
   $file="/var/run/mysql/mysql.sock";
   if(!file_exists($file))
   {
    $this->e->add("MySQL socket doesn't exist: ".$file);
    return UT_SKIPPED;
   }

   if(!$s->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    return UT_FAILED;
   }


   //fail (still connected)
   if($s->connect("tcp://localhost:80"))
   {
    $this->e->add("Connect should have failed to localhost:80");
    $this->e->add($s->e);
    $s->close();
    return UT_FAILED;
   }

   if($s->mode!="connection")
   {
    $this->e->add("Connect set mode incorrectly");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $s->close();

   return UT_OK;
  }

  function Listen()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->mode!="listen")
   {
    $this->e->add("Listen set mode incorrectly");
    $this->e->add($s->e);
    $s->close();
    return UT_FAILED;
   }

   if(!$s->socket)
   {
    $this->e->add("Listen socket not set");
    $s->close();
    return UT_FAILED;
   }

   if(!$s->socket6)
   {
    $this->e->add("Listen socket6 not set");
    $s->close();
    return UT_FAILED;
   }

   $s->close();

   //file
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if($s->mode!="listen")
   {
    $this->e->add("Listen set mode incorrectly");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $s->close();

   //drop priveledges
   if(!$s->listen("tcp://localhost:18080", posix_getuid(), posix_getgid())) //to current user
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $s->close();

   if($s->listen("tcp://localhost:18080", "nobody", "nogroup")) //to current user
   {
    $this->e->add("Listen should have failed to drop priveledges");
    $s->close();
    return UT_FAILED;
   }


   return UT_OK;
  }

  function Accept()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(true);
   $client->setTimeout(5);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if($server->mode!="connection")
   {
    $this->e->add("Connections mode doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->ipv4!=$server->ipv4)
   {
    $this->e->add("Connections ipv4 doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->ipv6!=$server->ipv6)
   {
    $this->e->add("Connections ipv6 doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->blocking!=$server->blocking)
   {
    $this->e->add("Connections blocking doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->timeout!=$server->timeout)
   {
    $this->e->add("Connections timeout doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->reread!=$server->reread)
   {
    $this->e->add("Connections reread doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->shost!=$server->shost)
   {
    $this->e->add("Connections shost doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->sport!=$server->sport)
   {
    $this->e->add("Connections sport doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //file
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->setTimeout(1.1))
   {
    $this->e->add("SetTimeout shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if($server->mode!="connection")
   {
    $this->e->add("Connections mode doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->ipv4!=$server->ipv4)
   {
    $this->e->add("Connections ipv4 doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->ipv6!=$server->ipv6)
   {
    $this->e->add("Connections ipv6 doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->blocking!=$server->blocking)
   {
    $this->e->add("Connections blocking doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->reread!=$server->reread)
   {
    $this->e->add("Connections reread doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->timeout!=$server->timeout)
   {
    $this->e->add("Connections timeout doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->shost!=$server->shost)
   {
    $this->e->add("Connections shost doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($s->sport!=$server->sport)
   {
    $this->e->add("Connections sport doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //fail
   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(($server=$s->accept()))
   {
    $this->e->add("Accept should have failed");
    $this->e->add($s->e);
    $s->close();
    $server->close();
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AcceptShutdown()
  {
   $s=new Socket();
   $s->acceptShutdown=false;

   $s->acceptShutdown();

   if(!$s->acceptShutdown)
   {
    $this->e->add("Accept shutdown should have been set");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(false); //ensure tests don't block
   $client->setTimeout(1);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   socket_write($client->socket, "test", 4);

   if(!($data=$server->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->bytesReceived!=4)
   {
    $this->e->add("Read bytes received doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   socket_write($server->socket, "test", 4);

   if(!($data=$client->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->bytesReceived!=4)
   {
    $this->e->add("Read bytes received doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //file
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   socket_write($client->socket, "test", 4);

   if(!($data=$server->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->bytesReceived!=4)
   {
    $this->e->add("Read bytes received doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   socket_write($server->socket, "test", 4);

   if(!($data=$client->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->bytesReceived!=4)
   {
    $this->e->add("Read bytes received doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //fail
   if($s->read())
   {
    $this->e->add("Read should have failed");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ReRead()
  {
   global $_path;

   $s=new Socket();
   $s->setReRead(true);
   $client=new Socket();

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   socket_write($client->socket, "test", 4);

   if(!($data=$server->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->reread["counter"]!==1)
   {
    $this->e->add("ReRead counter doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   socket_write($client->socket, "test2", 5);

   if(($data=$server->read())===false)
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->reread["counter"]!==2)
   {
    $this->e->add("ReRead counter doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test2")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$server->write("something")) //should reset reread data
   {
    $this->e->add("Write shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   socket_write($client->socket, "test3", 5);

   if(!($data=$server->read()))
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="test3")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->reread["counter"]!==1)
   {
    $this->e->add("ReRead counter doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(($data=$server->read())===false)
   {
    $this->e->add("Read shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($data!="")
   {
    $this->e->add("Read data doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->reread["counter"]!==2)
   {
    $this->e->add("ReRead counter doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }


   $s->close();
   $client->close();
   $server->close();

   return UT_OK;
  }


  function Write()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(false); //ensure tests don't block
   $client->setTimeout(1);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!($client->write("test")))
   {
    $this->e->add("Write shouldn't have failed");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->bytesSend!=4)
   {
    $this->e->add("Write bytes send doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!($server->write("test")))
   {
    $this->e->add("Write shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->bytesSend!=4)
   {
    $this->e->add("Write bytes send doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //file
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!($client->write("test")))
   {
    $this->e->add("Write shouldn't have failed");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->bytesSend!=4)
   {
    $this->e->add("Write bytes send doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!($server->write("test")))
   {
    $this->e->add("Write shouldn't have failed");
    $this->e->add($server->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($client->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($client->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->bytesSend!=4)
   {
    $this->e->add("Write bytes send doesn't match");
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   $s->close();
   $client->close();
   $server->close();


   //fail
   if($s->write("test"))
   {
    $this->e->add("Write should have failed");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Close()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(false); //ensure tests don't block
   $client->setTimeout(1);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!$s->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$client->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$server->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $server->close();
    return UT_FAILED;
   }


   //message
   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!$s->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$client->close("test"))
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($server->e);
    $server->close();
    return UT_FAILED;
   }

   if(!$server->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $server->close();
    return UT_FAILED;
   }


   //file
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!$s->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$client->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$server->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $server->close();
    return UT_FAILED;
   }


   //message
   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   if(!$s->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if(!$client->close("test"))
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $client->close();
    $server->close();
    return UT_FAILED;
   }

   if($server->read()!="test")
   {
    $this->e->add("Read data doesn't match");
    $this->e->add($server->e);
    $server->close();
    return UT_FAILED;
   }

   if(!$server->close())
   {
    $this->e->add("Close shouldn't have failed");
    $this->e->add($s->e);
    $server->close();
    return UT_FAILED;
   }


   //fail
   if($s->close())
   {
    $this->e->add("Close should have failed");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Destructor()
  {
   global $_path;

   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(false); //ensure tests don't block
   $client->setTimeout(1);

   if(!$s->listen("tcp://localhost:18080"))
   {
    $this->e->add("Listen shouldn't have failed to localhost:18080");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$client->connect("tcp://localhost:18080"))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   $sock=&$s->socket;
   $clientCopy=&$client->socket;
   $serverCopy=&$server->socket;
   unset($s, $client, $server);

   if($sock!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }

   if($clientCopy!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }

   if($serverCopy!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }


   //file
   $s=new Socket();
   $s->setBlocking(false); //ensure tests don't block
   $s->setTimeout(1);
   $client=new Socket();
   $client->setBlocking(false); //ensure tests don't block
   $client->setTimeout(1);

   $file=getcwd()."/".$_path["plugins"]."unit_testing/misc/test.sock";
   if(!touch($file))
   {
    $this->e->add("Unable to create file: ".$file);
    return UT_FAILED;
   }

   @unlink($file);

   if(!$s->listen("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to ".$file);
    $this->e->add($s->e);
    @unlink($file);
    return UT_FAILED;
   }

   if(!$client->connect("file://".$file))
   {
    $this->e->add("Connect shouldn't have failed to localhost:18080");
    $this->e->add($client->e);
    $s->close();
    return UT_FAILED;
   }

   if(!($server=$s->accept()))
   {
    $this->e->add("Accept shouldn't have failed");
    $this->e->add($s->e);
    $s->close();
    $client->close();
    return UT_FAILED;
   }

   $sock=&$s->socket;
   $clientCopy=&$client->socket;
   $serverCopy=&$server->socket;
   unset($s, $client, $server);

   if($sock!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }

   if($clientCopy!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }

   if($serverCopy!==false)
   {
    $this->e->add("Socket should have been close");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function FromIP()
  {
   $s=new Socket();

   //ipv4
   $ips=Array(
    "0.0.0.0" => 0,
    "0.0.0.1" => 1,
    "0.0.1.0" => bcpow(2, 8),
    "0.1.0.0" => bcpow(2, 16),
    "1.0.0.0" => bcpow(2, 24),

    "127.0.0.1" => "2130706433",
    "192.168.0.1" => "3232235521",
    "255.255.255.255" => "4294967295"
   );

   foreach($ips as $ip => $nr)
   {
    if($s->fromIP($ip)!=$nr)
    {
     $this->e->add("From IP4: ".$ip." failed, output doesn't match: ".$nr, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   //ipv6
   $ips=Array(
    "0:0:0:0:0:0:0:0" => 0,
    "0:0:0:0:0:0:0:1" => 1,
    "0:0:0:0:0:0:1:0" => bcpow(2, 16),
    "0:0:0:0:0:1:0:0" => bcpow(2, 32),
    "0:0:0:0:1:0:0:0" => bcpow(2, 48),
    "0:0:0:1:0:0:0:0" => bcpow(2, 64),
    "0:0:1:0:0:0:0:0" => bcpow(2, 80),
    "0:1:0:0:0:0:0:0" => bcpow(2, 96),
    "1:0:0:0:0:0:0:0" => bcpow(2, 112),

    "1:2:3:4:5:6:7:8" => "5192455318486707404433266433261576",
    "2001:1234:5678:90AB:CDEF:1234:5678:90AB" => "42540857391974624365358842897856237739",
    "fe80:0000:0000:0000:1234:5678:90ab:cdef" => "338288524927261089655330665308642594287",
    "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff" => "340282366920938463463374607431768211455",

    "2001:1234::1" => "42540857365213159232363542340108812289",
    "2001::1:1" => "42540488161975842760550356425300312065",
    "::1" => 1,
   );

   foreach($ips as $ip => $nr)
   {
    if($s->fromIP($ip)!=$nr)
    {
     $this->e->add("From IP6: ".$ip." failed, output doesn't match: ".$nr, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function FromIP_Subnet()
  {
   $s=new Socket();

   //ipv4
   $ips=Array(
    "0.0.0.0/0" => Array(0, bcpow(2, 32)-1),
    "0.0.0.1/8" => Array(1, bcpow(2, 24)),
    "0.0.0.1/16" => Array(1, bcpow(2, 16)),
    "0.0.0.1/24" => Array(1, bcpow(2, 8)),
    "0.0.0.1/32" => Array(1, 1),

    "127.0.0.0/8" => Array("2130706432", "2147483647"),
    "192.168.0.1/24" => Array("3232235521", "3232235776"),
    "169.254.10.25/29" => Array("2851998233", "2851998240"),

    "0.0.0.*" => Array(0, bcpow(2, 8)-1),
    "0.0.*" => Array(0, bcpow(2, 16)-1),
    "0.*" => Array(0, bcpow(2, 24)-1),

    "127.*" => Array("2130706432", "2147483647"),
    "192.168.*" => Array("3232235520", "3232301055"),
    "169.254.10.*" => Array("2851998208", "2851998463"),
   );

   foreach($ips as $ip => $nr)
   {
    if($s->fromIP($ip)!=$nr)
    {
     $this->e->add("From IP4 subnet: ".$ip." failed, output doesn't match: ".$nr[0]." ".$nr[1], "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }


   //ipv6
   $ips=Array(
    "::/0" => Array(0, bcsub(bcpow(2, 16*8), 1)),
    "::1/8" => Array(1, bcpow(2, 15*8)),
    "::1/16" => Array(1, bcpow(2, 14*8)),
    "::1/24" => Array(1, bcpow(2, 13*8)),
    "::1/32" => Array(1, bcpow(2, 12*8)),
    "::1/40" => Array(1, bcpow(2, 11*8)),
    "::1/48" => Array(1, bcpow(2, 10*8)),
    "::1/56" => Array(1, bcpow(2, 9*8)),
    "::1/64" => Array(1, bcpow(2, 8*8)),
    "::1/72" => Array(1, bcpow(2, 7*8)),
    "::1/80" => Array(1, bcpow(2, 6*8)),
    "::1/88" => Array(1, bcpow(2, 5*8)),
    "::1/96" => Array(1, bcpow(2, 4*8)),
    "::1/104" => Array(1, bcpow(2, 3*8)),
    "::1/112" => Array(1, bcpow(2, 2*8)),
    "::1/120" => Array(1, bcpow(2, 1*8)),
    "::1/128" => Array(1, 1),

    "1:2:3:4:5:6:7:8/64" => Array("5192455318486707404433266433261576", "5192455318486725851177340142813191"),
    "2001:1234:5678:90AB:CDEF:1234:5678:0/112" => Array("42540857391974624365358842897856200704", "42540857391974624365358842897856266239"),

    "::/128" => Array(0, 0),
    "fe80::/10" => Array("338288524927261089654018896841347694592", "338620831926207318622244848606417780735"),
    "2001::/32" => Array("42540488161975842760550356425300246528", "42540488241204005274814694018844196863"),
    "ff00::/8" => Array("338953138925153547590470800371487866880", "340282366920938463463374607431768211455"),

    "2001:1234:5678:90AB:*" => Array("42540857391974624350519743584648822784", "42540857391974624368966487658358374399"),
    "ff00:*" => Array("338953138925153547590470800371487866880", "338958331222012082418099330867817086975"),
    "fe80::*" => Array("338288524927261089654018896841347694592", "338288524927261089654018896841347760127"),
    "::*" => Array(0, 65535),
   );

   foreach($ips as $ip => $nr)
   {
    if($s->fromIP($ip)!=$nr)
    {
     $this->e->add("From IP6 subnet: ".$ip." failed, output doesn't match: ".$nr[0]." ".$nr[1], "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function ToIP()
  {
   $s=new Socket();

   //ipv4
   $ips=Array(
    "0.0.0.0" => 0,
    "0.0.0.1" => 1,
    "0.0.1.0" => bcpow(2, 8),
    "0.1.0.0" => bcpow(2, 16),
    "1.0.0.0" => bcpow(2, 24),

    "127.0.0.1" => "2130706433",
    "192.168.0.1" => "3232235521",
    "255.255.255.255" => "4294967295"
   );

   foreach($ips as $ip => $nr)
   {
    if($s->toIP($nr, 4)!=$ip)
    {
     $this->e->add("To IP4: ".$nr." failed, output doesn't match: ".$ip, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   //ipv6
   $ips=Array(
    "0:0:0:0:0:0:0:0" => 0,
    "0:0:0:0:0:0:0:1" => 1,
    "0:0:0:0:0:0:1:0" => bcpow(2, 16),
    "0:0:0:0:0:1:0:0" => bcpow(2, 32),
    "0:0:0:0:1:0:0:0" => bcpow(2, 48),
    "0:0:0:1:0:0:0:0" => bcpow(2, 64),
    "0:0:1:0:0:0:0:0" => bcpow(2, 80),
    "0:1:0:0:0:0:0:0" => bcpow(2, 96),
    "1:0:0:0:0:0:0:0" => bcpow(2, 112),

    "1:2:3:4:5:6:7:8" => "5192455318486707404433266433261576",
    "2001:1234:5678:90ab:cdef:1234:5678:90ab" => "42540857391974624365358842897856237739",
    "fe80:0:0:0:1234:5678:90ab:cdef" => "338288524927261089655330665308642594287",
    "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff" => "340282366920938463463374607431768211455",

    "2001:1234:0:0:0:0:0:1" => "42540857365213159232363542340108812289",
    "2001:0:0:0:0:0:1:1" => "42540488161975842760550356425300312065",
   );

   foreach($ips as $ip => $nr)
   {
    if($s->toIP($nr, 6, false)!=$ip)
    {
     $this->e->add("To IP6: ".$nr." failed, output doesn't match: ".$ip, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   //autodetect / short mode
   $ips=Array(
    "127.0.0.1" => "2130706433",
    "192.168.0.1" => "3232235521",
    "255.255.255.255" => "4294967295",

    "2001:1234:5678:90ab:cdef:1234:5678:90ab" => "42540857391974624365358842897856237739",
    "fe80::1234:5678:90ab:cdef" => "338288524927261089655330665308642594287",
    "2001:1234::" => "42540857365213159232363542340108812288",
    "2001::1:1" => "42540488161975842760550356425300312065",
    "::1:0:1" => "4294967297",
   );

   foreach($ips as $ip => $nr)
   {
    if($s->toIP($nr)!=$ip)
    {
     $this->e->add("To IP: ".$nr." failed, output doesn't match: ".$ip, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function InSubnet()
  {
   $s=new Socket();

   //ipv4
   $ips=Array(
    Array("0.0.0.0/0", "0.0.0.0", true),
    Array("0.0.0.0/0", "128.0.0.0", true),
    Array("0.0.0.0/0", "255.255.255.255", true),

    Array("127.0.0.0/8", "127.0.0.1", true),
    Array("127.0.0.0/8", "127.255.255.255", true),
    Array("127.0.0.0/8", "0.0.0.1", false),
    Array("127.0.0.0/8", "126.255.255.255", false),
    Array("127.0.0.0/8", "128.0.0.1", false),

    Array("192.168.0.0/21", "192.168.0.1", true),
    Array("192.168.0.0/21", "192.168.1.1", true),
    Array("192.168.0.0/21", "192.168.3.1", true),
    Array("192.168.0.0/21", "192.168.7.1", true),
    Array("192.168.0.0/21", "192.167.255.255", false),
    Array("192.168.0.0/21", "192.168.8.1", false),

    Array("169.254.10.25/32", "169.254.10.25", true),
    Array("169.254.10.25/32", "169.254.10.24", false),
    Array("169.254.10.25/32", "169.254.10.26", false),

    Array("127.*", "127.0.0.1", true),
    Array("127.*", "127.255.255.255", true),
    Array("127.*", "0.0.0.1", false),
    Array("127.*", "126.255.255.255", false),
    Array("127.*", "128.0.0.1", false),

    Array("192.168.*", "192.168.0.1", true),
    Array("192.168.*", "192.168.1.1", true),
    Array("192.168.*", "192.168.3.1", true),
    Array("192.168.*", "192.168.7.1", true),
    Array("192.168.*", "192.167.255.255", false),
    Array("192.168.*", "192.169.0.0", false),

    Array("169.254.10.*", "169.254.10.25", true),
    Array("169.254.10.*", "169.254.9.255", false),
    Array("169.254.10.*", "169.254.11.1", false),
   );

   foreach($ips as $ip)
   {
    if($s->inSubnet($ip[0], $ip[1])!=$ip[2])
    {
     $this->e->add("In IP4 subnet: ".$ip[0]." failed, ip".($ip[2]?" not":"")." in scope: ".$ip[1], "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   //ipv6
   $ips=Array(
    Array("::/0", "::", true),
    Array("::/0", "8000::", true),
    Array("::/0", "FFFF:FFFF:FFFF:FFFF:FFFF:FFFF", true),

    Array("::1/128", "::1", true),
    Array("::1/128", "::0", false),
    Array("::1/128", "::2", false),

    Array("fe80::/10", "fe80::", true),
    Array("fe80::/10", "fe80:ffff:ffff:ffff:ffff:ffff", true),
    Array("fe80::/10", "fec0::", false),
    Array("fe80::/10", "fe79:ffff:ffff:ffff:ffff:ffff", false),

    Array("2001:1234:5678:90ab:/64", "2001:1234:5678:90ab::1", true),
    Array("2001:1234:5678:90ab:/64", "2001:1234:5678:90ab:ffff:ffff:ffff:ffff", true),
    Array("2001:1234:5678:90ab:/64", "2001:1234:5678:90ac::", false),
    Array("2001:1234:5678:90ab:/64", "2001:1234:5678:90aa:ffff:ffff:ffff:ffff", false),

    Array("ff00::/8", "ff00::1", true),
    Array("ff00::/8", "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", true),
    Array("ff00::/8", "feff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", false),

    Array("fe80:*", "fe80::", true),
    Array("fe80:*", "fe80:ffff:ffff:ffff:ffff:ffff", true),
    Array("fe80:*", "fe81::", false),
    Array("fe80:*", "fe79:ffff:ffff:ffff:ffff:ffff", false),

    Array("fe80::*", "fe80::", true),
    Array("fe80::*", "fe80::ffff", true),
    Array("fe80::*", "fe80::1:1", false),
    Array("fe80::*", "fe79:ffff:ffff:ffff:ffff:ffff", false),

    Array("::*", "::", true),
    Array("::*", "::ffff", true),
    Array("::*", "::1:1", false),
   );

   foreach($ips as $ip)
   {
    if($s->inSubnet($ip[0], $ip[1])!=$ip[2])
    {
     $this->e->add("In IP6 subnet: ".$ip[0]." failed, ip".($ip[2]?" not":"")." in scope: ".$ip[1], "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }
   }


   //failures

   return UT_OK;
  }

 }

?>
