<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/about.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/projects.php");
 require_once($_path["include"]."jas.php");

 class Backend_About extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->project=false;
   $this->projects=Array();
  }

  function main($section=false, $project=false, $plugin=false)
  {
   global $_path, $_session;

   //section
   if(!parent::main($section))
    return false;

   if($this->section=="changelog")
   {
    foreach(Backend_Projects::projects() as $p => $title)
    {
     if(file_exists($_path["projects"].$p."/include/changelog.txt"))
     {
      $this->projects[$p]=Array(
       "title" => $title,
       "changelog" => $_path["projects"].$p."/include/changelog.txt",
       "icon" => false,
      );

      if(file_exists($_path["projects"].$p."/images/favicon.ico"))
       $this->projects[$p]["icon"]=$_path["projects"].$p."/images/favicon.ico";
     }

     if($project&&isset($this->projects[$project]))
      $this->project=$project;
    }
   }

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   $this->backend->setTitle($this->icon("jas_dim"),
    $this->link("JAS v".JAS::versionS().(JAS::versionS("tag")?" ".JAS::versionS("tag"):"").(JAS::versionS("development")?"*":""), Array("about/")));

   $r="<div class=\"twothird\">\n";
   $r.=JAS::licenseS(true)."\n";
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   return $r;
  }


  function output_changelog()
  {
   global $_session;

   $current=$_session->get("backendProject");

   if(!$this->project)
    $this->backend->setTitle($this->icon("changelog_dim"),
     $this->link("JAS", Array("about/")),
     $this->link($this->language->t("Changelog"), Array("about/changelog/")));
   else
    $this->backend->setTitle($this->icon("changelog_dim"),
     $this->link("JAS", Array("about/")),
     $this->link($this->language->t("Changelog"), Array("about/changelog/")),
     $this->link($this->projects[$this->project]["title"], Array("about/changelog/".$this->project."/")));


   $r="<div class=\"twothird\">\n";

   if(!$this->project)
    $r.=JAS::changelogS(true)."\n";
   else
    $r.=JAS::changelogFormatS(file_get_contents($this->projects[$this->project]["changelog"]));

   $r.="</div>\n";


   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   //project changelogs
   if($this->projects)
   {
    $r.="<h3>".$this->icon("project_dim", "small")." ".$this->language->t("Projects")."</h3>\n";

    foreach($this->projects as $project => $info)
     $this->menuItem("projects", $this->icon($info["icon"])." ".$info["title"], $this->url("about/changelog/".$project."/", ($project!=$current?Array("backendProject" => $project):false)), $this->language->t("Show changelog for: ".$info["title"]), false, ($project==$this->project?Array("selected"):false));

    $r.=$this->menu("projects");
   }

   $r.="</div>\n";

   return $r;
  }

  function output_license()
  {
   $this->backend->setTitle($this->icon("license_dim"),
    $this->link("JAS", Array("about/")),
    $this->link($this->language->t("License"), Array("about/license/")));

   $r="<div class=\"twothird\">\n";
   $r.="<h3 id=\"GPL\">".$this->language->t("The GNU/GPL license version 3")."</h3>\n";
   $r.="<br/>".JAS::licenseS(true, "GPL")."<br/><br/>\n";
   $r.="<h3 id=\"LGPL\">".$this->language->t("The GNU/LGPL license version 3")."</h3>\n";
   $r.="<br/>".JAS::licenseS(true, "LGPL")."<br/>\n";
   $r.="</div>\n";

   //actions
   $this->actionsMenu(true);

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   return $r;
  }


  /*
   * Support
   */

  function actionsMenu($license=false)
  {
   $this->menuItem("actions", $this->language->t("About JAS"), $this->url("about/"), $this->language->t("About JAS"), "jas", Array("menu" => "about"));

   $this->menuItem("about", $this->language->t("Changelog"), $this->url("about/changelog/"), $this->language->t("View the changelog"), "changelog");
   $this->menuItem("about", $this->language->t("Full license"), $this->url("about/license/"), $this->language->t("View the full license"), "license", Array("menu" => "license"));

   if($license)
   {
    $this->menuItem("about", "GNU/GPL", $this->url("about/license/#GPL"), $this->language->t("The GNU/GPL license"));
    $this->menuItem("about", "GNU/LGPL", $this->url("about/license/#LGPL"), $this->language->t("The GNU/LGPL license"));
   }

   $this->menuItem("actions", $this->language->t("Check for updates"), $this->url("online/updates/"), $this->language->t("Check for updates"), "updates");
   $this->menuItem("actions", $this->language->t("JAS website"), JAS::URLS(), $this->language->t("Goto the JAS website"), "online", Array("rel" => "external"));
   $this->menuItem("actions", $this->language->t("Help"), JAS::URLS()."doc/", $this->language->t("Online help"), "help", Array("rel" => "external"));

   return true;
  }

 }
?>
