<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/import.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["api"]."csv.php");
 require_once($_path["api"]."data.php");
 require_once($_path["api"]."xml.php");

 class Backend_Import extends Backend_Modules
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->data=Array();
   $this->import=false;
  }

  function main($module=false, $nr=false)
  {
   global $_session, $_config;

   $_session->frontend();

   if(request("file", "tmp_name", "FILES")) //import file uploaded
   {
    if(!($import=@file_get_contents(request("file", "tmp_name", "FILES"))))
    {
     $this->e->add("Unable to read import file: \"".request("file", "name", "FILES")."\"", "error");
     $_session->backend();
     return false;
    }
   }
   else if(request("s")) //file contents submitted
    $import=request("import");
   else
    $import=false;

   if($import)
   {
    if(strpos($import, "<?xml")===0) //XML
     $api=new API_XML();
    else if(strpos($import, "O:4:\"Data\"")===0) //Data
     $api=new API_Data();
    else //assume CSV
     $api=new API_CSV();

    if(!$api->parse($import))
    {
     $this->e->add($api->e);
     $_session->backend();
     return false;
    }

    foreach($api->commands as $command)
     if($command["type"]=="data")
      $this->data[]=$command["data"];

    if(!$this->data)
    {
     $this->e->add("Invalid import, no data objects found", "error");
     $_session->backend();
     return false;
    }

    //set submitted action + catnr
    foreach($this->data as $d)
    {
     $d->requestVar=$d->module;
     $d->setRequestValue("a");

     if($catnr=$d->getRightsReference())
      $d->setRequestValue($catnr);
    }

    $this->section="import";
    $this->import=$import;
   }

   //submit
   if(request("s")&&$this->data)
   {
    foreach($this->data as $d)
    {
     //determine which entries to add with current id
     $daid=clone $d;
     for($c=$daid->values()-1;$c>=0;$c--)
      if($daid->getValue("a", $c)!="addid")
       $daid->removeEntry($c);

     //determine which entries to add
     $da=clone $d;
     for($c=$da->values()-1;$c>=0;$c--)
      if($da->getValue("a", $c)!="add")
       $da->removeEntry($c);

     //determine which entries to update
     $du=clone $d;
     for($c=$du->values()-1;$c>=0;$c--)
      if($du->getValue("a", $c)!="edit")
       $du->removeEntry($c);

     if($daid->values()) //add entries with current id
     {
      $daid->setAction("add");
      $daid->setExtra("item");

      //convert to SQL values
      foreach(array_keys($daid->field) as $field)
       if($daid->field[$field]["type"]!="filedata")
        foreach(array_keys($daid->field[$field]["value"]) as $entry)
         $daid->setValue($field, $daid->sqlValue($field, $entry), $entry);

      $daid->dataformat="sql"; //force sql data format to ensure IDs are inserted

      $daid->submit();
      $this->e->add($daid->e);

      $d->clearValues("a"); //reset action
     }

     if($da->values()) //add entries
     {
      $da->setAction("add");
      $da->setExtra("item");

      $da->submit();
      $this->e->add($da->e);

      $d->clearValues("a"); //reset action
     }

     if($du->values()) //update entries
     {
      $du->setAction("edit");
      $du->setExtra("item");

      $du->submit();
      $this->e->add($du->e);

      $d->clearValues("a"); //reset action
     }
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("import_dim"),
    $this->link($this->language->t("Import"), Array("import/")));

   $this->tableRow("modules", true);
   $this->tableColumn("modules", $this->language->t("Module"), false, false, Array("class" => "module"));
   $this->tableColumn("modules", $this->language->t("Type"), false, false, Array("class" => "type"));

   $r="<div class=\"twothird\">\n";
   $form=$this->field($this->language->t("File"), $this->form->input("file", "file"));
   $form.=$this->field($this->language->t("Items"), $this->form->button("confirm", $this->icon("import", "small")." ".$this->language->t("Import"), false, false, "submit"));
   $r.=$this->form($form);
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_import()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("import_dim"),
    $this->link($this->language->t("Import"), Array("import/")));

   //gather categories
   $categories=Array();
   $dc=Data::cache("categories", "tree", "down");
   $dc->setShowFields("name", "treedepth");
   $dc->setValue("catnr", 0);

   if(!$dc->get())
    $this->e->add($dc->e);

   while($dc->next())
   {
    if($dc->v["treedepth"])
     $depth=str_repeat("&nbsp; ", ($dc->v["treedepth"]-1)*3)."|_ ";
    else
     $depth="";

    $categories[$dc->v["nr"]]=$depth.$dc->v["name"];
   }

   //render import tables
   foreach($this->data as $d)
   {
    $this->language->read("module_".$d->module);

    $id=$d->getID();
    $catnr=$d->getRightsReference();
    $titles=$d->check("properties", "title");

    $updated=false;
    foreach($d->check("type", "datetime") as $field)
    {
     if($d->check("properties", "forceUpdate", $field))
     {
      $updated=$field;
      break;
     }
    }

    foreach($d->check("use", "sql") as $field)
     if(!$d->check("use", "show", $field))
      $d->field[$field]["use"].=";show";

    //lookup default category if none provided
    if($catnr&&$d->check("valuetype", "none", $catnr))
    {
     $tmp=Data::cache($d->module, "add", "item");

     for($c=0;$c<$d->vmax;$c++)
      $d->setValue($catnr, $tmp->getValue($catnr), $c);

     unset($tmp);
    }

    //get existing IDs
    if($id)
    {
     $ids=Array();

     $d2=clone $d;
     $d2->setAction("view");
     $d2->setExtra("item");

     if($updated)
     {
      $d2->setShowFields($updated);
      $d2->addMutilation($updated, "dateConvert", "U");
     }
     else
      $d2->setShowFields();

     if(!$d2->get())
      $this->e->add($d2->e);

     if($updated)
     {
      while($d2->next())
       $ids[$d2->v[$id]]=$d2->v[$updated];
     }
     else
     {
      while($d2->next())
       $ids[$d2->v[$id]]=true;
     }

     if($d->check("properties", "deleted")) //deleted field
     {
      $d2->setExtra("deleted");

      if(!$d2->get())
       $this->e->add($d2->e);

      if($updated)
      {
       while($d2->next())
        $ids[$d2->v[$id]]=$d2->v[$updated];
      }
      else
      {
       while($d2->next())
        $ids[$d2->v[$id]]=true;
      }
     }
    }

    $this->tableRow("import_".$d->module, true, Array("class" => $d->module));
    $this->tableColumn("import_".$d->module, $d->title(false, true), Array("modules/".$d->module."/"), $this->language->t("Show ".$d->title." items"), Array("colspan" => sizeof($titles)+($id?1:0)));

    if($catnr)
     $this->tableColumn("import_".$d->module, $d->title($catnr, true));

    $this->tableColumn("import_".$d->module, $this->language->t("Action"));

    $d->reset();
    while($d->next())
    {
     $this->tableRow("import_".$d->module);

     if($id)
     {
      $this->_viewField($d, $id, $value, $url, $class);
      $this->tableColumn("import_".$d->module, $value, false, false, Array("class" => $class));
     }

     foreach($titles as $title)
     {
      $this->_viewField($d, $title, $value, $url, $class);
      $this->tableColumn("import_".$d->module, $value, false, false, Array("class" => $class));
     }

     if($catnr)
     {
      $value=$this->form->select(false, $d->module."[".$catnr."][".($d->vc-1)."]", $categories, (isset($d->v[$catnr])?$d->v[$catnr]:0), false, false, false, true);
      $this->tableColumn("import_".$d->module, $value, false, false, Array("class" => "category"));
     }

     //actions
     $action=$d->getValue("a", $d->vc-1);
     if($id&&isset($ids[$d->v[$id]])) //lookup existing item with same id
     {
      $actionList=Array("" => "Ignore", "add" => "Import", "edit" => "Overwrite");

      if($action===false) //no previously submitted action
      {
       if($updated) //check if import item is newer
       {
        if($ids[$d->v[$id]]<DT::dateConvertS($d->v[$updated], $d->field[$updated]["format"][$d->dataformat]["datetimeformat"])) //newer
         $action="edit"; //overwrite
        else //older
         $action=""; //ignore
       }
       else //no last update field
        $action="edit"; //overwrite by default
      }
     }
     else //no existing entry
     {
      $actionList=Array("" => "Ignore", "addid" => "Import");

      if($action===false) //no previously submitted action
       $action="addid"; //import by default
     }

     $value=str_replace("<br/>", " ", $this->form->listview(false, $d->module."[a][".($d->vc-1)."]", $actionList, $action));
     $this->tableColumn("import_".$d->module, $value, false, false, Array("class" => "actions"));
    }

    //total
    $this->tableRow("import_".$d->module, false, Array("class" => "total"));
    $this->tableColumn("import_".$d->module, $this->language->t("Total"), false, false, Array("colspan" => sizeof($titles)+($id?1:0)+($catnr?1:0)));
    $this->tableColumn("import_".$d->module, $d->vmax, false, false, Array("class" => "nr"));
   }

   $r="<div class=\"twothird\">\n";

   foreach($this->data as $d)
    $r.=$this->table("import_".$d->module, "import")."<br/>";

   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   $r.="<div class=\"third\">\n";

   if(request("search"))
    $r.=$this->form->input("hidden", "search", strip_tags(request("search")));
   else if(request("catnr"))
    $r.=$this->form->input("hidden", "catnr", strip_tags(request("catnr")));

   //import submit
   $r.="<h3>".$this->icon("import_dim", "small")." ".$this->language->t("Import")."</h3>\n";
   $r.=$this->confirm();

   //actions
   $this->actionsMenu($d);

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   $r.="</div>\n";

   //make full page part of the form
   $r.=$this->form->input("hidden", "import", $this->import);
   $r=$this->form($r);

   $_session->backend();

   return $r;
  }

 }
?>
