<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/login.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/projects.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."cms.php");

 class Backend_Login extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->backendRootPasswordSet=NULL;
  }

  function main($section=false)
  {
   global $_config, $_session, $didLogin;

   //section
   if(!parent::main($section))
    return false;

   if(!$this->section&&!$this->backendRootPasswordSet()) //backend root password not setup
   {
    $this->section="backendPassword";

    if(!request("setRootPassword"))
     $this->e->add("Please set the root username and password for the JAS Management Tool", "note");
   }

   if(request("setRootPassword")) //set backend password form submitted, try to set new password
   {
    if($this->backendRootPasswordSet())
     return false;

    $d=Data::cache($_config->get("usermodule"), "edit", "item", Array("format" => "form", "request" => "setRootPassword"));
    $d->clearValues();

    $d->setValue($d->getID(), $_config->get("rootnr"));
    $d->setRequestValue("username");
    $d->setRequestValue("password");
    $d->setRequestValue("verifypassword");

    //sudo
    $tmp=$_config->get("usernr");
    $_config->set("usernr", $_config->get("rootnr"));
    if(!$d->submit())
    {
     $_config->set("usernr", $tmp);
     $this->e->add($d->e);
     return false;
    }

    $_config->set("usernr", $tmp);
    $this->backendRootPasswordSet=true;

    $this->e->add("Root user: \"".$d->getValue("username")."\" successfully set up", "note");
    $this->section=false;
   }
   else if(request("doLogin")&&!$didLogin) //login request via default/quick form
   {
    if(!$this->backendRootPasswordSet())
     return false;

    $didLogin=true;

    $_session->frontend();

    $d=Data::cache($_config->get("usermodule"), "user", request("extra"));

    if(!$d->submit())
    {
     $this->e->add($d->e);
     $_session->backend();
     return false;
    }

    $this->e->add($d->e);

    if($d->extra=="login"&&!$_config->userBackend())
     $this->e->add("This is not a backend user", "warning");

    $_session->backend();
   }

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($_config->get("usermodule"), "user", "login");

   $_session->backend();
   $d->l->read("jas");
   $_session->frontend();

   //add projects drop down
   if($projects=Backend_Projects::projects())
   {
    $projects=array_merge(Array("" => ""), $projects); //add empty entry

    //determine current active project
    if(($tmp=request("backendProject"))&&isset($projects[$tmp]))
     $project=$tmp;
    else if($_session->getKey()!="default")
     $project=$_session->getKey();
    else
     $project="";

    $d->addField("backendProject", "list", "form", "reload", false, false, $project, $this->language->t("Project"));
    $d->listOptions("backendProject", $projects);

    $d->field=array_merge(Array("backendProject" => $d->field["backendProject"]), $d->field); //set as first field
   }

   //add custom submit
   $d->addField("doLogin", "submit", "form", false, false, false, $this->language->t("Login"), "");

   //only show: project, username, password and submit
   $fields=Array("reload_trigger", "doLogin", "backendProject");
   foreach($d->check("use", "login") as $field)
    if($d->check("properties", "username", $field)||$d->check("properties", "password", $field))
     $fields[]=$field;

   $d->setShowFields($fields);

   //render cms
   $r=$this->cms($d, $this->url(false, Array("extra" => "login")), Array("cmsOptions" => false));

   $_session->backend();

   return $r;
  }

  function output_backendPassword()
  {
   global $_session, $_config;

   $d=Data::cache($_config->get("usermodule"), "edit", "item", Array("request" => "setRootPassword"));

   $cms=new CMS();
   $cms->setDataForm($d, $this->form);

   $r=$this->form->start()."\n";
   $r.=$this->form->input("hidden", "setRootPassword", "1")."\n";

   //add custom submit
   $d->addField("s", "submit", "form", false, false, false, $this->language->t("Set"), "");

   //only show: username, password and submit
   $d->setShowFields("username", "password", "verifypassword", "s");

   //render cms
   $r=$this->cms($d, $this->url(false, Array("setRootPassword" => 1)));

   return $r;
  }

  function output_quick()
  {
   global $_session, $_config, $_jasPath;

   $_session->frontend();

   //get username
   $d=Data::cache($_config->get("usermodule"), "view", "item");
   $d->setValue($d->getID(), $_config->get("usernr"));
   $d->setShowFields($d->check("properties", "title"));

   if($d->get()&&$d->next())
    $username=$d->v("title");
   else
    $username=$this->language->t("Unknown");

   //menu
   if($_config->userAnonymous())
   {
    $this->menuItem("login", $username, false, false, "user_disabled");

    $cms=new CMS();
    $cms->setDataForm($d, $this->form);

    foreach($d->check("use", "login") as $this->formield)
    {
     if($d->check("properties", "username", $this->formield))
      $this->menuItem("login", $this->form->input("text", $this->formield, $d->title($this->formield, true), "form", false, "onfocus=\"this.value='';\""));
     else if($d->check("properties", "password", $this->formield))
      $this->menuItem("login", $this->form->input("password", $this->formield, $d->title($this->formield, true), "form", false, "onfocus=\"this.value='';\""));
    }

    $this->menuItem("login", $this->form->input("submit", "", $this->language->t("Login"), "submit"));

    $r=$this->form($this->menu("login"), $this->url(false, Array("extra" => "login", "doLogin" =>1)));
   }
   else
   {
    $this->menuItem("login", $username, $this->url("users/".$_config->get("usernr")."/"), false, "user");

    $this->menuItem("login", $this->form->input("submit", "", $this->language->t("Logout"), "submit"));

    $r=$this->form($this->menu("login"), $this->url(false, Array("extra" => "logout", "doLogin" =>1)));
   }

   $_session->backend();

   return $r;
  }


  /*
   * Support
   */

  function backendLoggedIn()
  {
   global $_session, $_config;

   $current=$_session->getKey(); //store current session
   foreach($_session->getKeys() as $project)
   {
    if($_session->setKey($project)) //switch key
    {
     if(!$_config->userAnonymous()&&$_config->userBackend()) //logged in backend user
     {
      $_session->setKey($current); //restore current session
      return true;
     }
    }
   }

   $_session->setKey($current); //restore current session
   return false;
  }


  function backendRootPasswordSet()
  {
   global $_config;

   if($this->backendRootPasswordSet!==NULL)
    return $this->backendRootPasswordSet;

   //get (sql) password for root user
   $d=Data::cache($_config->get("usermodule"), "view", "item", Array("format" => "sql", "request" => "setRootPassword"));
   $d->setValue($d->getID(), $_config->get("rootnr"));
   $d->setShowFields("password");

   if(!$d->get()) //failure getting root password
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$d->getValue("password", 0)) //password not set
    $this->backendRootPasswordSet=false;
   else
    $this->backendRootPasswordSet=true;

   return $this->backendRootPasswordSet;
  }
 }
?>
