

 /*****************************************************************************
 * Filename: script/cms.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(onload)!='function')
 alert("Script not loaded: functions.js");
else if(typeof(setGetElementFunctions)!="function")
 alert("Script not loaded: getelements.js");
else
 onload("initCMS();");


function initCMS()
{
 setGetElementFunctions();

 initCMSOptions();

 focusFirstField();
}

function initCMSOptions()
{
 var fields=document.getElementsByClassName("field");

 for(var c=0;c<fields.length;c++)
  if(!fields[c].getElementsByClassName("field").length) //no sub field (reference manipulation
   addCMSOptions(fields[c]);
}


/*
 * Misc
 */

function focusFirstField()
{
 var anchor;
 if(anchor=document.location.toString().match(/\#(\w+)$/)) //anchor in URL
 {
  if(anchor=document.getElementById(anchor[1])) //focus anchor's first input element
   return _focusFirstField(anchor.parentNode);
 }
 else //no anchor
 {
  var forms=document.getElementsByTagName("form");
  for(var f=0;f<forms.length;f++) //find first CMS form
  {
   if(forms[f].className!="cms")
    continue;

   var fieldInput;
   var fields=forms[f].getElementsByClassName("field");
   for(var c=0;c<fields.length;c++) //focus first input field in form
    if((fieldInput=fields[c].getElementsByClassName("input")).length)
     if(_focusFirstField(fieldInput[0]))
      return true;
  }
 }

 return false;
}
function _focusFirstField(fieldInput)
{
 var fieldIframe;
 if((fieldIframe=fieldInput.getElementsByTagName("iframe")).length&&fieldIframe[0].className.match(/wysiwy/))
 {
  fieldIframe[0].contentWindow.focus();
  return true;
 }

 var elements=fieldInput.getAllElements();
 for(var x=0;x<elements.length;x++)
 {
  if(
    (elements[x].nodeName.match(/input/i)&&elements[x].type!="hidden")||
    elements[x].nodeName.match(/select/i)||
    elements[x].nodeName.match(/textarea/i)
   )
  {
   elements[x].focus();
   return true;
  }
 }

 return false;
}

function stopFormSubmit(e)
{
 if(e==undefined)
  e=window.event;

 if(e.keyCode==13) //ensure enter doesn't submit the form
  return false;

 return true;
}


/*
 * CMS options
 */

function addCMSOptions(field)
{
 var fieldInput;
 var fieldOptions;

 //get input+options nodes
 if(!(fieldInput=field.getElementsByClassName("input")).length)
  return false;

 fieldInput=fieldInput[0];

 if(!(fieldOptions=field.getElementsByClassName("options")).length)
  return false;

 fieldOptions=fieldOptions[0];

 //add search
 if(
   fieldInput.getElementsByClassName("listview").length||
   fieldInput.getElementsByClassName("multilistview").length||
   fieldInput.getElementsByTagName("select").length
  )
 {
  //ensure height remains the same if less options are shown
  if(fieldInput.getElementsByTagName("div").length)
   fieldInput.getElementsByTagName("div")[0].style.height=fieldInput.getElementsByTagName("div")[0].offsetHeight+"px";

  //set reference
  field.options=fieldOptions; //field options container
  fieldOptions.options=Array(); //reference to various options
  fieldOptions.field=field; //field container
  fieldOptions.fieldInput=fieldInput; //field input container
  fieldOptions.fieldInputSelectMultiple=false; //multiple options not allowed, overwritten below
  fieldOptions.fieldInputSelectType=(fieldInput.getElementsByTagName("select").length?"select":"input"); //select type: select / checkbox / radio

  //input field
  if(fieldOptions.fieldInputSelectType=="select")
  {
   fieldOptions.fieldInputSelect=fieldInput.getElementsByTagName("select")[0]; //select
   fieldOptions.fieldInputSelectBackup=fieldOptions.fieldInputSelect.cloneNode(true); //create backup

   if(fieldOptions.fieldInputSelect.disabled) //field is disabled
    return false; //don't add options
  }
  else
  {
   fieldOptions.fieldInputSelect=fieldInput.getElementsByTagName("div")[0]; //inputs div container

  if(fieldOptions.fieldInputSelect.getElementsByTagName("input").length) //field has entries
   if(fieldOptions.fieldInputSelect.getElementsByTagName("input")[0].disabled) //first entry disabled
    return false; //don't add options
  }

  //create search box
  var search=document.createElement("input");
  search.type="text";
  search.className="form search";
  search.placeholder="Search";
  search.onkeypress=stopFormSubmit;

  search=fieldOptions.insertBefore(search, fieldOptions.firstChild);
  search.addEvent("keyup", cmsSearch);
//  search.addEvent("blur", cmsSearch);

  if(fieldOptions.fieldInputSelectType=="select") //ensure all select options are visible on submmit
   search.form.addEvent("submit", function(){cmsSearch(search);});

  fieldOptions.options["search"]=search; //add reference

  //create checkbox x button
  var x=document.createElement("input");
  x.type="button";
  x.className="form x";
  x.value="X";

  x=fieldOptions.insertBefore(x, search.nextSibling);
  x.addEvent("click", function(){cmsSearch(search);});

  fieldOptions.options["x"]=x; //add reference

  //detect multiSelect (must be either checkbox or multiple select) and add focussedIndex
  var multiSelect;
  if(fieldOptions.fieldInputSelectType=="select"&&fieldOptions.fieldInputSelect.multiple&&!fieldOptions.fieldInputSelect.className.match(/catnr/)) //multi select
  {
   fieldOptions.fieldInputSelectMultiple=true; //allow multiple options

   //store focussedIndex
   fieldOptions.fieldInputSelect.focussedIndex=0;
   for(var c=0;c<fieldOptions.fieldInputSelect.options.length;c++)
    fieldOptions.fieldInputSelect.options[c].addEvent("mouseup", setFocussedIndex);
  }
  else if((multiSelect=fieldInput.getElementsByClassName("multilistview")).length) //checkboxes
  {
   fieldOptions.fieldInputSelect=multiSelect[0]; //overwrite actual select
   fieldOptions.fieldInputSelectMultiple=true; //allow multiple options

   //store focussedIndex
   fieldOptions.fieldInputSelect.focussedIndex=0;
   var inputs=fieldOptions.fieldInputSelect.getElementsByTagName("input");
   for(var c=0;c<inputs.length;c++)
    if(inputs[c].type.match(/checkbox/i))
     inputs[c].addEvent("change", setFocussedIndex);
   }

  if(fieldOptions.fieldInputSelectMultiple)
  {
   //add break
   var br=document.createElement("br");
   br=fieldOptions.insertBefore(br, x.nextSibling);

   //create checkbox invert button
   var invert=document.createElement("input");
   invert.type="button";
   invert.className="form invert";
   invert.value="Invert";

   invert=fieldOptions.insertBefore(invert, br.nextSibling);
   invert.addEvent("click", cmsInvertClear);

   fieldOptions.options["invert"]=invert; //add reference

   //create checkbox clear button
   var clear=document.createElement("input");
   clear.type="button";
   clear.className="form clear";
   clear.value="Clear";

   clear=fieldOptions.insertBefore(clear, invert.nextSibling);
   clear.addEvent("click", cmsInvertClear);

   fieldOptions.options["clear"]=clear; //add reference

   //create order up buttons
   var up=document.createElement("input");
   up.type="button";
   up.className="form order up";
   up.value="Up";

   up=fieldOptions.insertBefore(up, clear.nextSibling);
   up.addEvent("click", cmsOrder);

   fieldOptions.options["up"]=up; //add reference

   //create order down buttons
   var down=document.createElement("input");
   down.type="button";
   down.className="form order down";
   down.value="Down";

   down=fieldOptions.insertBefore(down, up.nextSibling);
   down.addEvent("click", cmsOrder);

   fieldOptions.options["down"]=down; //add reference

   //add break
   var br=document.createElement("br");
   br=fieldOptions.insertBefore(br, down.nextSibling);

   //change edit button to edit "focussed" option
   var inputs=fieldOptions.getElementsByTagName("input");
   var last=inputs.length-1;
   if(inputs[last].type=="submit")
    inputs[last].addEvent("click", cmsEditFocussed);
  }

  //set show/hide triggers
  fieldOptions.fieldInputSelect.addEvent("focus", cmsShowHide);
  fieldOptions.fieldInputSelect.addEvent("blur", cmsShowHide);

  var elements=fieldOptions.field.getElementsByTagName("input");
  for(var c=0;c<elements.length;c++)
  {
   elements[c].addEvent("focus", cmsShowHide);
   elements[c].addEvent("blur", cmsShowHide);
  }

  var elements=fieldOptions.field.getElementsByTagName("select");
  for(var c=0;c<elements.length;c++)
  {
   elements[c].addEvent("focus", cmsShowHide);
   elements[c].addEvent("blur", cmsShowHide);
  }
 }
 else if(fieldInput.getElementsByClassName("password").length)
 {
  if(typeof(Passwords)!='function'||!Passwords.toString().match(/function generate/))
   alert("Script not loaded: passwords.js");

  //set reference
  field.options=fieldOptions; //field options container
  fieldOptions.options=Array(); //reference to various options
  fieldOptions.field=field; //field container
  fieldOptions.fieldInput=fieldInput; //field input container
  fieldOptions.fieldInputSelect=fieldInput.getElementsByTagName("input")[0]; //input field

  if(fieldOptions.fieldInputSelect.disabled) //field is disabled
   return false; //don't add options

  if(fieldOptions.fieldInputSelect.placeholder) //get minimum password requirements from placeholder (HTML5) attribute
  {
   //default minimum password requirements
   fieldOptions.minimum={
    length: 7,
    lower: 1,
    upper: 0,
    numbers: 0,
    specials: 0,
   };

   //overwrite individual password requirements
   var value;
   if(value=fieldOptions.fieldInputSelect.placeholder.match(/(\d+)\s*long/))
    fieldOptions.minimum["length"]=parseInt(value[1]);
   if(value=fieldOptions.fieldInputSelect.placeholder.match(/(\d+)\s*lower/))
    fieldOptions.minimum["lower"]=parseInt(value[1]);
   if(value=fieldOptions.fieldInputSelect.placeholder.match(/(\d+)\s*upper/))
    fieldOptions.minimum["upper"]=parseInt(value[1]);
   if(value=fieldOptions.fieldInputSelect.placeholder.match(/(\d+)\s*number/))
    fieldOptions.minimum["numbers"]=parseInt(value[1]);
   if(value=fieldOptions.fieldInputSelect.placeholder.match(/(\d+)\s*special/))
    fieldOptions.minimum["specials"]=parseInt(value[1]);
  }
  else //no predefined minimum password requirements, make dropdown
  {
   //create generate password buttons
   var strength=document.createElement("select");
   strength.className="form password strength";

   //match minimum password requirements of first option
   fieldOptions.minimum={
    length: 0, //defined by individual types
    lower: 6,
    upper: 2,
    numbers: 0,
    specials: 0,
   };

   var option=document.createElement("option");
   option.value="6,2,0,0";
   option.innerHTML="Simple";
   strength.appendChild(option);

   var option=document.createElement("option");
   option.value="4,3,2,0";
   option.innerHTML="Medium";
   strength.appendChild(option);

   var option=document.createElement("option");
   option.value="4,3,3,3";
   option.innerHTML="High";
   strength.appendChild(option);

   var option=document.createElement("option");
   option.value="6,5,5,5";
   option.innerHTML="Extreme";
   strength.appendChild(option);

   strength=fieldOptions.appendChild(strength);
   fieldOptions.appendChild(document.createTextNode(" "));
   strength.addEvent("change", passwordStrength);
   strength.addEvent("change", generatePassword);

   fieldOptions.options["strength"]=strength; //add reference
  }

  //create generate password buttons
  var generate=document.createElement("input");
  generate.type="button";
  generate.className="form password generate";
  generate.value="Generate";

  generate=fieldOptions.appendChild(generate);
  generate.addEvent("click", generatePassword);

  fieldOptions.options["generate"]=generate; //add reference

  //create show password buttons
  var show=document.createElement("label");
  show.for="showpass"+document.getAllElements().length;
  show.appendChild(document.createTextNode(" "));

  show=fieldOptions.insertBefore(show, generate.nextSibling);

  var checkbox=document.createElement("input");
  checkbox.type="checkbox";
  checkbox.id=show.for;
  checkbox.className="form checkbox show";

  if(fieldOptions.fieldInputSelect.type=="text")
   checkbox.checked=true;
  else
   checkbox.checked=false;

  checkbox=show.appendChild(checkbox);
  show.appendChild(document.createTextNode(" Show password"));

  checkbox.addEvent("click", showPassword);

  fieldOptions.options["show"]=show; //add reference
 }

 return true;
}

function cmsSearch(e)
{
 var searchInput;
 var selectVisibile=false; //enter pressed, select current visible

 if(!e.target) //clear search
 {
  searchInput=e;
  searchInput.value="";
 }
 else //event
 {
  searchInput=(e.target?e.target:e.srcElement);

  if(e.keyCode==13) //enter pressed
   selectVisibile=true; //select current visible

  if(e.type=="blur") //search reset
   searchInput.value="";
 }

 var search=searchInput.value; //get search string from input field
 var regexp=new RegExp(search, "i"); //make regexp search

 //search all input entries
 if(searchInput.parentNode.fieldInputSelectType=="input")
 {
  var options=searchInput.parentNode.fieldInputSelect.getElementsByTagName("input");
  for(var c=0;c<options.length;c++)
  {
   if(
     options[c].parentNode==searchInput.parentNode|| //ignore options container
     options[c].nextSibling==undefined|| //option must have a next (text) element
     !options[c].nextSibling.nodeName.match(/\#text/i)
   )
    continue;

   var text=options[c].nextSibling.nodeValue;
   text=text.replace(/(^\s*|\s*$)/, ""); //trim text

   if(!search||text.match(regexp)) //no search or search matches
   {
    options[c].parentNode.style.display="inline"; //show label
    options[c].parentNode.nextSibling.style.display="block"; //show line break

    if(selectVisibile) //select visible
    {
     options[c].checked=true;

     if(!searchInput.parentNode.fieldInputSelectMultiple) //stop selecting subsequent options
      selectVisibile=false;
    }
   }
   else //search doesn't match
   {
    options[c].parentNode.style.display="none"; //hide label
    options[c].parentNode.nextSibling.style.display="none"; //hide line break
   }
  }
 }
 else //search select options
 {
  var select=searchInput.parentNode.fieldInputSelect;
  var focussedIndex=select.focussedIndex;

  //merge selection to backup
  var options=searchInput.parentNode.fieldInputSelectBackup.options;
  var optionsNew=select.options;
  for(var c=0;c<options.length;c++)
  {
   for(var x=0;x<optionsNew.length;x++)
   {
    if(options[c].value==optionsNew[x].value)
    {
     options[c].selected=optionsNew[x].selected;
     break;
    }
   }
  }

  //restore backup
  searchInput.parentNode.fieldInput.replaceChild(searchInput.parentNode.fieldInputSelectBackup.cloneNode(true), select); //clone backup
  searchInput.parentNode.fieldInputSelect=searchInput.parentNode.fieldInput.getElementsByTagName("select")[0]; //update reference
  select=searchInput.parentNode.fieldInputSelect;

  //force selected options update
  var options=searchInput.parentNode.fieldInputSelectBackup.options;
  for(var c=0;c<options.length;c++)
   searchInput.parentNode.fieldInputSelect.options[c].selected=options[c].selected;

  if(search)
  {
   //remove options that don't match search
   var options=select.options;
   for(var c=0;c<options.length;c++)
   {
    //get text
    var text="";
    if(options[c].firstChild!=undefined)
    {
     text=options[c].firstChild.nodeValue;
     text=text.replace(/(^\s*|\s*$)/, ""); //trim text
    }

    if(!text) //no text, ignore
     continue;

    if(text.match(regexp)) //matches search
    {
     if(selectVisibile)
     {
      options[c].selected=true;

      //correct selection for single select
      if(!searchInput.parentNode.fieldInputSelectMultiple)
      {
       selectVisibile=false;

       for(var x=c+1;x<options.length;x++)
        if(options[x].selected) //unselect
         options[x].selected=false;
      }
     }
    }
    else //doesn't match search
    {
     select.remove(c);
     c--;
    }
   }
  }

  if(searchInput.parentNode.fieldInputSelectMultiple)
  {
   //restore focussedIndex
   select.focussedIndex=focussedIndex;

   //readd setFocussedIndex
   var options=select.options;
   for(var c=0;c<options.length;c++)
    options[c].addEvent("mouseup", setFocussedIndex);
  }
 }

 //enable/disable order fields
 if(searchInput.parentNode.fieldInputSelectMultiple)
 {
  if(search) //no ordering if search is active
  {
   searchInput.parentNode.options["up"].disabled=true;
   searchInput.parentNode.options["down"].disabled=true;
  }
  else //enable order fields for unfiltered list
  {
   searchInput.parentNode.options["up"].disabled=false;
   searchInput.parentNode.options["down"].disabled=false;
  }
 }

 return true;
}

function cmsInvertClear(e)
{
 var button=(e.target?e.target:e.srcElement);

 var invert=(button.className.match("invert")?true:false);
 var clear=(button.className.match("clear")?true:false);

 //invert/clear all input entries
 if(button.parentNode.fieldInputSelectType=="input")
 {
  var options=button.parentNode.fieldInputSelect.getElementsByTagName("input");
  for(var c=0;c<options.length;c++)
  {
   if(options[c].type.match(/(checkbox|radio)/i)&&options[c].parentNode.style.display!="none") //only visible checkbox/radio options
   {
    if(clear)
     options[c].checked=false;
    else if(invert)
     options[c].checked=!options[c].checked;
   }
  }
 }
 else //invert/clear select options
 {
  var options=button.parentNode.fieldInputSelect.options;
  for(var c=0;c<options.length;c++)
  {
   if(clear)
    options[c].selected=false;
   else if(invert)
    options[c].selected=!options[c].selected;
  }
 }

 return true;
}

function cmsOrder(e)
{
 var button=(e.target?e.target:e.srcElement);

 var up=(button.className.match("up")?true:false);
 var down=(button.className.match("down")?true:false);

 //order selected input entries up/down
 if(button.parentNode.fieldInputSelectType=="input")
 {
  var select=button.parentNode.fieldInputSelect;
  var options=select.getElementsByTagName("input");

  if(up)
  {
   if(select.focussedIndex>0)
   {
    //flip selected with previous
    var c=select.focussedIndex;
    var br=select.insertBefore(options[c].parentNode.nextSibling, options[c-1].parentNode);
    select.insertBefore(options[c].parentNode, br);

    //correct focussedIndex
    select.focussedIndex--;
   }
  }
  else if(down)
  {
   if(select.focussedIndex<options.length-1)
   {
    //flip selected with next
    var c=select.focussedIndex;
    var br=select.insertBefore(options[c+1].parentNode.nextSibling, options[c].parentNode);
    select.insertBefore(options[c+1].parentNode, br);

    //correct focussedIndex
    select.focussedIndex++;
   }
  }
 }
 else //order selected options up/down
 {
  var select=button.parentNode.fieldInputSelect;

  if(up)
  {
   if(select.focussedIndex>0)
   {
    //flip selected with previous
    var c=select.focussedIndex;
    var tmp=select.options[c-1];
    select.options[c-1]=select.options[c].cloneNode(true);
    select.options[c]=tmp.cloneNode(true);

    //readd setFocussedIndex
    select.options[c-1].addEvent("mouseup", setFocussedIndex);
    select.options[c].addEvent("mouseup", setFocussedIndex);

    //correct focussedIndex
    select.focussedIndex--;
   }
  }
  else if(down)
  {
   if(select.focussedIndex<select.options.length-1)
   {
    //flip selected with previous
    var c=select.focussedIndex;
    var tmp=select.options[c+1];
    select.options[c+1]=select.options[c].cloneNode(true);
    select.options[c]=tmp.cloneNode(true);

    //readd setFocussedIndex
    select.options[c+1].addEvent("mouseup", setFocussedIndex);
    select.options[c].addEvent("mouseup", setFocussedIndex);

    //correct focussedIndex
    select.focussedIndex++;
   }
  }

  //update backup
  button.parentNode.fieldInputSelectBackup=select.cloneNode(true);
 }

 return true;
}

function cmsEditFocussed(e)
{
 var button=(e.target?e.target:e.srcElement);

 //move focussed option end of the list so CMS uses it
 if(button.parentNode.fieldInputSelectType=="input")
 {
  var select=button.parentNode.fieldInputSelect;
  var options=select.getElementsByTagName("input");

  //flip selected with last
  var c=select.focussedIndex;
  var last=options.length-1;
  var br=select.insertBefore(options[c].parentNode.nextSibling, options[last].parentNode.nextSibling.nextSibling);
  select.insertBefore(options[c].parentNode, br);

  //correct focussedIndex
  select.focussedIndex=last;
 }
 else //move to last item
 {
  var select=button.parentNode.fieldInputSelect;

  //flip selected with last
  var c=select.focussedIndex;
  var last=select.options.length-1;
  var tmp=select.options[last];
  select.options[last]=select.options[c].cloneNode(true);
  select.options[c]=tmp.cloneNode(true);

  //readd setFocussedIndex
  select.options[last].addEvent("mouseup", setFocussedIndex);
  select.options[c].addEvent("mouseup", setFocussedIndex);

  //correct focussedIndex
  select.focussedIndex=last;

  //update backup
  button.parentNode.fieldInputSelectBackup=select.cloneNode(true);
 }

 return true;
}

function cmsShowHide(e)
{
 var target=(e.target?e.target:e.srcElement);

 var show;
 if(e.type=="focus"||e.type=="mouseover") //show
  show=true;
 else //hide (blur, mouseout)
  show=false;

 //find field options
 var fieldOptions;
 var parent=target;
 while(true)
 {
  if(parent.nodeName.match(/div/i)&&parent.className.match(/field/i)) //field container found
  {
   fieldOptions=parent.options;
   break;
  }

  parent=parent.parentNode;
 }

 if(show)
 {
  if(!fieldOptions.className.match(" optionsShow"))
   fieldOptions.className+=" optionsShow"; //add show class
 }
 else //hide
  fieldOptions.className=fieldOptions.className.replace(" optionsShow", ""); //remove show class

 return true;
}

function setFocussedIndex(e)
{
 var option=(e.target?e.target:e.srcElement);

 if(option.nodeName.match(/option/i)) //multi select
  option.parentNode.focussedIndex=option.index;
 else //multi checkboxes
 {
  var inputs=option.parentNode.parentNode.getElementsByTagName("input");
  for(var c=0;c<inputs.length;c++)
  {
   if(inputs[c]==option)
   {
    option.parentNode.parentNode.focussedIndex=c;
    break;
   }
  }
 }
}


/*
 * Passwords
 */

function passwordStrength(e)
{
 var select=(e.target?e.target:e.srcElement);
 var minimum=select.options[select.selectedIndex].value.split(",");

 select.parentNode.minimum={
  length: 0,
  lower: parseInt(minimum[0]),
  upper: parseInt(minimum[1]),
  numbers: parseInt(minimum[2]),
  specials: parseInt(minimum[3])
 };

 return true;
}

function generatePassword(e)
{
 var button=(e.target?e.target:e.srcElement);
 var password=new Passwords();

 password.setLength(button.parentNode.minimum["length"]);

 password.setMimimum(
  button.parentNode.minimum["lower"],
  button.parentNode.minimum["upper"],
  button.parentNode.minimum["numbers"],
  button.parentNode.minimum["specials"]
 );

 button.parentNode.fieldInputSelect.value=password.generate();
 return true;
}

function showPassword(e)
{
 var checkbox=(e.target?e.target:e.srcElement);

 if(checkbox.checked)
  checkbox.parentNode.parentNode.fieldInputSelect.type="text";
 else
  checkbox.parentNode.parentNode.fieldInputSelect.type="password";

 return true;
}

