

 /*****************************************************************************
 * Filename: script/flash.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


function dec(hex)
{
 return parseInt(hex,16);
}
function hex(dec, len)
{
 var hexString=Number(dec).toString(16);

 if(len&&hexString.length<len)
  for(var c=hexString.length;c<len;c++)
   hexString="0"+hexString;

 return hexString;
}

//flash class
function Flash(obj)
{
 var self=this;
 self.obj=obj;

 self.init=init;

 self.speed=speed;
 self.accelerate=accelerate;
 self.decelerate=decelerate;

 self.move=move;
 self.resize=resize;
 self.moveResize=moveResize;

 self.fontSize=fontSize;
 self.fontColor=fontColor;

 self.borderSize=borderSize;
 self.borderColor=borderColor;

 self.backgroundMove=backgroundMove;
 self.backgroundColor=backgroundColor;

 self.opacity=opacity;

 self.end=end;
 self.pause=pause;
 self.go=go;
 self.stop=stop;
 self.loop=loop;

 self.init();

 function init()
 {
  if(typeof(obj.tagName)=="undefined")
  {
   alert("Flash object not a tag");
   return false;
  }

  //speed
  self.updateSpeed=10;

  //waypoints
  self.wp=Array();

  //looping
  self.stopLooping=false;
  self.looping=false;

  //save original style
  if(typeof(window.getComputedStyle)!="undefined") //Opera, Firefox, Safari
   self.originalStyle=window.getComputedStyle(self.obj, null);
  else //IE
  {
   self.originalStyle=Object();

   for(var style in self.obj.currentStyle)
    eval("self.originalStyle."+style+"=self.obj.currentStyle[style]");
  }

  //current object size and position
  self.height=self.obj.offsetHeight;
  self.width=self.obj.offsetWidth;
  self.left=self.obj.offsetLeft;
  self.top=self.obj.offsetTop;

  //current X, Y, Z coordinate
  self.x=0;  //top and left are relative to page, 0 is relative to parent object
  self.y=0;
  self.z=0;

  if(self.originalStyle.zIndex&&self.originalStyle.zIndex!="auto")
   self.z=self.originalStyle.zIndex;

  self.xSpeed=0; //X, Y, Z speed (can be negative) 
  self.ySpeed=0;
  self.zSpeed=0;

  //current width and height
  self.w=self.width; 
  self.h=self.height;
  self.wSpeed=0; //width and height speed (can be negative) 
  self.hSpeed=0;

  //font size
  self._fontSize=0;
  self._fontSizeSpeed=0; 

  if(self.originalStyle.fontSize)
   self.fontSize(self.originalStyle.fontSize, 0, 0);

  //font color
  self._fontRed=0; 
  self._fontGreen=0;
  self._fontBlue=0;
  self._fontRedSpeed=0; 
  self._fontGreenSpeed=0;
  self._fontBlueSpeed=0;

  if(self.originalStyle.color&&self.originalStyle.color!="transparent")
   self.fontColor(self.originalStyle.color, 0, 0);

  //border size
  self._borderSize=0;
  self._borderSizeSpeed=0; 

  if(self.originalStyle.borderWidth)
   self.borderSize(self.originalStyle.borderWidth, 0, 0);

  //border color
  self._borderRed=0; 
  self._borderGreen=0;
  self._borderBlue=0;
  self._borderRedSpeed=0; 
  self._borderGreenSpeed=0;
  self._borderBlueSpeed=0;

  if(self.originalStyle.borderColor)
  {
   var m;
   if(m=self.originalStyle.borderColor.match(/#[0-F]{6}/)) //match any border color
    self.borderColor(m[0], 0, 0);  
  }

  //background position
  self._backgroundX=0;
  self._backgroundY=0;
  self._backgroundXSpeed=0;
  self._backgroundYSpeed=0;

  if(self.originalStyle.backgroundPosition)
  {
   var m;
   if(m=self.originalStyle.backgroundPosition.match(/([^\s]+)/g)) //match background positions
    self.backgroundMove(m[0], m[1], 0, 0);  
  }


  //background color
  self._backgroundRed=0; 
  self._backgroundGreen=0;
  self._backgroundBlue=0;
  self._backgroundRedSpeed=0; 
  self._backgroundGreenSpeed=0;
  self._backgroundBlueSpeed=0;

  if(self.originalStyle.backgroundColor&&self.originalStyle.backgroundColor!="transparent"&&self.originalStyle.backgroundColor.match(/rgb\(.*,\s*0/i))
   self.backgroundColor(self.originalStyle.backgroundColor, 0, 0);

  //opacity
  self._opacity=0;
  self._opacitySpeed=0; 

  self.go();
 }


//SPEED

 function speed(updateSpeed)
 {
  self.updateSpeed=updateSpeed;
  return true;
 }

 function accelerate(entry, speed)
 {
  if(!speed)
   speed=4;

  if(self.wp[entry])
  {
   self.wp[entry].accelerate=speed;
   return true;
  }

  return false;
 }

 function decelerate(entry)
 {
  return self.accelerate(entry, 0.25);
 }


//MOVE+RESIZE

 function move(x, y, z, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.move=Object();

  if(tmp=x.toString().match(/(-?\d+)([^\d]+)?/))
  {
   wp.move.x=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.move.xt=tmp[2];
   else
    wp.move.xt="px";
  }

  if(tmp=y.toString().match(/(-?\d+)([^\d]+)?/))
  {
   wp.move.y=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.move.yt=tmp[2];
   else
    wp.move.yt="px";
  }

  wp.move.z=z;

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function resize(w, h, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.size=Object();

  if(tmp=w.toString().match(/(\d+)([^\d]+)?/))
  {
   wp.size.w=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.size.wt=tmp[2];
   else
    wp.size.wt="px";
  }

  if(tmp=h.toString().match(/(\d+)([^\d]+)?/))
  {
   wp.size.h=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.size.ht=tmp[2];
   else
    wp.size.ht="px";
  }


  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function moveResize(x, y, z, w, h, ms)
 {
  entry=self.move(x, y, z, ms);
  entry=self.resize(w, h, ms, entry);
  return entry;
 }


//FONT

 function fontSize(size, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.fontSize=Object();

  if(tmp=size.toString().match(/(\d+)([^\d]+)?/))
  {
   wp.fontSize.size=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.fontSize.sizeType=tmp[2];
   else
    wp.fontSize.sizeType="px";
  }

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function fontColor(color, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.fontColor=Object();

  var m;
  if(m=color.match(/([0-9A-F]{2})([0-9A-F]{2})([0-9A-F]{2})/i))
  {
   wp.fontColor.r=dec(m[1]);
   wp.fontColor.g=dec(m[2]);
   wp.fontColor.b=dec(m[3]);
  }
  else if(m=color.match(/([0-9]+),\s?([0-9]+),\s+([0-9]+)/i))
  {
   wp.fontColor.r=m[1];
   wp.fontColor.g=m[2];
   wp.fontColor.b=m[3];
  }
  else
   alert("Can't match color code: "+color);

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }


//BORDER

 function borderSize(size, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.borderSize=Object();

  if(tmp=size.toString().match(/(\d+)(\w+)?/))
  {
   wp.borderSize.size=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.borderSize.sizeType=tmp[2];
   else
    wp.borderSize.sizeType="px";
  }

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function borderColor(color, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.borderColor=Object();

  var m;
  if(m=color.match(/([0-9A-F]{2})([0-9A-F]{2})([0-9A-F]{2})/i))
  {
   wp.borderColor.r=dec(m[1]);
   wp.borderColor.g=dec(m[2]);
   wp.borderColor.b=dec(m[3]);
  }
  else if(m=color.match(/([0-9]+),\s?([0-9]+),\s+([0-9]+)/i))
  {
   wp.borderColor.r=m[1];
   wp.borderColor.g=m[2];
   wp.borderColor.b=m[3];
  }
  else
   alert("Can't match color code: "+color);

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }


//BACKGROUND

 function backgroundMove(x, y, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.backgroundMove=Object();

  if(tmp=x.toString().match(/(-?\d+)([^\d]+)?/))
  {
   wp.backgroundMove.x=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.backgroundMove.xt=tmp[2];
   else
    wp.backgroundMove.xt="px";
  }

  if(tmp=y.toString().match(/(-?\d+)([^\d]+)?/))
  {
   wp.backgroundMove.y=tmp[1];

   if(typeof(tmp[2])!="undefined")
    wp.backgroundMove.yt=tmp[2];
   else
    wp.backgroundMove.yt="px";
  }

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function backgroundColor(color, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.backgroundColor=Object();

  var m;
  if(m=color.match(/([0-9A-F]{2})([0-9A-F]{2})([0-9A-F]{2})/i))
  {
   wp.backgroundColor.r=dec(m[1]);
   wp.backgroundColor.g=dec(m[2]);
   wp.backgroundColor.b=dec(m[3]);
  }
  else if(m=color.match(/([0-9]+),\s?([0-9]+),\s+([0-9]+)/i))
  {
   wp.backgroundColor.r=m[1];
   wp.backgroundColor.g=m[2];
   wp.backgroundColor.b=m[3];
  }
  else
   alert("Can't match color code: "+color);

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }


//OPACITY
 
 function opacity(opacity, ms, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.opacity=Object();

  if(opacity<1)
   opacity*=100;

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.opacity.o=opacity;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }


//CONTROL

 function end(end, entry)
 {
  if(self.wp[entry])
   var wp=self.wp[entry];
  else
  {
   var wp=Object();
   entry=self.wp.length;
  }

  wp.onEnd=function End(){eval(end)};

  self.wp[entry]=wp;

  return entry
 }

 function pause(ms)
 {
  var wp=Object();
  entry=self.wp.length;

  if(ms<self.updateSpeed)
   ms=self.updateSpeed;

  wp.step=0;
  wp.steps=ms/self.updateSpeed;
  wp.ms=ms;

  self.wp[entry]=wp;
  return entry;
 }

 function go()
 {
  if(self.looping==false)
  {
   self.looping=true;
   self.loop();
  }
 }

 function stop()
 {
  if(self.looping==true)
  {
   self.stopLooping=true;
   self.loop();
  }
 }

 function loop()
 {
  if(self.wp.length==0)
   return;

  var wp=self.wp[0];

  if(wp.step==0) //new waypoint
  {
   //set travel speed and direction
   if(wp.move)
   {
    self.xSpeed=(wp.move.x-self.x)/wp.steps;
    self.ySpeed=(wp.move.y-self.y)/wp.steps;
    self.zSpeed=(wp.move.z-self.z)/wp.steps;
   }
   
   if(wp.size)
   {
    self.wSpeed=(wp.size.w-self.w)/wp.steps;
    self.hSpeed=(wp.size.h-self.h)/wp.steps;
   }


   if(wp.fontSize)
   {
    self._fontSizeSpeed=(wp.fontSize.size-self._fontSize)/wp.steps;
   }

   if(wp.fontColor)
   {
    self._fontRedSpeed=(wp.fontColor.r-self._fontRed)/wp.steps;
    self._fontGreenSpeed=(wp.fontColor.g-self._fontGreen)/wp.steps;
    self._fontBlueSpeed=(wp.fontColor.b-self._fontBlue)/wp.steps;
   }
  

   if(wp.borderSize)
   {
    self._borderSizeSpeed=(wp.borderSize.size-self._borderSize)/wp.steps;
   }

   if(wp.borderColor)
   {
    self._borderRedSpeed=(wp.borderColor.r-self._borderRed)/wp.steps;
    self._borderGreenSpeed=(wp.borderColor.g-self._borderGreen)/wp.steps;
    self._borderBlueSpeed=(wp.borderColor.b-self._borderBlue)/wp.steps;
   }
  

   if(wp.backgroundMove)
   {
    self._backgroundXSpeed=(wp.backgroundMove.x-self._backgroundX)/wp.steps;
    self._backgroundYSpeed=(wp.backgroundMove.y-self._backgroundY)/wp.steps;
   }

   if(wp.backgroundColor)
   {
    self._backgroundRedSpeed=(wp.backgroundColor.r-self._backgroundRed)/wp.steps;
    self._backgroundGreenSpeed=(wp.backgroundColor.g-self._backgroundGreen)/wp.steps;
    self._backgroundBlueSpeed=(wp.backgroundColor.b-self._backgroundBlue)/wp.steps;
   }
   

   if(wp.opacity)
   {
    self._opacitySpeed=(wp.opacity.o-self._opacity)/wp.steps;
   }
   
  }

  wp.step++;

  if(wp.accelerate)
  {
   step=Math.pow((wp.step/wp.steps), wp.accelerate)*wp.steps;

   if(step<0.01)
    step=0.01;
  }
  else
   step=wp.step;


  if(wp.move)
  {
   var left=Math.round(self.x+(self.xSpeed*step));
   var top=Math.round(self.y+(self.ySpeed*step));
   var zIndex=Math.round(self.z+(self.zSpeed*step));
   self.obj.style.marginLeft=left+wp.move.xt;
   self.obj.style.marginTop=top+wp.move.yt;
   self.obj.style.zIndex=zIndex;
  }

  if(wp.size)
  {
   var width=Math.round(self.w+(self.wSpeed*step));
   var height=Math.round(self.h+(self.hSpeed*step));

   if(width<0)
    width=0;
   if(height<0)
    height=0;

   self.obj.style.width=width+wp.size.wt;
   self.obj.style.height=height+wp.size.ht;
  }


  if(wp.fontSize)
  {
   var fontSize=self._fontSize+(self._fontSizeSpeed*step);
   self.obj.style.fontSize=fontSize+wp.fontSize.sizeType;
  }

  if(wp.fontColor)
  {
   var font_red=Math.round(self._fontRed+(self._fontRedSpeed*step));
   var font_green=Math.round(self._fontGreen+(self._fontGreenSpeed*step));
   var font_blue=Math.round(self._fontBlue+(self._fontBlueSpeed*step));

   tmp=Array();
   tmp[0]=hex(font_red);
   tmp[1]=hex(font_green);
   tmp[2]=hex(font_blue);
   if(tmp[0].length==1)
    tmp[0]="0"+tmp[0];
   if(tmp[1].length==1)
    tmp[1]="0"+tmp[1];
   if(tmp[2].length==1)
    tmp[2]="0"+tmp[2];

   self.obj.style.color="#"+tmp[0]+tmp[1]+tmp[2];
  }


  if(wp.borderSize)
  {
   var borderSize=self._borderSize+(self._borderSizeSpeed*step);
   self.obj.style.borderWidth=borderSize+wp.borderSize.sizeType;
  }

  if(wp.borderColor)
  {
   var border_red=Math.round(self._borderRed+(self._borderRedSpeed*step));
   var border_green=Math.round(self._borderGreen+(self._borderGreenSpeed*step));
   var border_blue=Math.round(self._borderBlue+(self._borderBlueSpeed*step));

   tmp=Array();
   tmp[0]=hex(border_red);
   tmp[1]=hex(border_green);
   tmp[2]=hex(border_blue);
   if(tmp[0].length==1)
    tmp[0]="0"+tmp[0];
   if(tmp[1].length==1)
    tmp[1]="0"+tmp[1];
   if(tmp[2].length==1)
    tmp[2]="0"+tmp[2];

   self.obj.style.borderColor="#"+tmp[0]+tmp[1]+tmp[2];
  }


  if(wp.backgroundMove)
  {
   var bg_left=Math.round(self._backgroundX+(self._backgroundXSpeed*step));
   var bg_top=Math.round(self._backgroundY+(self._backgroundYSpeed*step));
   self.obj.style.backgroundPosition=bg_left+wp.backgroundMove.xt+" "+bg_top+wp.backgroundMove.yt;
  }

  if(wp.backgroundColor)
  {
   var bg_red=Math.round(self._backgroundRed+(self._backgroundRedSpeed*step));
   var bg_green=Math.round(self._backgroundGreen+(self._backgroundGreenSpeed*step));
   var bg_blue=Math.round(self._backgroundBlue+(self._backgroundBlueSpeed*step));

   tmp=Array();
   tmp[0]=hex(bg_red);
   tmp[1]=hex(bg_green);
   tmp[2]=hex(bg_blue);
   if(tmp[0].length==1)
    tmp[0]="0"+tmp[0];
   if(tmp[1].length==1)
    tmp[1]="0"+tmp[1];
   if(tmp[2].length==1)
    tmp[2]="0"+tmp[2];

   self.obj.style.backgroundColor="#"+tmp[0]+tmp[1]+tmp[2];
  }


  if(wp.opacity)
  {
   var opacity=self._opacity+(self._opacitySpeed*step);

   if(typeof self.obj.style.opacity!='undefined')
    self.obj.style.opacity=opacity/100;
   else
    self.obj.style.filter="alpha(opacity="+opacity+")";
  }


  if(!self.stopLooping&&wp.step<wp.steps)
  {
   setTimeout(self.loop, self.updateSpeed);
  }
  else
  {
   if(wp.move)
   {
    self.x=left; //set new coordinates
    self.y=top;
    self.z=zIndex;
   }

   if(wp.size)
   {
    self.w=width;
    self.h=height;
   }


   if(wp.fontSize)
   {
    self._fontSize=fontSize;
   }

   if(wp.fontColor)
   {
    self._fontRed=font_red;
    self._fontGreen=font_green;
    self._fontBlue=font_blue;
   }


   if(wp.borderSize)
   {
    self._borderSize=borderSize;
   }

   if(wp.borderColor)
   {
    self._borderRed=border_red;
    self._borderGreen=border_green;
    self._borderBlue=border_blue;
   }


   if(wp.backgroundMove)
   {
    self._backgroundX=bg_left;
    self._backgroundY=bg_top;
   }

   if(wp.backgroundColor)
   {
    self._backgroundRed=bg_red;
    self._backgroundGreen=bg_green;
    self._backgroundBlue=bg_blue;
   }


   if(wp.opacity)
   {
    self._opacity=opacity;
   }


   if(self.wp.length>0)
   {
    var lll=self.wp.length;
    for(ccc=1;ccc<lll;ccc++) 
     self.wp[ccc-1]=self.wp[ccc];

    self.wp.length=lll-1;
   }

   if(self.wp.length>0)
    setTimeout(self.loop, self.updateSpeed);

   if(wp.onEnd)
    wp.onEnd();

   self.stopLooping=false;
   self.looping=false;
  }
 }
}
