

 /*****************************************************************************
 * Filename: script/functions.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


/*
 *
 * onload
 */

function onload(code)
{
 var alreadyrunflag=0;

 if(/Safari/i.test(navigator.userAgent))
 {
  var _timer=setInterval(function()
  {
   if(/loaded|complete/.test(document.readyState))
   {
    clearInterval(_timer)
    alreadyrunflag=1;
    eval(code);
   }
  }, 10);
 }
 else if(document.addEventListener)
 {
  document.addEventListener("DOMContentLoaded", function(){alreadyrunflag=1; eval(code);}, false);
 }
 else if (document.all && !window.opera)
 {
  var contentloaded="contentloaded"+code.replace(/[^a-z0-9]/gi, "");
  document.write('<script type="text/javascript" id="'+contentloaded+'" defer="defer" src=""><\/script>');

  var contentloadtag=document.getElementById(contentloaded);
  contentloadtag.onreadystatechange=function()
  {
   if(this.readyState=="complete")
   {
    alreadyrunflag=1;
    eval(code);
   }
  }
 }
}


/*
 *
 * in_array (automatically initialized)
 * */

Array.prototype.in_array=function(val)
{
 var l=this.length;
 for(var c=0;c<l;c++)
  if(this[c]==val)
   return true;

 return false;
}


/*
 *
 * htmlEncode/htmlDecode (automatically initialized)
 * */

var htmlEntities=Array();
htmlEntities["&"]="&amp;";
htmlEntities['"']="&quot;";
htmlEntities["'"]="&apos;";
htmlEntities["<"]="&lt;";
htmlEntities[">"]="&gt;";
htmlEntities[" "]="&nbsp;";
htmlEntities[""]="&iexcl;";
htmlEntities[""]="&cent;";
htmlEntities[""]="&pound;";
htmlEntities[""]="&curren;";
htmlEntities[""]="&yen;";
htmlEntities[""]="&brvbar;";
htmlEntities[""]="&sect;";
htmlEntities[""]="&uml;";
htmlEntities[""]="&copy;";
htmlEntities[""]="&ordf;";
htmlEntities[""]="&laquo;";
htmlEntities[""]="&not;";
htmlEntities[""]="&reg;";
htmlEntities[""]="&macr;";
htmlEntities[""]="&deg;";
htmlEntities[""]="&plusmn;";

String.prototype.htmlEncode=function()
{
 var text=this;
 for(var key in htmlEntities)
  if(key!=" ")
   text=text.replace(key, htmlEntities[key]);

 return text;
}
String.prototype.htmlDecode=function()
{
 var text=this;

 for(var key in htmlEntities)
  if(key!=" ")
   text=text.replace(htmlEntities[key], key);

 return text;
}


/*
 *
 * Debug
 * */

function d()
{
 alert(_d(arguments));
}

function _d(args, level)
{
 if(typeof(level)=="undefined")
  level=0;

 if(args===null)
  return Array(level+1).join("   ")+"null";

 var debug="";
 for(_c in args)
 {
  if(_c=="in_array")
   continue;

  debug+=Array(level+1).join("   ");

  if(typeof(args[_c])=="undefined")
   debug+="undefined\n";
  else if(args[_c]==null)
   debug+="null\n";
  else if(typeof(args[_c])=="object")
  {
   if(typeof(args[_c].length)!="undefined")
    debug+="Array["+_c+"]: ";
   else
    debug+="Object."+_c+": ";

   debug+="\n"+_d(args[_c], level+1);
  }
  else
  {
   debug+=typeof(args[_c])+": "+_c+": ";
   debug+=args[_c]+"\n";
  }
 }

 return debug;
}


/*
 *
 * Diff
 */

function initDiffFunction()
{
 String.prototype.diff=function(string, split, full)
 {
  //what level to diff (i.e. how to split the string): line, word or char (default)
  if(typeof(split)=="undefined")
   split="character";

  if(split=="line")
   split="\n";
  else if(split=="word")
   split=" ";
  else //character
   split=false;

  //output full diffed string? (-+===-+===== vs -j+Java-s+Script)
  if(typeof(full)=="undefined"||!full)
   full=true;
  else
   full=true;

  //split string according to split character
  var text1, text2;

  if(split)
  {
   text1=this.split(split);
   text2=string.split(split);
  }
  else
  {
   text1=this;
   text2=string;
  }

  //determine longest common subsequence
  var lcs=Array();

  for(var c=-1;c<text1.length;c++)
  {
   lcs[c]=Array();
   lcs[c][-1]=0;
  }

  for(var r=-1;r<text2.length;r++)
   lcs[-1][r]=0;

  for(var c=0;c<text1.length;c++)
  {
   for(var r=0;r<text2.length;r++)
   {
    if(text1[c]==text2[r])
     lcs[c][r]=lcs[c-1][r-1]+1;
    else
     lcs[c][r]=Math.max(lcs[c][r-1], lcs[c-1][r]);
   }
  }

  //convert lcs to diff string
  var diff=this.lcs2diff(text1, text2, text1.length-1, text2.length-1, lcs);

  //return diff if not in full mode
  if(!full)
   return diff;

  //generate full diff string
  var output="";

  var offset1=0;
  var offset2=0;

  for(var c=0;c<diff.length;c++)
  {
   if(split&&c>0)
    output+=split;

   if(diff[c]=="-") //deleted
   {
    output+="-"+text1[c+offset1];
    offset2--;
   }
   else if(diff[c]=="+") //added
   {
    output+="+"+text2[c+offset2];
    offset1--;
   }
   else //same
    output+=text1[c+offset1];
  }

  return output;
 }

 String.prototype.lcs2diff=function(text1, text2, c, r, lcs)
 {
  var diff="";

  if(c>-1&&r>-1&&text1[c]==text2[r]) //same
  {
   diff+=this.lcs2diff(text1, text2, c-1, r-1, lcs);
   diff+="=";
  }
  else if(r>-1&&(c==-1||lcs[c][r-1]>=lcs[c-1][r])) //added
  {
   diff+=this.lcs2diff(text1, text2, c, r-1, lcs);
   diff+="+";
  }
  else if(c>-1&&(r==-1||lcs[c][r-1]<lcs[c-1][r])) //deleted
  {
   diff+=this.lcs2diff(text1, text2, c-1, r, lcs);
   diff+="-";
  }

  return diff;
 }
}


/*
 *
 * Add/Remove event
 * */

Element.prototype.addEvent=function(event, action)
{
 if(window.addEventListener)
  this.addEventListener(event, action, false);
 else if(window.attachEvent)
  this.attachEvent("on"+event, action);
}

Element.prototype.removeEvent=function(event, action)
{
 if(window.addEventListener)
  this.removeEventListener(event, action, false);
 else if(window.attachEvent)
  this.detachEvent("on"+event, action);
}

