

 /*****************************************************************************
 * Filename: script/http.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(browser)=="undefined"||typeof(path)=="undefined"||typeof(rootpath)=="undefined")
 alert("JAS::JavaScript() not loaded");

function HTTP()
{
 var self=this;

 self.lastUpdated=Array();
 self.callbacks=Array();

 self.init=init;

 self.abort=abort;
 self.clearRequest=clearRequest;

 self.addVar=addVar;
 self.addHeader=addHeader;

 self.request=request;
 self.response=response;

 self.apiRequest=apiRequest;
 self.apiResponse=apiResponse;

 self.db=db;
 self.format=format;

 self.clear=clear;
 self.setValue=setValue;
 self.show=show;
 self.sort=sort;
 self.next=next;
 self.reset=reset;

 self.data=data;
 self.dataResponse=dataResponse;

 self.updated=updated;
 self.updatedResponse=updatedResponse;

 self.projects=projects;
 self.projectsResponse=projectsResponse;

 self.modules=modules;
 self.modulesResponse=modulesResponse;

 self.module=module;
 self.moduleResponse=moduleResponse;

 self.init();

 function init()
 {
  if(typeof(window.XMLHttpRequest)!="undefined")
   self.xmlhttp=new XMLHttpRequest();
  else if(typeof(window.ActiveXObject)!="undefined")
   self.xmlhttp=new ActiveXObject("Microsoft.XMLHTTP");
  else
  {
   self.xmlhttp=false;
   return false;
  }

  self.clearRequest();
  self.clear();

  return true;
 }


 function abort()
 {
  if(!self.xmlhttp) //xmlhttp not initialized
   return false;

  self.xmlhttp.abort();

  return true;
 }

 function clearRequest()
 {
  self.vars=Array();
  self.header=Array();

  self.url=false;
  self.xml=false;
  self.txt=false;
  self.error="";
 }


 function addVar(key, value)
 {
  self.vars[self.vars.length]=Array(key, value);
 }

 function addHeader(key, value)
 {
  self.header[self.header.length]=Array(key, value);
 }


//REQUEST
 function request(url, method, async)
 {
  if(!self.xmlhttp) //xmlhttp not initialized
   return false;

  //get arguments
  if(typeof(method)=="undefined")
   method="GET";

  if(typeof(async)=="undefined"||!async)
   async=false;
  else
   async=true;

  //prepare request
  self.url=url;
  self.xmlhttp.open(method, self.url, async);

  for(var c=0;c<self.header.length;c++)
   self.xmlhttp.setRequestHeader(self.header[c][0], self.header[c][1]);

  if(async) //asynchronous, use callbacks
  {
   self.xmlhttp.onreadystatechange=self.response
   self.xmlhttp.send(null);
   return true;
  }

  //synchronous
  self.xmlhttp.send(null);
  return self.response();
 }

 function response()
 {
  switch(self.xmlhttp.readyState)
  {
   case 0:
   {
    self.error="Request \""+self.url+"\" failed: request not initialized";
    return false;
   }
   case 1:
   {
    self.error="Request \""+self.url+"\" in progress: server connection established";
    break;
   }
   case 2:
   {
    self.error="Request \""+self.url+"\" in progress: request received";
    break;
   }
   case 3:
   {
    self.error="Request \""+self.url+"\" in progress: processing request";
    break;
   }
   case 4:
   {
    self.error="Request \""+self.url+"\" successful";

    //getAllResponseHeaders();

    if(self.xmlhttp.status!=200)
    {
     self.error="Request \""+self.url+"\" failed: "+self.xmlhttp.statusText+" ("+self.xmlhttp.status+")";
     return false;
    }

    self.xml=self.xmlhttp.responseXML;
    self.txt=self.xmlhttp.responseText;

    if(callback=self.callbacks.pop()) //asynchronous, use callbacks
     callback(self);

    break;
   }
   default:
   {
    self.error="Request \""+self.url+"\" failed: unknown ready state: "+self.xmlhttp.readyState;
    return false;
   }
  }

  return true;
 }


//XMLI
 function apiRequest(xml, async)
 {
  //get arguments
  if(typeof(async)=="undefined"||!async)
   async=false;
  else
   async=true;

  //set cookies so session is kept
  self.clearRequest();
  self.addHeader("Cookie", document.cookie);

  if(async) //asynchronous, use callback
  {
   self.callbacks.push(self.apiResponse);

   if(!self.request(rootpath+"api/xml/?request="+xml, "GET", async))
    return false;

   return true;
  }

  //synchronous
  if(!self.request(rootpath+"api/xml/?request="+xml, "GET", async))
   return false;

  return self.apiResponse();
 }

 function apiResponse()
 {
  //validate response
  if(self.xml==null)
  {
   self.error="No XML returned";
   return false;
  }

  if(self.xml.firstChild==null)
  {
   self.error="Invalid XML returned: "+self.txt.replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/\n/g, "<br/>").replace(/  /g, "&nbsp; ");
   return false;
  }

  if(browser.match(/internet explorer/i))
   var body=self.xml.childNodes[1];
  else
   var body=self.xml.childNodes[0];

  if(body.nodeName!="jas")
  {
   self.error="Unknown XML returned: "+self.txt.replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/\n/g, "<br/>").replace(/  /g, "&nbsp; ");
   return false;
  }

  var content=false;
  for(var c=0;c<body.childNodes.length;c++)
  {
   if(body.childNodes[c].nodeName!="#text")
   {
    content=body.childNodes[c];
    break;
   }
  }

  if(!content)
  {
   self.error="Unknown XML returned: "+self.txt.replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/\n/g, "<br/>").replace(/  /g, "&nbsp; ");
   return false;
  }

  if(content.nodeName=="errors") //error response
  {
   fatal=false;
   self.error="";
   var errors=content.getElementsByTagName("error"); //errors->error

   for(var c=0;c<errors.length;c++)
   {
    self.error+=errors[c].getAttribute("level").substr(0, 1).toUpperCase()+errors[c].getAttribute("level").substr(1)+
     ": "+errors[c].getAttribute("error")+
     (errors[c].getAttribute("extra")?" ("+errors[c].getAttribute("extra")+")":"")+
     "<br/>";

    if(errors[c].getAttribute("level")!="debug"&&errors[c].getAttribute("level")!="note"&&errors[c].getAttribute("level")!="warning")//fatal error
     fatal=true;
   }

   if(fatal)
    return false;
  }


  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }


//DATA
 function db(db)
 {
  self.datadb=db;
  return true;
 }

 function format(format)
 {
  self.dataformat=format;
  return true;
 }


 function clear()
 {
  self.v=Array();
  self.vc=0;

  self.show(); //default show fields
  self.sort(); //no sorting
 }

 function setValue(key, value, row)
 {
  if(typeof(row)=="undefined")
   row=0;

  if(typeof(self.v[row])=="undefined")
   self.v[row]=Array();

  self.v[row][key]=value;
  return true;
 }

 function show()
 {
  var fields;

  if(arguments.length==1&&typeof(arguments[0].length)!="undefined")
   fields=arguments[0];
  else
   fields=arguments;

  self.showFields=fields;
  return true;
 }

 function sort()
 {
  var fields;

  if(arguments.length==1&&typeof(arguments[0].length)!="undefined")
   fields=arguments[0];
  else
   fields=arguments;

  self.sortFields=fields;
  return true;
 }

 function next()
 {
  if(self.vc<self.v.length)
   return self.v[self.vc++];

  self.vc=0;
  return false;
 }

 function reset()
 {
  self.vc=0;
  return true;
 }


 function data(module, action, extra, callback)
 {
  if(typeof(action)=="undefined")
   action="view";
  if(typeof(extra)=="undefined")
   extra="all";

  var xml='<?xml version="1.0" encoding="ISO-8859-1"?>\n';
  xml+='<jas version="1.0">\n';
  xml+=' <data '+
   ' module="'+module+'"'+
   ' action="'+action+'"'+
   ' extra="'+extra+'"'+
   (typeof(self.dataformat)&&self.dataformat?' format="'+self.dataformat+'"':'')+
   (typeof(self.datadb)&&self.datadb?' db="'+self.datadb+'"':'')+
   '>\n';

  for(c=0;c<self.v.length;c++)
  {
   xml+='  <row>\n';

   for(key in self.v[c])
    if(key!="in_array")
     xml+='   <field name="'+key+'" value="'+self.v[c][key].toString().htmlEncode()+'" />\n'

   xml+='  </row>\n';
  }

  if(self.showFields.length)
  {
   xml+='  <show>\n';

   for(c=0;c<self.showFields.length;c++)
    xml+='   <field name="'+self.showFields[c]+'" />\n'

   xml+='  </show>\n';
  }

  if(self.sortFields.length)
  {
   xml+='  <sort>\n';

   for(c=0;c<self.sortFields.length;c++)
    xml+='   <field name="'+self.sortFields[c]+'" />\n'

   xml+='  </sort>\n';
  }

  xml+=' </data>\n';
  xml+='</jas>\n';

  if(typeof(callback)!="undefined") //asynchronous, use callbacks
  {
   self.callbacks.push(callback);
   self.callbacks.push(self.dataResponse);

   return self.apiRequest(xml, true);
  }

  //synchronous
  if(!self.apiRequest(xml, false))
   return false;

  return self.dataResponse();
 }

 function dataResponse()
 {
  //get data from response
  if(browser.match(/internet explorer/i))
   var jas=self.xml.childNodes[1];
  else
   var jas=self.xml.childNodes[0];

  var data=false;
  for(var c=0;c<jas.childNodes.length;c++)
  {
   if(jas.childNodes[c].nodeName!="#text")
   {
    data=jas.childNodes[c];
    break;
   }
  }

  if(!data)
   return false;

  if(data.nodeName=="data")
  {
   self.clear();

   var rows=data.getElementsByTagName("row"); //data->row

   for(var c=0;c<rows.length;c++)
   {
    self.v[c]=Array();
    fields=rows[c].getElementsByTagName("field"); //data->row->field

    for(var x=0;x<fields.length;x++)
     self.v[c][fields[x].getAttribute("name")]=(fields[x].getAttribute("value")!=undefined?fields[x].getAttribute("value"):fields[x].innerHTML);
   }
  }
  //not a data response, probably a non fatal error....

  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }


//UPDATED
 function updated(module, callback)
 {
  //get last updated
  if(typeof(self.lastUpdated[module])=="undefined")
  {
   self.clear(); //don't set info_updated, will return last updated immediately

   if(!(jas=self.data(module, "database", "info"))) //synchronous call
    return false;

   var row=self.next();
   self.lastUpdated[module]=row["info_updated"];
  }

  //wait for update
  self.clear();
  self.setValue("info_updated", self.lastUpdated[module]); //set last updated

  self.updatedModule=module; //set module for later

  if(typeof(callback)!="undefined") //asynchronous, use callbacks
  {
   self.callbacks.push(callback);
   return self.data(module, "database", "info", self.updatedResponse);
  }

  //synchronous
  if(!self.data(module, "database", "info"))
   if(!self.error.match(/no updates/)) //not a timeout error
    return false; //fail

  return self.updatedResponse();
 }

 function updatedResponse()
 {
  if(self.error.match(/no updates/)) //timeout error
  {
   //wait for next update
   if(callback=self.callbacks.pop()) //asynchronous, use callbacks
    return self.updated(self.updatedModule, callback);

   return self.updated(self.updatedModule);
  }

  var row=self.next();

  self.lastUpdated[self.updatedModule]=row["info_updated"]; //save value

  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }


//PROJECTS

 function projects(callback)
 {
  var xml='<?xml version="1.0" encoding="ISO-8859-1"?>\n';
  xml+='<jas version="1.0">\n';
  xml+=' <projects/>\n';
  xml+='</jas>\n';

  if(typeof(callback)!="undefined") //asynchronous, use callbacks
  {
   self.callbacks.push(callback);
   self.callbacks.push(self.projectsResponse);

   return self.apiRequest(xml, true);
  }

  //synchronous
  if(!self.apiRequest(xml, false))
   return false;

  return self.projectsResponse();
 }

 function projectsResponse()
 {
  //get projects from response
  if(browser.match(/internet explorer/i))
   var jas=self.xml.childNodes[1];
  else
   var jas=self.xml.childNodes[0];

  var projects=false;
  for(var c=0;c<jas.childNodes.length;c++)
  {
   if(jas.childNodes[c].nodeName!="#text")
   {
    projects=jas.childNodes[c];
    break;
   }
  }

  if(!projects)
   return false;

  if(projects.nodeName=="projects")
  {
   self.clear();

   var projects=projects.getElementsByTagName("project"); //projects->project

   for(var c=0;c<projects.length;c++)
   {
    self.v[c]=Array();

    self.v[c]["name"]=projects[c].getAttribute("name");
   }
  }
  //not a projects response, probably a non fatal error....

  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }


//MODULES

 function modules(all, callback)
 {
  if(typeof(all)=="undefined")
   all=false;

  var xml='<?xml version="1.0" encoding="ISO-8859-1"?>\n';
  xml+='<jas version="1.0">\n';
  xml+=' <modules'+(all?' all="true"':'')+'/>\n';
  xml+='</jas>\n';

  if(typeof(callback)!="undefined") //asynchronous, use callbacks
  {
   self.callbacks.push(callback);
   self.callbacks.push(self.modulesResponse);

   return self.apiRequest(xml, true);
  }

  //synchronous
  if(!self.apiRequest(xml, false))
   return false;

  return self.modulesResponse();
 }

 function modulesResponse()
 {
  //get modules from response
  if(browser.match(/internet explorer/i))
   var jas=self.xml.childNodes[1];
  else
   var jas=self.xml.childNodes[0];

  var modules=false;
  for(var c=0;c<jas.childNodes.length;c++)
  {
   if(jas.childNodes[c].nodeName!="#text")
   {
    modules=jas.childNodes[c];
    break;
   }
  }

  if(!modules)
   return false;

  if(modules.nodeName=="modules")
  {
   self.clear();

   var modules=modules.getElementsByTagName("module"); //modules->module

   for(var c=0;c<modules.length;c++)
   {
    self.v[c]=Array();

    self.v[c]["name"]=modules[c].getAttribute("name");
   }
  }
  //not a modules response, probably a non fatal error....

  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }


//MODULE

 function module(module, callback)
 {
  var xml='<?xml version="1.0" encoding="ISO-8859-1"?>\n';
  xml+='<jas version="1.0">\n';
  xml+=' <modules>\n';
  xml+='  <module name="'+module+'" />\n';
  xml+=' </modules>\n';
  xml+='</jas>\n';

  if(typeof(callback)!="undefined") //asynchronous, use callbacks
  {
   self.callbacks.push(callback);
   self.callbacks.push(self.moduleResponse);

   return self.apiRequest(xml, true);
  }

  //synchronous
  if(!self.apiRequest(xml, false))
   return false;

  return self.moduleResponse();
 }

 function moduleResponse()
 {
  //get modules from response
  if(browser.match(/internet explorer/i))
   var jas=self.xml.childNodes[1];
  else
   var jas=self.xml.childNodes[0];

  var modules=false;
  for(var c=0;c<jas.childNodes.length;c++)
  {
   if(jas.childNodes[c].nodeName!="#text")
   {
    modules=jas.childNodes[c];
    break;
   }
  }

  if(!modules)
   return false;

  if(modules.nodeName=="modules")
  {
   self.clear();

   var modules=modules.getElementsByTagName("module"); //modules->module

   for(var c=0;c<modules.length;c++)
   {
    self.v[c]=Array();

    self.v[c]["name"]=modules[c].getAttribute("name");
    self.v[c]["title"]=modules[c].getAttribute("title");

    self.v[c]["fields"]=Array();
    var fields=modules[c].getElementsByTagName("field"); //modules->module->field

    for(var x=0;x<fields.length;x++)
    {
     self.v[c]["fields"][fields[x].getAttribute("name")]=Array();
     self.v[c]["fields"][fields[x].getAttribute("name")]["type"]       = fields[x].getAttribute("type");
     self.v[c]["fields"][fields[x].getAttribute("name")]["use"]        = fields[x].getAttribute("use");
     self.v[c]["fields"][fields[x].getAttribute("name")]["properties"] = (fields[x].getAttribute("properties")!==null?fields[x].getAttribute("properties"):false);
     self.v[c]["fields"][fields[x].getAttribute("name")]["min"]        = (fields[x].getAttribute("min")!==null?parseFloat(fields[x].getAttribute("min")):false);
     self.v[c]["fields"][fields[x].getAttribute("name")]["max"]        = (fields[x].getAttribute("max")!==null?parseFloat(fields[x].getAttribute("max")):false);
     self.v[c]["fields"][fields[x].getAttribute("name")]["title"]      = fields[x].getAttribute("title");
    }

    self.v[c]["references"]=Array();
    var references=modules[c].getElementsByTagName("reference"); //modules->module->reference

    for(var x=0;x<references.length;x++)
    {
     self.v[c]["references"][references[x].getAttribute("name")]=Array();
     self.v[c]["references"][references[x].getAttribute("name")]["name"]       = references[x].getAttribute("name");
     self.v[c]["references"][references[x].getAttribute("name")]["module"]     = references[x].getAttribute("module");
     self.v[c]["references"][references[x].getAttribute("name")]["from"]       = references[x].getAttribute("from");
     self.v[c]["references"][references[x].getAttribute("name")]["to"]         = references[x].getAttribute("to");
     self.v[c]["references"][references[x].getAttribute("name")]["type"]       = references[x].getAttribute("type");
    }
   }
  }
  //not a modules response, probably a non fatal error....

  if(callback=self.callbacks.pop()) //asynchronous, use callbacks
   callback(self);

  return true;
 }
}
