

 /*****************************************************************************
 * Filename: project/jas/script/search.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(onload)!='function')
 alert("Script not loaded: functions.js");
else if(typeof(HTTP)!="function")
 alert("Script not loaded: http.js");
else
 onload("initSearch();");

function initSearch()
{
 //get search options elements
 var search;
 if(!(search=document.getElementsByName("_searchOptions")))
  return false;

 for(var c=0;c<search.length;c++)
 {
  //parse ; separated list of key=value options
  var options=Array();
  var tmp=search[c].value.split(";");
  for(var x=0;x<tmp.length;x++)
  {
   var tmp2=tmp[x].split("=");
   options[tmp2[0]]=tmp2[1];
  }

  if(typeof(options["request"])=="undefined") //options need at least request type set
   continue; //ignore

  //find next input type="text" element
  var input=false;
  var next=search[c];
  while(next=next.nextSibling)
  {
   if(next.nodeName.toLowerCase()=="input"&&next.type.toLowerCase()=="text")
   {
    input=next;
    break;
   }
  }

  //search options and input text field found
  if(input)
  {
   search[c].parentNode.removeChild(search[c]); //ensure _searchOptions aren't submitted
   c--; //step back for subsequent _searchOptions

   //create results container
   var container=document.createElement("div");
   container.className="search";
   container.style.display="inline-block";
   container.input=input;
   input.container=container;
   input.parentNode.replaceChild(container, input); //replace current input with container
   container.appendChild(input); //add input to container

   //set options and events
   input.options=options;
   input.onkeyup=searchRequest;
   input.onfocus=searchShow;
   input.onblur=searchHide;

   //create results list
   var results=document.createElement("ul");
   results.className="results";
   results.style.position="absolute";
   results.input=input;
   input.results=results;
   container.appendChild(results); //add results list to container
  }
 }
}


function searchShow(input)
{
 if(typeof(input.srcElement)!="undefined") //event
  input=input.srcElement;
 //else //input = element

 input.results.style.visibility="visible";
}
function searchHide(input)
{
 if(typeof(input.srcElement)!="undefined") //event
  input=input.srcElement;
 //else //input = element

 if(input!=document.activeElement) //only hide if focus hasn't returned
  setTimeout(_searchHide, 100, input);
}
function _searchHide(input)
{
 input.results.style.visibility="hidden";
}


function searchRequest(e)
{
 var input;
 if(typeof(e.srcElement)!="undefined") //event
  input=e.srcElement;
 else //input = element
  input=e;

 var options=input.results.getElementsByTagName("li");

 //check if up/down error has been pressed
 if(e.keyCode==38) //up
 {
  select=options.length-1;
  for(var c=0;c<options.length;c++)
  {
   if(options[c].className=="selected")
   {
    if(c-1>=0)
     select=c-1;

    break;
   }
  }

  setSearchValue(input, options[select]);
  return true;
 }
 else if(e.keyCode==40) //down
 {
  select=0;
  for(var c=0;c<options.length;c++)
  {
   if(options[c].className=="selected")
   {
    if(c+1<options.length)
     select=c+1;

    break;
   }
  }

  setSearchValue(input, options[select]);
  return true;
 }

 //handle request
 var stop=false;
 var http=new HTTP();

 switch(input.options["request"])
 {
  case "projects":
  {
   if(input.value.length<1)
   {
    setSearchResults(input, Array());
    break;
   }

   //get all projects (just once)
   if(!input.projects)
   {
    searchLoading(input);
    input.projects=Array();
    if(!http.projects()&&http.error)
    {
     stop=true;
     break;
    }

    while(v=http.next())
     input.projects.push(v["name"]);
   }

   //search regexp, with prefix/suffix
   var regexp=input.value.toLowerCase()
   if(typeof(input.options["prefix"])!="undefined")
    regexp="^"+input.options["prefix"]+".*"+regexp;
   if(typeof(input.options["suffix"])!="undefined")
    regexp=regexp+".*"+input.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<input.projects.length;c++)
    if(input.projects[c].match(regexp))
     results.push(input.projects[c]);

   setSearchResults(input, results);

   break;
  }
  case "modules":
  {
   if(input.value.length<1)
   {
    setSearchResults(input, Array());
    break;
   }

   //get all modules (just once)
   if(!input.modules)
   {
    searchLoading(input);
    input.modules=Array();
    if(!http.modules()&&http.error)
    {
     stop=true;
     break;
    }

    while(v=http.next())
     input.modules.push(v["name"]);
   }

   //search regexp, with prefix/suffix
   var regexp=input.value.toLowerCase()
   if(typeof(input.options["prefix"])!="undefined")
    regexp="^"+input.options["prefix"]+".*"+regexp;
   if(typeof(input.options["suffix"])!="undefined")
    regexp=regexp+".*"+input.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<input.modules.length;c++)
    if(input.modules[c].match(regexp))
     results.push(input.modules[c]);

   setSearchResults(input, results);

   break;
  }
  case "plugins":
  {
   if(input.value.length<1)
   {
    setSearchResults(input, Array());
    break;
   }

   //get all modules (just once)
   if(!input.plugins)
   {
    searchLoading(input);
    input.plugins=Array();
    if(!http.modules()&&http.error)
    {
     stop=true;
     break;
    }

    var m;
    while(v=http.next())
     if(m=v["name"].match(/^(.+)_[a-z]+$/i)) //get plugin name from module: plugin_module
      if(!input.plugins.in_array(m[1])) //plugin not yet in results
       input.plugins.push(m[1]);
   }

   //search regexp, with prefix/suffix
   var regexp=input.value.toLowerCase()
   if(typeof(input.options["prefix"])!="undefined")
    regexp="^"+input.options["prefix"]+".*"+regexp;
   if(typeof(input.options["suffix"])!="undefined")
    regexp=regexp+".*"+input.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<input.plugins.length;c++)
    if(input.plugins[c].match(regexp))
     results.push(input.plugins[c]);

   setSearchResults(input, results);

   break;
  }
  case "data":
  {
   //validate input and options
   if(input.value.length<2)
   {
    setSearchResults(input, Array());
    break;
   }

   if(!http.setValue("search", input.value))
    break;

   if(!http.setValue("show_total", 10))
    break;

   if(!http.setValue("aggregation_field", input.options["field"])) //show unique values only
    break;

   if(!http.show(Array(input.options["field"])))
    break;

   //get data
   searchLoading(input);
   if(!http.data(input.options["module"], "search", input.options["extra"])&&http.error)
   {
    stop=true;
    break;
   }

   //handle results
   var results=Array();
   while(v=http.next())
    results.push(v[input.options["field"]]);

   setSearchResults(input, results);

   break;
  }
  default:
  {
   stop=true;
   break;
  }
 }

 if(stop) //failed to get data
 {
  //disabled search triggers
  var search=document.getElementById("search");

  input.onkeyup="";
  input.onfocus="";
  input.onblur="";

  setSearchResults(input, Array());
 }

 return true;
}

function setSearchResults(input, results)
{
 //clear results list
 while(input.results.firstChild)
  input.results.removeChild(input.results.firstChild);

 //add new options to results list
 var option;
 for(var c=0;c<results.length;c++)
 {
  option=document.createElement("li");
  option.innerHTML=results[c];
  option.onclick=setSearchValue;

  input.results.appendChild(option);
 }
}

function searchLoading(input)
{
 if(typeof(input.srcElement)!="undefined") //event
  input=input.srcElement;
 //else //input = element

 //clear results list
 while(input.results.firstChild)
  input.results.removeChild(input.results.firstChild);

 //add new options to results list
 var option=document.createElement("li");
 option.innerHTML="Loading...";
 option.className="loading";

 input.results.appendChild(option);
 input.results.style.visibility="visible";
}


function setSearchValue(input, option)
{
 if(typeof(input.srcElement)!="undefined") //event
 {
  option=input.srcElement; //event send from option
  input=option.parentNode.input; //get input from option's parent (ul)
  input.value=option.innerHTML;
  input.form.submit();
  return true;
 }
 //else //input = element

 //clear selected flag of other options
 var options=option.parentNode.getElementsByTagName("li");
 for(var c=0;c<options.length;c++)
 {
  if(options[c].className=="selected")
  {
   options[c].className="";
   break;
  }
 }

 //set option to search field and mark options as selected
 input.value=option.innerHTML;
 input.focus();
 option.className="selected";

 return true;
}
