

 /*****************************************************************************
 * Filename: script/selection.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(browser)=="undefined"||typeof(path)=="undefined"||typeof(rootpath)=="undefined")
 alert("JAS::JavaScript() not loaded");

function Selection()
{
 var self=this;

 //variables
 self.selections=Array();

 //functions
 self.init=init;
 self.error=error;
 self._getID=_getID;

 self.store=store;
 self.restore=restore;

 self.stored=stored;
 self.changed=changed;

 self.getText=getText;
 self.getHTML=getHTML;


 //initialize
 self.init();


 /*
  * Support functions
  */

 function init()
 {
 }


 function error(error, node)
 {
  //added node name to error
  if(typeof(node)!="undefined")
   error+=" ("+node.nodeName.toLowerCase()+")";

  //output error
  alert(error);

  return true;
 }

 function _getID(node)
 {
  for(var c=0;c<self.selections.length;c++)
   if(self.selections[c].node==node)
    return c;

  return false;
 }

 /*
  * store / restore
  */

 function store(node)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);
  if(ID===false) //not found, add new entry
  {
   ID=self.selections.length;
   self.selections[ID]={node:node};
  }

  //store current selection
  if(browser.match(/internet explorer/i)) //internet explorer specific logic
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     var activeNode=node.ownerDocument.activeElement; //store current active node
     node.focus(); //focus node

     //get selections
     var range=node.ownerDocument.selection.createRange();
     var range2=node.createTextRange();

     //get start position
     var range3=range2.duplicate();
     range2.moveToBookmark(range.getBookmark()); //set current selection to range
     range3.setEndPoint("EndToStart", range2); //move selection end to start of real selection
     self.selections[ID].start=range3.text.length; //selection length is start of real selection

     var range3=range2.duplicate();
     range2.moveToBookmark(range.getBookmark()); //set current selection to range
     range3.setEndPoint("StartToStart", range2); //move selection start to start of real selection
     self.selections[ID].end=self.selections[ID].start+range3.text.length; //selection length is length of selection

     activeNode.focus(); //restore focus

     break;
    }
    default: //html
    {
     self.selections[ID].range=node.ownerDocument.selection.createRange();

     break;
    }
   }
  }
  else //Opera, Mozilla, Chrome, Safari
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     self.selections[ID].start=node.selectionStart;
     self.selections[ID].end=node.selectionEnd;

     break;
    }
    default: //html
    {
     var selection=node.ownerDocument.defaultView.getSelection();

     if(!selection.rangeCount) //nothing selected
      self.selections[ID].range=false; //store no selection
     else
     {
      var range=selection.getRangeAt(0);

      self.selections[ID].range={
       startContainer:range.startContainer,
       startOffset:range.startOffset,
       endContainer:range.endContainer,
       endOffset:range.endOffset
      };
     }

     break;
    }
   }
  }

  return true;
 }


 function restore(node)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);
  if(ID===false) //not found, no selection
  {
   self.error("No stored selection for node", node);
   return false;
  }

  //restore selection
  if(browser.match(/internet explorer/i)) //internet explorer specific logic
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     var range=node.createTextRange();
     range.collapse(true);
     range.moveStart("character", self.selections[ID].start);
     range.moveEnd("character", self.selections[ID].end-self.selections[ID].start); //length
     range.select();

     break;
    }
    default: //html
    {
     self.selections[ID].range.select();

     break;
    }
   }
  }
  else //Opera, Mozilla, Chrome, Safari
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     node.setSelectionRange(self.selections[ID].start, self.selections[ID].end);

     break;
    }
    default: //html
    {
     var selection=node.ownerDocument.defaultView.getSelection();
     var range=node.ownerDocument.createRange();

     if(!self.selections[ID].range) //nothing selected
     {
      range.setStart(node, 0);
      range.setEnd(node, 0);
     }
     else
     {
      range.setStart(self.selections[ID].range.startContainer, self.selections[ID].range.startOffset);
      range.setEnd(self.selections[ID].range.endContainer, self.selections[ID].range.endOffset);
     }

     selection.removeAllRanges();
     selection.addRange(range);

     break;
    }
   }
  }

  return true;
 }


 /*
  * stored
  */

 function stored(node)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);
  if(ID===false) //not found, no selection
   return false;

  return true;
 }

 /*
  * changed
  */

 function changed(node)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);
  if(ID===false) //not found, no selection
  {
   self.error("No stored selection for node", node);
   return false;
  }

  //store current selection
  if(browser.match(/internet explorer/i)) //internet explorer specific logic
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     var activeNode=node.ownerDocument.activeElement; //store current active node
     node.focus(); //focus node

     //get selections
     var range=node.ownerDocument.selection.createRange();
     var range2=node.createTextRange();

     //get start position
     var range3=range2.duplicate();
     range2.moveToBookmark(range.getBookmark()); //set current selection to range
     range3.setEndPoint("EndToStart", range2); //move selection end to start of real selection

     if(self.selections[ID].start!=range3.text.length) //selection length is start of real selection
     {
      activeNode.focus(); //restore focus
      return true;
     }

     var range3=range2.duplicate();
     range2.moveToBookmark(range.getBookmark()); //set current selection to range
     range3.setEndPoint("StartToStart", range2); //move selection start to start of real selection

     if(self.selections[ID].end!=self.selections[ID].start+range3.text.length) //selection length is length of selection
     {
      activeNode.focus(); //restore focus
      return true;
     }

     activeNode.focus(); //restore focus

     break;
    }
    default: //html
    {
     if(!self.selections[ID].range.isEqual(node.ownerDocument.selection.createRange()))
      return true;

     break;
    }
   }
  }
  else //Opera, Mozilla, Chrome, Safari
  {
   switch(node.nodeName.toLowerCase())
   {
    case "textarea":
    {
     if( //compare selection
      self.selections[ID].start!=node.selectionStart||
      self.selections[ID].end!=node.selectionEnd
     )
      return true; //changed

     break;
    }
    default: //html
    {
     var selection=node.ownerDocument.defaultView.getSelection();

     if(!selection.rangeCount) //nothing selected
     {
      if(self.selections[ID].range) //selection stored
       return true; //changed
     }
     else
     {
      var range=selection.getRangeAt(0);

      if( //compare selection
        self.selections[ID].range.startContainer!=range.startContainer||
        self.selections[ID].range.startOffset!=range.startOffset||
        self.selections[ID].range.endContainer!=range.endContainer||
        self.selections[ID].range.endOffset!=range.endOffset
       )
       return true; //changed
     }

     break;
    }
   }
  }

  return false;
 }


 /*
  * getHTML / getText
  */

 function getText(node, stored)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);

  //check parameter: stored
  if(typeof(stored)=="undefined"||!stored)
   stored=false;
  else
   stored=true;

  //get selected text
  var text="";

  if(stored) //stored selection
  {
   if(ID===false)
   {
    self.error("No stored selection for node", node);
    return text;
   }

   //store current selection
   var current=new Selection();
   current.store(node);

   //restore selection
   self.restore(node);

   //get select text
   text=self.getText(node, false);

   //restore current selection
   current.restore(node);
  }
  else //current selection
  {
   if(browser.match(/internet explorer/i)) //internet explorer specific logic
   {
    switch(node.nodeName.toLowerCase())
    {
     default: //html
     {
      var range=node.ownerDocument.selection.createRange();

      if(typeof(range.text)!="undefined")
       text=range.text;

      break;
     }
    }
   }
   else //Opera, Mozilla, Chrome, Safari
   {
    switch(node.nodeName.toLowerCase())
    {
     case "textarea":
     {
      text=node.value.substr(node.selectionStart, node.selectionEnd);

      break;
     }
     default: //html
     {
      var selection=node.ownerDocument.defaultView.getSelection();
      text=selection.toString();

      break;
     }
    }
   }
  }

  return text;
 }


 function getHTML(node, stored)
 {
  //check parameter: node
  if(typeof(node.nodeName)=="undefined")
  {
   self.error("Selection node not a tag");
   return false;
  }

  //use document body in case of document node
  if(node.nodeName.toLowerCase()=="#document")
   node=node.body;

  //find ID
  var ID=self._getID(node);

  //check parameter: stored
  if(typeof(stored)=="undefined"||!stored)
   stored=false;
  else
   stored=true;

  //get selected html
  var html="";

  if(stored) //stored selection
  {
   if(ID===false)
   {
    self.error("No stored selection for node", node);
    return html;
   }

   //store current selection
   var current=new Selection();
   current.store(node);

   //restore selection
   self.restore(node);

   //get select text
   html=self.getHTML(node, false);

   //restore current selection
   current.restore(node);
  }
  else //current selection
  {
   if(browser.match(/internet explorer/i)) //internet explorer specific logic
   {
    switch(node.nodeName.toLowerCase())
    {
     case "textarea":
     {
      self.error("Can't get HTML for node", node);

      break;
     }
     default: //html
     {
      var range=node.ownerDocument.selection.createRange();

      if(typeof(range.htmlText)!="undefined") //html selected
       html=range.htmlText;
      else if(typeof(range.length)!="undefined") //image selected
       html=range.item(0).outerHTML;

      break;
     }
    }
   }
   else //Opera, Mozilla, Chrome, Safari
   {
    switch(node.nodeName.toLowerCase())
    {
     case "textarea":
     {
      self.error("Can't get HTML for node", node);

      break;
     }
     default: //html
     {
      var selection=node.ownerDocument.defaultView.getSelection();

      if(!selection.rangeCount) //nothing selected
       break;
      else
      {
       var range=selection.getRangeAt(0);
       var contents=range.cloneContents(); //copy selected nodes

       //append selected nodes contents to html
       for(var c=0;c<contents.childNodes.length;c++)
       {
        if(contents.childNodes[c].nodeName.toLowerCase()=="#text")
         html+=contents.childNodes[c].nodeValue;
        else //normal node
         html+=contents.childNodes[c].outerHTML;
       }

       break;
      }
     }
    }
   }
  }

  return html;
 }
}
